@@ -62,7 +62,10 @@ GQuark CONFIG_FILE_ERROR;
  * @CONFIG_FILE_ERROR_CREATED_ELEMENT: error when creating a new #VisuElement.
  * @CONFIG_FILE_ERROR_VALUE: error of file format, values read are out of bounds.
  * @CONFIG_FILE_ERROR_READ: error of file format (can't read variables...).
- * @CONFIG_FILE_ERROR_MISSING: error of file format (missing variables...).
+ * @CONFIG_FILE_ERROR_MISSING: error of file format (missing
+ * variables...).
+ * @CONFIG_FILE_ERROR_TAG: error dealing with a tag.
+ * @CONFIG_FILE_ERROR_MARKUP: error dealing with a markup (unkown one...).
  *
  * Possible errors when parsing a config file.
  */
@@ -74,7 +77,9 @@ typedef enum
     CONFIG_FILE_ERROR_CREATED_ELEMENT,
     CONFIG_FILE_ERROR_VALUE,
     CONFIG_FILE_ERROR_READ,
-    CONFIG_FILE_ERROR_MISSING
+    CONFIG_FILE_ERROR_MISSING,
+    CONFIG_FILE_ERROR_TAG,
+    CONFIG_FILE_ERROR_MARKUP
   } toolConfigFileError;
 
 /**
@@ -129,6 +134,40 @@ gboolean configFileRead_floatWithElement(gchar *line, int position, float *value
 					 guint size, VisuElement **ele, GError **error);
 
 
+/**
+ * configFileRead_integerFromTokens:
+ * @tokens: array of tokens resulting from a call to g_strsplit() with " " as separator ;
+ * @position: IN, the position of the beginning in @tokens ; OUT, one token
+ *            after the last read ;
+ * @values: allocated area to store read values ;
+ * @size: the number of floating point values to be read ;
+ * @lineId: the number of the line of the config
+ *          file which the @line argument is taken from ;
+ * @error: a location to store a possible reading error.
+ *
+ * Read @size integer values from @tokens, store them in @values and returns
+ * the new head in @tokens.
+ *
+ * Returns: TRUE if no error occured.
+ */
+gboolean configFileRead_integerFromTokens(gchar **tokens, int *position, int *values,
+					  guint size, int lineId, GError **error);
+/**
+ * configFileRead_integer:
+ * @line: string where values are read from ;
+ * @position: the number of the line of the config
+ *            file which the @line argument is taken from ;
+ * @values: allocated area to store read values ;
+ * @size: the number of floating point values to be read ;
+ * @error: a location to store a possible reading error.
+ *
+ * Read @size integers from @line and store them in @values.
+ *
+ * Returns: TRUE if no error occured.
+ */
+gboolean configFileRead_integer(gchar *line, int position, int *values,
+				guint size, GError **error);
+
 /**
  * configFileRead_booleanFromTokens:
  * @tokens: array of tokens resulting from a call to g_strsplit() with " " as separator ;
@@ -207,16 +246,20 @@ gboolean configFileRead_stringFromTokens(gchar **tokens, int *position, gchar **
  *            file which the @line argument is taken from ;
  * @values: a location to point on a gchar** ;
  * @size: the number of strings to be read ;
+ * @join: a boolean ;
  * @error: a location to store a possible reading error.
  *
- * Read @size strings from @line and @values points on them.
+ * Read @size strings from @line and @values points on them. If more
+ * strings than @size are available, an error is raised ; except if
+ * @join is TRUE. In that case, the method return @size tokens,
+ * joining all remaining ones.
  *
  * Returns: TRUE if no error occured, then @values point on an allocated
  *          memory area that is NULL terminated and that must be freed
  *          with g_strfreev().
  */
 gboolean configFileRead_string(gchar *line, int position, gchar ***values,
-			       guint size, GError **error);
+			       guint size, gboolean join, GError **error);
 /**
  * configFileRead_stringWithElement:
  * @line: string where values are read from ;
@@ -244,7 +287,7 @@ gboolean configFileRead_stringWithElement(gchar *line, int position, gchar ***va
  * @position: IN, the position of the beginning in @tokens ; OUT, one token
  *            after the last read ;
  * @values: allocated area to store read values ;
- * @size: the number of floating point values to be read ;
+ * @size: the number of #VisuElement to be read ;
  * @lineId: the number of the line of the config
  *          file which the @line argument is taken from ;
  * @error: a location to store a possible reading error.
@@ -254,7 +297,27 @@ gboolean configFileRead_stringWithElement(gchar *line, int position, gchar ***va
  *
  * Returns: TRUE if no error occured.
  */
-gboolean configFileRead_elementFromTokens(gchar **tokens, int *position, VisuElement **values,
-					  guint size, int lineId, GError **error);
+gboolean configFileRead_elementFromTokens(gchar **tokens, int *position,
+					  VisuElement **values, guint size,
+					  int lineId, GError **error);
+
+/**
+ * configFileClamp_float:
+ * @variable: a pointer to a storage for the value ;
+ * @value: the value to put in the storage ;
+ * @min: a lower bound ;
+ * @max: a upper bound.
+ *
+ * It puts value in a variable if value is in min and max
+ * or put min or max in if not. It return true if value
+ * is out of bounds. Min and max are inclusive values.
+ * If there is no max bounds then put max at a lower
+ * value than min and if there is no min bound, put min
+ * at a higher value than max.
+ *
+ * Returns: TRUE if the value is out of bounds.
+ */
+gboolean configFileClamp_float(float *variable, float value, float min, float max);
+
 
 #endif
