@@ -48,6 +48,30 @@
 #include <string.h>
 #include <stdlib.h>
 
+/**
+ * SECTION:toolFileFormat
+ * @short_description: Describes file format objects (name patterns,
+ * description, properties...).
+ *
+ * <para>When dealing with files, it is convenient to class then by
+ * formats, one format for JPEG files, one other for
+ * postscript... Such file formats are characterized by their
+ * description, explaining what they are, one or more name pattern
+ * (e.g. "*.jpg") and some properties (e.g. compression level for JPEG
+ * file).</para>
+ * <para>This module describes objects that can store all these
+ * informations and deals with them. To create a new file format, use
+ * fileFormatNew(). A #FileFormat object can be used in a
+ * GtkFileChooser object, using a GtkFileFilter. fileFormatGet_label()
+ * will return directly the label to give to the GtkFileFilter and the
+ * name patterns can be passed to it also.</para>
+ * <para>A file format property can be an integer, a boolean or a
+ * string. For instance, use fileFormatAdd_propertyBoolean() to add a
+ * boolean property to a given #FileFormat object. Then the stored
+ * value can be changed using fileFormatSet_propertyBoolean() and
+ * retrieve with fileFormatGet_propertyBoolean().</para>
+ */
+
 /* Private structure to store some properties associated to FileFormat objects. */
 struct FileFormatProperty_struct
 {
@@ -78,8 +102,8 @@ FileFormat* fileFormatNew(gchar* descr, gchar** patterns)
   lg = strlen(descr);
   if (lg > nbCharacterMax)
     {
-      fprintf(stderr, "WARNING! The descr parameter given to 'fileFormatNew'"
-	      " is bigger than %d characters and it will be truncated.\n", nbCharacterMax);
+      g_warning("The descr parameter given to 'fileFormatNew' is bigger than"
+		" %d characters and it will be truncated.\n", nbCharacterMax);
       lg = nbCharacterMax;
     }
 
@@ -92,12 +116,6 @@ FileFormat* fileFormatNew(gchar* descr, gchar** patterns)
   res->description = g_strndup(descr, lg);
 
   tmpStr = g_string_new(res->description);
-  if (!tmpStr)
-    {
-      allocationProblems();
-      fileFormatFree(res);
-      return (FileFormat*)0;
-    }
   g_string_append_printf(tmpStr, " (");
   n = 0;
   while (patterns[n])
