@@ -158,16 +158,20 @@ void shadeGet_valueTransformedInRGB(Shade *shade, float rgba[4], float value);
  * @heightVal: a floating point value ;
  * @minMax: the min & max value of the represented shaded color ;
  * @marks: an array of values bounded by @minMax ;
- * @nbMarks: the number of elements in the @marks array.
+ * @nbMarks: the number of elements in the @marks array ;
+ * @smin: a pointer to a floating point value or NULL.
  *
  * This method calls OpenGL primitive to draw a coloured legend on the rendering
  * surface. If @widthVal is negative then the default value in pixel is used. If @widthVal
  * is smaller than 1., the width is taken as a percentage of the width of the rendering area.
  * Finaly, if @widthVal is larger than 1., the width of the legend in pixel is the given value.
- * If @marks is not NULL, some marks are set on the legend for the given values.
+ * If @marks is not NULL, some marks are set on the legend for the
+ * given values. If @smin is given, the legend is drawn in log scale
+ * using the given value as secondary minimum.
+ *
  */
 void shadeDraw_legend(Shade *shade, float widthVal, float heightVal,
-		      float minMax[2], float *marks, int nbMarks);
+		      float minMax[2], float *marks, int nbMarks, float *smin);
 
 
 
