@@ -56,58 +56,87 @@
 #include <renderingBackend/visu_windowInterface.h>
 #include <openGLFunctions/objectList.h>
 #include <coreTools/toolColor.h>
+#include <coreTools/toolConfigFile.h>
 
 #include <math.h>
 
+/**
+ * SECTION:axes
+ * @short_description: Defines methods to draw axes.
+ *
+ * <para>The axes are the X, Y and Z lines drawn on the bottom right of the
+ * screen defining a given orthogonal basis set in which the box is
+ * projected.</para>
+ * <para>The axis may be different, depending on the rendering method
+ * currently used. For instance, when the spin is used, a projection
+ * of the colour scheme is added to the simple lines of the basis
+ * set. Besides that, axes are defined by their width (see
+ * axesSet_lineWidth()) and their colour (see
+ * axesSet_RGBValues()).</para>
+ */
+
 /* Parameters & resources*/
 /* This is a boolean to control is the axes is render or not. */
 #define FLAG_RESOURCE_AXES_USED   "axes_are_on"
 #define DESC_RESOURCE_AXES_USED   "Control if the axes are drawn ; boolean (0 or 1)"
 #define RESOURCE_AXES_USED_DEFAULT 0
-gboolean readAxesAreOn(gchar **lines, int nbLines,
-		       int position, GString *errorMessage);
+static gboolean readAxesAreOn(gchar **lines, int nbLines,
+			      int position, VisuData *dataObj, GError **error);
 /* A resource to control the color used to render the lines of the axes. */
 #define FLAG_RESOURCE_AXES_COLOR   "axes_color"
 #define DESC_RESOURCE_AXES_COLOR   "Define the color of the axes ; three floating point values (0. <= v <= 1.)"
-float axesRGB[3];
-gboolean readAxesColor(gchar **lines, int nbLines,
-		       int position, GString *errorMessage);
+static float axesRGB[3];
+static float axesRGBDefault[3] = {1.0, 0.5, 0.1};
+static gboolean readAxesColor(gchar **lines, int nbLines,
+			      int position, VisuData *dataObj, GError **error);
 /* A resource to control the width to render the lines of the axes. */
 #define FLAG_RESOURCE_AXES_LINE   "axes_line_width"
 #define DESC_RESOURCE_AXES_LINE   "Define the width of the lines of the axes ; one floating point values (1. <= v <= 10.)"
 #define RESOURCE_AXES_LINE_DEFAULT 1.
-float axesLineWidth;
-gboolean readAxesLineWidth(gchar **lines, int nbLines,
-			   int position, GString *errorMessage);
+static float axesLineWidth;
+static gboolean readAxesLineWidth(gchar **lines, int nbLines,
+				  int position, VisuData *dataObj, GError **error);
+/* A resource to control the stipple to render the lines of the box. */
+#define FLAG_RESOURCE_AXES_STIPPLE   "axes_line_stipple"
+#define DESC_RESOURCE_AXES_STIPPLE   "Dot scheme detail for the lines of the axes ; 0 < integer < 2^16"
+#define RESOURCE_AXES_STIPPLE_DEFAULT 65535
+static guint16 axesLineStipple;
+static gboolean readAxesLineStipple(gchar **lines, int nbLines,
+				    int position, VisuData *dataObj, GError **error);
 
 /* Export function that is called by visu_module to write the
    values of resources to a file. */
-gboolean exportResourcesAxes(GString *data, int *nbLinesWritten, VisuData *dataObj);
+static gboolean exportResourcesAxes(GString *data, int *nbLinesWritten,
+				    VisuData *dataObj);
 
-static float axesRGBDefault[3] = {1.0, 0.5, 0.1};
 
-void rebuildAxes(VisuData *dataObj);
+static OpenGLExtension* extensionAxes;
+
+
+static void rebuildAxes(VisuData *dataObj);
 
 /* This flag can be set internally to specify that axes should be recreated. */
-gboolean axesHaveBeenBuilt;
+static gboolean axesHaveBeenBuilt;
 
 /* Local callbacks */
 static void onAxesParametersChange(VisuData *dataObj, OpenGLView *view, gpointer data);
 static void onAxesConfigChange(GObject *visu, VisuData *dataObj, gpointer data);
-static void rebuildAxesOnResourcesLoaded(GObject *visu, gpointer data);
+static void rebuildAxesOnResourcesLoaded(GObject *visu, VisuData *dataObj,
+					 gpointer data);
 static void onDataReadySignal(GObject *visu, VisuData *dataObj, gpointer data);
 static void onDataNewSignal(GObject *visu, VisuData *dataObj, gpointer data);
 
 OpenGLExtension* initExtensionAxes()
 {
-  char *name = "Axes";
+  char *name = EXT_AXES_ID;
   char *description = _("Draw the axes at the bottom right of the rendering area.");
   int listAxes;
   VisuConfigFileEntry *resourceEntry;
 
   DBG_fprintf(stderr,"Initialising the axes OpenGL extension...\n");
   listAxes = openGLObjectList_new(1);
-  extensionAxes = OpenGLExtension_new(name, description, listAxes, rebuildAxes);
+  extensionAxes = OpenGLExtension_new(name, _(name), description,
+				      listAxes, rebuildAxes);
 
   resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_RESOURCE,
 					  FLAG_RESOURCE_AXES_USED,
@@ -121,6 +150,11 @@ OpenGLExtension* initExtensionAxes()
 					  FLAG_RESOURCE_AXES_LINE,
 					  DESC_RESOURCE_AXES_LINE,
 					  1, readAxesLineWidth);
+  resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_RESOURCE,
+					  FLAG_RESOURCE_AXES_STIPPLE,
+					  DESC_RESOURCE_AXES_STIPPLE,
+					  1, readAxesLineStipple);
+  visuConfigFileSet_version(resourceEntry, 3.4f);
   visuConfigFileAdd_exportFunction(VISU_CONFIGFILE_RESOURCE,
 				   exportResourcesAxes);
 
@@ -129,7 +163,8 @@ OpenGLExtension* initExtensionAxes()
   extensionAxes->used = RESOURCE_AXES_USED_DEFAULT;
   OpenGLExtensionSet_priority(extensionAxes, OPENGL_EXTENSION_PRIORITY_LAST);
   OpenGLExtensionSet_saveOpenGLState(extensionAxes, TRUE);
-  axesLineWidth = RESOURCE_AXES_LINE_DEFAULT;
+  axesLineWidth   = RESOURCE_AXES_LINE_DEFAULT;
+  axesLineStipple = RESOURCE_AXES_STIPPLE_DEFAULT;
 
   g_signal_connect(G_OBJECT(visu), "dataReadyForRendering",
 		   G_CALLBACK(onDataReadySignal), (gpointer)0);
@@ -148,7 +183,7 @@ OpenGLExtension* initExtensionAxes()
 }
 
 /* Method used to change the value of the parameter axes_color. */
-int axesSet_RGBValues(float rgb[3], int mask)
+gboolean axesSet_RGBValues(float rgb[3], int mask)
 {
   int diff = 0;
   
@@ -168,51 +203,46 @@ int axesSet_RGBValues(float rgb[3], int mask)
       diff = 1;
     }
   if (!diff)
-    return 0;
+    return FALSE;
 
-  /* Il faut recrer la bote puisque des param ont changs. */
   axesHaveBeenBuilt = FALSE;
-/*   axesDraw((GObject*)0, (gpointer)0); */
-  return extensionAxes->used;
+  return (gboolean)extensionAxes->used;
 }
 
 /* Method used to change the value of the parameter axes_line_width. */
-int axesSet_lineWidth(float width)
+gboolean axesSet_lineWidth(float width)
 {
   if (width < 1. || width > 10. || width == axesLineWidth)
-    return 0;
+    return FALSE;
 
   axesLineWidth = width;
-  /* Il faut recrer la bote puisque des param ont changs. */
   axesHaveBeenBuilt = FALSE;
-/*   axesDraw((GObject*)0, (gpointer)0); */
-  return extensionAxes->used;
+  return (gboolean)extensionAxes->used;
+}
+gboolean axesSet_lineStipple(guint16 stipple)
+{
+  if (stipple == axesLineStipple)
+    return FALSE;
+
+  axesLineStipple = stipple;
+  axesHaveBeenBuilt = FALSE;
+  return (gboolean)extensionAxes->used;
 }
 
 /* Method used to change the value of the parameter axes_is_on. */
-int axesSet_areOn(int value)
+gboolean axesSet_areOn(gboolean value)
 {
   if (value == extensionAxes->used)
-    return 0;
+    return FALSE;
 
   extensionAxes->used = value;
   return (value && !axesHaveBeenBuilt);
 }
 
 /* Get methods. */
-float axesGet_redValue()
+float* axesGet_RGBvalues()
 {
-  return axesRGB[0];
-}
-
-float axesGet_greenValue()
-{
-  return axesRGB[1];
-}
-
-float axesGet_blueValue()
-{
-  return axesRGB[2];
+  return axesRGB;
 }
 
 int axesGet_areOn()
@@ -224,31 +254,32 @@ float axesGet_lineWidth()
 {
   return axesLineWidth;
 }
+guint16 axesGet_lineStipple()
+{
+  return axesLineStipple;
+}
 
 
 /****************/
 /* Private part */
 /****************/
 
-void rebuildAxes(VisuData *dataObj)
+static void rebuildAxes(VisuData *dataObj)
 {
   openGLText_rebuildFontList();
   /* Set flag to false to force recreation of axes. */
   axesHaveBeenBuilt = FALSE;
   axesDraw(dataObj);
 }
-static void rebuildAxesOnResourcesLoaded(GObject *visu, gpointer data)
+static void rebuildAxesOnResourcesLoaded(GObject *visu _U_, VisuData *dataObj,
+					 gpointer data _U_)
 {
-  GList *tmpLst;
-
-  tmpLst = visuDataGet_allObjects();
-  while (tmpLst)
-    {
-      axesDraw(VISU_DATA(tmpLst->data));
-      tmpLst = g_list_next(tmpLst);
-    }
+  if (!dataObj)
+    return;
+  axesDraw(dataObj);
 }
-static void onDataReadySignal(GObject *visu, VisuData *dataObj, gpointer data)
+static void onDataReadySignal(GObject *visu _U_, VisuData *dataObj,
+			      gpointer data _U_)
 {
   if (!dataObj)
     return;
@@ -256,7 +287,8 @@ static void onDataReadySignal(GObject *visu, VisuData *dataObj, gpointer data)
   axesHaveBeenBuilt = FALSE;
   axesDraw(dataObj);
 }
-static void onDataNewSignal(GObject *visu, VisuData *dataObj, gpointer data)
+static void onDataNewSignal(GObject *visu _U_, VisuData *dataObj,
+			    gpointer data _U_)
 {
   g_signal_connect(G_OBJECT(dataObj), "OpenGLNearFar",
 		   G_CALLBACK(onAxesParametersChange), (gpointer)0);
@@ -277,7 +309,7 @@ void drawAxes(float length, GLsizei w, GLsizei h, float width,
 	      char *legend, gboolean long_axes) 
 {
   glLineWidth(width);
-  glColor3f(axesRGB[0], axesRGB[1], axesRGB[2]);
+  glColor3fv(axesRGB);
   glBegin(GL_LINES);
   glVertex3f(long_axes ? -length : 0, 0.0f, 0.0f); glVertex3f(length, 0.0f, 0.0f);
   glVertex3f(0.0f, long_axes ? -length : 0, 0.0f); glVertex3f(0.0f, length, 0.0f);
@@ -337,21 +369,18 @@ void draw_coloured_cone(double r, double h, int n, float phi_prime_zero)
 
   glRotatef(-90, 1, 0, 0);
 
-#define TWOPI 6.283185307179586
-#define PID2 1.570796326794897
-
   hsv[1] = 0;
   hsv[2] = 1;
 
   for (j=0;j<n/2;j++) 
     {
-      theta1 = j * TWOPI / n - PID2;
-      theta2 = (j + 1) * TWOPI / n - PID2;
+      theta1 = j * 2 * G_PI / n - G_PI_2;
+      theta2 = (j + 1) * 2 * G_PI / n - G_PI_2;
       
       glBegin(GL_QUAD_STRIP);
       for (i=0;i<=n;i++) 
 	{
-	  theta3 = i * TWOPI / n;
+	  theta3 = i * 2 * G_PI / n;
 	  
 	  hsv[0] = /*1-*/(float)i/(float)n;
 
@@ -438,14 +467,15 @@ void draw_coloured_cone(double r, double h, int n, float phi_prime_zero)
 /* /\*   glDisable(GL_POLYGON_SMOOTH); *\/ */
 
 /* } */
-static void onAxesConfigChange(GObject *visu, VisuData *dataObj, gpointer data)
+static void onAxesConfigChange(GObject *visu _U_, VisuData *dataObj, gpointer data)
 {
   g_return_if_fail(dataObj);
 
   onAxesParametersChange(dataObj, visuDataGet_openGLView(dataObj), data);
 }
 
-static void onAxesParametersChange(VisuData *dataObj, OpenGLView *view, gpointer data)
+static void onAxesParametersChange(VisuData *dataObj, OpenGLView *view _U_,
+				   gpointer data _U_)
 {
   axesHaveBeenBuilt = FALSE;
   axesDraw(dataObj);
@@ -456,8 +486,7 @@ void axesDraw(VisuData *dataObj)
   float length;
   GLsizei w, h;
   GLint xx, yy;
-  double mini;
-  double maxi;
+  double mini, maxi, near, far;
   OpenGLView *view;
 
   /* Nothing to draw; */
@@ -471,13 +500,17 @@ void axesDraw(VisuData *dataObj)
   DBG_fprintf(stderr, "Extension axes : creating axes.\n");
 
   view = visuDataGet_openGLView(dataObj);
-  length = 0.5f*view->box->extens/1.91f;
-  w = 0.2*view->window->width;
-  h = 0.2*view->window->height;
+  length = 0.26178f * view->camera->length0;
+  w = 0.2f * MIN(view->window->width, view->window->height);
+  h = w;
   xx = view->window->width - w;
   yy = 0;
-  mini = -0.5 * view->box->extens/view->camera->gr;
+  mini = -0.5f * view->camera->length0 *
+    (view->camera->d_red - 1.f) / view->camera->d_red;
   maxi = -mini;
+  near = far = view->camera->d_red * view->camera->length0;
+  near -= view->camera->length0;
+  far  += view->camera->length0;
 
   axesHaveBeenBuilt = TRUE;
 
@@ -494,8 +527,19 @@ void axesDraw(VisuData *dataObj)
   glDisable(GL_FOG);
 
   /* Resetting the depth buffer. */
-  glClear(GL_DEPTH_BUFFER_BIT);
+  glDisable(GL_DEPTH_TEST);
 
+  if (axesLineStipple != 65535)
+    {
+      glEnable(GL_LINE_STIPPLE);
+      glLineStipple(1, axesLineStipple);
+    }
+
+  glMatrixMode(GL_PROJECTION);
+  glPushMatrix();
+  glLoadIdentity();
+  glFrustum(mini, maxi, mini, maxi, near, far);
+  glMatrixMode(GL_MODELVIEW);
   if(pointerOnRenderingSpinMethod == getRenderingMethodInUse())
     {
       float phi_prime_zero, phi_a, theta_a;
@@ -504,34 +548,17 @@ void axesDraw(VisuData *dataObj)
       phi_a          = rspin_getGlobalResource_float(spin_globalConePhi);
       theta_a        = rspin_getGlobalResource_float(spin_globalConeTheta);
 
-      glMatrixMode(GL_PROJECTION);
-      glPushMatrix();
-      glLoadIdentity();
-      glFrustum(mini, maxi, mini, maxi, view->window->near, view->window->far);
-      glMatrixMode(GL_MODELVIEW);
-      glPushMatrix();
       glViewport(xx, yy, w, h);
 
       /* Draw the first color cone */
+      drawAxes(1.5*length, w, h, axesLineWidth, "front", TRUE);
+
       glPushMatrix();
       glRotatef(phi_a, 0, 0, 1);
       glRotatef(theta_a, 0, 1, 0);
       draw_coloured_cone(length, 1.2*length, 16, phi_prime_zero);
       glPopMatrix();
 
-      drawAxes(1.5*length, w, h, axesLineWidth, "front", TRUE);
-
-      glPopMatrix();
-      glMatrixMode(GL_PROJECTION);
-      glPopMatrix();
-      glMatrixMode(GL_MODELVIEW);
-
-      glMatrixMode(GL_PROJECTION);
-      glPushMatrix();
-      glLoadIdentity();
-      glFrustum(mini, maxi, mini, maxi, view->window->near, view->window->far);
-      glMatrixMode(GL_MODELVIEW);
-      glPushMatrix();
       glViewport(xx, yy+h, w, h);
 
       /* Enabling front culling and drawing the second color cone */
@@ -544,84 +571,56 @@ void axesDraw(VisuData *dataObj)
       glPopMatrix();
 
       drawAxes(1.5*length, w, h, axesLineWidth, "back", TRUE);
-
-      /* Back to the main viewport. */
-      glViewport(0, 0, view->window->width, view->window->height);
-
-      glPopMatrix();
-      glMatrixMode(GL_PROJECTION);
-      glPopMatrix();
-      glMatrixMode(GL_MODELVIEW);
     }
   else if(pointerOnRenderingAtomicMethod == getRenderingMethodInUse())
     {
-      glMatrixMode(GL_PROJECTION);
-      glPushMatrix();
-      glLoadIdentity();
-      glFrustum(mini, maxi, mini, maxi, view->window->near, view->window->far);
-      glMatrixMode(GL_MODELVIEW);
-      glPushMatrix();
       glViewport(xx, yy, w, h);
 
       drawAxes(length, w, h, axesLineWidth, NULL, FALSE);
-
-      /* Back to the main viewport. */
-      glViewport(0, 0, view->window->width, view->window->height);
-
-      glPopMatrix();
-      glMatrixMode(GL_PROJECTION);
-      glPopMatrix();
-      glMatrixMode(GL_MODELVIEW);
     }
+  glMatrixMode(GL_PROJECTION);
+  glPopMatrix();
+  glMatrixMode(GL_MODELVIEW);
+  /* Back to the main viewport. */
+  glViewport(0, 0, view->window->width, view->window->height);
+
   glEndList();
 }
 
 /* Parameters & resources*/
 
 /* This is a boolean to control is the axes is render or not. */
-gboolean readAxesAreOn(gchar **lines, int nbLines,
-		       int position, GString *errorMessage)
+static gboolean readAxesAreOn(gchar **lines, int nbLines,
+			      int position, VisuData *dataObj _U_, GError **error)
 {
-  int res, val;
-/*   VisuData *data; */
+  gboolean val;
 
-/*   if (visuRenderingWindowGet_current()) */
-/*     data = visuRenderingWindowGet_visuData(visuRenderingWindowGet_current()); */
-/*   else */
-/*     data = (VisuData*)0; */
-  res = sscanf(lines[0],"%d", &val);
-  if (res != 1)
-    {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d,"
-					       " 1 boolean value must appear"
-					       " after the %s markup.\n"),
-			       position, FLAG_RESOURCE_AXES_USED);
-      axesSet_areOn(RESOURCE_AXES_USED_DEFAULT);
-      return FALSE;
-    }
+  g_return_val_if_fail(nbLines == 1, FALSE);
 
+  if (!configFileRead_boolean(lines[0], position, &val, 1, error))
+    return FALSE;
   axesSet_areOn(val);
   return TRUE;
 }
 /* A resource to control the color used to render the lines of the axes. */
-gboolean readAxesColor(gchar **lines, int nbLines,
-		       int position, GString *errorMessage)
+static gboolean readAxesColor(gchar **lines, int nbLines,
+			      int position, VisuData *dataObj _U_, GError **error)
 {
-  int res;
   float rgb[3];
   
-  res = sscanf(lines[0],"%f %f %f", &rgb[0], &rgb[1], &rgb[2]);
-  if (res != 3 ||
-      visuConfigFileSet_floatValue(&rgb[0], rgb[0], 0., 1.) ||
-      visuConfigFileSet_floatValue(&rgb[1], rgb[1], 0., 1.) ||
-      visuConfigFileSet_floatValue(&rgb[2], rgb[2], 0., 1.))
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
+  if (!configFileRead_float(lines[0], position, rgb, 3, error))
+    return FALSE;
+
+  if (configFileClamp_float(&rgb[0], rgb[0], 0., 1.) ||
+      configFileClamp_float(&rgb[1], rgb[1], 0., 1.) ||
+      configFileClamp_float(&rgb[2], rgb[2], 0., 1.))
     {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d, 3 floating "
-					       "point values (0 <= v <= 1) must appear after"
-					       " the %s markup.\n"),
-			       position, FLAG_RESOURCE_AXES_COLOR);
+      *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_VALUE,
+			   _("Parse error at line %d: 3 floating points"
+			     "(0 <= v <= 1) must appear after the %s markup.\n"),
+			   position, FLAG_RESOURCE_AXES_COLOR);
       axesSet_RGBValues(axesRGBDefault, MASK_RGB_ALL);
       return FALSE;
     }
@@ -630,21 +629,21 @@ gboolean readAxesColor(gchar **lines, int nbLines,
   return TRUE;
 }
 /* A resource to control the width to render the lines of the axes. */
-gboolean readAxesLineWidth(gchar **lines, int nbLines,
-			   int position, GString *errorMessage)
+static gboolean readAxesLineWidth(gchar **lines, int nbLines,
+				  int position, VisuData *dataObj _U_, GError **error)
 {
-  int res;
   float width;
   
-  res = sscanf(lines[0],"%f", &width);
-  if (res != 1 ||
-      visuConfigFileSet_floatValue(&width, width, 1., 10.))
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
+  if (!configFileRead_float(lines[0], position, &width, 1, error))
+    return FALSE;
+  if (configFileClamp_float(&width, width, 1., 10.))
     {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d, 1 floating "
-					       "point value (1 <= v <= 10) must appear after"
-					       " the %s markup.\n"),
-			       position, FLAG_RESOURCE_AXES_COLOR);
+      *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_VALUE,
+			   _("Parse error at line %d: 1 floating point"
+			     "(1 <= v <= 10) must appear after the %s markup.\n"),
+			   position, FLAG_RESOURCE_AXES_LINE);
       axesSet_lineWidth(RESOURCE_AXES_LINE_DEFAULT);
       return FALSE;
     }
@@ -652,10 +651,25 @@ gboolean readAxesLineWidth(gchar **lines, int nbLines,
 
   return TRUE;
 }
+/* A resource to control the width to render the lines of the axes. */
+static gboolean readAxesLineStipple(gchar **lines, int nbLines,
+				    int position, VisuData *dataObj _U_, GError **error)
+{
+  gint stipple;
+  
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
+  if (!configFileRead_integer(lines[0], position, &stipple, 1, error))
+    return FALSE;
+  axesSet_lineStipple((guint16)stipple);
+
+  return TRUE;
+}
 
 /* Export function that is called by visu_module to write the
    values of resources to a file. */
-gboolean exportResourcesAxes(GString *data, int *nbLinesWritten, VisuData *dataObj)
+static gboolean exportResourcesAxes(GString *data, int *nbLinesWritten,
+				    VisuData *dataObj _U_)
 {
   g_string_append_printf(data, "# %s\n", DESC_RESOURCE_AXES_USED);
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_AXES_USED);
@@ -667,6 +681,9 @@ gboolean exportResourcesAxes(GString *data, int *nbLinesWritten, VisuData *dataO
   g_string_append_printf(data, "# %s\n", DESC_RESOURCE_AXES_LINE);
   g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_AXES_LINE);
   g_string_append_printf(data, "    %4.0f\n", axesLineWidth);
+  g_string_append_printf(data, "# %s\n", DESC_RESOURCE_AXES_STIPPLE);
+  g_string_append_printf(data, "%s:\n", FLAG_RESOURCE_AXES_STIPPLE);
+  g_string_append_printf(data, "    %d\n", axesLineStipple);
   g_string_append_printf(data, "\n");
   *nbLinesWritten += 10;
 
