@@ -49,7 +49,7 @@
 
 
 /**
- * initExtensionFog
+ * extFogAndBGInit:
  *
  * It initialises all variables of the OpenGL extension.
  * It creates all resources and parameters introduced by
@@ -58,14 +58,22 @@
  * be called automatically by the initOpenGLExtensions() at V_Sim
  * startup.
  *
- * Returns a pointer to the OpenGLExtension it created or
+ * Returns: a pointer to the OpenGLExtension it created or
  * NULL otherwise.
  */
-OpenGLExtension* initExtensionFogAndBGColor();
+OpenGLExtension* extFogAndBGInit();
 
 /**
- * setFogRGBValues:
- * @rgb: a three floats array with values (0 <= values <= 1) for the
+ * EXT_FOG_AND_BG_ID:
+ *
+ * The id used to identify this extension, see
+ * OpenGLExtensionRebuild_list() for instance.
+ */
+#define EXT_FOG_AND_BG_ID "FogAndColor"
+
+/**
+ * fogSet_values:
+ * @rgba: a three floats array with values (0 <= values <= 1) for the
  * red, the green and the blue color. Only values specified by the mask
  * are really relevant.
  * @mask: use #MASK_RGB_R, #MASK_RGB_G, #MASK_RGB_B, #MASK_RGB_ALL or a
@@ -76,32 +84,32 @@ OpenGLExtension* initExtensionFogAndBGColor();
  * This color is actually used only if fog_color_is_specific is set to 1,
  * use setFogColorSpecific() to do that.
  *
- * Returns 0 if redrawing is not necessary.
+ * Returns: 0 if redrawing is not necessary.
  * 1 if the rendering window must be redrawn.
  */
-int setFogRGBValues(float rgb[3], int mask);
+int fogSet_values(float rgba[4], int mask);
 /**
- * setFogIsOn:
- * @value: 1 if fog must be rendered, 0 otherwise.
+ * fogSet_isOn:
+ * @value: TRUE if fog must be rendered, FALSE otherwise.
  *
  * Method used to change the value of the parameter fog_is_on.
  *
- * Returns 0 if redrawing is not necessary.
- * 1 if the rendering window must be redrawn.
+ * Returns: FALSE if redrawing is not necessary.
+ * TRUE if the rendering window must be redrawn.
  */
-int setFogIsOn(int value);
+gboolean fogSet_isOn(gboolean value);
 /**
- * setFogColorSpecific:
- * @value: 1 if fog is rendered with its own color specified
- * by setFogRGBValues() or 0 if the fog uses the background
+ * fogSet_useSpecificColor:
+ * @value: TRUE if fog is rendered with its own color specified
+ * by setFogRGBValues() or FALSE if the fog uses the background
  * color.
  *
  * Method used to change the value of the parameter fog_color_is_specific.
  *
- * Returns: 1 if createFog() should be called. In all cases, 'OpenGLAskForReDraw'
+ * Returns: TRUE if fogCreate() should be called. In all cases, 'OpenGLAskForReDraw'
  *          signal should then be emitted.
  */
-int setFogColorSpecific(int value);
+gboolean fogSet_useSpecificColor(gboolean value);
 /**
  * MASK_FOG_START:
  *
@@ -119,7 +127,7 @@ int setFogColorSpecific(int value);
  */
 #define MASK_FOG_END (1 << 1)
 /**
- * setFogStartEndValues:
+ * fogSet_startEndValues:
  * @startEnd: a two floats array with values (0 <= values <= 1) for the
  * beging and the ending of the fog position. Only values specified by the mask
  * are really relevant.
@@ -129,57 +137,54 @@ int setFogColorSpecific(int value);
  * Method used to change the value of the parameters fog_start and
  * fog_end.
  *
- * Returns: 1 if createFog() should be called and then 'OpenGLAskForReDraw'
+ * Returns: TRUE if fogCreate() should be called and then 'OpenGLAskForReDraw'
  *          signal be emitted.
  */
-int setFogStartEndValues(float startEnd[2], int mask);
+gboolean fogSet_startEndValues(float startEnd[2], int mask);
 
 /**
- * getFogRedValue:
+ * fogGet_value:
+ * @rgba: a storage for four values.
  *
- * Returns: the red value of the specific color of the fog.
+ * Read the RGBA value of the specific fog colour (in [0;1]).
  */
-float getFogRedValue();
+void fogGet_value(float rgba[4]);
 /**
- * getFogGreenValue:
+ * fogGet_isOn:
  *
- * Returns: the green value of the specific color of the fog.
+ * Read if fog is used or not.
+ *
+ * Returns: TRUE if the fog is rendered, FALSE otherwise.
  */
-float getFogGreenValue();
+gboolean fogGet_isOn();
 /**
- * getFogBlueValue:
+ * fogGet_useSpecificColor:
  *
- * Returns: the blue value of the specific color of the fog.
- */
-float getFogBlueValue();
-/**
- * isFogOn:
+ * Read if fog uses a specific colour or not.
  *
- * Returns: 1 if the fog is rendered, 0 otherwise.
- */
-int isFogOn();
-/**
- * getFogColorSpecific:
- *
- * Returns: 1 if the fog uses its own color or 0 if it uses
+ * Returns: TRUE if the fog uses its own color or FALSE if it uses
  * the color of the background.
  */
-int getFogColorSpecific();
+gboolean fogGet_useSpecificColor();
 /**
- * getFogStart:
+ * fogGet_start:
+ *
+ * Read the starting value of the fog (in [0;1]).
  *
  * Returns: the position where the fog starts.
  */
-float getFogStart();
+float fogGet_start();
 /**
- * getFogEnd:
+ * fogGet_end:
+ *
+ * Read the ending value of the fog (in [0;1]).
  *
  * Returns: the position where the fog ends.
  */
-float getFogEnd();
+float fogGet_end();
 /**
- * setBgRGBValues:
- * @rgb: a three floats array with values (0 <= values <= 1) for the
+ * bgSet_values:
+ * @rgba: a three floats array with values (0 <= values <= 1) for the
  * red, the green and the blue color. Only values specified by the mask
  * are really relevant.
  * @mask: use #MASK_RGB_R, #MASK_RGB_G, #MASK_RGB_B, #MASK_RGB_ALL or a
@@ -188,41 +193,46 @@ float getFogEnd();
  *
  * Method used to change the value of the parameter background_color.
  *
- * Returns 0 if redrawing is not necessary.
+ * Returns: 0 if redrawing is not necessary.
  * 1 if the rendering window must be redrawn.
  */
-int setBgRGBValues(float rgb[3], int mask);
+int bgSet_values(float rgba[3], int mask);
 /**
- * getBgRedValue:
+ * bgGet_value:
+ * @rgba: a storage for four values.
  *
- * Returns: the red value of the current background color.
+ * Read the RGBA value of the specific background colour (in [0;1]).
  */
-float getBgRedValue();
+void bgGet_value(float rgba[4]);
+
 /**
- * getBgGreenValue:
+ * fogCreate:
+ * @data: the #VisuData object the fog apply to.
  *
- * Returns: the gree value of the current background color.
+ * Call the OpenGL routine to enable and initialise the fog.
  */
-float getBgGreenValue();
+void fogCreate(VisuData *data);
 /**
- * getBgBlueValue:
+ * fogCreate_color:
  *
- * Returns: the blue value of the current background color.
+ * Call the OpenGL routines that change the colour of the fog.
  */
-float getBgBlueValue();
-
-void createFog(VisuData *data);
-void createFogColor();
-
-
-
-
-/****************/
-/* Private part */
-/****************/
-
-OpenGLExtension* extensionFog;
-
+void fogCreate_color();
 
+/**
+ * bgSet_image:
+ * @imageData: raw image data in RGB or RGBA format ;
+ * @width: the width ;
+ * @height: the height ;
+ * @alpha: TRUE if the image is RGBA ;
+ * @title: an optional title (can be NULL).
+ *
+ * Draw the @imageData on the background. The image is scaled to the
+ * viewport dimensions, keeping the width/height ratio. If @title is
+ * not NULL, the title is also printed on the background. The image
+ * data are copied and can be free after this call.
+ */
+void bgSet_image(const guchar *imageData, guint width, guint height,
+		 gboolean alpha, const gchar *title);
 
 #endif
