@@ -50,6 +50,36 @@
 #include <renderingBackend/visu_windowInterface.h>
 #include <coreTools/toolColor.h>
 
+/**
+ * SECTION:dataFile
+ * @short_description: Adds a possibility to colorize nodes depending
+ * on data read in an input file.
+ *
+ * <para>With this module, it is possible to colorize nodes depending
+ * on data read in an input file. An input file can be associated to a
+ * #VisuData object using dataFileSet_file(). Doing this, the
+ * rendering is changed and nodes are colorized following a scheme
+ * describe later. To turn off colorization without removing the data
+ * file (for temporary turn off for instance), use
+ * dataFileSet_used().</para>
+ * <para>The input file must have the same numbers of uncommented
+ * lines as there are nodes in the #VisuData associated with. If less
+ * data is given, missing data are treaded as min values data. The
+ * input data file can has as much column as desired. The colorization
+ * is based on a linear color transformation. This transformation is
+ * applied on color channel in RGB mode or in HSV mode. Resulting
+ * color is given by : [resulting color vect] = [vectB] + [input
+ * data][vectA], where [input data] are input data scaled to [0;1]. It
+ * is possible to choose which column multiplies which color
+ * channel.</para>
+ * <para>Using dataFileApply_hideOnMinValue(), it is possible to hide
+ * some nodes depending on given input data. If the @column argument
+ * is -1, it unmask all nodes.</para>
+ * <para>Using this module stores several #Shade that can be used by
+ * other modules after being initialized using
+ * dataFileBuild_presetShadeList().</para>
+ */
+
 struct DataFile_struct
 {
   /* Set to TRUE to colorize. */
@@ -85,6 +115,7 @@ typedef struct DataFile_struct DataFile;
    This cache is usefull to avoid too much visuDataGet_property().
    This cache is refresh as soon as currentVisuData is changed. */
 DataFile *cacheDataFile;
+VisuData *cacheVisuData;
 
 /* Default values */
 #define DATAFILE_NB_COLUMN_DEFAULT  -1
@@ -97,28 +128,34 @@ DataFile *cacheDataFile;
 
 /* Internal variables. */
 static DataNode *dataNode;
+static GValue dataValue = {0, {{0}, {0}}};
 
+/* local methods. */
 void dataFileFree(gpointer data);
 DataFile* dataFileNew(VisuData *visuData);
 void colorFromUserData(VisuData *visuData, float rgba[4], VisuElement *ele, VisuNode* node);
 float dataFileGet_valuesFromData(VisuData *visuData, int column, float fromVal);
+static void freeData(gpointer obj, gpointer data);
+static gpointer newOrCopyData(gconstpointer orig, gpointer user_data);
 
 int initDataFileModule()
 {
   cacheDataFile = (DataFile*)0;
+  cacheVisuData = (VisuData*)0;
 
   /* Register a new NodeData. */
   dataNode = nodeDataNew(DATAFILE_ID, G_TYPE_FLOAT);
   nodeDataSet_label(dataNode, _("Colorisation data"));
 
+  g_value_init(&dataValue, G_TYPE_POINTER);
+
   return 1;
 }
 
 void dataFileReDraw(VisuData *data)
 {
   visuData_createAllNodes(data);
-  g_signal_emit(visu, VISU_GET_CLASS(visu)->OpenGLAskForReDraw_signal_id,
-		0 , NULL);
+  g_idle_add(visuObjectRedraw, (gpointer)0);
 }
 
 
@@ -148,6 +185,11 @@ DataFile* dataFileNew(VisuData *visuData)
     }
   visuDataSet_propertyWithDestroyFunc(visuData, "dataColor_parameters",
 				      (gpointer)dataFile, dataFileFree);
+
+  /* Set the cache. */
+  cacheDataFile = dataFile;
+  cacheVisuData = visuData;
+
   return dataFile;
 }
 
@@ -163,6 +205,14 @@ void dataFileFree(gpointer data)
   if (dataFile->file)
     g_free(dataFile->file);
   g_free(dataFile);
+
+  /* Update the cache. */
+  if (dataFile == cacheDataFile)
+    {
+      cacheDataFile = (DataFile*)0;
+      cacheVisuData = (VisuData*)0;
+    }
+
 }
 
 void dataFileActivate(VisuData *visuData, gboolean status)
@@ -181,9 +231,38 @@ void dataFileActivate(VisuData *visuData, gboolean status)
     }
 }
 
-int dataFileSet_file(VisuData *attachedVisuData, char* filename, GString *message, int *errors)
+static void freeData(gpointer obj, gpointer data)
 {
-  int i, j, res, err, nb;
+#if GLIB_MINOR_VERSION > 9
+  g_slice_free1(sizeof(float) * GPOINTER_TO_INT(data), obj);
+#else
+  if (GPOINTER_TO_INT(data) > 0)
+    g_free(obj);
+#endif
+}
+static gpointer newOrCopyData(gconstpointer orig, gpointer user_data)
+{
+  float *data;
+  int nb;
+
+  nb = GPOINTER_TO_INT(user_data);
+#if GLIB_MINOR_VERSION > 9
+  data = g_slice_alloc(sizeof(float) * nb);
+#else
+  data = g_malloc(sizeof(float) * nb);
+#endif
+  if (orig)
+    memcpy(data, orig, sizeof(float) * nb);
+  else
+    memset(data, 0, sizeof(float) * nb);
+
+  return (gpointer)data;
+}
+
+int dataFileSet_file(VisuData *attachedVisuData, char* filename,
+		     gboolean *new, GString *message, int *errors)
+{
+  int i, err, nb, colUsed[3];
   FILE *file;
   char line[MAX_LINE_LENGTH];
   char *msg;
@@ -192,9 +271,18 @@ int dataFileSet_file(VisuData *attachedVisuData, char* filename, GString *messag
   gchar **dataRead;
   gchar *fileUTF8;
   DataFile *dataFile;
+  VisuDataIter iter;
+  gboolean voidLine;
+  VisuNodeArray *nodeArray;
+  VisuNodeProperty *prop;
+#if DEBUG == 1
+  GTimer *timer;
+  gulong fractionTimer;
+#endif
 
-  DBG_fprintf(stderr, "Visu dataFile : set data file to '%s'.\n", filename);
-  g_return_val_if_fail(attachedVisuData && filename && message, 0);
+  DBG_fprintf(stderr, "Visu dataFile: set data file to '%s'.\n", filename);
+  g_return_val_if_fail(attachedVisuData && filename && message && new, 0);
+  *new = TRUE;
 
   /* Test if this VisuData has already been colorized by testing
      the existence of the 'dataColor_used' property. If not, all required
@@ -202,6 +290,8 @@ int dataFileSet_file(VisuData *attachedVisuData, char* filename, GString *messag
   dataFile = visuDataGet_property(attachedVisuData, "dataColor_parameters");
   if (!dataFile)
     dataFile = dataFileNew(attachedVisuData);
+  else
+    *new = FALSE;
   g_return_val_if_fail(dataFile, 0);
 
   if (!errors)
@@ -211,7 +301,10 @@ int dataFileSet_file(VisuData *attachedVisuData, char* filename, GString *messag
   /* erase previously stored dataFile informations. */
   dataFile->nbColumns = DATAFILE_NB_COLUMN_DEFAULT;
   for (i = 0; i < 3; i++)
-    dataFile->colUsed[i] = -1;
+    {
+      colUsed[i] = dataFile->colUsed[i];
+      dataFile->colUsed[i] = -1;
+    }
   if (dataFile->file)
     {
       g_free(dataFile->file);
@@ -225,49 +318,76 @@ int dataFileSet_file(VisuData *attachedVisuData, char* filename, GString *messag
 
   /* We clear previously stored user data from the current
      visuData structure. */
-  if (attachedVisuData->nodes)
-    {
-      visuDataRemove_nodeProperty(attachedVisuData, DATAFILE_ID);
-      nodeDataSet_used(dataNode, attachedVisuData, 0);
-    }
+  nodeArray = visuDataGet_nodeArray(attachedVisuData);
+  visuNodeFree_property(nodeArray, DATAFILE_ID);
+  prop = (VisuNodeProperty*)0;
+  nodeDataSet_used(dataNode, attachedVisuData, 0);
 
   file = fopen(filename, "r");
   if (!file)
     {
-      fileUTF8 = getStringInUTF8(filename);
-      g_string_append_printf(message, _("WARNING! Can't find the '%s' data file.\n"),
-			     fileUTF8);
-      g_free(fileUTF8);
+      fileUTF8 = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
+      if (fileUTF8)
+	{
+	  g_string_append_printf(message, _("Can't find the '%s' data file.\n"),
+				 fileUTF8);
+	  g_free(fileUTF8);
+	}
+      else
+	g_warning("Can't convert '%s' to UTF8.\n", filename);
       *errors = 1;
       return 0;
     }
   nAtome = 0;
-  visuDataAdd_nodeProperty(attachedVisuData, DATAFILE_ID, freeFloat);
-  while (!feof(file))
+
+#if DEBUG == 1
+  timer = g_timer_new();
+  g_timer_start(timer);
+#endif
+  visuDataIter_new(attachedVisuData, &iter);
+  for (visuDataIter_start(attachedVisuData, &iter); iter.node;
+       visuDataIter_next(attachedVisuData, &iter))
     {
-      msg = fgets(line, MAX_LINE_LENGTH, file);
-      if (!msg)
-	continue;
-      if (line[0] == '#' || line[0] == '!' || line[0] == '\n')
-	continue;
-      /* Test the number of columns. */
+      /* If the file is not finished, we read until a non-void line
+	 and return this line. */
+      voidLine = TRUE;
+      if (!feof(file))
+	{
+	    do
+	    {
+	      msg = fgets(line, MAX_LINE_LENGTH, file);
+	      if (!msg)
+		continue;
+	      g_strchug(line);
+	      voidLine = (line[0] == '#' || line[0] == '!' || line[0] == '\0');
+	    }
+	    while (!feof(file) && voidLine);
+	}
+      else
+	line[0] = '\0';
+      voidLine = (line[0] == '#' || line[0] == '!' || line[0] == '\0');
+
+      /* If still not done, we test the number of columns. */
       if (dataFile->nbColumns == DATAFILE_NB_COLUMN_DEFAULT)
 	{
-	  dataRead = g_strsplit_set(msg, " \t;:\n", MAX_LINE_LENGTH);
-	  DBG_fprintf(stderr, "Visu dataFile : looking for number of columns.\n");
-	  dataFile->nbColumns = 0;
-	  for (i = 0; dataRead[i]; i++)
-	    if (dataRead[i][0])
-	      {
-		DBG_fprintf(stderr, "                 | one token : '%s'\n", dataRead[i]);
-		dataFile->nbColumns += 1;
-	      }
-	  g_strfreev(dataRead);
-	  DBG_fprintf(stderr, "Visu dataFile : detected number of columns : %d\n",
-		      dataFile->nbColumns);
-	  if (dataFile->nbColumns <= 0)
+	  if (!voidLine)
 	    {
-	      g_string_append_printf(message, _("WARNING! Can't find any column of"
+	      dataRead = g_strsplit_set(line, " \t;:\n", MAX_LINE_LENGTH);
+	      DBG_fprintf(stderr, "Visu dataFile: looking for number of columns.\n");
+	      dataFile->nbColumns = 0;
+	      for (i = 0; dataRead[i]; i++)
+		if (dataRead[i][0])
+		  {
+		    DBG_fprintf(stderr, "   | one token : '%s'\n", dataRead[i]);
+		    dataFile->nbColumns += 1;
+		  }
+	      g_strfreev(dataRead);
+	      DBG_fprintf(stderr, "Visu dataFile : detected number of columns : %d\n",
+			  dataFile->nbColumns);
+	    }
+	  if (dataFile->nbColumns <= 0 || voidLine)
+	    {
+	      g_string_append_printf(message, _("Can't find any column of"
 						" data in the given file.\n"));
 	      *errors = 1;
 	      dataFileActivate(attachedVisuData, FALSE);
@@ -275,79 +395,71 @@ int dataFileSet_file(VisuData *attachedVisuData, char* filename, GString *messag
 	      return 0;
 	    }
 	  dataFile->readMinMax = g_malloc(sizeof(float) * 2 * (dataFile->nbColumns));
+	  /* Add a node property with the number of columns. */
+	  prop = visuNodeNew_pointerProperty(nodeArray, DATAFILE_ID,
+					     freeData, newOrCopyData,
+					     GINT_TO_POINTER(dataFile->nbColumns));
 	  /* Set the dimensions for the node property. */
 	  nodeDataSet_used(dataNode, attachedVisuData, dataFile->nbColumns);
 	}
-      if (nAtome >= attachedVisuData->nbOfAllStoredNodes)
+
+      /* We read the data from the line, if not void. */
+      data = newOrCopyData((gconstpointer)0, GINT_TO_POINTER(dataFile->nbColumns));
+      if (!voidLine)
 	{
-	  g_string_append_printf(message, _("WARNING! There are more data than nodes.\n"));
-	  *errors = 1;
-	  break;
-	}
-      data = g_malloc(sizeof(float) * dataFile->nbColumns);
-      dataRead = g_strsplit_set(msg, " \t;:\n", MAX_LINE_LENGTH);
-      res = 1;
-      nb = 0;
-      for (i = 0; dataRead[i]; i++)
-	{
-	  if (dataRead[i][0])
+	  dataRead = g_strsplit_set(line, " \t;:\n", MAX_LINE_LENGTH);
+	  nb = 0;
+	  for (i = 0; dataRead[i] && nb < dataFile->nbColumns; i++)
 	    {
-	      res = res && (1 == sscanf(dataRead[i], "%f", data + nb));
-	      DBG_fprintf(stderr, "                 | one value : '%s' -> '%f'\n",
-			  dataRead[i], data[nb]);
-	      nb += 1;
-	    }
-	}
-      res = res && (nb == dataFile->nbColumns);
-      g_strfreev(dataRead);
-      if (res)
-	{
-	  /* Associates the values to the node. */
-	  visuDataSet_nodeProperty(attachedVisuData, attachedVisuData->fromNumberToVisuNode[nAtome],
-				   DATAFILE_ID, (gpointer)data);
-	  /* Check for minMax values for each column. */
-	  for (i = 0; i < dataFile->nbColumns; i++)
-	    {
-	      if (nAtome == 0)
+	      if (1 == sscanf(dataRead[i], "%f", data + nb))
 		{
-		  dataFile->readMinMax[i * 2 + 0] = data[i];
-		  dataFile->readMinMax[i * 2 + 1] = data[i];
-		}
-	      else
-		{
-		  if (data[i] < dataFile->readMinMax[i * 2 + 0])
-		    dataFile->readMinMax[i * 2 + 0] = data[i];
-		  if (data[i] > dataFile->readMinMax[i * 2 + 1])
-		    dataFile->readMinMax[i * 2 + 1] = data[i];
+		  DBG_fprintf(stderr, "   '%s' - '%f'",
+			      dataRead[i], data[nb]);
+		  nb += 1;
 		}
 	    }
-	  nAtome++;
+	  DBG_fprintf(stderr, "\n");
+	  g_strfreev(dataRead);
 	}
       else
-	free(data);
+	{
+	  if (*errors == 0)
+	    {
+	      g_string_append_printf(message,
+				     _("There are more nodes than data.\n"));
+	      *errors = 1;
+	    }
+	}
+      /* Associates the values to the node. */
+      g_value_set_pointer(&dataValue, data);
+      visuNodePropertySet_value(prop, iter.node, &dataValue);
+      /* Check for minMax values for each column. */
+      for (i = 0; i < dataFile->nbColumns; i++)
+	{
+	  if (nAtome == 0)
+	    {
+	      dataFile->readMinMax[i * 2 + 0] = data[i];
+	      dataFile->readMinMax[i * 2 + 1] = data[i];
+	    }
+	  else
+	    {
+	      if (data[i] < dataFile->readMinMax[i * 2 + 0])
+		dataFile->readMinMax[i * 2 + 0] = data[i];
+	      if (data[i] > dataFile->readMinMax[i * 2 + 1])
+		dataFile->readMinMax[i * 2 + 1] = data[i];
+	    }
+	}
+      nAtome++;
     }
   fclose(file);
-  if (nAtome < attachedVisuData->nbOfAllStoredNodes)
-    {
-      g_string_append_printf(message, _("WARNING! There are less data than nodes.\n"));
-      /* Padding with min values for other nodes. */
-      for (i = nAtome; i < attachedVisuData->nbOfAllStoredNodes; i++)
-	{
-	  data = g_malloc(sizeof(float) * dataFile->nbColumns);
-	  for (j = 0; j < dataFile->nbColumns; j++)
-	    data[j] = dataFile->readMinMax[j * 2];
-	  visuDataSet_nodeProperty(attachedVisuData, attachedVisuData->fromNumberToVisuNode[i],
-				   DATAFILE_ID, (gpointer)data);
-	}
-      *errors = 1;
-    }
   dataFile->file = g_strdup(filename);
 
-  if (dataFile->nbColumns < 0)
+  if (dataFile->nbColumns <= 0)
     {
-      g_string_append_printf(message, _("WARNING! Can't find any columns with numbers.\n"
-					"Valid format are as much numbers as desired, separated"
-					" by any of the following characters : [ ;:\\t].\n"));
+      g_string_append_printf(message,
+			     _("Can't find any columns with numbers.\n"
+			       "Valid format are as much numbers as desired, separated"
+			       " by any of the following characters : [ ;:\\t].\n"));
       dataFile->used = FALSE;
       return 0;
     }
@@ -359,9 +471,18 @@ int dataFileSet_file(VisuData *attachedVisuData, char* filename, GString *messag
 	DBG_fprintf(stderr, "Visu dataFile :  col %d -> %f (min) and %f (max).\n",
 		    i, dataFile->readMinMax[i * 2 + 0], dataFile->readMinMax[i * 2 + 1]);
     }
+  for (i = 0; i < 3; i++)
+    dataFile->colUsed[i] = (colUsed[i] >= dataFile->nbColumns)?-1:colUsed[i];
 
   dataFileActivate(attachedVisuData, dataFile->used);
   cacheDataFile = dataFile;
+  cacheVisuData = attachedVisuData;
+
+#if DEBUG == 1
+  g_timer_stop(timer);
+  fprintf(stderr, "Visu dataFile: data associated in %g micro-s.\n", g_timer_elapsed(timer, &fractionTimer)/1e-6);
+  g_timer_destroy(timer);
+#endif
 
   return 1;
 }
@@ -369,7 +490,15 @@ gchar* dataFileGet_file(VisuData *visuData)
 {
   DataFile *dataFile;
 
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
+  
   if (dataFile)
     return dataFile->file;
   else
@@ -381,7 +510,14 @@ gboolean dataFileGet_fileMinMaxFromColumn(VisuData *visuData, float minMax[2], i
 
   g_return_val_if_fail(visuData && minMax, FALSE);
 
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
   g_return_val_if_fail(dataFile, FALSE);
   g_return_val_if_fail(column >= 0 && column < dataFile->nbColumns, FALSE);
 
@@ -397,7 +533,14 @@ int dataFileSet_used(VisuData *data, int val)
 
   DBG_fprintf(stderr, "Visu dataFile : set using flag for dataFile module to %d .\n", val);
 
-  dataFile = (DataFile*)visuDataGet_property(data, "dataColor_parameters");
+  if (data == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(data, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = data;
+    }
   if (!dataFile)
     return 0;
 
@@ -416,7 +559,14 @@ int dataFileGet_used(VisuData *data)
 {
   DataFile *dataFile;
 
-  dataFile = (DataFile*)visuDataGet_property(data, "dataColor_parameters");
+  if (data == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(data, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = data;
+    }
   
   if (!dataFile)
     return 0;
@@ -429,7 +579,14 @@ int dataFileSet_scaleType(VisuData *visuData, DataFileInputScaleId scale)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData, 0);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
   g_return_val_if_fail(dataFile, 0);
 
   DBG_fprintf(stderr, "Visu dataFile : set the scale type to %d (previuosly %d).\n",
@@ -445,7 +602,14 @@ DataFileInputScaleId dataFileGet_scaleType(VisuData *visuData)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData, DATAFILE_SCALE_TYPE_DEFAULT);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
 
   if (!dataFile)
     return DATAFILE_SCALE_TYPE_DEFAULT;
@@ -457,15 +621,17 @@ int dataFileSet_min(VisuData *visuData, float min)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData, 0);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-  g_return_val_if_fail(dataFile, 0);
-
-  if (min >= dataFile->minMaxValues[1])
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
     {
-      fprintf(stderr, "WARNING! Can't put a minimum value equal or greater than"
-			" the maximum value.\n");
-      return 0;
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
     }
+  g_return_val_if_fail(dataFile, 0);
+/*   g_return_val_if_fail(min < dataFile->minMaxValues[1], 0); */
+
   DBG_fprintf(stderr, "Visu dataFile : set the min value to %f (previuosly %f).\n",
 	      min, dataFile->minMaxValues[0]);
   if (dataFile->minMaxValues[0] == min)
@@ -479,15 +645,17 @@ int dataFileSet_max(VisuData *visuData, float max)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData, 0);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
-  g_return_val_if_fail(dataFile, 0);
-
-  if (max <= dataFile->minMaxValues[0])
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
     {
-      fprintf(stderr, "WARNING! Can't put a maximum value equal or smaller than"
-	      " the minimum value.\n");
-      return 0;
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
     }
+  g_return_val_if_fail(dataFile, 0);
+/*   g_return_val_if_fail(max > dataFile->minMaxValues[0], 0); */
+
   DBG_fprintf(stderr, "Visu dataFile : set the max value to %f (previuosly %f).\n",
 	      max, dataFile->minMaxValues[1]);
   if (dataFile->minMaxValues[1] == max)
@@ -501,7 +669,14 @@ float dataFileGet_min(VisuData *visuData)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData, DATAFILE_MIN_NORM_DEFAULT);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
 
   if (!dataFile)
     return DATAFILE_MIN_NORM_DEFAULT;
@@ -513,7 +688,14 @@ float dataFileGet_max(VisuData *visuData)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData, DATAFILE_MAX_NORM_DEFAULT);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
 
   if (!dataFile)
     return DATAFILE_MAX_NORM_DEFAULT;
@@ -526,7 +708,14 @@ int dataFileGet_nbColumns(VisuData *visuData)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData, DATAFILE_NB_COLUMN_DEFAULT);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
 
   if (!dataFile)
     return DATAFILE_NB_COLUMN_DEFAULT;
@@ -539,7 +728,14 @@ int dataFileSet_vectA(VisuData *visuData, float val, int pos)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData && pos > DATAFILE_NB_COLUMN_DEFAULT && pos < 3, 0);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
   g_return_val_if_fail(dataFile, 0);
 
   DBG_fprintf(stderr, "Visu dataFile : set the %d value of vectA to %f (previuosly %f).\n",
@@ -555,7 +751,14 @@ int dataFileSet_vectB(VisuData *visuData, float val, int pos)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData && pos > DATAFILE_NB_COLUMN_DEFAULT && pos < 3, 0);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
   g_return_val_if_fail(dataFile, 0);
 
   DBG_fprintf(stderr, "Visu dataFile : set the %d value of vectB to %f (previuosly %f).\n",
@@ -571,7 +774,14 @@ float* dataFileGet_vectA(VisuData *visuData)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData, (float*)0);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
 
   if (!dataFile)
     return (float*)0;
@@ -583,7 +793,14 @@ float* dataFileGet_vectB(VisuData *visuData)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData, (float*)0);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
 
   if (!dataFile)
     return (float*)0;
@@ -595,7 +812,14 @@ int dataFileSet_colUsed(VisuData *visuData, int val, int pos)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData && pos > DATAFILE_NB_COLUMN_DEFAULT && pos < 3, 0);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
   g_return_val_if_fail(dataFile, 0);
 
   g_return_val_if_fail(val < dataFile->nbColumns || val == DATAFILE_NB_COLUMN_DEFAULT, 0);
@@ -613,7 +837,14 @@ int* dataFileGet_colUsed(VisuData *visuData)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData, (int*)0);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
 
   if (!dataFile)
     return (int*)0;
@@ -625,7 +856,14 @@ int dataFileSet_colorType(VisuData *visuData, DataFileColorModeId color)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData, 0);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
   g_return_val_if_fail(dataFile, 0);
 
   DBG_fprintf(stderr, "Visu dataFile : set the color type to %d (previuosly %d).\n",
@@ -641,7 +879,14 @@ DataFileColorModeId dataFileGet_colorType(VisuData *visuData)
   DataFile *dataFile;
 
   g_return_val_if_fail(visuData, DATAFILE_COLOR_TYPE_DEFAULT);
-  dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+  if (visuData == cacheVisuData)
+    dataFile = cacheDataFile;
+  else
+    {
+      dataFile = (DataFile*)visuDataGet_property(visuData, "dataColor_parameters");
+      cacheDataFile = dataFile;
+      cacheVisuData = visuData;
+    }
 
   if (!dataFile)
     return DATAFILE_COLOR_TYPE_DEFAULT;
@@ -680,9 +925,11 @@ int dataFileSet_shade(VisuData *visuData, Shade *shade)
 gboolean dataFileApply_hideOnMinValue(VisuData *visuData, int column, float value)
 {
   gboolean redraw;
-  int i;
   float *storedValues;
   DataFile *dataFile;
+  VisuDataIter iter;
+  VisuNodeProperty *prop;
+  GValue data = {0, {{0}, {0}}};
 
   g_return_val_if_fail(visuData, FALSE);
 
@@ -700,15 +947,19 @@ gboolean dataFileApply_hideOnMinValue(VisuData *visuData, int column, float valu
 
   g_return_val_if_fail(column > DATAFILE_NB_COLUMN_DEFAULT && column < dataFile->nbColumns, FALSE);
   
+  g_value_init(&data, G_TYPE_POINTER);
+  prop = visuNodeGet_property(visuDataGet_nodeArray(visuData), DATAFILE_ID);
+
   redraw = FALSE;
-  for (i = 0; i < visuData->nbOfAllStoredNodes; i++)
+  visuDataIter_new(visuData, &iter);
+  for (visuDataIter_startVisible(visuData, &iter); iter.node;
+       visuDataIter_nextVisible(visuData, &iter))
     {
-      storedValues =
-	(float*)visuDataGet_nodeProperty(visuData,
-					 visuData->fromNumberToVisuNode[i],
-					 DATAFILE_ID);
-      if (storedValues[column] < value && -storedValues[column] < value)
-	redraw = visuNodeSet_visibility(visuData->fromNumberToVisuNode[i], FALSE) || redraw;
+      visuNodePropertyGet_value(prop, iter.node, &data);
+      storedValues = (float*)g_value_get_pointer(&data);
+      g_return_val_if_fail(storedValues, FALSE);
+      if (storedValues[column] < value) /* && -storedValues[column] < value) */
+	redraw = visuNodeSet_visibility(iter.node, FALSE) || redraw;
     }
 
   return TRUE;
@@ -723,7 +974,7 @@ gboolean dataFileApply_hideOnMinValue(VisuData *visuData, int column, float valu
    pointing to the right place in the visuData object that needs to be rendered. */
 
 /* Normalization method */
-float dataFileGet_valuesFromData(VisuData *visuData, int column, float fromVal)
+float dataFileGet_valuesFromData(VisuData *visuData _U_, int column, float fromVal)
 {
   float res;
 
@@ -739,11 +990,7 @@ float dataFileGet_valuesFromData(VisuData *visuData, int column, float fromVal)
     case dataFile_minMax:
       res = ( (fromVal - cacheDataFile->minMaxValues[0]) /
 	      (cacheDataFile->minMaxValues[1] - cacheDataFile->minMaxValues[0]) );
-      if (res > 1.)
-	res = 1.;
-      else if (res < 0.)
-	res = 0.;
-      return res;
+      return CLAMP(res, 0., 1.);
     }
   return 0.;
 }
@@ -784,7 +1031,9 @@ void colorFromUserData(VisuData *visuData, float rgba[4], VisuElement *ele, Visu
 
   g_return_if_fail(visuData && node && ele && rgba);
 
-  storedValues = (float*)visuDataGet_nodeProperty(visuData, node, DATAFILE_ID);
+  visuNodeGet_propertyValue(visuDataGet_nodeArray(visuData), node,
+			    DATAFILE_ID, &dataValue);
+  storedValues = (float*)g_value_get_pointer(&dataValue);
   if (!storedValues)
     return;
 
