@@ -49,9 +49,29 @@
 #include <opengl.h>
 #include <visu_object.h>
 
+/**
+ * SECTION:plane
+ * @short_description: Adds capabilities to draw and handle planes.
+ *
+ * <para>A #Plane is a GObject. It is defined by its normal vector and
+ * the distance of the plane with the origin (see
+ * planeSet_normalVector() and planeSet_distanceFromOrigin()). When
+ * these informations are given and an #OpenGLView is used to render
+ * the plane, V_Sim computes the intersections of the plane with the
+ * bounding box (see planeGet_intersection()).</para>
+ * <para>Planes can be used to hide nodes defining their
+ * planeSet_hiddenState() and planeSet_hidingMode(). A list of planes
+ * can also be exported or imported from an XML file using
+ * planesExport_XMLFile() and planesParse_XMLFile().</para>
+ * <para>Planes can have transparency but the support of it is limited
+ * to one plane. If several planes are drawn with transparency, they
+ * may hide each other because of the implementation of transparency
+ * in OpenGL (planes are treated as single polygons).</para>
+ */
+
 Plane_hidingMode plane_hidingMode = plane_hideUnion;
 
-struct Plane_struct
+struct _Plane
 {
   /* Internal object gestion. */
   GObject parent;
@@ -99,52 +119,31 @@ struct PlaneClass_struct
 };
 
 /* Internal variables. */
-static GObjectClass *parent_class = NULL;
 static guint plane_signals[PLANE_NB_SIGNAL] = { 0 };
 
 /* Object gestion methods. */
-static void plane_class_init(PlaneClass *klass);
-static void plane_init      (Plane *obj);
 static void plane_dispose   (GObject* obj);
 static void plane_finalize  (GObject* obj);
 
 /* Local methods. */
 static int comparePolygonPoint(gconstpointer pointA, gconstpointer pointB, gpointer data);
 
-GType plane_get_type(void)
-{
-  static GType plane_type = 0;
-
-  if (!plane_type)
-    {
-      static const GTypeInfo plane_info =
-      {
-        sizeof (PlaneClass),
-        NULL, /* base_init */
-        NULL, /* base_finalize */
-        (GClassInitFunc)plane_class_init,
-        NULL, /* class_finalize */
-        NULL, /* class_data */
-        sizeof (Plane),
-        0,
-        (GInstanceInitFunc)plane_init,
-	NULL
-      };
-      plane_type = g_type_register_static(G_TYPE_OBJECT, "PlaneType",
-						 &plane_info, 0);
-      DBG_fprintf(stderr, "Plane : creating the type Plane %d.\n",
-		  (int)plane_type);
-    }
-
-  return plane_type;
-}
+G_DEFINE_TYPE(Plane, plane, G_TYPE_OBJECT)
 
 static void plane_class_init(PlaneClass *klass)
 {
   DBG_fprintf(stderr, "Plane : creating the class of the object.\n");
-  parent_class = g_type_class_peek_parent(klass);
 
   DBG_fprintf(stderr, "                - adding new signals ;\n");
+  /**
+   * Plane::moved:
+   * @plane: the object emitting the signal.
+   *
+   * This signal is emitted each time the plane position is changed
+   * (either distance or normal).
+   *
+   * Since: 3.3
+   */
   plane_signals[PLANE_MOVED_SIGNAL] =
     g_signal_newv("moved", G_TYPE_FROM_CLASS(klass),
 		  G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
@@ -183,7 +182,7 @@ static void plane_dispose(GObject* obj)
 
   PLANE(obj)->dispose_has_run = TRUE;
   /* Chain up to the parent class */
-  G_OBJECT_CLASS(parent_class)->dispose(obj);
+  G_OBJECT_CLASS(plane_parent_class)->dispose(obj);
 }
 /* This method is called once only. */
 static void plane_finalize(GObject* obj)
@@ -198,12 +197,12 @@ static void plane_finalize(GObject* obj)
   tmpLst = PLANE(obj)->inter;
   while (tmpLst)
     {
-      free(tmpLst->data);
+      g_free(tmpLst->data);
       tmpLst = g_list_next(tmpLst);
     }
 
   /* Chain up to the parent class */
-  G_OBJECT_CLASS(parent_class)->finalize(obj);
+  G_OBJECT_CLASS(plane_parent_class)->finalize(obj);
 }
 
 Plane* planeNew_undefined()
@@ -252,12 +251,9 @@ int planeSet_normalVector(Plane *plane, float vect[3])
       vect[1] == plane->nVectUser[1] &&
       vect[2] == plane->nVectUser[2])
     return 0;
-  if (vect[0] * vect[0] + vect[1] * vect[1] + vect[2] * vect[2] == 0.)
-    {
-      fprintf(stderr, "WARNING! '%s' has been called with a null value for parameter '%s'.\n",
-	      "planeSet_normalVector", "vect");
-      return -1;
-    }
+  g_return_val_if_fail(vect[0] * vect[0] +
+		       vect[1] * vect[1] +
+		       vect[2] * vect[2] != 0., -1);
 
   norm = 0.;
   for (i = 0; i < 3; i++)
@@ -272,6 +268,9 @@ int planeSet_normalVector(Plane *plane, float vect[3])
 
   DBG_fprintf(stderr, "Visu Plane : set normal vector (%f,%f,%f) for plane %p.\n",
 	      plane->nVect[0], plane->nVect[1], plane->nVect[2], (gpointer)plane);
+  DBG_fprintf(stderr, "Visu Plane : set user vector (%f,%f,%f) for plane %p.\n",
+	      plane->nVectUser[0], plane->nVectUser[1], plane->nVectUser[2],
+	      (gpointer)plane);
   g_signal_emit(G_OBJECT(plane), plane_signals[PLANE_MOVED_SIGNAL], 0, NULL);
   return 1;
 }
@@ -335,7 +334,7 @@ GList* planeGet_intersection(Plane *plane)
 int planeComputeInter(Plane* plane, OpenGLView *view)
 {
   float lambda, denom;
-  float l[12][3], a[12][3];
+  float l[12][3], a[12][3], vertices[8][3];
   int i, j, n;
   float *inter;
   GList *tmpLst;
@@ -348,39 +347,40 @@ int planeComputeInter(Plane* plane, OpenGLView *view)
       tmpLst = plane->inter;
       while(tmpLst)
 	{
-	  free((float*)tmpLst->data);
+	  g_free((float*)tmpLst->data);
 	  tmpLst = g_list_next(tmpLst);
 	}
       g_list_free(plane->inter);
       plane->inter = (GList*)0;
     }
   /* Compute, vector and position for the box. */
+  openGLViewGet_boxVertices(view, vertices, TRUE);
   for (i = 0; i < 3; i++)
     {
-      l[0][i] = view->box->p2[i] - view->box->p1[i];
-      l[1][i] = view->box->p3[i] - view->box->p2[i];
-      l[2][i] = view->box->p4[i] - view->box->p3[i];
-      l[3][i] = view->box->p1[i] - view->box->p4[i];
-      l[4][i] = view->box->p5[i] - view->box->p1[i];
-      l[5][i] = view->box->p6[i] - view->box->p2[i];
-      l[6][i] = view->box->p7[i] - view->box->p3[i];
-      l[7][i] = view->box->p8[i] - view->box->p4[i];
-      l[8][i] = view->box->p6[i] - view->box->p5[i];
-      l[9][i] = view->box->p7[i] - view->box->p6[i];
-      l[10][i] = view->box->p8[i] - view->box->p7[i];
-      l[11][i] = view->box->p5[i] - view->box->p8[i];
-      a[0][i] = view->box->p1[i];
-      a[1][i] = view->box->p2[i];
-      a[2][i] = view->box->p3[i];
-      a[3][i] = view->box->p4[i];
-      a[4][i] = view->box->p1[i];
-      a[5][i] = view->box->p2[i];
-      a[6][i] = view->box->p3[i];
-      a[7][i] = view->box->p4[i];
-      a[8][i] = view->box->p5[i];
-      a[9][i] = view->box->p6[i];
-      a[10][i] = view->box->p7[i];
-      a[11][i] = view->box->p8[i];
+      l[0][i] = vertices[1][i] - vertices[0][i];
+      l[1][i] = vertices[2][i] - vertices[1][i];
+      l[2][i] = vertices[3][i] - vertices[2][i];
+      l[3][i] = vertices[0][i] - vertices[3][i];
+      l[4][i] = vertices[4][i] - vertices[0][i];
+      l[5][i] = vertices[5][i] - vertices[1][i];
+      l[6][i] = vertices[6][i] - vertices[2][i];
+      l[7][i] = vertices[7][i] - vertices[3][i];
+      l[8][i] = vertices[5][i] - vertices[4][i];
+      l[9][i] = vertices[6][i] - vertices[5][i];
+      l[10][i] = vertices[7][i] - vertices[6][i];
+      l[11][i] = vertices[4][i] - vertices[7][i];
+      a[0][i] = vertices[0][i];
+      a[1][i] = vertices[1][i];
+      a[2][i] = vertices[2][i];
+      a[3][i] = vertices[3][i];
+      a[4][i] = vertices[0][i];
+      a[5][i] = vertices[1][i];
+      a[6][i] = vertices[2][i];
+      a[7][i] = vertices[3][i];
+      a[8][i] = vertices[4][i];
+      a[9][i] = vertices[5][i];
+      a[10][i] = vertices[6][i];
+      a[11][i] = vertices[7][i];
     }
   n = 0;
   for (i = 0; i < 3; i++)
@@ -406,12 +406,7 @@ int planeComputeInter(Plane* plane, OpenGLView *view)
 	  lambda /= denom;
 	  if (lambda >= 0. && lambda <= 1.)
 	    {
-	      inter = malloc(sizeof(float) * 3);
-	      if (!inter)
-		{
-		  allocationProblems();
-		  exit(1);
-		}
+	      inter = g_malloc(sizeof(float) * 3);
 	      for (j = 0; j < 3; j++)
 		{
 		  inter[j] = a[i][j] + lambda * l[i][j];
@@ -433,64 +428,53 @@ int planeComputeInter(Plane* plane, OpenGLView *view)
 
   return 0;
 }
-int planesComputeInter_list(OpenGLView *view, GList *list)
+int planesComputeInter_list(OpenGLView *view, Plane **list)
 {
-  int res;
-  GList *tmpLst;
+  int res, i;
 
-  g_return_val_if_fail(view, 1);
+  g_return_val_if_fail(view && list, 1);
   
   res = 0;
-  tmpLst = list;
-  while(tmpLst)
-    {
-      res = planeComputeInter((Plane*)tmpLst->data, view) || res;
-      tmpLst = g_list_next(tmpLst);
-    }
+  for (i = 0; list[i]; i++)
+    res = planeComputeInter(list[i], view) || res;
   return res;
 }
 
-void planeDraw(Plane* plane, OpenGLView *view)
+void planeDraw(Plane* plane)
 {
   GList *tmpLst;
-  float mat[5] = {1., 0.0, 1.0, 0.0, 0.0};
+/*   float mat[5] = {1., 0.0, 1.0, 0.0, 0.0}; */
 
-  g_return_if_fail(view && IS_PLANE_TYPE(plane));
+  g_return_if_fail(IS_PLANE_TYPE(plane));
 
   tmpLst = plane->inter;
   if (tmpLst && plane->rendered)
     {
       glDisable(GL_CULL_FACE);
-      setOpenGlMaterial(mat, plane->color->rgba);
-      glPushMatrix();
-      glTranslated(-view->box->dxxs2, -view->box->dyys2, -view->box->dzzs2);
+/*       setOpenGlMaterial(mat, plane->color->rgba); */
+      glColor4fv(plane->color->rgba);
       glBegin(GL_POLYGON);
       while (tmpLst)
 	{
-	  glVertex3f(((float*)tmpLst->data)[0],
-		     ((float*)tmpLst->data)[1],
-		     ((float*)tmpLst->data)[2]);
+	  glVertex3fv((float*)tmpLst->data);
 	  tmpLst = g_list_next(tmpLst);
 	}
       glEnd();
-      glPopMatrix();  
       glEnable(GL_CULL_FACE);
       glCullFace(GL_BACK);
     }
 }
-void planesDraw_list(OpenGLView *view, GList *list)
+void planesDraw_list(Plane **list)
 {
-  GList *tmpLst;
+  int i;
 
-  g_return_if_fail(view);
+  g_return_if_fail(list);
 
   DBG_fprintf(stderr, "Plane : drawing list of planes.\n");
-  tmpLst = list;
-  while(tmpLst)
+  for (i = 0; list[i]; i++)
     {
-      DBG_fprintf(stderr, " | plane %p\n", tmpLst->data);
-      planeDraw((Plane*)tmpLst->data, view);
-      tmpLst = g_list_next(tmpLst);
+      DBG_fprintf(stderr, " | plane %p\n", (gpointer)list[i]);
+      planeDraw(list[i]);
     }
 }
 
@@ -561,70 +545,139 @@ int planeGet_hiddenState(Plane *plane)
   g_return_val_if_fail(IS_PLANE_TYPE(plane), 0);
   return plane->hiddenSide;
 }
-int planeShowHide_all(VisuData *visuData, GList *listOfPlanes)
+
+gboolean planesGet_visibility(Plane **listOfPlanes, float point[3])
 {
-  int i, j, k;
+  int i, k;
+  float pScal;
+  gboolean visibility;
+
+  visibility = (plane_hidingMode == plane_hideUnion) || !listOfPlanes[0];
+
+  for (i = 0; listOfPlanes[i]; i++)
+    {
+      pScal = 0.;
+      for (k = 0; k < 3; k++)
+	pScal += listOfPlanes[i]->nVect[k] *
+	  (point[k] - listOfPlanes[i]->pointG[k]);
+      switch (plane_hidingMode)
+	{
+	case plane_hideUnion:
+	  visibility =
+	    visibility && (pScal * listOfPlanes[i]->hiddenSide >= 0.);
+	  break;
+	case plane_hideInter:
+	  visibility =
+	    visibility || (pScal * listOfPlanes[i]->hiddenSide >= 0.);
+	  break;
+	default: break;
+	}
+    }
+  return visibility;
+}
+gboolean planesGet_intersection(Plane **listOfPlanes, float pointA[3],
+				float pointB[3], float inter[3])
+{
+  float lambda, denom;
+  int i;
+  int iMin;
+  float lambdaMin;
+
+/*   DBG_fprintf(stderr, "Plane: intersection A(%g;%g;%g) B(%g;%g;%g).\n", */
+/* 	      pointA[0], pointA[1], pointA[2], pointB[0], pointB[1], pointB[2]); */
+  lambdaMin = 2.f;
+  for (i = 0; listOfPlanes[i]; i++)
+    {
+      /* Compute lambda and keep the smallest. */
+      lambda = -listOfPlanes[i]->dist + listOfPlanes[i]->nVect[0] * pointA[0] +
+	listOfPlanes[i]->nVect[1] * pointA[1] + listOfPlanes[i]->nVect[2] * pointA[2];
+      denom = listOfPlanes[i]->nVect[0] * (pointB[0] - pointA[0]) +
+	listOfPlanes[i]->nVect[1] * (pointB[1] - pointA[1]) +
+	listOfPlanes[i]->nVect[2] * (pointB[2] - pointA[2]);
+      if (denom == 0.f)
+	{
+	  /* if B is in the plane, we put lambdaMin to 1. */
+	  if (listOfPlanes[i]->nVect[0] * pointB[0] +
+	      listOfPlanes[i]->nVect[1] * pointB[1] +
+	      listOfPlanes[i]->nVect[2] * pointB[2] -
+	      listOfPlanes[i]->dist == 0.f)
+	    lambdaMin = 1.f;
+	  /* if not, let's test other planes and return with lambdaMin
+	     == 2.f if necessary. */
+	}
+      else
+	{
+	  lambda /= - denom;
+	  if (lambda >= 0. && lambda <= 1. && lambda < lambdaMin)
+	    {
+	      lambdaMin = lambda;
+	      iMin = i;
+	    }
+	}
+    }
+  if (lambdaMin == 2.f)
+    return FALSE;
+
+  inter[0] = pointA[0] + lambdaMin * (pointB[0] - pointA[0]);
+  inter[1] = pointA[1] + lambdaMin * (pointB[1] - pointA[1]);
+  inter[2] = pointA[2] + lambdaMin * (pointB[2] - pointA[2]);
+  return TRUE;
+}
+
+
+int planeShowHide_all(VisuData *visuData, Plane **listOfPlanes)
+{
+  int i, n;
   int reDraw;
   float point[3];
-  float pScal;
-  gboolean hidingMode, visibility;
-  GList *tmpLst;
+  gboolean visibility;
+  Plane **tmpLst;
+  VisuDataIter iter;
+
+  g_return_val_if_fail(visuData && listOfPlanes, 0);
 
   DBG_fprintf(stderr, "Planes : applying masking properties of planes.\n");
 
   /* Remove planes of the given list that doesn't have a masking action. */
-  tmpLst = listOfPlanes;
-  listOfPlanes = (GList*)0;
-  while(tmpLst)
+  for (n = 0; listOfPlanes[n]; n++);
+  tmpLst = g_malloc(sizeof(Plane*) * (n + 1));
+  n = 0;
+  for (i = 0; listOfPlanes[i]; i++)
     {
-      if (((Plane*)tmpLst->data)->hiddenSide != PLANE_SIDE_NONE)
-	listOfPlanes = g_list_append(listOfPlanes, tmpLst->data);
+      if (listOfPlanes[i]->hiddenSide != PLANE_SIDE_NONE)
+	{
+	  tmpLst[n] = listOfPlanes[i];
+	  n += 1;
+	}
       else
-	DBG_fprintf(stderr, " | plane %p has a masking property.\n", tmpLst->data);
-      tmpLst = g_list_next(tmpLst);
-    }
-
-  hidingMode = TRUE;
-  switch (plane_hidingMode)
-    {
-    case plane_hideUnion: hidingMode = TRUE; break;
-    case plane_hideInter: hidingMode = FALSE; break;
-    default: break;
+	DBG_fprintf(stderr, " | plane %p has a masking property.\n",
+		    (gpointer)listOfPlanes[i]);
     }
+  tmpLst[n] = (Plane*)0;
 
   reDraw = 0;
   /* We change the rendered attribute of all nodes, very expensive... */
-  if (listOfPlanes)
-    for (i = 0; i < visuData->ntype; i++)
-      for(j = 0; j < visuData->numberOfStoredNodes[i]; j++)
-	{
-	  visuDataGet_nodePosition(visuData, &visuData->nodes[i][j], point);
-	
-	  visibility = hidingMode;
-	  tmpLst = listOfPlanes;
-	  while(tmpLst)
+  if (tmpLst[0])
+    {
+      visuDataIter_new(visuData, &iter);
+      for (visuDataIter_start(visuData, &iter); iter.element;
+	   visuDataIter_nextElement(visuData, &iter))
+	if (iter.element->sensitiveToMaskingPlanes && iter.element->rendered)
+	  for (visuDataIter_restartNode(visuData, &iter); iter.node;
+	       visuDataIter_nextNode(visuData, &iter))
 	    {
-	      pScal = 0.;
-	      for (k = 0; k < 3; k++)
-		pScal += ((Plane*)tmpLst->data)->nVect[k] *
-		  (point[k] - ((Plane*)tmpLst->data)->pointG[k]);
-	      switch (plane_hidingMode)
-		{
-		case plane_hideUnion:
-		  visibility =
-		    visibility && (pScal * ((Plane*)tmpLst->data)->hiddenSide > 0.);
-		  break;
-		case plane_hideInter:
-		  visibility =
-		    visibility || (pScal * ((Plane*)tmpLst->data)->hiddenSide > 0.);
-		  break;
-		default: break;
-		}
-	      tmpLst = g_list_next(tmpLst);
+	      /* If node is already hiden, well, we go to the next. */
+	      if (!iter.node->rendered)
+		continue;
+
+	      visuDataGet_nodePosition(visuData, iter.node, point);
+	    
+	      visibility = planesGet_visibility(tmpLst, point);
+	      if (!visibility)
+		reDraw = visuNodeSet_visibility(iter.node, FALSE) || reDraw;
 	    }
-	  if (!visibility)
-	    reDraw = visuNodeSet_visibility(&visuData->nodes[i][j], visibility) || reDraw;
-	}
+    }
+  g_free(tmpLst);
   if (reDraw)
     return 1;
   else
@@ -663,7 +716,7 @@ int planeSet_hidingMode(Plane_hidingMode mode)
    element, a new plane is created and prepend in the list.
    When 'geometry' or other qualificative elements are
    found, the first plane of the list is modified accordingly. */
-void listOfPlanes_element(GMarkupParseContext *context,
+void listOfPlanes_element(GMarkupParseContext *context _U_,
                           const gchar         *element_name,
                           const gchar        **attribute_names,
                           const gchar        **attribute_values,
@@ -837,7 +890,7 @@ void listOfPlanes_element(GMarkupParseContext *context,
 }
 
 /* Check when a element is closed that everything required has been set. */
-void listOfPlanes_end(GMarkupParseContext *context,
+void listOfPlanes_end(GMarkupParseContext *context _U_,
 		      const gchar         *element_name,
 		      gpointer             user_data,
 		      GError             **error)
@@ -872,54 +925,40 @@ void listOfPlanes_end(GMarkupParseContext *context,
 }
 
 /* What to do when an error is raised. */
-void listOfPlanes_error(GMarkupParseContext *context,
+void listOfPlanes_error(GMarkupParseContext *context _U_,
                           GError              *error,
-                          gpointer             user_data)
+                          gpointer             user_data _U_)
 {
   DBG_fprintf(stderr, "Planes parser : error raised '%s'.\n", error->message);
 }
 
-gboolean planesParse_XMLFile(gchar* filename, GList **list, GError **error)
+gboolean planesParse_XMLFile(gchar* filename, Plane ***planes, GError **error)
 {
   GMarkupParseContext* xmlContext;
   GMarkupParser parser;
   gboolean res;
-  GIOChannel *xmlData;
   gsize size;
   gchar *buffer;
-  GIOStatus ioRes;
+  GList *list, *tmpLst;
+  int i;
 
-  g_return_val_if_fail(filename && list, FALSE);
+  g_return_val_if_fail(filename && planes && !*planes, FALSE);
 
-  xmlData = g_io_channel_new_file(filename, "r", error);
-  if (!xmlData)
-    return FALSE;
+  *planes = g_malloc(sizeof(Plane*));
+  *planes[0] = (Plane*)0;
 
   buffer = (gchar*)0;
-  ioRes = g_io_channel_read_to_end(xmlData, &buffer, &size, error);
-  if (ioRes != G_IO_STATUS_NORMAL)
-    {
-      g_io_channel_unref(xmlData);
-      return FALSE;
-    }
-
-  ioRes = g_io_channel_shutdown(xmlData, FALSE, error);
-  if (ioRes != G_IO_STATUS_NORMAL)
-    {
-      g_io_channel_unref(xmlData);
-      return FALSE;
-    }
-  
-  g_io_channel_unref(xmlData);
+  if (!g_file_get_contents(filename, &buffer, &size, error))
+    return FALSE;
 
   /* Create context. */
-  *list = (GList*)0;
+  list = (GList*)0;
   parser.start_element = listOfPlanes_element;
   parser.end_element   = listOfPlanes_end;
   parser.text          = NULL;
   parser.passthrough   = NULL;
   parser.error         = listOfPlanes_error;
-  xmlContext = g_markup_parse_context_new(&parser, 0, list, NULL);
+  xmlContext = g_markup_parse_context_new(&parser, 0, &list, NULL);
 
   /* Parse data. */
   res = g_markup_parse_context_parse(xmlContext, buffer, size, error);
@@ -929,17 +968,32 @@ gboolean planesParse_XMLFile(gchar* filename, GList **list, GError **error)
   g_free(buffer);
 
   /* Need to reverse the list since elements have been prepended. */
-  *list = g_list_reverse(*list);
+  list = g_list_reverse(list);
+
+  /* Convert the list to an array. */
+  DBG_fprintf(stderr, "Planes: create array of planes (%d).\n", g_list_length(list));
+  *planes = g_realloc(*planes, sizeof(Plane*) * (g_list_length(list) + 1));
+  i = 0;
+  tmpLst = list;
+  while (tmpLst)
+    {
+      DBG_fprintf(stderr, " | %d -> %p\n", i, tmpLst->data);
+      (*planes)[i] = (Plane*)tmpLst->data;
+      i += 1;
+      tmpLst = g_list_next(tmpLst);
+    }
+  (*planes)[i] = (Plane*)0;
+  g_list_free(list);
 
   return res;
 }
-gboolean planesExport_XMLFile(gchar* filename, GList *list, GError **error)
+gboolean planesExport_XMLFile(gchar* filename, Plane **list, GError **error)
 {
   GIOChannel *xmlData;
   GIOStatus ioRes;
   GString *buffer;
-  Plane *plane;
   gsize sizeWritten;
+  int i;
 
   g_return_val_if_fail(filename && list, FALSE);
 
@@ -949,18 +1003,17 @@ gboolean planesExport_XMLFile(gchar* filename, GList *list, GError **error)
 
   buffer = g_string_new("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
   g_string_append(buffer, "<planes>\n");
-  while (list)
+  for (i = 0; list[i]; i++)
     {
-      plane = (Plane*)list->data;
-      if (plane->rendered)
+      if (list[i]->rendered)
 	g_string_append(buffer, "  <plane rendered=\"yes\">\n");
       else
 	g_string_append(buffer, "  <plane rendered=\"no\">\n");
       g_string_append_printf(buffer, "    <geometry normal-vector=\"%g %g %g\""
-			     " distance=\"%g\" />\n", plane->nVectUser[0],
-			     plane->nVectUser[1], plane->nVectUser[2],
-			     plane->dist);
-      switch (plane->hiddenSide)
+			     " distance=\"%g\" />\n", list[i]->nVectUser[0],
+			     list[i]->nVectUser[1], list[i]->nVectUser[2],
+			     list[i]->dist);
+      switch (list[i]->hiddenSide)
 	{
 	case PLANE_SIDE_NONE:
 	  g_string_append(buffer, "    <hide status=\"no\" invert=\"no\" />\n");
@@ -975,10 +1028,9 @@ gboolean planesExport_XMLFile(gchar* filename, GList *list, GError **error)
 	  g_warning("Unknown hiddenSide attribute ofr the given plane.");
 	};
       g_string_append_printf(buffer, "    <color rgba=\"%g %g %g %g\" />\n",
-			     plane->color->rgba[0], plane->color->rgba[1],
-			     plane->color->rgba[2], plane->color->rgba[3]);
+			     list[i]->color->rgba[0], list[i]->color->rgba[1],
+			     list[i]->color->rgba[2], list[i]->color->rgba[3]);
       g_string_append(buffer, "  </plane>\n");
-      list = g_list_next(list);
     }
   g_string_append(buffer, "</planes>\n");
 
