@@ -58,6 +58,48 @@
 #include "gtk_colorComboBoxWidget.h"
 #include <visu_tools.h>
 
+/**
+ * SECTION:colorcombobox
+ * @short_description: Defines a specialised #GtkComboBox to choose
+ * stored colours.
+ * @see_also: #Color, #StippleComboBox, #ShadeComboBox
+ * @include: coreTools/toolColor.h
+ *
+ * <para>This widget looks like a #GtkComboBox and it displays a list
+ * of stored colours. These colours may come from the configuration
+ * files or can be selected and stored by the user actions. To do it,
+ * the first entry of the combo box is 'new / modify', that opens a GTK
+ * colour picker. The new values are used to craete a new colour entry
+ * in the combo box. In a complete version, the combo box can have a
+ * #GtkVBox associated to modify colours without creating new entries, see
+ * colorComboBox_newWithRanges() method. Otherwise, only already
+ * selected colours are available. The stored colours are shared by
+ * all widgets of this class. It is thus a convenient way to have
+ * coherent colour picker through V_Sim.</para>
+ *
+ * <para>When the widget is created with ranges, the additional part
+ * can be retrieve with colorComboBoxGet_rangesWidgets() and attached
+ * whereever is convenient. When the ranges are modified, the new
+ * colour is not added to the combo box. It can be read using
+ * colorComboBoxGet_color() or colorComboBoxGet_material(). The combo
+ * box is set thus to an unselected state and
+ * colorComboBoxGet_selectedColor() will return a NULL
+ * pointer. Besides the colour ranges, there is an add button to
+ * insert the newly defined colour into the combo box.</para>
+ *
+ * <para>This widget can emit a #ColorComboBox::color-selected
+ * signal that is a wrapper around the #GtkComboBox::changed signal,
+ * but it is emitted only when a new colour is selected (either an
+ * existing one or a newly created from the picker). This colour
+ * is passed to the callback. The two other signals,
+ * #ColorComboBox::color-value-changed and
+ * #ColorComboBox::material-value-changed, are generated when the
+ * widget have been created with ranges and that one of these ranges
+ * is modified.</para>
+ *
+ * Since: 3.1
+ */
+
 enum {
   COLOR_SELECTED_SIGNAL,
   COLOR_VALUE_CHANGED_SIGNAL,
@@ -102,16 +144,13 @@ enum
 #define SPE_ELE_LABEL   _("spe:")
 #define EMI_ELE_LABEL   _("emi:")
 
-static void colorComboBox_class_init          (ColorComboBoxClass *klass);
-static void colorComboBox_init                (ColorComboBox *colorComboBox);
-static void colorComboBox_changed             (GtkWidget *widget, ColorComboBox *colorComboBox);
-static void colorComboBox_dispose             (GObject *obj);
-static void colorComboBox_finalize            (GObject *obj);
+static void colorComboBox_changed (GtkWidget *widget, ColorComboBox *colorComboBox);
+static void colorComboBox_dispose (GObject *obj);
+static void colorComboBox_finalize(GObject *obj);
 
 static guint colorComboBox_signals[LAST_SIGNAL] = { 0 };
-static GObjectClass *parent_class = NULL;
 
-struct ColorComboBox_struct
+struct _ColorComboBox
 {
   GtkComboBox comboColor;
   Color* previouslySelectedColor;
@@ -127,11 +166,13 @@ struct ColorComboBox_struct
 
   gboolean hasAlphaChannel;
 
+  GtkCellRenderer *rendererRGB;
+
   /* Memory gestion. */
   gboolean dispose_has_run;
 };
 
-struct ColorComboBoxClass_struct
+struct _ColorComboBoxClass
 {
   GtkComboBoxClass parent_class;
 
@@ -151,37 +192,11 @@ static void colorComboBox_rgbChanged(GtkRange *rg, gpointer data);
 static void colorComboBox_addButtonClicked(GtkButton *button, gpointer data);
 
 /* Local methods. */
-void addColorToModel(GtkTreeIter *iter, ColorComboBoxClass* klass, Color* color);
-void onNewColorAvailable(GObject *obj, Color* newColor, gpointer data);
+static void addColorToModel(GtkTreeIter *iter, ColorComboBoxClass* klass,
+			    Color* color);
+static void onNewColorAvailable(GObject *obj, Color* newColor, gpointer data);
 
-
-GType colorComboBox_get_type(void)
-{
-  static GType colorComboBox_type = 0;
-
-  if (!colorComboBox_type)
-    {
-      static const GTypeInfo colorComboBox_info =
-      {
-        sizeof (ColorComboBoxClass),
-        NULL, /* base_init */
-        NULL, /* base_finalize */
-        (GClassInitFunc) colorComboBox_class_init,
-        NULL, /* class_finalize */
-        NULL, /* class_data */
-        sizeof (ColorComboBox),
-        0,
-        (GInstanceInitFunc) colorComboBox_init,
-        NULL
-      };
-      colorComboBox_type = g_type_register_static (GTK_TYPE_COMBO_BOX, "ColorComboBox",
-						   &colorComboBox_info, 0);
-      DBG_fprintf(stderr, "Gtk ColorComboBox : creating the type ColorComboBox %d.\n",
-		  (int)colorComboBox_type);
-    }
-
-  return colorComboBox_type;
-}
+G_DEFINE_TYPE(ColorComboBox, colorComboBox, GTK_TYPE_COMBO_BOX)
 
 static void colorComboBox_class_init(ColorComboBoxClass *klass)
 {
@@ -189,10 +204,17 @@ static void colorComboBox_class_init(ColorComboBoxClass *klass)
   GList *colorLst;
   
   DBG_fprintf(stderr, "Gtk ColorComboBox : creating the class of the widget.\n");
-
-  parent_class = g_type_class_peek_parent(klass);
-
   DBG_fprintf(stderr, "                     - adding new signals ;\n");
+  /**
+   * ColorComboBox::color-selected:
+   * @combo: the #ColorComboBox that emits the signal ;
+   * @color: the newly selected #Color.
+   *
+   * This signal is emitted when a new valid colour is selected,
+   * either an existing one or newly created one.
+   *
+   * Since: 3.1
+   */
   colorComboBox_signals[COLOR_SELECTED_SIGNAL] =
     g_signal_new ("color-selected",
 		  G_TYPE_FROM_CLASS (klass),
@@ -202,6 +224,15 @@ static void colorComboBox_class_init(ColorComboBoxClass *klass)
 		  NULL,                
 		  g_cclosure_marshal_VOID__POINTER,
 		  G_TYPE_NONE, 1, G_TYPE_POINTER);
+  /**
+   * ColorComboBox::color-value-changed:
+   * @combo: the #ColorComboBox that emits the signal ;
+   * @RGBA: the modified channel.
+   *
+   * This signal is emitted when the range of a colour is modified.
+   *
+   * Since: 3.3
+   */
   colorComboBox_signals[COLOR_VALUE_CHANGED_SIGNAL] =
     g_signal_new ("color-value-changed",
 		  G_TYPE_FROM_CLASS (klass),
@@ -211,6 +242,15 @@ static void colorComboBox_class_init(ColorComboBoxClass *klass)
 		  NULL,                
 		  g_cclosure_marshal_VOID__UINT,
 		  G_TYPE_NONE, 1, G_TYPE_UINT);
+  /**
+   * ColorComboBox::material-value-changed:
+   * @combo: the #ColorComboBox that emits the signal ;
+   * @mat: the modified material channel (see #Material).
+   *
+   * This signal is emitted when the range of a material is modified.
+   *
+   * Since: 3.3
+   */
   colorComboBox_signals[MATERIAL_VALUE_CHANGED_SIGNAL] =
     g_signal_new ("material-value-changed",
 		  G_TYPE_FROM_CLASS (klass),
@@ -240,6 +280,7 @@ static void colorComboBox_class_init(ColorComboBoxClass *klass)
   klass->colorAddedSignalId = g_signal_connect(G_OBJECT(visu), "colorNewAvailable",
 					       G_CALLBACK(onNewColorAvailable),
 					       (gpointer)klass);
+  DBG_fprintf(stderr, "                     - add stored colours.\n");
   colorLst = colorGet_storedColors();
   while (colorLst)
     {
@@ -247,6 +288,9 @@ static void colorComboBox_class_init(ColorComboBoxClass *klass)
       colorLst = g_list_next(colorLst);
     }
 
+  /* Connect freeing methods. */
+  G_OBJECT_CLASS(klass)->dispose = colorComboBox_dispose;
+  G_OBJECT_CLASS(klass)->finalize = colorComboBox_finalize;
 }
 
 static void colorComboBox_dispose(GObject *obj)
@@ -258,7 +302,7 @@ static void colorComboBox_dispose(GObject *obj)
 
   COLOR_COMBOX(obj)->dispose_has_run = TRUE;
   /* Chain up to the parent class */
-  G_OBJECT_CLASS(parent_class)->dispose(obj);
+  G_OBJECT_CLASS(colorComboBox_parent_class)->dispose(obj);
 }
 static void colorComboBox_finalize(GObject *obj)
 {
@@ -273,7 +317,7 @@ static void colorComboBox_finalize(GObject *obj)
     gtk_widget_destroy(colorComboBox->expandRanges);
 
   /* Chain up to the parent class */
-  G_OBJECT_CLASS(parent_class)->finalize(obj);
+  G_OBJECT_CLASS(colorComboBox_parent_class)->finalize(obj);
 
   DBG_fprintf(stderr, "Gtk ColorComboBox : freeing ... OK.\n");
 }
@@ -319,6 +363,7 @@ static void buildWidgets(ColorComboBox *colorComboBox)
   else
     gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(colorComboBox), renderer,
 				  "text", COLUMN_COLOR_LABEL);
+  colorComboBox->rendererRGB = renderer;
 
   gtk_combo_box_set_active(GTK_COMBO_BOX(colorComboBox), 1);
 
@@ -464,6 +509,11 @@ void colorComboBoxSet_expanded(ColorComboBox *colorComboBox, gboolean value)
 
   gtk_expander_set_expanded(GTK_EXPANDER(colorComboBox->expandRanges), value);
 }
+void colorComboBoxSet_printValues(ColorComboBox *colorComboBox, gboolean value)
+{
+  g_object_set(G_OBJECT(colorComboBox->rendererRGB),
+	       "visible", value, NULL);
+}
 GtkWidget* colorComboBoxGet_rangesWidgets(ColorComboBox *colorComboBox)
 {
   g_return_val_if_fail(IS_COLOR_COMBOX(colorComboBox), (GtkWidget*)0);
@@ -471,7 +521,7 @@ GtkWidget* colorComboBoxGet_rangesWidgets(ColorComboBox *colorComboBox)
   return colorComboBox->expandRanges;
 }
 
-static void colorComboBox_changed(GtkWidget *widget, ColorComboBox *colorComboBox)
+static void colorComboBox_changed(GtkWidget *widget _U_, ColorComboBox *colorComboBox)
 {
   int selected, i;
   GdkColor gdkcolor;
@@ -628,7 +678,7 @@ static void colorComboBox_rgbChanged(GtkRange *rg, gpointer data)
       }
   g_warning("Internal error, unrecognized range.");
 }
-static void colorComboBox_addButtonClicked(GtkButton *button, gpointer data)
+static void colorComboBox_addButtonClicked(GtkButton *button _U_, gpointer data)
 {
   GtkTreeIter iter;
   GObjectClass *klass;
@@ -702,7 +752,8 @@ GdkPixbuf* colorComboBoxBuild_colorStamp(Color *color, gboolean alpha)
   return pixbufColorBox;
 }
 
-void addColorToModel(GtkTreeIter *iter, ColorComboBoxClass* klass, Color* color)
+static void addColorToModel(GtkTreeIter *iter, ColorComboBoxClass* klass,
+			    Color* color)
 {
   char str[20], strAlpha[20];
   GdkPixbuf *pixbufColorBox;
@@ -750,7 +801,7 @@ gboolean colorComboBoxSet_selectionByColor(ColorComboBox* colorComboBox, Color *
       gtk_tree_model_get(GTK_TREE_MODEL(model), &iter,
 			 COLUMN_COLOR_POINTER_TO, &tmpColor,
 			 -1);
-      if (tmpColor == color)
+      if (tmpColor && colorEqual_color(tmpColor, color))
 	{
 	  gtk_combo_box_set_active_iter(GTK_COMBO_BOX(colorComboBox), &iter);
 	  return TRUE;
@@ -763,9 +814,10 @@ void colorComboBoxSet_color(ColorComboBox *colorComboBox, float rgba[4],
 			    gboolean raiseSignal)
 {
   int pos, i;
+  Color *color;
 
-  colorGet_byValues(&pos, rgba[0], rgba[1], rgba[2], rgba[3]);
-  if (pos < 0)
+  color = colorGet_byValues(&pos, rgba[0], rgba[1], rgba[2], rgba[3]);
+  if (!color)
     {
       gtk_combo_box_set_active(GTK_COMBO_BOX(colorComboBox), -1);
       if (raiseSignal)
@@ -789,6 +841,7 @@ void colorComboBoxSet_color(ColorComboBox *colorComboBox, float rgba[4],
 	gtk_combo_box_set_active(GTK_COMBO_BOX(colorComboBox), pos + 1);
       else
 	{
+	  colorComboBox->previouslySelectedColor = color;
 	  g_signal_handler_block(G_OBJECT(colorComboBox),
 				 colorComboBox->comboSignal);
 	  gtk_combo_box_set_active(GTK_COMBO_BOX(colorComboBox), pos + 1);
@@ -808,16 +861,12 @@ void colorComboBoxSet_color(ColorComboBox *colorComboBox, float rgba[4],
 }
 
 
-void onNewColorAvailable(GObject *obj, Color* newColor, gpointer data)
+static void onNewColorAvailable(GObject *obj _U_, Color* newColor, gpointer data)
 {
   ColorComboBoxClass *klass;
   GtkTreeIter iter;
 
-  if (!data)
-    {
-      fprintf(stderr, "INTERNAL ERROR! 'onNewColorAvailable' has been"
-	      " called without 'data' argument.\n");
-    }
+  g_return_if_fail(data);
 
   DBG_fprintf(stderr, "Gtk ColorComboBoxClass : catch the 'colorNewAvailable' signal.\n");
   klass = COLOR_COMBOX_CLASS(data);
@@ -831,12 +880,8 @@ GdkPixbuf* colorComboBoxGet_selectedPixbuf(ColorComboBox *colorComboBox)
   GdkPixbuf *pixbuf;
   GObjectClass *klass;
 
-  if (!colorComboBox)
-    {
-      fprintf(stderr, "WARNING! 'colorComboBoxGet_selectedPixbuf' has been"
-	      " called with a null 'colorComboBox' argument.\n");
-      return (GdkPixbuf*)0;
-    }
+  g_return_val_if_fail(IS_COLOR_COMBOX(colorComboBox), (GdkPixbuf*)0);
+
   validIter = gtk_combo_box_get_active_iter(GTK_COMBO_BOX(colorComboBox), &iter);
   if (!validIter)
     return (GdkPixbuf*)0;
@@ -860,12 +905,8 @@ Color* colorComboBoxGet_selectedColor(ColorComboBox *colorComboBox)
   Color *color;
   GObjectClass *klass;
 
-  if (!colorComboBox)
-    {
-      fprintf(stderr, "WARNING! 'colorComboBoxGet_selectedColor' has been"
-	      " called with a null 'colorComboBox' argument.\n");
-      return (Color*)0;
-    }
+  g_return_val_if_fail(IS_COLOR_COMBOX(colorComboBox), (Color*)0);
+
   validIter = gtk_combo_box_get_active_iter(GTK_COMBO_BOX(colorComboBox), &iter);
   if (!validIter)
     return (Color*)0;
@@ -886,14 +927,7 @@ GdkPixbuf* colorComboBoxGet_pixbufFromColor(ColorComboBox *colorComboBox, Color
   Color *cl;
   GtkListStore *model;
 
-  if (!colorComboBox)
-    {
-      fprintf(stderr, "WARNING! 'colorComboBoxGet_pixbufFromColor' has"
-	      " been called with a null 'colorComboBox' argument.\n");
-      return (GdkPixbuf*)0;
-    }
-  if (!color)
-    return (GdkPixbuf*)0;
+  g_return_val_if_fail(colorComboBox && color, (GdkPixbuf*)0);
 
   model = COLOR_COMBOX_CLASS(G_OBJECT_GET_CLASS(colorComboBox))->listStoredColors;
   validIter = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(model), &iter);
@@ -911,7 +945,7 @@ GdkPixbuf* colorComboBoxGet_pixbufFromColor(ColorComboBox *colorComboBox, Color
 			   COLUMN_COLOR_PIXBUF, &pixbuf,
 			   COLUMN_COLOR_POINTER_TO, &cl,
 			   -1);
-      if (color == cl)
+      if (cl && colorEqual_color(color, cl))
 	return pixbuf;
       validIter = gtk_tree_model_iter_next(GTK_TREE_MODEL(model), &iter);
     }
