@@ -50,6 +50,24 @@
 #include "gtk_shadeComboBoxWidget.h"
 #include <visu_tools.h>
 
+/**
+ * SECTION:shadecombobox
+ * @short_description: Defines a specialised #GtkComboBox to choose
+ * pre-built shades.
+ * @see_also: #StippleComboBox, #Shade
+ * @include: coreTools/toolShade.h
+ *
+ * <para>This widget looks like a #GtkComboBox and it displays a list
+ * of preset colour shades. These patterns are defined by colour
+ * parameters, see #Shade.</para>
+ * <para>This widget can emit a #ShadeComboBox::shade-selected
+ * signal that is a wrapper around the #GtkComboBox::changed signal,
+ * but it is emitted only when a new shade is selected and this shade
+ * is passed to the callback.</para>
+ *
+ * Since: 3.3
+ */
+
 enum {
   SHADE_SELECTED_SIGNAL,
   LAST_SIGNAL
@@ -59,11 +77,11 @@ enum {
    that contains the informations of stroed shades. */
 enum
   {
-    /* This has a pointer to a 16x16 image to represent the shade. */
+    /* This has a pointer to a 48x16 image to represent the shade. */
     COLUMN_SHADE_PIXBUF,
     /* This is a pointer to a label that describes the shade. */
     COLUMN_SHADE_LABEL,
-    /* This a pointer to the #Shade as defined in visuTools.h */
+    /* This a pointer to the #Shade as defined in toolShade.h */
     COLUMN_SHADE_POINTER_TO,
 
     N_COLUMN_SHADE
@@ -74,15 +92,12 @@ enum
 #define SHADE_BOX_HEIGHT 16
 #define SHADE_BOX_BITS   8
 
-static void shadeComboBox_class_init          (ShadeComboBoxClass *klass);
-static void shadeComboBox_init                (ShadeComboBox *shadeComboBox);
-static void shadeComboBox_dispose             (GObject *obj);
-static void shadeComboBox_finalize            (GObject *obj);
+static void shadeComboBox_dispose (GObject *obj);
+static void shadeComboBox_finalize(GObject *obj);
 
 static guint shadeComboBox_signals[LAST_SIGNAL] = { 0 };
-static GObjectClass *parent_class = NULL;
 
-struct ShadeComboBox_struct
+struct _ShadeComboBox
 {
   GtkComboBox comboShade;
   Shade* previouslySelectedShade;
@@ -93,7 +108,7 @@ struct ShadeComboBox_struct
   gboolean dispose_has_run;
 };
 
-struct ShadeComboBoxClass_struct
+struct _ShadeComboBoxClass
 {
   GtkComboBoxClass parent_class;
 
@@ -111,37 +126,11 @@ struct ShadeComboBoxClass_struct
 static void shadeComboBox_changed(GtkWidget *widget, ShadeComboBox *shadeComboBox);
 
 /* Local methods. */
-void addShadeToModel(GtkTreeIter *iter, ShadeComboBoxClass* klass, Shade* shade);
+static void addShadeToModel(GtkTreeIter *iter, ShadeComboBoxClass* klass,
+			    Shade* shade);
 static void buildWidgets(ShadeComboBox *shadeComboBox);
 
-
-GType shadeComboBox_get_type(void)
-{
-  static GType shadeComboBox_type = 0;
-
-  if (!shadeComboBox_type)
-    {
-      static const GTypeInfo shadeComboBox_info =
-      {
-        sizeof (ShadeComboBoxClass),
-        NULL, /* base_init */
-        NULL, /* base_finalize */
-        (GClassInitFunc) shadeComboBox_class_init,
-        NULL, /* class_finalize */
-        NULL, /* class_data */
-        sizeof (ShadeComboBox),
-        0,
-        (GInstanceInitFunc) shadeComboBox_init,
-        NULL
-      };
-      shadeComboBox_type = g_type_register_static (GTK_TYPE_COMBO_BOX, "ShadeComboBox",
-						   &shadeComboBox_info, 0);
-      DBG_fprintf(stderr, "Gtk ShadeComboBox : creating the type ShadeComboBox %d.\n",
-		  (int)shadeComboBox_type);
-    }
-
-  return shadeComboBox_type;
-}
+G_DEFINE_TYPE(ShadeComboBox, shadeComboBox, GTK_TYPE_COMBO_BOX)
 
 static void shadeComboBox_class_init(ShadeComboBoxClass *klass)
 {
@@ -149,10 +138,16 @@ static void shadeComboBox_class_init(ShadeComboBoxClass *klass)
   GList *shadeLst;
   
   DBG_fprintf(stderr, "Gtk ShadeComboBox : creating the class of the widget.\n");
-
-  parent_class = g_type_class_peek_parent(klass);
-
   DBG_fprintf(stderr, "                     - adding new signals ;\n");
+  /**
+   * ShadeComboBox::shade-selected:
+   * @combo: the #ShadeComboBox that emits the signal ;
+   * @shade: the newly selected #Shade.
+   *
+   * This signal is emitted when a new valid colour shade is selected.
+   *
+   * Since: 3.3
+   */
   shadeComboBox_signals[SHADE_SELECTED_SIGNAL] =
     g_signal_new ("shade-selected",
 		  G_TYPE_FROM_CLASS (klass),
@@ -175,6 +170,10 @@ static void shadeComboBox_class_init(ShadeComboBoxClass *klass)
       addShadeToModel(&iter, klass, (Shade*)shadeLst->data);
       shadeLst = g_list_next(shadeLst);
     }
+
+  /* Connect freeing methods. */
+  G_OBJECT_CLASS(klass)->dispose = shadeComboBox_dispose;
+  G_OBJECT_CLASS(klass)->finalize = shadeComboBox_finalize;
 }
 
 static void shadeComboBox_dispose(GObject *obj)
@@ -186,7 +185,7 @@ static void shadeComboBox_dispose(GObject *obj)
 
   SHADE_COMBOX(obj)->dispose_has_run = TRUE;
   /* Chain up to the parent class */
-  G_OBJECT_CLASS(parent_class)->dispose(obj);
+  G_OBJECT_CLASS(shadeComboBox_parent_class)->dispose(obj);
 }
 static void shadeComboBox_finalize(GObject *obj)
 {
@@ -195,7 +194,7 @@ static void shadeComboBox_finalize(GObject *obj)
   DBG_fprintf(stderr, "Gtk ShadeComboBox : finalize object %p.\n", (gpointer)obj);
 
   /* Chain up to the parent class */
-  G_OBJECT_CLASS(parent_class)->finalize(obj);
+  G_OBJECT_CLASS(shadeComboBox_parent_class)->finalize(obj);
 
   DBG_fprintf(stderr, " | freeing ... OK.\n");
 }
@@ -219,6 +218,7 @@ static void buildWidgets(ShadeComboBox *shadeComboBox)
   klass = G_OBJECT_GET_CLASS(shadeComboBox);
   gtk_combo_box_set_model(GTK_COMBO_BOX(shadeComboBox),
 			  GTK_TREE_MODEL(SHADE_COMBOX_CLASS(klass)->listStoredShades));
+
   renderer = gtk_cell_renderer_pixbuf_new();
   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(shadeComboBox), renderer, FALSE);
   gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(shadeComboBox), renderer,
@@ -228,6 +228,7 @@ static void buildWidgets(ShadeComboBox *shadeComboBox)
   gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(shadeComboBox), renderer,
 				"text", COLUMN_SHADE_LABEL);
 
+  gtk_combo_box_set_active(GTK_COMBO_BOX(shadeComboBox), -1);
   g_signal_connect(G_OBJECT(shadeComboBox), "changed",
 		   G_CALLBACK(shadeComboBox_changed), (gpointer)shadeComboBox);
 }
@@ -249,7 +250,7 @@ GtkWidget* shadeComboBox_new(gboolean hasAlphaChannel)
   return GTK_WIDGET(shadeComboBox);
 }
 
-static void shadeComboBox_changed(GtkWidget *widget, ShadeComboBox *shadeComboBox)
+static void shadeComboBox_changed(GtkWidget *widget _U_, ShadeComboBox *shadeComboBox)
 {
   int selected;
   GtkTreeIter iter;
@@ -261,9 +262,9 @@ static void shadeComboBox_changed(GtkWidget *widget, ShadeComboBox *shadeComboBo
   if (selected < 0)
     {
       shadeComboBox->previouslySelectedShade = (Shade*)0;
-      DBG_fprintf(stderr, "Gtk ShadeComboBox : emitting 'shade-selected' signal.\n");
-      g_signal_emit(G_OBJECT(shadeComboBox),
-		    shadeComboBox_signals[SHADE_SELECTED_SIGNAL], 0, (gpointer)0, NULL);
+/*       DBG_fprintf(stderr, "Gtk ShadeComboBox : emitting 'shade-selected' signal.\n"); */
+/*       g_signal_emit(G_OBJECT(shadeComboBox), */
+/* 		    shadeComboBox_signals[SHADE_SELECTED_SIGNAL], 0, (gpointer)0, NULL); */
       return;
     }
 
@@ -338,7 +339,8 @@ GdkPixbuf* shadeComboBoxBuild_shadeStamp(Shade *shade, GdkPixbuf *pixbuf)
   return pixbufShadeBox;
 }
 
-void addShadeToModel(GtkTreeIter *iter, ShadeComboBoxClass* klass, Shade* shade)
+static void addShadeToModel(GtkTreeIter *iter, ShadeComboBoxClass* klass,
+			    Shade* shade)
 {
   g_return_if_fail(iter && klass && shade);
 
