@@ -51,6 +51,20 @@
 #include <visu_tools.h>
 #include <gtk_main.h>
 
+/**
+ * SECTION:toolpanel
+ * @short_description: Defines a widget that hold a set of V_Sim panel.
+ * 
+ * <para>This widget is a complex association of a #GtkComboBox and a
+ * #GtkNotebook with V_Sim panels. It can have its own window or be
+ * attached into the main command panel.</para>
+ *
+ * <para>This widget also has a built-in menu to exchange V_Sim panel
+ * between different instances of a #ToolPanel. A #ToolPanel is
+ * refered by its name and has a position and a size. These values can
+ * be stored in the parameter files.</para>
+ */
+
 enum {
   NOTEBOOK_ENTERED,
   LAST_SIGNAL
@@ -58,13 +72,14 @@ enum {
 
 enum
   {
+    TOOL_LIST_ICON,            /* The icon shown */
     TOOL_LIST_NAME,            /* The label shown */
     TOOL_LIST_POINTER_TO_DATA, /* Pointer to the tool panel. */
     TOOL_LIST_N_COLUMNS
   };
 
 /* Local structures. */
-struct DockWindow_struct
+struct _DockWindow
 {
   /* The name of the dock window. */
   gchar *name;
@@ -74,6 +89,8 @@ struct DockWindow_struct
   GtkWidget *window;
   /* The top container widget for the DockWindow. */
   GtkWidget *vbox;
+  /* The header line with the combobox. */
+  GtkWidget *hbox;
   /* The notebook holding the tools. */
   GtkWidget *notebook;
   /* The combo list of accessible tool panels. */
@@ -87,7 +104,7 @@ struct DockWindow_struct
 };
 
 /* Object structures. */
-struct ToolPanel_struct
+struct _ToolPanel
 {
   GtkAlignment alignment;
 
@@ -121,7 +138,7 @@ struct ToolPanel_struct
   gboolean dispose_has_run;
 };
 
-struct ToolPanelClass_struct
+struct _ToolPanelClass
 {
   GtkAlignmentClass parent_class;
 
@@ -142,12 +159,16 @@ struct ToolPanelClass_struct
   /* Pointer on current handled VisuData.
      No reference is hold. */
   VisuData *dataObj;
+
+  /* If TRUE, the labels are always shown in the tabs. */
+  gboolean showHeader;
 };
 
+G_DEFINE_TYPE(ToolPanel, toolPanel, GTK_TYPE_ALIGNMENT)
+
 #define MAIN_PANEL_NAME _("Command panel")
 
 /* Local variables. */
-static GObjectClass *parent_class = NULL;
 static ToolPanelClass *local_class = NULL;
 static GQuark CURRENT_TOOLPANEL_POINTER;
 static guint toolPanel_signals[LAST_SIGNAL] = { 0 };
@@ -177,42 +198,22 @@ static void onPageChanged(GtkNotebook *book, GtkNotebookPage *page,
 			  guint num, gpointer data);
 static void onComboChanged(GtkComboBox *combo, gpointer data);
 
-GType toolPanel_get_type(void)
-{
-  static GType toolPanel_type = 0;
-
-  if (!toolPanel_type)
-    {
-      static const GTypeInfo toolPanel_info =
-      {
-        sizeof (ToolPanelClass),
-        NULL, /* base_init */
-        NULL, /* base_finalize */
-        (GClassInitFunc) toolPanel_class_init,
-        NULL, /* class_finalize */
-        NULL, /* class_data */
-        sizeof (ToolPanel),
-        0,
-        (GInstanceInitFunc) toolPanel_init,
-        NULL
-      };
-      toolPanel_type = g_type_register_static (GTK_TYPE_ALIGNMENT, "ToolPanel",
-					       &toolPanel_info, 0);
-      DBG_fprintf(stderr, "Gtk ToolPanel : creating the type ToolPanel %d.\n",
-		  (int)toolPanel_type);
-    }
-
-  return toolPanel_type;
-}
-
 static void toolPanel_class_init(ToolPanelClass *klass)
 {
   DBG_fprintf(stderr, "Gtk ToolPanel : creating the class of the widget.\n");
 
-  parent_class = g_type_class_peek_parent(klass);
   local_class  = klass;
 
   DBG_fprintf(stderr, "                     - adding new signals ;\n");
+  /**
+   * ToolPanel::page-entered:
+   * @panel: the #ToolPanel that emits the signal.
+   *
+   * This signal is emitted when a page of the #GtkNotebook with all
+   * the V_Sim panels is entered.
+   *
+   * Since: 3.3
+   */
   toolPanel_signals[NOTEBOOK_ENTERED] =
     g_signal_new ("page-entered",
 		  G_TYPE_FROM_CLASS(klass),
@@ -230,8 +231,10 @@ static void toolPanel_class_init(ToolPanelClass *klass)
   klass->dataObj = (VisuData*)0;
   klass->allToolPanels = g_hash_table_new_full(g_str_hash, g_str_equal,
 					       NULL, NULL);
+  klass->showHeader = FALSE;
 
-  CURRENT_TOOLPANEL_POINTER = g_quark_from_static_string("ToolPanel_currentToolPanel");
+  CURRENT_TOOLPANEL_POINTER =
+    g_quark_from_static_string("ToolPanel_currentToolPanel");
 
   /* Connect freeing methods. */
   G_OBJECT_CLASS(klass)->dispose = toolPanel_dispose;
@@ -251,7 +254,7 @@ static void toolPanel_dispose(GObject *toolPanel)
 
   TOOL_PANEL(toolPanel)->dispose_has_run = TRUE;
   /* Chain up to the parent class */
-  G_OBJECT_CLASS(parent_class)->dispose(toolPanel);
+  G_OBJECT_CLASS(toolPanel_parent_class)->dispose(toolPanel);
 }
 /* This method is called once only. */
 static void toolPanel_finalize(GObject *obj)
@@ -271,16 +274,11 @@ static void toolPanel_finalize(GObject *obj)
     g_free(toolPanel->comboLabel);
   if (toolPanel->tabLabel)
     g_free(toolPanel->tabLabel);
-  if (toolPanel->icon)
-    g_object_unref(G_OBJECT(toolPanel->icon));
   if (toolPanel->id)
     g_free(toolPanel->id);
 
-  if (toolPanel->headerWidget)
-    g_object_unref(G_OBJECT(toolPanel->headerWidget));
-
   /* Chain up to the parent class */
-  G_OBJECT_CLASS(parent_class)->finalize(obj);
+  G_OBJECT_CLASS(toolPanel_parent_class)->finalize(obj);
 
   DBG_fprintf(stderr, "Gtk ToolPanel : freeing ... OK.\n");
 }
@@ -358,6 +356,7 @@ GtkWidget* toolPanelNew_withIconFromStock(gchar *id, gchar* name, gchar *tabName
 GtkWidget* toolPanelGet_headerWidget(ToolPanel *toolPanel)
 {
   GtkWidget *label, *wd, *image, *align;
+  GtkTooltips *tooltips;
 
   g_return_val_if_fail(toolPanel, (GtkWidget*)0);
 
@@ -365,12 +364,24 @@ GtkWidget* toolPanelGet_headerWidget(ToolPanel *toolPanel)
      we do it now. */
   if (!toolPanel->headerWidget)
     {
+      tooltips = gtk_tooltips_new ();
+
       toolPanel->headerWidget = gtk_hbox_new(FALSE, 0);
+      wd = gtk_event_box_new();
+      gtk_event_box_set_visible_window(GTK_EVENT_BOX(wd), FALSE);
+      gtk_box_pack_start(GTK_BOX(toolPanel->headerWidget),
+			 wd, FALSE, FALSE, 0);
+      gtk_tooltips_set_tip(tooltips, wd, toolPanel->comboLabel, NULL);
       if (toolPanel->icon)
-	gtk_box_pack_start(GTK_BOX(toolPanel->headerWidget),
-			   toolPanel->icon, FALSE, FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(wd), toolPanel->icon);
+      else
+	gtk_container_add(GTK_CONTAINER(wd),
+			  gtk_image_new_from_stock(GTK_STOCK_MISSING_IMAGE,
+						   GTK_ICON_SIZE_MENU));
       label = gtk_label_new(toolPanel->tabLabel);
+      gtk_misc_set_padding(GTK_MISC(label), 2, 0);
       gtk_box_pack_start(GTK_BOX(toolPanel->headerWidget), label, FALSE, FALSE, 0);
+
       if (toolPanel->dockable)
 	{
 	  wd = gtk_button_new();
@@ -382,10 +393,18 @@ GtkWidget* toolPanelGet_headerWidget(ToolPanel *toolPanel)
 	  image = create_pixmap((GtkWidget*)0, "stock-menu-detach.png");
 	  gtk_container_add(GTK_CONTAINER(wd), image);
 	  gtk_container_add(GTK_CONTAINER(align), wd);
+
+	  gtk_tooltips_set_tip(tooltips, wd,
+			       _("Manage this subpanel: attach/detach"
+				 " or hide it."), NULL);
 	}
+      else
+	align = (GtkWidget*)0;
 
       g_object_ref(G_OBJECT(toolPanel->headerWidget));
       gtk_widget_show_all(toolPanel->headerWidget);
+      gtk_widget_hide(label);
+      if (align) gtk_widget_hide(align);
     }
 
   return toolPanel->headerWidget;
@@ -418,6 +437,20 @@ DockWindow* toolPanelGet_container(ToolPanel *toolPanel)
 
   return toolPanel->container;
 }
+GtkWindow* toolPanelGet_containerWindow(ToolPanel *toolPanel)
+{
+  g_return_val_if_fail(IS_TOOL_PANEL(toolPanel), (GtkWindow*)0);
+
+  if (toolPanel->container)
+    {
+      if (toolPanel->container->window)
+	return GTK_WINDOW(toolPanel->container->window);
+      else
+	return GTK_WINDOW(gtkMainClassGet_currentPanel());
+    }
+  else
+    return (GtkWindow*)0;
+}
 void toolPanelSet_container(ToolPanel *toolPanel, DockWindow *container)
 {
   g_return_if_fail(IS_TOOL_PANEL(toolPanel));
@@ -468,12 +501,27 @@ VisuData* toolPanelGet_visuData(ToolPanel *toolPanel)
   
   return TOOL_PANEL_CLASS(G_OBJECT_GET_CLASS(toolPanel))->dataObj;
 }
+gboolean toolPanelGet_visible(ToolPanel *toolPanel)
+{
+  g_return_val_if_fail(IS_TOOL_PANEL(toolPanel), FALSE);
+
+  /* If the tool panel is not shown, we return fFALSE. */
+  if (!toolPanel->container || !toolPanel->container->show)
+    return FALSE;
+
+  /* If the tool panel is attached to a visible dock, we
+     test if the current page of the notebook is our tool panel. */
+  return (toolPanel == TOOL_PANEL(gtk_notebook_get_nth_page
+	  (GTK_NOTEBOOK(toolPanel->container->notebook),
+	   gtk_notebook_get_current_page
+	   (GTK_NOTEBOOK(toolPanel->container->notebook)))));
+}
 
 
 /*****************/
 /* Menu gestion. */
 /*****************/
-static void onDockButtonClicked(ToolPanel *toolPanel, gpointer data)
+static void onDockButtonClicked(ToolPanel *toolPanel, gpointer data _U_)
 {
   GtkWidget *wd;
 
@@ -541,7 +589,7 @@ static void onDockMenuClicked(GtkMenuItem *menuitem, gpointer user_data)
 
   toolPanelSet_container(toolPanel, (DockWindow*)user_data);
 }
-static void onDockMenuNewClicked(GtkMenuItem *menuitem, gpointer user_data)
+static void onDockMenuNewClicked(GtkMenuItem *menuitem _U_, gpointer user_data)
 {
   GList **lst;
   ToolPanel *toolPanel;
@@ -560,13 +608,13 @@ static void onDockMenuNewClicked(GtkMenuItem *menuitem, gpointer user_data)
   if (toolPanel)
     toolPanelSet_container(toolPanel, window);
 }
-static void onDockMenuHideClicked(GtkMenuItem *menuitem, gpointer user_data)
+static void onDockMenuHideClicked(GtkMenuItem *menuitem _U_, gpointer user_data)
 {
   DBG_fprintf(stderr, "Gtk ToolPanel : hide the panel %p.\n", (gpointer)user_data);
   toolPanelSet_container(TOOL_PANEL(user_data), (DockWindow*)0);
 }
 
-static void onDockMenuSelected(GtkMenuShell *menushell, gpointer user_data)
+static void onDockMenuSelected(GtkMenuShell *menushell, gpointer user_data _U_)
 {
   DBG_fprintf(stderr, "Gtk ToolPanel : destroy the dock menu %p.\n", (gpointer)menushell);
   gtk_widget_destroy(GTK_WIDGET(menushell));
@@ -575,7 +623,7 @@ static void onDockMenuSelected(GtkMenuShell *menushell, gpointer user_data)
 static DockWindow* buildDockWindow(gchar *name, gboolean withWindow)
 {
   DockWindow *window;
-  GtkWidget *hbox, *wd, *image, *align;
+  GtkWidget *wd, *image, *align;
   GtkTooltips *tooltips;
   GtkCellRenderer *renderer;
 
@@ -586,7 +634,7 @@ static DockWindow* buildDockWindow(gchar *name, gboolean withWindow)
 
   window->name = name;
 
-  window->show = TRUE;
+  window->show = !strcmp(name, MAIN_PANEL_NAME);
 
   window->vbox = gtk_vbox_new(FALSE, 0);
 
@@ -604,7 +652,7 @@ static DockWindow* buildDockWindow(gchar *name, gboolean withWindow)
 
       gtk_container_set_border_width(GTK_CONTAINER(window->window), 3);
       gtk_container_add(GTK_CONTAINER(window->window), window->vbox);
-      gtk_widget_show(window->window);
+/*       gtk_widget_show(window->window); */
     }
   else
     window->window = (GtkWidget*)0;
@@ -614,56 +662,69 @@ static DockWindow* buildDockWindow(gchar *name, gboolean withWindow)
   gtk_box_pack_end(GTK_BOX(window->vbox), align, TRUE, TRUE, 0);
   
   window->notebook = gtk_notebook_new();
+  gtk_notebook_set_tab_border(GTK_NOTEBOOK(window->notebook), 1);
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(window->notebook), TRUE);
   gtk_container_add(GTK_CONTAINER(align), window->notebook);
-  window->notebookChanged = g_signal_connect(G_OBJECT(window->notebook), "switch-page",
-					     G_CALLBACK(onPageChanged), (gpointer)window);
+  window->notebookChanged =
+    g_signal_connect(G_OBJECT(window->notebook), "switch-page",
+		     G_CALLBACK(onPageChanged), (gpointer)window);
 
-  hbox = gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(window->vbox), hbox, FALSE, FALSE, 0);
+  window->hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(window->vbox), window->hbox, FALSE, FALSE, 0);
 
   tooltips = gtk_tooltips_new ();
 
   /* Add a help tooltips. */
   wd = gtk_event_box_new();
-  gtk_tooltips_set_tip (tooltips, wd,
-			_("Positions, sizes, names, contains... of dock" \
-			  " windows are stored in the parameters file, " \
-			  "see the 'Config. files' button on the command panel."), NULL);
-  gtk_box_pack_end(GTK_BOX(hbox), wd, FALSE, FALSE, 2);
+  gtk_event_box_set_visible_window(GTK_EVENT_BOX(wd), FALSE);
+  gtk_tooltips_set_tip(tooltips, wd,
+		       _("Positions, sizes, names, contains... of dock" \
+			 " windows are stored in the parameters file, " \
+			 "see the 'Config. files' button on the command panel."), NULL);
+  gtk_box_pack_end(GTK_BOX(window->hbox), wd, FALSE, FALSE, 2);
   image = gtk_image_new_from_stock(GTK_STOCK_HELP, GTK_ICON_SIZE_MENU);
   gtk_container_add(GTK_CONTAINER(wd), image);
 
   /* Add a menu button. */
   wd = gtk_button_new();
   gtk_button_set_relief(GTK_BUTTON(wd), GTK_RELIEF_NONE);
-  gtk_box_pack_end(GTK_BOX(hbox), wd, FALSE, FALSE, 2);
+  gtk_box_pack_end(GTK_BOX(window->hbox), wd, FALSE, FALSE, 2);
   g_signal_connect(G_OBJECT(wd), "clicked",
 		   G_CALLBACK(onMainButtonClicked), (gpointer)window);
   image = create_pixmap((GtkWidget*)0, "stock-menu-detach.png");
   gtk_container_add(GTK_CONTAINER(wd), image);
+  gtk_tooltips_set_tip(tooltips, wd,
+		       _("Manage hidden subpanels and dock windows."), NULL);
 
   /* Little image. */
   image = create_pixmap((GtkWidget*)0, "icone-tool.png");
-  gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(window->hbox), image, FALSE, FALSE, 0);
 
   /* The Label to introduce the combo list. */
   wd = gtk_label_new(_("Tool: "));
-  gtk_box_pack_start(GTK_BOX(hbox), wd, FALSE, FALSE, 2);
+  gtk_box_pack_start(GTK_BOX(window->hbox), wd, FALSE, FALSE, 2);
 
   /* We create the list store. */
   window->list = gtk_list_store_new(TOOL_LIST_N_COLUMNS,
+				    GDK_TYPE_PIXBUF,
 				    G_TYPE_STRING, G_TYPE_POINTER);
   /* The combo list. */
   window->combo = gtk_combo_box_new_with_model(GTK_TREE_MODEL(window->list));
+  renderer = gtk_cell_renderer_pixbuf_new();
+  gtk_cell_renderer_set_fixed_size(renderer, 22, -1);
+  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(window->combo), renderer, FALSE);
+  gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(window->combo),
+				renderer, "pixbuf", TOOL_LIST_ICON);
   renderer = gtk_cell_renderer_text_new();
   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(window->combo), renderer, FALSE);
   gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(window->combo),
 				renderer, "text", TOOL_LIST_NAME);
-  gtk_box_pack_start(GTK_BOX(hbox), window->combo, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(window->hbox), window->combo, TRUE, TRUE, 0);
   window->comboChanged = g_signal_connect(G_OBJECT(window->combo), "changed",
-					  G_CALLBACK(onComboChanged), (gpointer)window);
+					  G_CALLBACK(onComboChanged),
+					  (gpointer)window);
 
+  gtk_widget_show_all(window->vbox);
   return window;
 }
 
@@ -678,11 +739,31 @@ static gboolean emitPageEnter(gpointer data)
   return FALSE;
 }
 
-static void onPageChanged(GtkNotebook *book, GtkNotebookPage *page,
+static void changeHeaderVisibility(ToolPanel *panel, gboolean status)
+{
+  GList *list, *tmpLst;
+  
+  g_return_if_fail(IS_TOOL_PANEL(panel) && panel->headerWidget);
+
+  list = gtk_container_get_children(GTK_CONTAINER(panel->headerWidget));
+  tmpLst = list->next;
+  while(tmpLst)
+    {
+      if (status)
+	gtk_widget_show(GTK_WIDGET(tmpLst->data));
+      else
+	gtk_widget_hide(GTK_WIDGET(tmpLst->data));
+      tmpLst = g_list_next(tmpLst);
+    }
+  g_list_free(list);
+}
+
+static void onPageChanged(GtkNotebook *book, GtkNotebookPage *page _U_,
 			  guint num, gpointer data)
 {
   GtkWidget *wd;
   DockWindow *window;
+  int numOld;
 
   wd = gtk_notebook_get_nth_page(book, num);
   g_return_if_fail(IS_TOOL_PANEL(wd));
@@ -691,28 +772,54 @@ static void onPageChanged(GtkNotebook *book, GtkNotebookPage *page,
   g_return_if_fail(window);
 
   DBG_fprintf(stderr, "Gtk ToolPanel : caught 'page-switch' signal, jump to %d.\n", num);
+  /* Change the visibility of the header labels. */
+  if (!local_class->showHeader)
+    {
+      numOld = gtk_combo_box_get_active(GTK_COMBO_BOX(window->combo));
+      changeHeaderVisibility
+	(TOOL_PANEL(gtk_notebook_get_nth_page(book, numOld)), FALSE);
+      changeHeaderVisibility
+	(TOOL_PANEL(gtk_notebook_get_nth_page(book, num)), TRUE);
+    }
+  
   /* Change the combo to the right one. */
   g_signal_handler_block(G_OBJECT(window->combo), window->comboChanged);
   gtk_combo_box_set_active(GTK_COMBO_BOX(window->combo), num);
   g_signal_handler_unblock(G_OBJECT(window->combo), window->comboChanged);
 
+
   g_idle_add(emitPageEnter, wd);
 }
 
 static void onComboChanged(GtkComboBox *combo, gpointer data)
 {
-  int index;
+  int index, numOld;
   DockWindow *window;
+  GtkNotebook *book;
   
   window = (DockWindow*)data;
   g_return_if_fail(window);
 
   index = gtk_combo_box_get_active(combo);
+
+  /* Change the visibility of the header labels. */
+  if (!local_class->showHeader)
+    {
+      book = GTK_NOTEBOOK(window->notebook);
+      numOld = gtk_notebook_get_current_page(book);
+      changeHeaderVisibility
+	(TOOL_PANEL(gtk_notebook_get_nth_page(book, numOld)), FALSE);
+      changeHeaderVisibility
+	(TOOL_PANEL(gtk_notebook_get_nth_page(book, index)), TRUE);
+    }
+
+  /* Change the page to the right one. */
   gtk_notebook_set_current_page(GTK_NOTEBOOK(window->notebook), index);
 }
 
 
-static gboolean onKillDockWindowEvent(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+static gboolean onKillDockWindowEvent(GtkWidget *widget, GdkEvent *event _U_,
+				      gpointer user_data)
 {
   g_return_val_if_fail(user_data, TRUE);
 
@@ -726,23 +833,36 @@ void toolPanelAttach(ToolPanel *toolPanel, DockWindow *dock)
 {
   GtkTreeIter iter;
   ToolPanelClass *klass;
+  GdkPixbuf *pixbuf;
 
   DBG_fprintf(stderr, "Gtk ToolPanel : attach a toolPanel (%p) to a dock"
 	      " window (%s).\n", (gpointer)toolPanel, dock->name);
 
   g_return_if_fail(dock && toolPanel);
 
+  if (gtk_image_get_storage_type(GTK_IMAGE(toolPanel->icon)) == GTK_IMAGE_PIXBUF)
+    pixbuf = gtk_image_get_pixbuf(GTK_IMAGE(toolPanel->icon));
+  else
+    pixbuf = (GdkPixbuf*)0;
+
   /* We add the window in the list store. */
   gtk_list_store_append(dock->list, &iter);
   gtk_list_store_set(dock->list, &iter,
+		     TOOL_LIST_ICON, pixbuf,
 		     TOOL_LIST_NAME, toolPanel->comboLabel,
 		     TOOL_LIST_POINTER_TO_DATA, toolPanel,
 		     -1);
 
   gtk_notebook_append_page(GTK_NOTEBOOK(dock->notebook), GTK_WIDGET(toolPanel),
 			   toolPanelGet_headerWidget(toolPanel));
+  gtk_widget_show(GTK_WIDGET(toolPanel));
   toolPanel->container = dock;
 
+  if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(dock->notebook)) > 1)
+    gtk_widget_show(dock->hbox);
+  else
+    gtk_widget_hide(dock->hbox);
+
   klass = TOOL_PANEL_CLASS(G_OBJECT_GET_CLASS(toolPanel));
   klass->orphanToolPanel = g_list_remove(klass->orphanToolPanel,
 					 (gpointer)toolPanel);
@@ -771,6 +891,9 @@ void toolPanelDetach(ToolPanel *toolPanel)
   /* We remove the page for the notebook. */
   gtk_notebook_remove_page(GTK_NOTEBOOK(window->notebook), page);
 
+  if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(window->notebook)) < 2)
+    gtk_widget_hide(window->hbox);
+
   /* We remove the tool from the combo list. */
   path = gtk_tree_path_new_from_indices(page, -1);
   valid = gtk_tree_model_get_iter(GTK_TREE_MODEL(window->list), &iter, path);
@@ -801,7 +924,7 @@ void toolPanelDetach(ToolPanel *toolPanel)
 	      g_list_length(klass->orphanToolPanel));
 }
 
-static void onMainButtonClicked(GtkButton *button, gpointer data)
+static void onMainButtonClicked(GtkButton *button _U_, gpointer data)
 {
   GtkWidget *wd;
 
@@ -906,7 +1029,7 @@ static void onMainMenuClicked(GtkMenuItem *menuitem, gpointer user_data)
   toolPanelAttach(toolPanel, window);
   g_object_unref(G_OBJECT(toolPanel));
 }
-static void onMainMenuShowClicked(GtkMenuItem *menuitem, gpointer user_data)
+static void onMainMenuShowClicked(GtkMenuItem *menuitem _U_, gpointer user_data)
 {
   DockWindow *dock;
 
@@ -917,7 +1040,7 @@ static void onMainMenuShowClicked(GtkMenuItem *menuitem, gpointer user_data)
     gtk_widget_show(dock->window);
   dock->show = TRUE;
 }
-static void onMainMenuHideClicked(GtkMenuItem *menuitem, gpointer user_data)
+static void onMainMenuHideClicked(GtkMenuItem *menuitem _U_, gpointer user_data)
 {
   DockWindow *dock;
 
@@ -928,7 +1051,7 @@ static void onMainMenuHideClicked(GtkMenuItem *menuitem, gpointer user_data)
     gtk_widget_hide(dock->window);
   dock->show = FALSE;
 }
-static void onMainMenuSelected(GtkMenuShell *menushell, gpointer user_data)
+static void onMainMenuSelected(GtkMenuShell *menushell, gpointer user_data _U_)
 {
   DBG_fprintf(stderr, "Gtk ToolPanel : destroy the main menu %p.\n", (gpointer)menushell);
   gtk_widget_destroy(GTK_WIDGET(menushell));
@@ -987,7 +1110,10 @@ void toolPanelClassSet_windowSize(const gchar *id, guint width, guint height)
 
   /* Before setting (width, height), we check that it is coherent with
      the screen size. */
-  screen = gdk_drawable_get_screen(GDK_DRAWABLE(dock->window->window));
+  if (GDK_IS_DRAWABLE(dock->window->window))
+    screen = gdk_drawable_get_screen(GDK_DRAWABLE(dock->window->window));
+  else
+    screen = gdk_screen_get_default();
   widthScreen = gdk_screen_get_width(screen);
   heightScreen = gdk_screen_get_height(screen);
 
@@ -995,6 +1121,8 @@ void toolPanelClassSet_windowSize(const gchar *id, guint width, guint height)
   heightSet = ((gint)height > heightScreen)?heightScreen:(gint)height;
   if (widthSet < 0) widthSet = 50;
   if (heightSet < 0) heightSet = 50;
+  DBG_fprintf(stderr, "Gtk ToolPanel: set window (%s) size %dx%d.\n",
+	      id, widthSet, heightSet);
   gtk_window_resize(GTK_WINDOW(dock->window), widthSet, heightSet);
 }
 void toolPanelClassSet_windowPosition(const gchar *id, guint x, guint y)
@@ -1013,7 +1141,10 @@ void toolPanelClassSet_windowPosition(const gchar *id, guint x, guint y)
 
   /* Before setting (x, y), we check that it is coherent with
      the screen definition. */
-  screen = gdk_drawable_get_screen(GDK_DRAWABLE(dock->window->window));
+  if (GDK_IS_DRAWABLE(dock->window->window))
+    screen = gdk_drawable_get_screen(GDK_DRAWABLE(dock->window->window));
+  else
+    screen = gdk_screen_get_default();
   xScreen = gdk_screen_get_width(screen);
   yScreen = gdk_screen_get_height(screen);
   gtk_window_get_size(GTK_WINDOW(dock->window), &width, &height);
@@ -1022,6 +1153,8 @@ void toolPanelClassSet_windowPosition(const gchar *id, guint x, guint y)
   yPosition = ((gint)y + height > yScreen)?yScreen - height:(gint)y;
   if (xPosition < 0) xPosition = 0;
   if (yPosition < 0) yPosition = 0;
+  DBG_fprintf(stderr, "Gtk ToolPanel: set window (%s) position %dx%d.\n",
+	      id, xPosition, yPosition);
   gtk_window_move(GTK_WINDOW(dock->window), xPosition, yPosition);
 }
 void toolPanelClassSet_windowVisibility(const gchar *id, gboolean visible)
@@ -1063,7 +1196,6 @@ DockWindow* toolPanelClassGet_container(const gchar *id)
 {
   GObject *dummy;
   DockWindow *window;
-  GtkWidget *child;
   GList *tmplst;
 
   if (!local_class)
@@ -1091,8 +1223,6 @@ DockWindow* toolPanelClassGet_container(const gchar *id)
   local_class->hostingWindows = g_list_prepend(local_class->hostingWindows,
 					      (gpointer)window);
 
-  child = gtk_bin_get_child(GTK_BIN(window->window));
-  gtk_widget_show_all(child);
   return window;
 }
 GList* toolPanelClassGet_allToolPanels()
@@ -1147,6 +1277,36 @@ GList* toolPanelClassGet_allWindows()
   returnlst = g_list_remove(returnlst, local_class->commandPanel);
   return returnlst;
 }
+void toolPanelClassSet_headerVisibility(gboolean status)
+{
+  GObject *dummy;
+  GList *tmpLst;
+  int num, i;
+  GtkNotebook *notebook;
+
+  if (!local_class)
+    {
+      dummy = g_object_new(toolPanel_get_type(), NULL);
+      g_object_ref(dummy);
+      g_object_unref(dummy);
+    }
+  
+  DBG_fprintf(stderr, "Gtk ToolPanel : set the header visibility.\n");
+  if (local_class->showHeader == status)
+    return;
+
+  tmpLst = local_class->hostingWindows;
+  while(tmpLst)
+    {
+      notebook = GTK_NOTEBOOK(((DockWindow*)tmpLst->data)->notebook);
+      num = gtk_notebook_get_current_page(notebook);
+      for (i = 0; i < gtk_notebook_get_n_pages(notebook); i++)
+	changeHeaderVisibility(TOOL_PANEL(gtk_notebook_get_nth_page(notebook, i)),
+			       status || (i == num));
+      tmpLst = g_list_next(tmpLst);
+    }
+  local_class->showHeader = status;
+}
 void toolPanelClassGet_windowCharacteristics(DockWindow *dock, gchar **id, 
 					     gboolean *visibility, 
 					     gint *x, gint *y, 
@@ -1171,3 +1331,12 @@ GtkWidget* dockWindowGet_notebook(DockWindow *dock)
   
   return dock->notebook;
 }
+GtkWidget* dockWindowGet_window(DockWindow *dock)
+{
+  g_return_val_if_fail(dock, (GtkWidget*)0);
+  
+  if (dock->window)
+    return dock->window;
+  else
+    return GTK_WIDGET(gtkMainClassGet_currentPanel());
+}
