@@ -53,17 +53,37 @@
 #include "OSOpenGL/visu_openGL.h"
 #include "visu_tools.h"
 
-/* OS dependent includes */
-#if SYSTEM_X11 == 1
-#include <GL/glx.h>
-#include <gdk/gdkx.h>
-#endif
-#if SYSTEM_WIN32 == 1
-#include <windows.h>
-#include <gdk/gdkwin32.h>
+/* OpenGL implementation dependencies */
+#ifdef HAVE_GTKGLEXT
+  #include <gtk/gtkgl.h>
+  #include <gdk/gdkx.h>
+  #define IMPL_GTKGLEXT
+#else
+  #if SYSTEM_X11 == 1
+  #include <GL/glx.h>
+  #include <gdk/gdkx.h>
+  #define IMPL_BUILTIN_X11
+  #endif
+  #if SYSTEM_WIN32 == 1
+  #include <windows.h>
+  #include <gdk/gdkwin32.h>
+  #define IMPL_BUILTIN_WIN32
+  #endif
 #endif
 
-struct OpenGLWidget_struct
+/**
+ * SECTION:openglwidget
+ * @short_description: defines an OpenGL capable GtkWidget.
+ *
+ * <para>This is a simple implementation of GtkGlExt to create an
+ * OpenGL surface that is a full GtkWidget. When creating such a
+ * widget, one should give then a RedrawMethod() to tell the widget
+ * how to redraw itself when needed.</para>
+ *
+ * <para>The current implementation is working on X11 (built-in or
+ * with GtkGlExt) and Win32.</para>
+ */
+struct _OpenGLWidget
 {
   GtkWidget parent;
 
@@ -75,40 +95,44 @@ struct OpenGLWidget_struct
   gpointer redrawData;
 
   /* OpenGL part, OS dependent. */
-#if SYSTEM_X11 == 1
-  Display *dpy;
   gboolean isContextDirect;
+#ifdef IMPL_GTKGLEXT
+  GdkGLConfig *glconfig;
+  GdkGLWindow *glwindow;
+  GdkGLContext *context;
+#endif
+#ifdef IMPL_BUILTIN_X11
+  Display *dpy;
+  XVisualInfo *vinfo;
   GLXContext context;
 #endif
-#if SYSTEM_WIN32 == 1
+#ifdef IMPL_BUILTIN_WIN32
   HDC hdc;
-  gboolean isContextDirect;
   HGLRC context;
   HWND windowId;
 #endif
 };
 
-struct OpenGLWidgetClass_struct
+struct _OpenGLWidgetClass
 {
   GtkWidgetClass parent_class;
 
   OpenGLWidget *contextCurrent;
 };
 
+G_DEFINE_TYPE(OpenGLWidget, openGLWidget, GTK_TYPE_WIDGET)
+
 /* Local variables. */
-static gpointer parent_class;
 static OpenGLWidgetClass *myClass = (OpenGLWidgetClass*)0;
 
 /* Local callbacks. */
 static gboolean openGLWidgetEvent_expose(GtkWidget *widget, GdkEventExpose *event);
+static gboolean openGLWidgetEvent_visibility(GtkWidget *widget, GdkEventVisibility *event);
 static void openGLWidgetEvent_sizeRequest(GtkWidget *widget, GtkRequisition *requisition);
 static void openGLWidgetEvent_sizeAllocate(GtkWidget *widget, GtkAllocation *allocation);
 static void openGLWidgetEvent_realise(GtkWidget *widget);
-static void openGLWidgetEvent_styleSet(GtkWidget *widget, GtkStyle *previous_style);
 
 /* Initialisation methods. */
-static void openGLWidgetInit_class(OpenGLWidgetClass *class);
-static void openGLWidgetInit_object(OpenGLWidget *render);
 static void openGLWidgetInit_context(OpenGLWidget *render, gboolean contextIsDirect);
 
 /* Freeing methods. */
@@ -120,41 +144,13 @@ static void openGLWidgetFree_openGL(OpenGLWidget *render);
 static void openGLWidgetSet_viewport(OpenGLWidget *render, guint width,
 				     guint height, gboolean redraw);
 static GdkColormap* openGLWidgetGet_openGLColormap(OpenGLWidget *render);
+static void openGLWidgetSet_pixelFormat(OpenGLWidget *render);
 
-/* Methods to deals with the handling of the object. */
-GType openGLWidgetGet_type(void)
-{
-  static GType openGLWidget_type = 0;
-
-  if (!openGLWidget_type)
-    {
-      static const GTypeInfo openGLWidget_info =
-      {
-        sizeof (OpenGLWidgetClass),
-        NULL, /* base_init */
-        NULL, /* base_finalize */
-        (GClassInitFunc) openGLWidgetInit_class,
-        NULL, /* class_finalize */
-        NULL, /* class_data */
-        sizeof (OpenGLWidget),
-        0,
-        (GInstanceInitFunc) openGLWidgetInit_object,
-        NULL
-      };
-      openGLWidget_type = g_type_register_static(GTK_TYPE_WIDGET, "OpenGLWidget",
-						 &openGLWidget_info, 0);
-    }
-
-  return openGLWidget_type;
-}
-static void openGLWidgetInit_class(OpenGLWidgetClass *class)
+static void openGLWidget_class_init(OpenGLWidgetClass *class)
 {
   GtkWidgetClass *widget_class;
   GObjectClass *gobject_class;
 
-  /* Put a static pointer on the parent class for chaining actions. */
-  parent_class = g_type_class_peek_parent(class);
-
   /* Dealing with GObject events. */
   gobject_class = G_OBJECT_CLASS(class);
   gobject_class->dispose  = openGLWidgetEvent_dispose;
@@ -163,6 +159,7 @@ static void openGLWidgetInit_class(OpenGLWidgetClass *class)
   /* Dealing with widget events. */
   widget_class = GTK_WIDGET_CLASS(class);
   widget_class->expose_event  = openGLWidgetEvent_expose;
+  widget_class->visibility_notify_event  = openGLWidgetEvent_visibility;
   widget_class->realize       = openGLWidgetEvent_realise;
   widget_class->size_request  = openGLWidgetEvent_sizeRequest;
   widget_class->size_allocate = openGLWidgetEvent_sizeAllocate;
@@ -172,7 +169,7 @@ static void openGLWidgetInit_class(OpenGLWidgetClass *class)
   myClass = class;
 }
 
-static void openGLWidgetInit_object(OpenGLWidget *render)
+static void openGLWidget_init(OpenGLWidget *render)
 {
   DBG_fprintf(stderr, "Gtk OpenGL (init) : create object %p.\n", (gpointer)render);
   render->sizeAllocation_has_run = FALSE;
@@ -182,19 +179,25 @@ static void openGLWidgetInit_object(OpenGLWidget *render)
   render->redrawData = (gpointer)0;
 
   render->isContextDirect = FALSE;
-#if SYSTEM_X11 == 1
-  render->dpy = (Display*)0;
-  render->context = (GLXContext)0;
+#ifdef IMPL_GTKGLEXT
+  render->glconfig = (GdkGLConfig*)0;
+  render->glwindow = (GdkGLWindow*)0;
+  render->context  = (GdkGLContext*)0;
 #endif
-#if SYSTEM_WIN32 == 1
-  render->hdc = (HDC)0;
-  render->context = (HGLRC)0;
+#ifdef IMPL_BUILTIN_X11
+  render->dpy      = (Display*)0;
+  render->vinfo    = (XVisualInfo*)0;
+  render->context  = (GLXContext)0;
+#endif
+#ifdef IMPL_BUILTIN_WIN32
+  render->hdc      = (HDC)0;
+  render->context  = (HGLRC)0;
   render->windowId = (HWND)0;
+#endif
 
   /* Cancel the GTK double buffering since it is taken into
      account by OpenGL itself. */
   gtk_widget_set_double_buffered(GTK_WIDGET(render), FALSE);
-#endif
 }
 
 GtkWidget* openGLWidgetNew(gboolean contextIsDirect)
@@ -218,7 +221,7 @@ static void openGLWidgetEvent_dispose(GObject *obj)
 
   OPENGL_WIDGET(obj)->dispose_has_run = TRUE;
   /* Chain up to the parent class */
-  G_OBJECT_CLASS(parent_class)->dispose(obj);
+  G_OBJECT_CLASS(openGLWidget_parent_class)->dispose(obj);
 }
 static void openGLWidgetEvent_finalize(GObject *obj)
 {
@@ -227,13 +230,13 @@ static void openGLWidgetEvent_finalize(GObject *obj)
   openGLWidgetFree_openGL(OPENGL_WIDGET(obj));
 
   /* Chain up to the parent class */
-  G_OBJECT_CLASS(parent_class)->finalize(obj);
+  G_OBJECT_CLASS(openGLWidget_parent_class)->finalize(obj);
 }
 static gboolean openGLWidgetEvent_expose(GtkWidget *widget, GdkEventExpose *event)
 {
   OpenGLWidget *render;
 
-  DBG_fprintf(stderr, "Gtk OpenGL (events) : expose event for object %p.\n", (gpointer)widget);
+  DBG_fprintf(stderr, "Gtk OpenGL (events): expose event for object %p.\n", (gpointer)widget);
 
   render = OPENGL_WIDGET(widget);
   if (render->redraw)
@@ -247,6 +250,25 @@ static gboolean openGLWidgetEvent_expose(GtkWidget *widget, GdkEventExpose *even
 
   return FALSE;
 }
+static gboolean openGLWidgetEvent_visibility(GtkWidget *widget, GdkEventVisibility *event)
+{
+  OpenGLWidget *render;
+
+  if (event->state != GDK_VISIBILITY_UNOBSCURED)
+    return FALSE;
+
+  DBG_fprintf(stderr, "Gtk OpenGL (events): visibility event for object %p.\n", (gpointer)widget);
+  render = OPENGL_WIDGET(widget);
+  if (render->redraw)
+    openGLWidgetRedraw(render);
+  else
+    {
+      DBG_fprintf(stderr, " | clear window.\n");
+      gdk_window_clear(widget->window);
+    }
+
+  return FALSE;
+}
 static void openGLWidgetEvent_sizeRequest(GtkWidget *widget, GtkRequisition *requisition)
 {
   OpenGLWidget *render;
@@ -268,7 +290,7 @@ static void openGLWidgetEvent_sizeRequest(GtkWidget *widget, GtkRequisition *req
   widget->requisition.width = width;
   widget->requisition.height = height;
   /* Chain up to default that simply reads current requisition */
-  GTK_WIDGET_CLASS(parent_class)->size_request(widget, requisition);
+  GTK_WIDGET_CLASS(openGLWidget_parent_class)->size_request(widget, requisition);
 }
 static void openGLWidgetEvent_sizeAllocate(GtkWidget *widget, GtkAllocation *allocation)
 {
@@ -283,13 +305,13 @@ static void openGLWidgetEvent_sizeAllocate(GtkWidget *widget, GtkAllocation *all
 
   render->sizeAllocation_has_run = TRUE;
   /* Chain up to default that simply reads current requisition */
-  GTK_WIDGET_CLASS(parent_class)->size_allocate(widget, allocation);
+  GTK_WIDGET_CLASS(openGLWidget_parent_class)->size_allocate(widget, allocation);
 
-#if SYSTEM_X11 == 1
+#ifdef IMPL_BUILTIN_X11
   glXWaitX();
 #endif
 
-  openGLWidgetSet_viewport(render, widget->allocation.width, widget->allocation.height, TRUE);
+  openGLWidgetSet_viewport(render, widget->allocation.width, widget->allocation.height, FALSE);
 }
 static void openGLWidgetEvent_realise(GtkWidget *widget)
 {
@@ -298,11 +320,18 @@ static void openGLWidgetEvent_realise(GtkWidget *widget)
   gint attributes_mask;
   GdkColormap *colormap;
 
-  DBG_fprintf(stderr, "Gtk OpenGL (events) : realise event for %p.\n", (gpointer)widget);
+  DBG_fprintf(stderr, "Gtk OpenGL (events): realise event for %p.\n",
+	      (gpointer)widget);
 
   render = OPENGL_WIDGET(widget);
 
+#ifndef IMPL_BUILTIN_WIN32
+  openGLWidgetSet_pixelFormat(render);
+#endif
+
   colormap = openGLWidgetGet_openGLColormap(render);
+  DBG_fprintf(stderr, "Gtk OpenGL (debug): colormap is %p.\n",
+	      (gpointer)colormap);
 
   attributes.window_type = GDK_WINDOW_CHILD;
   attributes.x = widget->allocation.x;
@@ -318,28 +347,38 @@ static void openGLWidgetEvent_realise(GtkWidget *widget)
     GDK_SCROLL_MASK |
     GDK_KEY_PRESS_MASK |
     GDK_POINTER_MOTION_MASK |
-    GDK_POINTER_MOTION_HINT_MASK |
-    GDK_VISIBILITY_NOTIFY_MASK;
+    GDK_POINTER_MOTION_HINT_MASK;
 
+#ifdef IMPL_GTKGLEXT
+  attributes_mask = GDK_WA_X | GDK_WA_Y;
+#else
   attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+#endif
 
   widget->window = gdk_window_new(gtk_widget_get_parent_window(widget), 
 				  &attributes, attributes_mask);
   gdk_window_set_user_data(widget->window, render);
+  DBG_fprintf(stderr, "Gtk OpenGL (debug): GdkWindow is %p.\n",
+	      (gpointer)widget->window);
 
   widget->style = gtk_style_attach(widget->style, widget->window);
   gtk_style_set_background(widget->style, widget->window, GTK_STATE_NORMAL);
 
+  GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_FOCUS);
+  GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
   GTK_WIDGET_SET_FLAGS(widget, GTK_REALIZED);
 
   gdk_display_sync(gtk_widget_get_display(widget));
-#if SYSTEM_X11 == 1
+#ifdef IMPL_BUILTIN_X11
   glXWaitX();
 #endif
 
   /* Initialize context for OpenGL, OS dependent. */
+#ifdef IMPL_BUILTIN_WIN32
+  openGLWidgetSet_pixelFormat(render);
+#endif
   openGLWidgetInit_context(render, render->isContextDirect);
-  openGLWidgetSet_current(render);
+  openGLWidgetSet_current(render, FALSE);
 }
 
 
@@ -389,18 +428,16 @@ void openGLWidgetRedraw(OpenGLWidget *render)
   if (render->redraw)
     {
       DBG_fprintf(stderr, " | set current.\n");
-      openGLWidgetSet_current(render);
+      openGLWidgetSet_current(render, FALSE);
       DBG_fprintf(stderr, " | redraw inside.\n");
-      render->redraw(GTK_WIDGET(render)->allocation.width,
-		     GTK_WIDGET(render)->allocation.height,
-		     render->redrawData);
+      render->redraw((const char**)0, render->redrawData);
       DBG_fprintf(stderr, " | swap buffers.\n");
       openGLWidgetSwap_buffers(render);
     }
 }
 
 guchar* openGLWidgetGet_pixmapData(OpenGLWidget *render, int *width,
-				   int *height, gboolean offScreen)
+				   int *height, gboolean offScreen, gboolean hasAlpha)
 {
   GtkWidget *wd;
   guchar *image;
@@ -417,7 +454,7 @@ guchar* openGLWidgetGet_pixmapData(OpenGLWidget *render, int *width,
     {
       *width = wd->allocation.width;
       *height = wd->allocation.height;
-      return visuOpenGLGet_pixmapData(*width, *height);
+      return visuOpenGLGet_pixmapData(*width, *height, hasAlpha);
     }
 
   /* If a method is given, then we draw
@@ -426,74 +463,192 @@ guchar* openGLWidgetGet_pixmapData(OpenGLWidget *render, int *width,
   *height = (*height > 0)?*height:wd->allocation.height;
   
   /* We create a pixmap context and make this context current. */
-  OPENGL_WIDGET_GET_CLASS(render)->contextCurrent = (OpenGLWidget*)0;
   dumpData = visuOpenGLNew_pixmapContext((guint)*width, (guint)*height);
   /* We set the glViewport of this new context. */
   glViewport(0, 0, *width, *height);
   /* We call the given draw method. */
-  render->redraw(*width, *height, render->redrawData);
+  render->redraw((const char**)0, render->redrawData);
   /* We copy the pixmap into generic data. */
-  image = visuOpenGLGet_pixmapData((guint)*width, (guint)*height);
+  image = visuOpenGLGet_pixmapData((guint)*width, (guint)*height, hasAlpha);
   /* We free the pixmap context. */
   visuOpenGLFree_pixmapContext(dumpData);
   /* We change back the context to the current rendering area. */
-  openGLWidgetSet_current(render);
+  openGLWidgetSet_current(render, TRUE);
 
   return image;
 }
 
 /* OpenGL functions, OS dependent. */
-#if SYSTEM_X11 == 1
+#ifdef IMPL_GTKGLEXT
+static void openGLWidgetSet_pixelFormat(OpenGLWidget *render)
+{
+  GdkScreen *screen;
+
+  DBG_fprintf(stderr, "Gtk OpenGL (glext): choose visual for %p.\n", render);
+  /* Get the screen. */
+  if (GDK_IS_DRAWABLE(GTK_WIDGET(render)->window))
+    screen = gdk_drawable_get_screen(GDK_DRAWABLE(GTK_WIDGET(render)->window));
+  else
+    screen = gdk_screen_get_default();
+  DBG_fprintf(stderr, " | get a screen number %d.\n",
+	      gdk_x11_screen_get_screen_number(screen));
+
+  render->glconfig = visuOpenGLGet_glConfig(screen);
+  DBG_fprintf(stderr, " | get a visual.\n");
+}
 static void openGLWidgetInit_context(OpenGLWidget *render, gboolean contextIsDirect)
 {
-  gint screenId;
-  XVisualInfo *vinfo;
   GtkWidget *wd;
 
-  DBG_fprintf(stderr, "Gtk OpenGL (init) : create an OpenGL context (%d).\n", contextIsDirect);
+  DBG_fprintf(stderr, "Gtk OpenGL (glext): create an OpenGL context (%d).\n",
+	      contextIsDirect);
 
   wd = GTK_WIDGET(render);
-  if (!render->dpy)
-    render->dpy = gdk_x11_drawable_get_xdisplay(GDK_DRAWABLE(wd->window));
-  DBG_fprintf(stderr, " | get the display %p.\n", (gpointer)render->dpy);
-  /* Check for GLX. */
-  if (!glXQueryExtension(render->dpy, 0, 0))
-    g_error("No GLX extension.\nYour X server"
-	    " does not support OpenGL extension. Please contact your"
-	    " system administrator to ask him to add the 'glx'"
-	    " extension to your X server.\n");
 
-  /* Get the screen id. */
-  screenId = gdk_x11_screen_get_screen_number
-    (gdk_drawable_get_screen(GDK_DRAWABLE(wd->window)));
-  DBG_fprintf(stderr, " | get a screen number %d.\n", screenId);
-
-  /* We don't cancel the double buffering since the backing store
-     need it. */
-/*   if (DoesBackingStore(ScreenOfDisplay(render->dpy, screenId)) == NotUseful) */
-    gtk_widget_set_double_buffered(GTK_WIDGET(render), FALSE);
-
-  /* Try to create a visual. */
-  vinfo = visuOpenGLGet_visualInfo(render->dpy, screenId);
+  /* Create a GL area on the widget window. */
+  render->glwindow = gdk_gl_window_new(render->glconfig, wd->window, NULL);
 
   /* Finaly, create the context. */
   if (contextIsDirect)
     {
-      render->context = glXCreateContext(render->dpy, vinfo, 0, GL_TRUE);
+      render->context = gdk_gl_context_new(GDK_GL_DRAWABLE(render->glwindow),
+					   (GdkGLContext*)0, TRUE, GDK_GL_RGBA_TYPE);
       if (!render->context)
 	{
 	  g_warning("Can't create a direct rendering context, try an inderect one.\n");
-	  render->context = glXCreateContext(render->dpy, vinfo, 0, GL_FALSE);
+	  render->context = gdk_gl_context_new(GDK_GL_DRAWABLE(render->glwindow),
+					       (GdkGLContext*)0, FALSE,
+					       GDK_GL_RGBA_TYPE);
 	  render->isContextDirect = FALSE;
 	}
     }
   else
-    render->context = glXCreateContext(render->dpy, vinfo, 0, GL_FALSE);
+    render->context = gdk_gl_context_new(GDK_GL_DRAWABLE(render->glwindow),
+					 (GdkGLContext*)0, FALSE, GDK_GL_RGBA_TYPE);
+  DBG_fprintf(stderr, " | create the context %p (%d).\n", (gpointer)render->context,
+	      (int)render->isContextDirect);
+
+  if (!render->context)
+    {
+      g_error("Cannot create a GtkGLExt context.\n");
+    }
+}
+static void openGLWidgetFree_openGL(OpenGLWidget *render)
+{
+  g_return_if_fail(IS_OPENGL_WIDGET(render));
+
+  DBG_fprintf(stderr, "Gtk OpenGL (glext): freeing context & window.\n");
+  if (render->glwindow)
+    gdk_gl_window_destroy(render->glwindow);
+  if (render->context)
+    gdk_gl_context_destroy(render->context);
+}
+gboolean openGLWidgetSet_current(OpenGLWidget *render, gboolean force)
+{
+  gboolean res;
+  GtkWidget *wd;
+  
+  g_return_val_if_fail(IS_OPENGL_WIDGET(render), FALSE);
+
+  if (!force && OPENGL_WIDGET_GET_CLASS(render)->contextCurrent == render)
+    return TRUE;
+
+  DBG_fprintf(stderr, "Gtk OpenGL (glext): %p is set current.\n", (gpointer)render);
+  res = gdk_gl_drawable_make_current(GDK_GL_DRAWABLE(render->glwindow),
+				     render->context);
+
+  if (!res)
+    {
+      g_warning("Cannot make the openGLWidget object %p current.\n",
+		(gpointer)render);
+      return FALSE;
+    }
+  DBG_fprintf(stderr, "Gtk OpenGL (glext): OK context changed.\n");
+
+  OPENGL_WIDGET_GET_CLASS(render)->contextCurrent = render;
+
+  wd = GTK_WIDGET(render);
+  openGLWidgetSet_viewport(render, wd->allocation.width, wd->allocation.height, FALSE);
+
+  return TRUE;
+}
+void openGLWidgetSwap_buffers(OpenGLWidget *render)
+{
+  g_return_if_fail(OPENGL_WIDGET_GET_CLASS(render)->contextCurrent == render);
+
+  DBG_fprintf(stderr, "Gtk OpenGL (glext): swap buffers of area %p.\n",
+	      (gpointer)render);
+  gdk_gl_drawable_swap_buffers(GDK_GL_DRAWABLE(render->glwindow));
+}
+static GdkColormap* openGLWidgetGet_openGLColormap(OpenGLWidget *render)
+{
+  g_return_val_if_fail(render->glconfig, (GdkColormap*)0);
+
+  return gdk_gl_config_get_colormap(render->glconfig);
+}
+#endif
+#ifdef IMPL_BUILTIN_X11
+static void openGLWidgetSet_pixelFormat(OpenGLWidget *render)
+{
+  gint screenId;
+  gboolean built;
+
+  built = GDK_IS_DRAWABLE(GTK_WIDGET(render)->window);
+  DBG_fprintf(stderr, "Gtk OpenGL (X11): choose visual for %p.\n", (gpointer)render);
+  if (built)
+    render->dpy = gdk_x11_drawable_get_xdisplay(GDK_DRAWABLE(GTK_WIDGET(render)->window));
+  else
+    render->dpy = gdk_x11_get_default_xdisplay();
+  DBG_fprintf(stderr, " | get the display %p.\n", (gpointer)render->dpy);
+
+  /* Get the screen id. */
+  if (built)
+    screenId = gdk_x11_screen_get_screen_number
+      (gdk_drawable_get_screen(GDK_DRAWABLE(GTK_WIDGET(render)->window)));
+  else
+    screenId = gdk_x11_get_default_screen();
+  DBG_fprintf(stderr, " | get a screen number %d.\n", screenId);
+
+  render->vinfo = visuOpenGLGet_visualInfo(render->dpy, screenId);
+  DBG_fprintf(stderr, " | get a visual %d.\n", (int)render->vinfo->visualid);
+}
+static void openGLWidgetInit_context(OpenGLWidget *render, gboolean contextIsDirect)
+{
+  GtkWidget *wd;
+
+  DBG_fprintf(stderr, "Gtk OpenGL (X11): create an OpenGL context (%d).\n",
+	      contextIsDirect);
+
+  wd = GTK_WIDGET(render);
+  /* Check for GLX. */
+  if (!glXQueryExtension(render->dpy, 0, 0))
+    {
+      g_error("No GLX extension.\nYour X server"
+	      " does not support OpenGL extension. Please contact your"
+	      " system administrator to ask him to add the 'glx'"
+	      " extension to your X server.\n");
+    }
+
+  /* Finaly, create the context. */
+  if (contextIsDirect)
+    {
+      render->context = glXCreateContext(render->dpy, render->vinfo, 0, GL_TRUE);
+      if (!render->context)
+	{
+	  g_warning("Can't create a direct rendering context, try an inderect one.\n");
+	  render->context = glXCreateContext(render->dpy, render->vinfo, 0, GL_FALSE);
+	  render->isContextDirect = FALSE;
+	}
+    }
+  else
+    render->context = glXCreateContext(render->dpy, render->vinfo, 0, GL_FALSE);
   DBG_fprintf(stderr, " | create the context %p (%d).\n", (gpointer)render->context,
 	    (int)render->isContextDirect);
 
   if (!render->context)
-    g_error("Cannot create a GLX context.\n");
+    {
+      g_error("Cannot create a GLX context.\n");
+    }
 }
 static void openGLWidgetFree_openGL(OpenGLWidget *render)
 {
@@ -501,7 +656,7 @@ static void openGLWidgetFree_openGL(OpenGLWidget *render)
 
   if (render->dpy)
     {
-      DBG_fprintf(stderr, "Free : freeing context.\n");
+      DBG_fprintf(stderr, "Free (X11): freeing context.\n");
       if (render->context)
 	glXDestroyContext(render->dpy, render->context);
       /* We do NOT close the display since it is shared
@@ -510,7 +665,7 @@ static void openGLWidgetFree_openGL(OpenGLWidget *render)
 /*       XCloseDisplay(render->dpy); */
     }
 }
-gboolean openGLWidgetSet_current(OpenGLWidget *render)
+gboolean openGLWidgetSet_current(OpenGLWidget *render, gboolean force)
 {
   int res;
   GtkWidget *wd;
@@ -518,22 +673,27 @@ gboolean openGLWidgetSet_current(OpenGLWidget *render)
   
   g_return_val_if_fail(IS_OPENGL_WIDGET(render), FALSE);
 
-  if (OPENGL_WIDGET_GET_CLASS(render)->contextCurrent == render)
+  if (!force && OPENGL_WIDGET_GET_CLASS(render)->contextCurrent == render)
     return TRUE;
 
-  DBG_fprintf(stderr, "Gtk OpenGL (debug) : widget visualID %d.\n", (int)gdk_x11_visual_get_xvisual
-	    (gtk_widget_get_visual(GTK_WIDGET(render)))->visualid);
+  DBG_fprintf(stderr, "Gtk OpenGL (X11): widget visualID %d.\n",
+	      (int)gdk_x11_visual_get_xvisual
+	      (gtk_widget_get_visual(GTK_WIDGET(render)))->visualid);
 
-  DBG_fprintf(stderr, "Gtk OpenGL (action) : %p is set current.\n", (gpointer)render);
+  DBG_fprintf(stderr, "Gtk OpenGL (X11): %p is set current.\n", (gpointer)render);
 
   windowId = gdk_x11_drawable_get_xid(GDK_DRAWABLE(GTK_WIDGET(render)->window));
+  DBG_fprintf(stderr, "Gtk OpenGL (X11): widget windowID %d.\n", (int)windowId);
+
   res = glXMakeCurrent(render->dpy, (GLXDrawable)windowId,
 		       render->context);
   if (!res)
     {
-      g_warning("Cannot make the openGLWidget object %p current.\n", (gpointer)render);
+      g_warning("Cannot make the openGLWidget object %p current.\n",
+		(gpointer)render);
       return FALSE;
     }
+  DBG_fprintf(stderr, "Gtk OpenGL (X11): OK context changed.\n");
 
   /* Now that the glx tunnel has been added, we need
      to specify again that we want a backing store because until now
@@ -554,43 +714,32 @@ void openGLWidgetSwap_buffers(OpenGLWidget *render)
 {
   g_return_if_fail(OPENGL_WIDGET_GET_CLASS(render)->contextCurrent == render);
 
-  DBG_fprintf(stderr, "Gtk OpenGL (action) : swap buffers of area %p.\n", (gpointer)render);
+  DBG_fprintf(stderr, "Gtk OpenGL (X11): swap buffers of area %p.\n", (gpointer)render);
   glXSwapBuffers(render->dpy, (GLXDrawable)gdk_x11_drawable_get_xid
 		 (GDK_DRAWABLE(GTK_WIDGET(render)->window)));
 }
 static GdkColormap* openGLWidgetGet_openGLColormap(OpenGLWidget *render)
 {
-  XVisualInfo *vinfo;
-  Display *dpy;
-  int screenId;
-  GdkVisual *visual;
-  GdkColormap *colormap;
-
   g_return_val_if_fail(IS_OPENGL_WIDGET(render), (GdkColormap*)0);
-  
-  dpy      = gdk_x11_get_default_xdisplay();
-  screenId = gdk_x11_get_default_screen();
-  vinfo    = visuOpenGLGet_visualInfo(dpy, screenId);
-  visual   = gdkx_visual_get(vinfo->visualid);
-  colormap = gdk_colormap_new(visual, FALSE);
+  g_return_val_if_fail(render->vinfo, (GdkColormap*)0);
 
-  return colormap;
+  return gdk_colormap_new(gdkx_visual_get(render->vinfo->visualid), FALSE);
 }
 #endif
-#if SYSTEM_WIN32 == 1
-static void openGLWidgetInit_context(OpenGLWidget *render, gboolean contextIsDirect)
+#ifdef IMPL_BUILTIN_WIN32
+static void openGLWidgetSet_pixelFormat(OpenGLWidget *render)
 {
-  GtkWidget *wd;
+  DBG_fprintf(stderr, "Gtk OpenGL (Win32): choose visual for %p.\n", render);
 
-  DBG_fprintf(stderr, "Gtk OpenGL (init) : create an OpenGL context (%d).\n", contextIsDirect);
-
-  wd = GTK_WIDGET(render);
-  render->windowId = (HWND)gdk_win32_drawable_get_handle(GDK_DRAWABLE(wd->window));
+  render->windowId = (HWND)gdk_win32_drawable_get_handle(GDK_DRAWABLE(GTK_WIDGET(render)->window));
   render->hdc = GetDC(render->windowId);
   DBG_fprintf(stderr, " | get the hdc %d.\n", (int)render->hdc);
 
-  /* Choose best matching format*/
   visuOpenGLSetup_pixelFormat(render->hdc);
+}
+static void openGLWidgetInit_context(OpenGLWidget *render, gboolean contextIsDirect)
+{
+  DBG_fprintf(stderr, "Gtk OpenGL (Win32): create an OpenGL context (%d).\n", contextIsDirect);
 
   /* Finaly, create the context. */
   render->context = wglCreateContext(render->hdc);
@@ -604,9 +753,8 @@ static void openGLWidgetFree_openGL(OpenGLWidget *render)
 /*   if (render->hdc) */
 /*     DeleteDC(render->hdc); */
 }
-gboolean openGLWidgetSet_current(OpenGLWidget *render)
+gboolean openGLWidgetSet_current(OpenGLWidget *render, gboolean force _U_)
 {
-  int res;
   GtkWidget *wd;
 
   g_return_val_if_fail(IS_OPENGL_WIDGET(render), FALSE);
