@@ -44,49 +44,35 @@
 #ifndef GTK_OPENGLWIDGET_H
 #define GTK_OPENGLWIDGET_H
 
-/**
- * OpenGLWidget_struct:
- *
- * Opaque structure to store informations for each objects.
- */
-struct OpenGLWidget_struct;
-/**
- * OpenGLWidgetClass_struct:
- *
- * Opaque structure to store informations about the class.
- */
-struct OpenGLWidgetClass_struct;
-
 /**
  * OpenGLWidget:
  *
  * Short name to address OpenGLWidget_struct objects.
  */
-typedef struct OpenGLWidget_struct OpenGLWidget;
+typedef struct _OpenGLWidget OpenGLWidget;
 /**
  * OpenGLWidgetClass:
  *
  * Short name to address OpenGLWidgetClass_struct objects.
  */
-typedef struct OpenGLWidgetClass_struct OpenGLWidgetClass;
+typedef struct _OpenGLWidgetClass OpenGLWidgetClass;
 
 /**
  * RedrawMethod:
- * @width: the width of the drawing area ;
- * @height: the height of the drawing area ;
- * @data: some user defined informations.
+ * @lists: a string array, NULL terminated.
+ * @data: some associated data.
  *
  * Methods of this prototype are called when the redraw of the OpenGL zone
  * should be done.
  */
-typedef void (*RedrawMethod)(guint width, guint height, gpointer data);
+typedef void (*RedrawMethod)(const char** lists, gpointer data);
 
 /**
  * TYPE_OPENGL_WIDGET:
  * 
  * The type of OpenGLWidget objects.
  */
-#define TYPE_OPENGL_WIDGET           (openGLWidgetGet_type())
+#define TYPE_OPENGL_WIDGET           (openGLWidget_get_type())
 /**
  * OPENGL_WIDGET:
  * @obj: a pointer.
@@ -125,20 +111,20 @@ typedef void (*RedrawMethod)(guint width, guint height, gpointer data);
  * OPENGL_WIDGET_GET_CLASS:
  * @obj: a pointer.
  *
- * Returns the class of the given OpenGLWidget object.
+ * Return the class of the given OpenGLWidget object.
  */
 #define OPENGL_WIDGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj),	\
 								TYPE_OPENGL_WIDGET, \
 								OpenGLWidgetClass))
 
 /**
- * openGLWidgetGet_type:
+ * openGLWidget_get_type:
  *
  * Retrive the type of OpenGLWidget objects.
  *
  * Returns: the id used by OBjects for OpenGLWidget objects.
  */
-GType openGLWidgetGet_type(void);
+GType openGLWidget_get_type(void);
 
 /**
  * openGLWidgetNew:
@@ -152,14 +138,17 @@ GType openGLWidgetGet_type(void);
 GtkWidget* openGLWidgetNew(gboolean contextIsDirect);
 /**
  * openGLWidgetSet_current:
- * @render: a #OpenGLWidget object.
+ * @render: a #OpenGLWidget object ;
+ * @force: a boolean.
  *
  * Make this object current. This means that all future OpenGL primitive will be
- * rendered on this surface.
+ * rendered on this surface. If @force is TRUE, the GL routine is
+ * actually called whereas in other cases, if @render believe being
+ * already current, nothing is done.
  *
  * Returns: TRUE if succeed.
  */
-gboolean openGLWidgetSet_current(OpenGLWidget *render);
+gboolean openGLWidgetSet_current(OpenGLWidget *render, gboolean force);
 /**
  * openGLWidgetSet_redraw:
  * @render: a #OpenGLWidget object ;
@@ -191,7 +180,8 @@ void openGLWidgetSwap_buffers(OpenGLWidget *render);
  * @render: a #OpenGLWidget object ;
  * @width: a pointer to the desired width or -1 ;
  * @height: a pointer to the desired height or -1 ;
- * @offScreen: a boolean.
+ * @offScreen: a boolean ;
+ * @hasAlpha: if TRUE, the returned data is RGBA, else only RGB.
  *
  * Create an image from the OpenGL area. The size can  be changed, using @width and
  * @height. If these pointers contains positive values, then they are used to set the
@@ -202,7 +192,8 @@ void openGLWidgetSwap_buffers(OpenGLWidget *render);
  * Returns: image data, row by row.
  */
 guchar* openGLWidgetGet_pixmapData(OpenGLWidget *render, int *width,
-				   int *height, gboolean offScreen);
+				   int *height, gboolean offScreen,
+				   gboolean hasAlpha);
 /**
  * openGLWidgetClassGet_currentContext:
  *
