@@ -49,6 +49,7 @@
 
 #include "gtk_save.h"
 
+#include "visu_gtk.h"
 #include "gtk_main.h"
 #include "visu_configFile.h"
 #include "visu_object.h"
@@ -57,20 +58,21 @@
 
 
 static guint saveResourcesContextId, saveParametersContextId;
-static gchar *lastEntryResources;
+static gchar *lastParsedDir;
+static GtkWidget *checkXMLResources, *checkXMLParameters;
 
 /* Local Callbacks. */
-void onLoadResButtonClicked(GtkButton *button, gpointer data);
-void onSaveResButtonClicked(GtkButton *button, gpointer data);
-void onSelectionResourcesChange(GtkFileChooser *fileChooser, gpointer *data);
-void onLoadResourcesSelected(GtkFileChooser *fileChooser, gpointer *data);
-void onTextEntryChange(GtkEditable *entry, gpointer data);
+static void onLoadResButtonClicked(GtkButton *button, gpointer data);
+static void onSaveResButtonClicked(GtkButton *button, gpointer data);
+static void onSelectionResourcesChange(GtkFileChooser *fileChooser, gpointer *data);
+static void onLoadResourcesSelected(GtkFileChooser *fileChooser, gpointer *data);
+static void onTextEntryChange(GtkEditable *entry, gpointer data);
 
-void onSaveParButtonClicked(GtkButton *button, gpointer data);
+static void onSaveParButtonClicked(GtkButton *button, gpointer data);
 
 /* Local methods. */
-void showAlertMessage(GtkWidget *saveDialog, gchar* message, gboolean warning, int type);
-void saveAction(int type, GtkWidget *saveDialog, GtkEntry *entry);
+static void showAlertMessage(GtkWidget *saveDialog, gchar* message, gboolean warning, int type);
+static void saveAction(int type, GtkWidget *saveDialog, GtkEntry *entry);
 
 /* The completion tree model. */
 enum
@@ -82,7 +84,7 @@ enum
 void gtkSaveInit()
 {
   GtkWidget *saveDialog;
-  GtkWidget *wd;
+  GtkWidget *wd, *label;
   GtkEntryCompletion *completion;
   gchar *directory, *file;
   GList *resList;
@@ -135,11 +137,11 @@ void gtkSaveInit()
 				   G_TYPE_STRING);
   gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(listOfFiles),
 				       GTK_SAVE_FILE_NAME, GTK_SORT_ASCENDING);
-  lastEntryResources = (gchar*)0;
+  lastParsedDir = (gchar*)0;
 
   /* Set initial values. */
   wd = lookup_widget(saveDialog, "filechooserwidgetResources");
-  directory = getLastOpenDirectory();
+  directory = gtkMainClassGet_lastOpenDirectory();
   if (directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(wd), directory);
   g_signal_connect(G_OBJECT(wd), "selection-changed",
@@ -151,7 +153,6 @@ void gtkSaveInit()
   wd = lookup_widget(saveDialog, "statusbarResources");
   saveResourcesContextId = gtk_statusbar_get_context_id(GTK_STATUSBAR(wd), "Resources");
 
-
   wd = lookup_widget(saveDialog, "comboboxentryParameters");
   gtk_widget_set_name(wd, "message_entry");
   resList = visuConfigFileGet_pathList(VISU_CONFIGFILE_PARAMETER);
@@ -165,6 +166,15 @@ void gtkSaveInit()
 	}
       while(file);
     }
+  completion = gtk_entry_completion_new();
+  gtk_entry_completion_set_model(completion, GTK_TREE_MODEL(listOfFiles));
+  gtk_entry_completion_set_text_column(completion, GTK_SAVE_FILE_NAME);
+#if GTK_MINOR_VERSION > 5
+  gtk_entry_completion_set_inline_completion(completion, TRUE);
+#endif
+  gtk_entry_set_completion(GTK_ENTRY(GTK_BIN(wd)->child), completion);
+  g_signal_connect(G_OBJECT(GTK_BIN(wd)->child), "changed",
+		   G_CALLBACK(onTextEntryChange), (gpointer)0);
   gtk_combo_box_set_active(GTK_COMBO_BOX(wd), 0);
 
 
@@ -184,11 +194,22 @@ void gtkSaveInit()
   completion = gtk_entry_completion_new();
   gtk_entry_completion_set_model(completion, GTK_TREE_MODEL(listOfFiles));
   gtk_entry_completion_set_text_column(completion, GTK_SAVE_FILE_NAME);
+#if GTK_MINOR_VERSION > 5
+  gtk_entry_completion_set_inline_completion(completion, TRUE);
+#endif
   gtk_entry_set_completion(GTK_ENTRY(GTK_BIN(wd)->child), completion);
   g_signal_connect(G_OBJECT(GTK_BIN(wd)->child), "changed",
 		   G_CALLBACK(onTextEntryChange), (gpointer)0);
   gtk_combo_box_set_active(GTK_COMBO_BOX(wd), 0);
 
+  wd = lookup_widget(saveDialog, "labelTipsResources");
+  gtk_label_set_line_wrap(GTK_LABEL(wd), FALSE);
+  gtk_label_set_markup(GTK_LABEL(wd), _("A description of all resource markups is"
+					" available on:"
+					"\n   <span font_desc=\"monospace\"><u>"
+					V_SIM_WEB_SITE"/sample.html#resources"
+					"</u></span>."));
+
   /* Set callbacks. */
   wd = lookup_widget(saveDialog, "buttonLoadResources");
   g_signal_connect(G_OBJECT(wd), "clicked",
@@ -200,20 +221,55 @@ void gtkSaveInit()
   g_signal_connect(G_OBJECT(wd), "clicked",
 		   G_CALLBACK(onSaveParButtonClicked), (gpointer)saveDialog);
 
+#if DEBUG == 1
+  wd = lookup_widget(saveDialog, "vbox16");
+  checkXMLResources =
+    gtk_check_button_new_with_mnemonic(_("Export all resource descriptions"
+					 " to an _XML file"));
+  gtk_box_pack_start(GTK_BOX(wd), checkXMLResources, FALSE, FALSE, 0);
+  gtk_container_set_border_width(GTK_CONTAINER(checkXMLResources), 5);
+  gtk_widget_set_name(checkXMLResources, "message_radio");
+  gtk_widget_show(checkXMLResources);
+
+  wd = lookup_widget(saveDialog, "vbox18");
+  checkXMLParameters =
+    gtk_check_button_new_with_mnemonic(_("Export all parameter descriptions"
+					 " to an _XML file"));
+  gtk_box_pack_start(GTK_BOX(wd), checkXMLParameters, FALSE, FALSE, 0);
+  gtk_container_set_border_width(GTK_CONTAINER(checkXMLParameters), 15);
+  gtk_widget_set_name(checkXMLParameters, "message_radio");
+  gtk_widget_show(checkXMLParameters);
+#else
+  checkXMLResources = (GtkWidget*)0;
+  checkXMLParameters = (GtkWidget*)0;
+#endif
+
+  wd = lookup_widget(saveDialog, "vbox18");
+  label = gtk_label_new("");
+  gtk_label_set_line_wrap(GTK_LABEL(label), FALSE);
+  gtk_label_set_markup(GTK_LABEL(label), _("A description of all parameter markups is"
+					   " available on:\n"
+					   "   <span font_desc=\"monospace\"><u>"
+					   V_SIM_WEB_SITE"/sample.html#parameters"
+					   "</u></span>."));
+  gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
+  gtk_box_pack_end(GTK_BOX(wd), label, FALSE, FALSE, 10);
+  gtk_widget_show(label);
+
   /* Run the dialog. */
   gtk_dialog_run(GTK_DIALOG(saveDialog));
   g_object_unref(listOfFiles);
-  if (lastEntryResources)
-    g_free(lastEntryResources);
+  if (lastParsedDir)
+    g_free(lastParsedDir);
   gtk_widget_destroy(saveDialog);
 }
 
-void onLoadResourcesSelected(GtkFileChooser *fileChooser, gpointer *data)
+static void onLoadResourcesSelected(GtkFileChooser *fileChooser _U_, gpointer *data)
 {
   onLoadResButtonClicked((GtkButton*)0, data);
 }
 
-void onSelectionResourcesChange(GtkFileChooser *fileChooser, gpointer *data)
+static void onSelectionResourcesChange(GtkFileChooser *fileChooser, gpointer *data)
 {
   gchar *filename;
 
@@ -233,13 +289,15 @@ void onSelectionResourcesChange(GtkFileChooser *fileChooser, gpointer *data)
   g_free(filename);
 }
 
-void onLoadResButtonClicked(GtkButton *button, gpointer data)
+static void onLoadResButtonClicked(GtkButton *button _U_, gpointer data)
 {
   GtkWidget *file_selector;
   gchar *filename, *basename;
   gchar *directory;
   int res;
-  GString *errorMessage, *message;
+  GString *message;
+  GError *error;
+  VisuData *dataObj;
 
   g_return_if_fail(GTK_DIALOG(data));
 
@@ -248,70 +306,81 @@ void onLoadResButtonClicked(GtkButton *button, gpointer data)
 
   filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_selector));
   directory = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(file_selector));
-  setLastOpenDirectory(directory);
+  gtkMainClassSet_lastOpenDirectory(directory);
   g_free(directory);
 
-  errorMessage = (GString*)0;
-  res = visuConfigFileLoad(VISU_CONFIGFILE_RESOURCE, filename, &errorMessage);
-  if (!res && errorMessage)
-    raiseAlertDialogWithScrollView(errorMessage->str);
-  if (errorMessage)
-    g_string_free(errorMessage, TRUE);
+  dataObj = renderingWindowGet_visuData(currentRenderingWindow);
+  error = (GError*)0;
+  res = visuConfigFileLoad(VISU_CONFIGFILE_RESOURCE, filename, dataObj, &error);
+  if (error)
+    {
+      visuGtkRaise_warningLong(_("Loading a file"), error->message,
+			       GTK_WINDOW(data));
+      g_error_free(error);
+    }
 
   basename = g_path_get_basename(filename);
+  g_free (filename);
   message = g_string_new("");
   if (res)
     g_string_append_printf(message, _("File '%s' succesfully loaded."), basename);
   else
-    g_string_append_printf(message, _("WARNING! File '%s' not loaded."), basename);
+    g_string_append_printf(message, _("File '%s' not or partially loaded."),
+			   basename);
   showAlertMessage(GTK_WIDGET(data), message->str, !res, VISU_CONFIGFILE_RESOURCE);
   g_string_free(message, TRUE);
   g_free(basename);
 
+  DBG_fprintf(stderr, "Gtk Save: emitting 'resourcesLoaded' signal.\n");
+  g_signal_emit(visu, VISU_GET_CLASS(visu)->resourcesLoaded_signal_id,
+		0, dataObj, NULL);
+  DBG_fprintf(stderr, "Gtk Save: emission done.\n");
+
   if (res)
-    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
-		   0 , NULL);
-  g_free (filename);
+    g_idle_add(visuObjectRedraw, (gpointer)0);
 }
 
-void onSaveResButtonClicked(GtkButton *button, gpointer data)
+static void onSaveResButtonClicked(GtkButton *button _U_, gpointer data)
 {
   GtkWidget *entry;
 
-  g_return_if_fail(GTK_DIALOG(data));
+  g_return_if_fail(GTK_IS_DIALOG(data));
 
   entry = GTK_BIN(lookup_widget(GTK_WIDGET(data), "comboboxentryResources"))->child;
   saveAction(VISU_CONFIGFILE_RESOURCE, GTK_WIDGET(data), GTK_ENTRY(entry));
+  gtk_widget_grab_default(lookup_widget(GTK_WIDGET(data), "closeButtonSave"));
 }
 
-void onSaveParButtonClicked(GtkButton *button, gpointer data)
+static void onSaveParButtonClicked(GtkButton *button _U_, gpointer data)
 {
   GtkWidget *entry;
 
-  g_return_if_fail(GTK_DIALOG(data));
+  g_return_if_fail(GTK_IS_DIALOG(data));
 
   entry = GTK_BIN(lookup_widget(GTK_WIDGET(data), "comboboxentryParameters"))->child;
   saveAction(VISU_CONFIGFILE_PARAMETER, GTK_WIDGET(data), GTK_ENTRY(entry));
+  gtk_widget_grab_default(lookup_widget(GTK_WIDGET(data), "closeButtonSave"));
 }
 
-void saveAction(int type, GtkWidget *saveDialog, GtkEntry *entry)
+static void saveAction(int type, GtkWidget *saveDialog, GtkEntry *entry)
 {
   GtkWidget *wd;
-  gchar *file, *tmpFile, *basename;
-  GString *errorMessage, *message;
+  gchar *file, *tmpFile, *basename, *fileUTF8;
+  GString *message;
   gboolean resOk;
   gsize ecrit, lu;
   int lines;
   VisuData *dataObj;
+  GError *error;
+  GtkTreeIter iter;
+  GtkListStore *listOfFiles;
+  GtkEntryCompletion *completion;
 
   g_return_if_fail(type == VISU_CONFIGFILE_RESOURCE ||
 		   type == VISU_CONFIGFILE_PARAMETER);
   g_return_if_fail(saveDialog && entry);
 
-  if (!getFileSystemInUTF8())
-    file = g_filename_from_utf8(gtk_entry_get_text(entry), -1, &lu, &ecrit, NULL);
-  else
-    file = g_strdup(gtk_entry_get_text(entry));
+  file = g_filename_from_utf8(gtk_entry_get_text(entry), -1, &lu, &ecrit, NULL);
   
   if (g_file_test(file, G_FILE_TEST_IS_DIR))
     {
@@ -321,7 +390,6 @@ void saveAction(int type, GtkWidget *saveDialog, GtkEntry *entry)
       file = tmpFile;
     }
   
-  errorMessage = (GString*)0;
   if (type == VISU_CONFIGFILE_RESOURCE)
     {
       wd = lookup_widget(saveDialog, "checkLimitOnVisuData");
@@ -333,17 +401,22 @@ void saveAction(int type, GtkWidget *saveDialog, GtkEntry *entry)
     }
   else
     dataObj = (VisuData*)0;
-  resOk = visuConfigFileSave(type, file, &errorMessage, &lines, dataObj);
+
+  error = (GError*)0;
+  lines = 0;
+  if (type == VISU_CONFIGFILE_RESOURCE && checkXMLResources &&
+      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkXMLResources)))
+    resOk = visuConfigFileExport_toXML(file, VISU_CONFIGFILE_RESOURCE, &error);
+  else if (type == VISU_CONFIGFILE_PARAMETER && checkXMLParameters &&
+	   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkXMLParameters)))
+    resOk = visuConfigFileExport_toXML(file, VISU_CONFIGFILE_PARAMETER, &error);
+  else
+    resOk = visuConfigFileSave(type, file, &lines, dataObj, &error);
   if (!resOk)
     {
-      if (errorMessage)
-	raiseAlertDialogWithScrollView(errorMessage->str);
-    }
-  if (errorMessage)
-    {
-      DBG_fprintf(stderr, "Gtk Save : trace from exporting par/res %d ...\n%s",
-		  type, errorMessage->str);
-      g_string_free(errorMessage, TRUE);
+      visuGtkRaise_warningLong(_("Saving a file"), error->message,
+			       GTK_WINDOW(saveDialog));
+      g_error_free(error);
     }
   
   basename = g_path_get_basename(file);
@@ -352,19 +425,40 @@ void saveAction(int type, GtkWidget *saveDialog, GtkEntry *entry)
     g_string_append_printf(message, _("File '%s' succesfully written (%d lines)."),
 			   basename, lines);
   else
-    g_string_append_printf(message, _("WARNING! File '%s' not written."), basename);
-				showAlertMessage(saveDialog, message->str, !resOk, type);
-				g_string_free(message, TRUE);
+    g_string_append_printf(message, _("File '%s' not written."), basename);
+  showAlertMessage(saveDialog, message->str, !resOk, type);
+  g_string_free(message, TRUE);
   g_free(basename);
 
+  /* If file is written, we add it to the completion list. */
+  if (resOk)
+    {
+      completion = gtk_entry_get_completion(GTK_ENTRY(entry));
+      g_return_if_fail(completion);
+
+      listOfFiles = GTK_LIST_STORE(gtk_entry_completion_get_model(completion));
+      g_return_if_fail(listOfFiles);
+
+      fileUTF8 = g_filename_from_utf8(file, -1, NULL, NULL, NULL);
+      if(fileUTF8)
+	{
+	  gtk_list_store_append(listOfFiles, &iter);
+	  gtk_list_store_set(listOfFiles, &iter,
+			     GTK_SAVE_FILE_NAME, fileUTF8,
+			     -1);
+	  g_free(fileUTF8);
+	}
+    }
+
   g_free(file);
 }
 
-void showAlertMessage(GtkWidget *saveDialog, gchar* message, gboolean warning, int type)
+static void showAlertMessage(GtkWidget *saveDialog, gchar* message, gboolean warning, int type)
 {
   GtkWidget *wd;
 
-  g_return_if_fail(type == VISU_CONFIGFILE_RESOURCE || type == VISU_CONFIGFILE_PARAMETER);
+  g_return_if_fail(type == VISU_CONFIGFILE_RESOURCE ||
+		   type == VISU_CONFIGFILE_PARAMETER);
   g_return_if_fail(saveDialog);
 
   if (type == VISU_CONFIGFILE_RESOURCE)
@@ -390,7 +484,7 @@ void showAlertMessage(GtkWidget *saveDialog, gchar* message, gboolean warning, i
     }
 }
 
-void onTextEntryChange(GtkEditable *entry, gpointer data)
+static void onTextEntryChange(GtkEditable *entry, gpointer data _U_)
 {
   gboolean reParse;
   GtkTreeIter iter;
@@ -414,14 +508,14 @@ void onTextEntryChange(GtkEditable *entry, gpointer data)
   g_return_if_fail(listOfFiles);
 
   reParse = FALSE;
-  if (!lastEntryResources)
+  if (!lastParsedDir)
     reParse = TRUE;
   else
     {
       dirEntry = g_path_get_dirname(fileEntry);
       DBG_fprintf(stderr, "Gtk Save : current and last directories...\n %s\n %s\n",
-		  dirEntry, lastEntryResources);
-      if (strcmp(dirEntry, lastEntryResources))
+		  dirEntry, lastParsedDir);
+      if (strcmp(dirEntry, lastParsedDir))
 	reParse = TRUE;
       g_free(dirEntry);
     }
@@ -436,12 +530,9 @@ void onTextEntryChange(GtkEditable *entry, gpointer data)
 
   gtk_list_store_clear(listOfFiles);
   dirEntry = g_path_get_dirname(fileEntry);
-  g_free(lastEntryResources);
-  lastEntryResources = dirEntry;
-  if (!getFileSystemInUTF8())
-    dirBrowsed = g_filename_from_utf8(dirEntry, -1, &lu, &ecrit, NULL);
-  else
-    dirBrowsed = g_strdup(dirEntry);
+  g_free(lastParsedDir);
+  lastParsedDir = dirEntry;
+  dirBrowsed = g_filename_from_utf8(dirEntry, -1, &lu, &ecrit, NULL);
   gdir = g_dir_open(dirBrowsed, 0, NULL);
   if (!gdir)
     {
@@ -452,7 +543,7 @@ void onTextEntryChange(GtkEditable *entry, gpointer data)
   while (fileFromDir)
     {
 /*       DBG_fprintf(stderr, "Gtk Save : adding '%s' to completion list.\n", fileFromDir); */
-      fileUTF8 = getStringInUTF8(fileFromDir);
+      fileUTF8 = g_filename_from_utf8(fileFromDir, -1, NULL, NULL, NULL);
       if(fileUTF8)
 	{
 	  saveFile = g_build_filename(dirBrowsed, fileFromDir, NULL);
