@@ -48,6 +48,8 @@
 
 #include <visu_nodes.h>
 #include <visu_data.h>
+#include <visu_pickMesure.h>
+#include <renderingBackend/visu_actionInterface.h>
 
 /**
  * OPENGL_OBSERVE_CONSTRAINED
@@ -68,31 +70,11 @@
 #define OPENGL_OBSERVE_WALKER 1
 
 /**
- * VisuPick_struct:
- * @data: a pointer to the #VisuData whose @node belongs to ;
- * @node: a pointer to a #VisuNode that has been selected ;
- * @button: an integer identifying the button that clicked on the node ;
- * @shiftMod: a boolean to set if shift button was pushed during the click ;
- * @x: x coordinate of the click in window pixels ;
- * @y: y coordinate of the click in window pixels
+ * VisuInteractive:
  *
- * This structure is used when the user uses the pick
- * interface. It stores some informations on the pick action.
+ * All fields are private.
  */
-struct VisuPick_struct
-{
-  VisuData *data;
-  VisuNode *node;
-  unsigned int button;
-  int shiftMod;
-  int x, y;
-};
-/**
- * VisuPick
- *
- * A short definition for #VisuPick_struct objects.
- */
-typedef struct VisuPick_struct VisuPick;
+typedef struct _VisuInteractive VisuInteractive;
 
 /**
  * openGLInteractiveSet_preferedObserveMethod:
@@ -118,14 +100,26 @@ int openGLInteractiveGet_preferedObserveMethod();
 
 /**
  * callBackOnNodeSelectedFunc:
- * @pickData: the structure containing the informations of the
- *            pick (mouse click, position...) ;
+ * @mesureData: the object that stores the pick information ;
  * @data: a pointer to user define data.
  *
  * This interface defines functions that are called whenever a
  * node is selected.
  */
-typedef void (*callBackOnNodeSelectedFunc) (const VisuPick *pickData, gpointer data);
+typedef void (*callBackOnNodeSelectedFunc)(PickMesure *mesureData, gpointer data);
+
+/**
+ * callBackOnEventFunc:
+ * @event: the event that triggered the action ;
+ * @inter: the #VisuInteractive object the callback deals with.
+ *
+ * An interface to methods that can be called whenever an action occurs.
+ *
+ * Returns: TRUE if action should be stopped after this call.
+ */
+typedef gboolean (*callBackOnEventFunc)(SimplifiedEvents *event,
+					VisuInteractive *inter);
+
 
 /* observe and pick functions. */
 /**
@@ -143,67 +137,129 @@ typedef void (*callBackOnNodeSelectedFunc) (const VisuPick *pickData, gpointer d
  * (observe, pick, move...). It changes cursors shapes and initialises different
  * internal variables. It can be stop by a call to openGLInteractiveEnd_session().
  * The default first interactive session is an observe session.
+ *
+ * Returns: a newly created object. This object is automatically
+ * associated to the #VisuData handled by the @window and will be
+ * destroyed with it.
  */
-void openGLInteractiveInit_session(GenericRenderingWindow window,
-				   callBackOnNodeSelectedFunc handler, gpointer data,
-				   GDestroyNotify stopPickObserve);
+VisuInteractive* openGLInteractiveInit_session(GenericRenderingWindow window,
+					       callBackOnNodeSelectedFunc handler,
+					       gpointer data,
+					       GDestroyNotify stopPickObserve);
 /**
  * openGLInteractiveEnd_session:
+ * @inter: a #VisuInteractive object.
  *
  * Stop the period when pick or observe sessions are allowed. Basically,
  * for the user point of view it returns the cursor to its initial shape.
  */
-void openGLInteractiveEnd_session();
+void openGLInteractiveEnd_session(VisuInteractive *inter);
 /**
  * openGLInteractiveBegin_observe:
+ * @inter: a #VisuInteractive object.
  * 
  * During a pick or observe session, the client can begin an observe session
  * by a call to this method. It actually stops also a possibly running
  * pick session.
  */
-void openGLInteractiveBegin_observe();
+void openGLInteractiveBegin_observe(VisuInteractive *inter);
 /**
  * openGLInteractiveBegin_pick:
+ * @inter: a #VisuInteractive object.
  * 
  * During a pick or observe session, the client can begin a pick session
  * by a call to this method. It actually stops also a possibly running
  * observe session.
  */
-void openGLInteractiveBegin_pick();
+void openGLInteractiveBegin_pick(VisuInteractive *inter);
 /**
  * openGLInteractiveBegin_move:
+ * @inter: a #VisuInteractive object.
  * 
  * During an interactive session, the client can begin a move session
  * by a call to this method. It enables the press-button (and release), the motion-notify
  * and the scroll event. It returns the displacement and the node selected.
  */
-void openGLInteractiveBegin_move();
+void openGLInteractiveBegin_move(VisuInteractive *inter);
 /**
  * openGLInteractiveBegin_mark:
- * @data: a #VisuData object.
+ * @inter: a #VisuInteractive object.
  * 
  * During an interactive session, the client can begin a mark session
- * by a call to this method. It enables the press-button (and release), the motion-notify
- * and the scroll event. A mark is drawn around the selected element (or removed) when clicked.
+ * by a call to this method. It enables the press-button (and
+ * release), the motion-notify and the scroll event. A mark is drawn
+ * around the selected element (or removed) when clicked. To begin an
+ * interactive session, see openGLInteractiveInit_session().
  */
-void openGLInteractiveBegin_mark(VisuData *data);
+void openGLInteractiveBegin_mark(VisuInteractive *inter);
 /**
  * openGLInteractiveBegin_pickAndObserve:
+ * @inter: a #VisuInteractive object.
  * 
  * During a pick or observe session, the client can both observe (normal way)
  * and do minimal pick with the right button.
  */
-void openGLInteractiveBegin_pickAndObserve();
+void openGLInteractiveBegin_pickAndObserve(VisuInteractive *inter);
 
 /**
- * openGLInteractiveRemove_allMarks:
+ * openGLInteractiveHandle_event:
+ * @ev: an event ;
+ * @inter: a #VisuInteractive object.
+ *
+ * This routine should be called by the rendering window when some
+ * event is raised on the rendering surface.
+ */
+void openGLInteractiveHandle_event(SimplifiedEvents *ev, VisuInteractive *inter);
+
+/**
+ * openGLInteractiveStart_move:
+ * @inter: a #VisuInteractive object ;
+ * @nodes: a list of #VisuNode (not ids).
+ *
+ * Update the internal of @inter when the given list is dragged. This
+ * means to create a new list with the given nodes and remove them
+ * from the classical #VisuData OpenGL lists. A redraw is
+ * automatically asked.
+ */
+void openGLInteractiveStart_move(VisuInteractive *inter, GList *nodes);
+/**
+ * openGLInteractiveStop_move:
+ * @inter: a #VisuInteractive object ;
+ * @nodes: a list of #VisuNode (not ids).
+ *
+ * Update the internal of @inter when the drag stops. The @nodes are
+ * recreated in the OpenGL lists of #VisuData. A redraw is
+ * automatically asked.
+ */
+void openGLInteractiveStop_move(VisuInteractive *inter, GList *nodes);
+/**
+ * openGLInteractiveMove:
+ * @inter: a #VisuInteractive object ;
+ * @nodes: a list of #VisuNode ids ;
+ * @drag: a translation (in cartesian coordinates).
+ *
+ * Update the internal of @inter when the given list is dragged.
+ */
+void openGLInteractiveMove(VisuInteractive *inter, GList *nodes, float drag[3]);
+
+/**
+ * openGLInteractiveGet_pickMesure:
+ * @inter: a #VisuInteractive object.
+ * 
+ * Get the #PickMesure object associated to an interactive session.
+ *
+ * Returns: a #PickMesure object, private.
+ */
+PickMesure* openGLInteractiveGet_pickMesure(VisuInteractive *inter);
+/**
+ * openGLInteractiveSet_visuData:
+ * @inter: a #VisuInteractive object ;
  * @data: a #VisuData object.
  *
- * It empties the list of all marks.
- *
- * Returns: TRUE if the redraw signal should be emitted.
+ * Change the #VisuData the interactive session is dealing with.
  */
-gboolean openGLInteractiveRemove_allMarks(VisuData *data);
+void openGLInteractiveSet_visuData(VisuInteractive *inter, VisuData *data);
+
 
 /**
  * openGLInteractive_init:
