@@ -67,13 +67,8 @@ void openGLObjectList_init()
 {
   /* Initialisation of the table that stores the user
      glObjectList. */
-  glObjectListRegistered = malloc(sizeof(int) * GLOBJECTLIST_INCREMENT);
-  if (!glObjectListRegistered)
-    {
-      allocationProblems();
-      exit(1);
-    }
-  glObjectListRegisteredSize = GLOBJECTLIST_INCREMENT;
+  glObjectListRegistered = (int*)0;
+  glObjectListRegisteredSize = 0;
   nbGlObjectListRegistered = 0;
   lastGlObjectListSize = 0;
 }
@@ -88,23 +83,20 @@ int openGLObjectList_new(int size)
 {
   if (nbGlObjectListRegistered >= glObjectListRegisteredSize)
     {
-      glObjectListRegistered = realloc(glObjectListRegistered,
-				       sizeof(int) * (glObjectListRegisteredSize + 
-						      GLOBJECTLIST_INCREMENT));
-      if (!glObjectListRegistered)
-	{
-	  allocationProblems();
-	  exit(1);
-	}
       glObjectListRegisteredSize += GLOBJECTLIST_INCREMENT;
+      glObjectListRegistered = g_realloc(glObjectListRegistered,
+					 sizeof(int) * glObjectListRegisteredSize);
     }
   if (nbGlObjectListRegistered > 0)
     glObjectListRegistered[nbGlObjectListRegistered] = 
-      glObjectListRegistered[nbGlObjectListRegistered - 1] + lastGlObjectListSize + 1;
+      glObjectListRegistered[nbGlObjectListRegistered - 1] + lastGlObjectListSize;
   else
     glObjectListRegistered[0] = GLOBJECTLIST_FIRST;
-  lastGlObjectListSize += size;
+  lastGlObjectListSize = size;
   nbGlObjectListRegistered++;
+  DBG_fprintf(stderr, "Object List: request %d OpenGL lists -> %d %d.\n",
+	      size, glObjectListRegistered[nbGlObjectListRegistered - 1],
+	      glObjectListRegistered[nbGlObjectListRegistered - 1] + size - 1);
   return glObjectListRegistered[nbGlObjectListRegistered - 1];
 }
 
