@@ -53,6 +53,41 @@
 #include <visu_configFile.h>
 #include <visu_data.h>
 #include <coreTools/toolMatrix.h>
+#include <coreTools/toolConfigFile.h>
+
+/**
+ * SECTION:view
+ * @short_description: Defines all necessary informations for the
+ * rendering of a view.
+ *
+ * <para>The #OpenGLView stores three basic informations: one for the
+ * position and orientation of the camera (#OpenGLCamera), one for the
+ * description of the bounding box in the OpenGL coordinates
+ * (#OpenGLBox, should be moved elsewhere later) and one last for the
+ * definition of the viewing window (#OpenGLWindow, including volumic
+ * informations).</para>
+ *
+ * <para>One resource is used by this part, defining the precision
+ * desired by the user when drawing OpenGL objects. This precision can
+ * be changed using OpenGLViewSet_precision() and all V_Sim part
+ * drawing something should use OpenGLViewGet_numberOfFacettes() to
+ * know the size of the vertices to be drawn depending on this
+ * precision and the level of zoom.</para>
+ *
+ * <para>The rendering is done in an OpenGl viewport whose size is
+ * given by the bounding box (plus 10%). The camera can be positionned
+ * with three angles (theta, phi and omega) and has a zoom factor
+ * (gross) and a perspective value (d_red). The angle theta is around
+ * the z axis (box coordinates), phi is around the new x axis (after
+ * the theta rotation) and omega is a rotation around the axis which
+ * goes from the observer to the center of the bounding box. By
+ * default the camera looks at the center of the bounding box but this
+ * can be changed with the Xs and Ys parameters. These values are
+ * stored and are readable through the #OpenGLCamera structure. They
+ * must be changed with the following methods :
+ * openGLViewSet_thetaPhiOmega(), openGLViewSet_gross(),
+ * openGLViewSet_persp() and openGLViewSet_XsYs().</para>
+ */
 
 /* Global value used for precision. */
 static float precisionOfRendering;
@@ -222,7 +257,6 @@ int openGLViewSet_persp(OpenGLView *view, float value)
     return 0;
 
   view->camera->d_red = val;
-  view->camera->gr = val / (val - 1.0);
 /*   g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLDRed_signal_id, */
 /* 		 0 , NULL); */
 /*   g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLFacetteChanged_signal_id, */
@@ -235,22 +269,17 @@ int OpenGLViewSet_windowSize(OpenGLView *view, guint width, guint height)
 {
   DBG_fprintf(stderr, "OpenGL View : set viewport size (%dx%d) for view %p.\n",
 	      width, height, (gpointer)view);
+  
+  g_return_val_if_fail(view, 0);
 
-  if (!view)
-    {
-      glViewport(0, 0, width, height);
-      return 0;
-    }
   if (view->window->width != width || view->window->height != height)
     {
+      DBG_fprintf(stderr, " | old values were %dx%d.\n", view->window->width,
+		  view->window->height);
       view->window->width = width;
       view->window->height = height;
       glViewport(0, 0, view->window->width, view->window->height);
       project(view);
-      /*       g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLWidthHeight_signal_id, */
-      /* 		     0 , NULL); */
-      /*       g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLFacetteChanged_signal_id, */
-      /* 		     0 , NULL); */
       return 1;
     }
   return 0;
@@ -264,14 +293,13 @@ static void modelize(OpenGLView *view)
   double sth, cth, sph, cph, com, som;
   double eyex, eyey, eyez;
   double distance;   
-  double upx, upy, upz;
 
-  g_return_if_fail(view && view->camera && view->box);
+  g_return_if_fail(view && view->camera);
  
   theta_rad = view->camera->theta * PI180;
   phi_rad   = view->camera->phi   * PI180; 
 
-  distance = view->camera->d_red * view->box->extens;
+  distance = view->camera->d_red * view->camera->length0;
 
   sth = sin(theta_rad);
   cth = cos(theta_rad);      
@@ -293,13 +321,16 @@ static void modelize(OpenGLView *view)
    
   /* Vecteur donnant la direction verticale.
      Dans le repre transform il est (-1,0,0). */
-  upx = -cth*cph*com + sph*som;
-  upy = -cth*sph*com - cph*som;
-  upz = sth*com;
+  view->camera->up[0] = -cth*cph*com + sph*som;
+  view->camera->up[1] = -cth*sph*com - cph*som;
+  view->camera->up[2] = sth*com;
    
   glMatrixMode(GL_MODELVIEW); 
   glLoadIdentity();
-  gluLookAt(eyex, eyey, eyez, 0.0, 0.0, 0.0, upx, upy, upz);
+  gluLookAt(eyex, eyey, eyez, 0.0, 0.0, 0.0,
+	    view->camera->up[0],
+	    view->camera->up[1],
+	    view->camera->up[2]);
 }
 
 static void project(OpenGLView *view)
@@ -308,29 +339,23 @@ static void project(OpenGLView *view)
   double rap_win;
  
   g_return_if_fail(view && view->camera && view->box && view->window);
- 
-  view->window->near = (view->camera->d_red - 1.0) * view->box->extens;
-  view->window->far  = (view->camera->d_red + 1.0) * view->box->extens;
 
-/*   g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLNearFar_signal_id, */
-/* 		 0 , NULL); */
+  fact = view->camera->d_red * view->camera->length0;
+  view->window->near = MAX(0.01, fact - view->box->extens);
+  view->window->far  = fact + view->box->extens;
 
   fact = view->window->near / view->camera->gross / view->camera->d_red;
   rap = 2. * view->window->near / (view->camera->d_red - 1.);
-  rap_win = (1.0*view->window->height)/view->window->width;
   x = (0.5 - view->camera->xs) * rap;
-  if (rap_win < 1.)
-    x /= rap_win;
   xmin = x - fact;
   xmax = x + fact;
   y = (0.5 - view->camera->ys) * rap;
-  if (rap_win > 1.)
-    y /= rap_win;
   ymin = y - fact;
   ymax = y + fact;
   view->window->left   = xmin;
   view->window->bottom = ymin;
     
+  rap_win = (1.0*view->window->height)/view->window->width;
   if ( 1. > rap_win )
     {
       view->window->top   = ymax;
@@ -380,14 +405,14 @@ OpenGLView* OpenGLViewNew()
   view->window->near   = 0.;
   view->window->far    = 0.;
 
-  view->camera->theta = anglesDefault[0];
-  view->camera->phi   = anglesDefault[1];
-  view->camera->omega = anglesDefault[2];
-  view->camera->xs    = translatDefault[0];
-  view->camera->ys    = translatDefault[1];
-  view->camera->gross = grossDefault;
-  view->camera->d_red = perspDefault;
-  view->camera->gr    = view->camera->d_red / (view->camera->d_red - 1.);
+  view->camera->theta   = anglesDefault[0];
+  view->camera->phi     = anglesDefault[1];
+  view->camera->omega   = anglesDefault[2];
+  view->camera->xs      = translatDefault[0];
+  view->camera->ys      = translatDefault[1];
+  view->camera->gross   = grossDefault;
+  view->camera->d_red   = perspDefault;
+  view->camera->length0 = -1.;
 
   DBG_fprintf(stderr, "OpenGL view : create a new view (%p).\n", (gpointer)view);
   DBG_fprintf(stderr, " | theta = %g\n", view->camera->theta);
@@ -432,34 +457,230 @@ OpenGLView* OpenGLViewCopy(OpenGLView *view)
   viewCopy->window->bottom = view->window->bottom;
   viewCopy->window->top    = view->window->top;
 
-  viewCopy->camera->theta = view->camera->theta;
-  viewCopy->camera->phi   = view->camera->phi;
-  viewCopy->camera->omega = view->camera->omega;
-  viewCopy->camera->xs    = view->camera->xs;
-  viewCopy->camera->ys    = view->camera->ys;
-  viewCopy->camera->gross = view->camera->gross;
-  viewCopy->camera->d_red = view->camera->d_red;
-  viewCopy->camera->gr    = view->camera->gr;
+  viewCopy->camera->theta   = view->camera->theta;
+  viewCopy->camera->phi     = view->camera->phi;
+  viewCopy->camera->omega   = view->camera->omega;
+  viewCopy->camera->xs      = view->camera->xs;
+  viewCopy->camera->ys      = view->camera->ys;
+  viewCopy->camera->gross   = view->camera->gross;
+  viewCopy->camera->d_red   = view->camera->d_red;
+  viewCopy->camera->length0 = view->camera->length0;
   
-  viewCopy->box->extens = view->box->extens;
-  viewCopy->box->dxxs2  = view->box->dxxs2;
-  viewCopy->box->dyys2  = view->box->dyys2;
-  viewCopy->box->dzzs2  = view->box->dzzs2;
+  viewCopy->box->extens  = view->box->extens;
+  viewCopy->box->dxxs2   = view->box->dxxs2;
+  viewCopy->box->dyys2   = view->box->dyys2;
+  viewCopy->box->dzzs2   = view->box->dzzs2;
   for (i = 0; i < 3; i++)
     {
-      viewCopy->box->p1[i]  = view->box->p1[i];
-      viewCopy->box->p2[i]  = view->box->p2[i];
-      viewCopy->box->p3[i]  = view->box->p3[i];
-      viewCopy->box->p4[i]  = view->box->p4[i];
-      viewCopy->box->p5[i]  = view->box->p5[i];
-      viewCopy->box->p6[i]  = view->box->p6[i];
-      viewCopy->box->p7[i]  = view->box->p7[i];
-      viewCopy->box->p8[i]  = view->box->p8[i];
+      viewCopy->box->extension[i] = view->box->extension[i];
+      viewCopy->box->p1[i] = view->box->p1[i];
+      viewCopy->box->p2[i] = view->box->p2[i];
+      viewCopy->box->p3[i] = view->box->p3[i];
+      viewCopy->box->p4[i] = view->box->p4[i];
+      viewCopy->box->p5[i] = view->box->p5[i];
+      viewCopy->box->p6[i] = view->box->p6[i];
+      viewCopy->box->p7[i] = view->box->p7[i];
+      viewCopy->box->p8[i] = view->box->p8[i];
     }
 
   return viewCopy;
 }
 
+void openGLViewGet_boxSpan(OpenGLView *view, float span[2])
+{
+  float diag[3], ext[3];
+
+  g_return_if_fail(view && view->box);
+
+  ext[0]  = view->box->extension[0] * view->box->extension[0];
+  ext[1]  = view->box->extension[1] * view->box->extension[1];
+  ext[2]  = view->box->extension[2] * view->box->extension[2];
+  diag[0] = view->box->p7[0] * view->box->p7[0];
+  diag[1] = view->box->p7[1] * view->box->p7[1];
+  diag[2] = view->box->p7[2] * view->box->p7[2];
+  span[0] = -sqrt(view->box->extension[0] * view->box->extension[0] * diag[0] +
+		  view->box->extension[1] * view->box->extension[1] * diag[1] +
+		  view->box->extension[2] * view->box->extension[2] * diag[2]);
+  span[1] = sqrt((view->box->extension[0] + 1.f) *
+		 (view->box->extension[0] + 1.f) * diag[0] +
+		 (view->box->extension[1] + 1.f) *
+		 (view->box->extension[1] + 1.f) * diag[1] +
+		 (view->box->extension[2] + 1.f) *
+		 (view->box->extension[2] + 1.f) * diag[2]);
+}
+static float computeExtens(float basisSet[6], float extension[3])
+{
+  float dz2, dy2, dx, dy, dz, su, sc, ext[3];
+
+  /* calculate bare = 1/2 radius of centered sample */
+  ext[0] = (1. + 2. * extension[0]);
+  ext[1] = (1. + 2. * extension[1]);
+  ext[2] = (1. + 2. * extension[2]);
+  dz = basisSet[VISU_DATA_BOX_DZZ] * ext[2];
+  dz2 = dz * dz;
+  dy = basisSet[VISU_DATA_BOX_DYY] * ext[1] + basisSet[VISU_DATA_BOX_DZY] * ext[2];
+  dy2 = dy * dy;
+  dx = (basisSet[VISU_DATA_BOX_DXX] * ext[0] +
+	basisSet[VISU_DATA_BOX_DYX] * ext[1] +
+	basisSet[VISU_DATA_BOX_DZX] * ext[2]);
+  su = dx * dx + dy2 + dz2;
+  dx = (-basisSet[VISU_DATA_BOX_DXX] * ext[0] +
+	basisSet[VISU_DATA_BOX_DYX] * ext[1] +
+	basisSet[VISU_DATA_BOX_DZX] * ext[2]);
+  sc = dx * dx + dy2 + dz2;
+  if (sc > su) su = sc;
+  dx = (basisSet[VISU_DATA_BOX_DXX] * ext[0] -
+	basisSet[VISU_DATA_BOX_DYX] * ext[1] +
+	basisSet[VISU_DATA_BOX_DZX] * ext[2]);
+  dy = basisSet[VISU_DATA_BOX_DYY] * ext[1] - basisSet[VISU_DATA_BOX_DZY] * ext[2];
+  dy2 = dy * dy;
+  sc = dx * dx + dy2 + dz2;
+  if (sc > su) su = sc;
+  dx = (basisSet[VISU_DATA_BOX_DXX] * ext[0] +
+	basisSet[VISU_DATA_BOX_DYX] * ext[1] -
+	basisSet[VISU_DATA_BOX_DZX] * ext[2]);
+  sc = dx * dx + dy2 + dz2;
+  if (sc > su) su = sc;
+
+  return sqrt(su);
+}
+void openGLViewSet_box(OpenGLView *view, float basisSet[6], float extension[3])
+{
+  g_return_if_fail(view && view->box);
+
+  DBG_fprintf(stderr, "OpenGLBox: set box vertex from basis-set:\n"
+	      " | %f %f %f\n | %f %f %f\n | %f %f %f\n",
+	      basisSet[0], basisSet[1], basisSet[2], basisSet[3],
+	      basisSet[4], basisSet[5], extension[0],
+	      extension[1], extension[2]);
+
+  view->box->dxxs2 = 0.5 * (basisSet[VISU_DATA_BOX_DXX] +
+			    basisSet[VISU_DATA_BOX_DYX] +
+			    basisSet[VISU_DATA_BOX_DZX]);
+  view->box->dyys2 = 0.5 * (basisSet[VISU_DATA_BOX_DYY] +
+			    basisSet[VISU_DATA_BOX_DZY]);
+  view->box->dzzs2 = 0.5 * (basisSet[VISU_DATA_BOX_DZZ]);
+  
+  view->box->extension[0] = extension[0];
+  view->box->extension[1] = extension[1];
+  view->box->extension[2] = extension[2];
+
+  view->box->p1[0] = 0.;
+  view->box->p1[1] = 0.;
+  view->box->p1[2] = 0.;
+
+  view->box->p2[0] = basisSet[VISU_DATA_BOX_DXX];
+  view->box->p2[1] = 0.;
+  view->box->p2[2] = 0.;
+
+  view->box->p4[0] = basisSet[VISU_DATA_BOX_DYX];
+  view->box->p4[1] = basisSet[VISU_DATA_BOX_DYY];
+  view->box->p4[2] = 0.;
+
+  view->box->p5[0] = basisSet[VISU_DATA_BOX_DZX];
+  view->box->p5[1] = basisSet[VISU_DATA_BOX_DZY];
+  view->box->p5[2] = basisSet[VISU_DATA_BOX_DZZ];
+
+  view->box->p3[0] = view->box->p2[0] + view->box->p4[0];
+  view->box->p3[1] = view->box->p2[1] + view->box->p4[1];
+  view->box->p3[2] = view->box->p2[2] + view->box->p4[2];
+      
+  view->box->p6[0] = view->box->p5[0] + view->box->p2[0];
+  view->box->p6[1] = view->box->p5[1] + view->box->p2[1];
+  view->box->p6[2] = view->box->p5[2] + view->box->p2[2];
+      
+  view->box->p7[0] = view->box->p5[0] + view->box->p3[0];
+  view->box->p7[1] = view->box->p5[1] + view->box->p3[1];
+  view->box->p7[2] = view->box->p5[2] + view->box->p3[2];
+      
+  view->box->p8[0] = view->box->p5[0] + view->box->p4[0];
+  view->box->p8[1] = view->box->p5[1] + view->box->p4[1];
+  view->box->p8[2] = view->box->p5[2] + view->box->p4[2];
+
+  view->box->extens = computeExtens(basisSet, extension) * 0.55f;
+  DBG_fprintf(stderr, " | extens = %g.\n", view->box->extens);
+}
+gboolean openGLViewSet_refLength(OpenGLView *view, float length)
+{
+  float basisSet[6], ext[3], l0;
+
+  g_return_val_if_fail(view && view->camera, FALSE);
+
+  if (length > 0.f)
+    l0 = length;
+  else
+    {
+      g_return_val_if_fail(view->box, FALSE);
+  
+      ext[0] = ext[1] = ext[2] = 0.f;
+      basisSet[VISU_DATA_BOX_DXX] = view->box->p2[0];
+      basisSet[VISU_DATA_BOX_DYX] = view->box->p4[0];
+      basisSet[VISU_DATA_BOX_DYY] = view->box->p4[1];
+      basisSet[VISU_DATA_BOX_DZX] = view->box->p5[0];
+      basisSet[VISU_DATA_BOX_DZY] = view->box->p5[1];
+      basisSet[VISU_DATA_BOX_DZZ] = view->box->p5[2];
+      l0 = computeExtens(basisSet, ext) * 0.55f;
+    }
+  if (l0 == view->camera->length0)
+    return FALSE;
+  else
+    view->camera->length0 = l0;
+  DBG_fprintf(stderr, "OpenGLCamera: set ref length to %g.\n",
+	      view->camera->length0);
+  return TRUE;
+}
+void openGLViewGet_boxVertices(OpenGLView *view, float vertices[8][3],
+			       gboolean withExtension)
+{
+  float transX[3], transY[3], transZ[3];
+  int i;
+
+  g_return_if_fail(view && view->box);
+
+  if (withExtension)
+    {
+      DBG_fprintf(stderr, "OpenGL View: get the box vertices with"
+		  " extension (%g;%g;%g).\n", view->box->extension[0],
+		  view->box->extension[1], view->box->extension[2]);
+      transX[0] = view->box->extension[0] * (view->box->p2[0] - view->box->p1[0]);
+      transX[1] = view->box->extension[0] * (view->box->p2[1] - view->box->p1[1]);
+      transX[2] = view->box->extension[0] * (view->box->p2[2] - view->box->p1[2]);
+      transY[0] = view->box->extension[1] * (view->box->p4[0] - view->box->p1[0]);
+      transY[1] = view->box->extension[1] * (view->box->p4[1] - view->box->p1[1]);
+      transY[2] = view->box->extension[1] * (view->box->p4[2] - view->box->p1[2]);
+      transZ[0] = view->box->extension[2] * (view->box->p5[0] - view->box->p1[0]);
+      transZ[1] = view->box->extension[2] * (view->box->p5[1] - view->box->p1[1]);
+      transZ[2] = view->box->extension[2] * (view->box->p5[2] - view->box->p1[2]);
+      for (i = 0; i < 3; i++)
+	{
+	  vertices[0][i] = view->box->p1[i] - transX[i] - transY[i] - transZ[i];
+	  vertices[1][i] = view->box->p2[i] + transX[i] - transY[i] - transZ[i];
+	  vertices[2][i] = view->box->p3[i] + transX[i] + transY[i] - transZ[i];
+	  vertices[3][i] = view->box->p4[i] - transX[i] + transY[i] - transZ[i];
+	  vertices[4][i] = view->box->p5[i] - transX[i] - transY[i] + transZ[i];
+	  vertices[5][i] = view->box->p6[i] + transX[i] - transY[i] + transZ[i];
+	  vertices[6][i] = view->box->p7[i] + transX[i] + transY[i] + transZ[i];
+	  vertices[7][i] = view->box->p8[i] - transX[i] + transY[i] + transZ[i];
+	}
+    }
+  else
+    {
+      DBG_fprintf(stderr, "OpenGL View: get the box vertices without extension.\n");
+      for (i = 0; i < 3; i++)
+	{
+	  vertices[0][i] = view->box->p1[i];
+	  vertices[1][i] = view->box->p2[i];
+	  vertices[2][i] = view->box->p3[i];
+	  vertices[3][i] = view->box->p4[i];
+	  vertices[4][i] = view->box->p5[i];
+	  vertices[5][i] = view->box->p6[i];
+	  vertices[6][i] = view->box->p7[i];
+	  vertices[7][i] = view->box->p8[i];
+	}
+    }
+}
+
+
 /* This is a function to get the number of facettes advised
    by the server (according to its policy on rendering)
    to draw an object according to a given dimension. */
@@ -482,7 +703,7 @@ int OpenGLViewGet_numberOfFacettes(OpenGLView *view, float dimension)
 
   static float fac = -1.0f, fac_v = -1.0f;
 
-  g_return_val_if_fail(view && view->camera && view->window && view->box, -1);
+  g_return_val_if_fail(view && view->camera && view->window, -1);
 
 /*   if (!visuCamera->gr || !visuCamera->gross) */
 /*     return -1; */
@@ -495,9 +716,9 @@ int OpenGLViewGet_numberOfFacettes(OpenGLView *view, float dimension)
     fac_v = ((float)(NLAT_V_MAX - NLAT_V_MIN))/(RSIZE_V_MAX - RSIZE_V_MIN);
   }
 
-  rsize = (int)(view->window->width * (dimension /
-				       (2.0 * view->box->extens / view->camera->gr /
-					view->camera->gross)));
+  rsize = (int)(MIN(view->window->width, view->window->height) *
+		(0.5 * dimension / view->camera->length0 * view->camera->gross *
+		 view->camera->d_red / (view->camera->d_red - 1.)));
   
   if(rsize < RSIZE_MIN) {
     nlat = (int)(NLAT_V_MIN + fac_v * (rsize - RSIZE_V_MIN));
@@ -784,7 +1005,7 @@ void openGLViewRotate_camera(OpenGLView *view, float dTheta, float dPhi, float a
   while (Omega - view->camera->omega > 90.)
     Omega -= 360.;
 
-/*   fprintf(stderr, "Theta phi omega : %f %f %f\n", Theta, Phi, Omega); */
+  /*   fprintf(stderr, "Theta phi omega : %f %f %f\n", Theta, Phi, Omega); */
   angles[0] = Theta;
   angles[1] = Phi;
   angles[2] = Omega;
@@ -798,29 +1019,29 @@ void openGLViewRotate_camera(OpenGLView *view, float dTheta, float dPhi, float a
 #define FLAG_PARAMETER_OPENGL_DETAILS   "opengl_details"
 #define DESC_PARAMETER_OPENGL_DETAILS   "Give a value to the quality of rendering (100 is normal) ; positive integer"
 #define PARAMETER_OPENGL_DETAILS_DEFAULT 100
-gboolean readOpenGLPrecision(gchar **lines, int nbLines,
-			     int position, GString *errorMessage);
-gboolean exportParametersOpenGLView(GString *data, int *nbLinesWritten,
-				    VisuData *dataObj);
+static gboolean readOpenGLPrecision(gchar **lines, int nbLines, int position,
+				    VisuData *dataObj, GError **error);
+static gboolean exportParametersOpenGLView(GString *data, int *nbLinesWritten,
+					   VisuData *dataObj);
 
 #define FLAG_RESOURCE_OPENGL_ANGLES "opengl_theta_phi_omega"
 #define DESC_RESOURCE_OPENGL_ANGLES "2 real values (degrees) for user orientation with respect to sample"
-gboolean readOpenGLThetaPhi(gchar **lines, int nbLines,
-			    int position, GString *errorMessage);
+static gboolean readOpenGLThetaPhi(gchar **lines, int nbLines, int position,
+				   VisuData *dataObj, GError **error);
 #define FLAG_RESOURCE_OPENGL_TRANSLAT "opengl_xs_ys"
 #define DESC_RESOURCE_OPENGL_TRANSLAT "2 real values for image position with respect to [0.0, 1.0]x[0.0, 1.0] window"
-gboolean readOpenGLXsYs(gchar **lines, int nbLines,
-			int position, GString *errorMessage);
+static gboolean readOpenGLXsYs(gchar **lines, int nbLines, int position,
+			       VisuData *dataObj, GError **error);
 #define FLAG_RESOURCE_OPENGL_GROSS "opengl_gross"
 #define DESC_RESOURCE_OPENGL_GROSS "gross factor (must be real > 0.0)"
-gboolean readOpenGLGross(gchar **lines, int nbLines,
-			 int position, GString *errorMessage);
+static gboolean readOpenGLGross(gchar **lines, int nbLines, int position,
+				VisuData *dataObj, GError **error);
 #define FLAG_RESOURCE_OPENGL_PERSP "opengl_d_red"
 #define DESC_RESOURCE_OPENGL_PERSP "reduced perspective distance (must be real > 1.0)"
-gboolean readOpenGLPersp(gchar **lines, int nbLines,
-			 int position, GString *errorMessage);
-gboolean exportResourcesOpenGLView(GString *data, int *nbLinesWritten,
-				   VisuData *dataObj);
+static gboolean readOpenGLPersp(gchar **lines, int nbLines, int position,
+				VisuData *dataObj, GError **error);
+static gboolean exportResourcesOpenGLView(GString *data, int *nbLinesWritten,
+					  VisuData *dataObj);
 
 
 void OpenGLViewInit()
@@ -843,6 +1064,7 @@ void OpenGLViewInit()
 					  FLAG_RESOURCE_OPENGL_ANGLES,
 					  DESC_RESOURCE_OPENGL_ANGLES,
 					  1, readOpenGLThetaPhi);
+  visuConfigFileSet_version(resourceEntry, 3.1f);
   resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_RESOURCE,
 					  FLAG_RESOURCE_OPENGL_TRANSLAT,
 					  DESC_RESOURCE_OPENGL_TRANSLAT,
@@ -860,29 +1082,27 @@ void OpenGLViewInit()
 
 }
 
-gboolean readOpenGLPrecision(gchar **lines, int nbLines,
-			     int position, GString *errorMessage)
+static gboolean readOpenGLPrecision(gchar **lines, int nbLines, int position,
+				    VisuData *dataObj _U_, GError **error)
 {
   int val;
-  int res;
 
-  res = sscanf(lines[0],"%d", &val);
-  if (res != 1)
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
+  if (!configFileRead_integer(lines[0], position, &val, 1, error))
+    return FALSE;
+  if (val <= 0 || val > 500)
     {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d,"
-					       " 1 integer value (v > 0) must appear"
-					       " after the %s markup.\n"),
-			       position, FLAG_PARAMETER_OPENGL_DETAILS);
-      OpenGLViewSet_precision((float)PARAMETER_OPENGL_DETAILS_DEFAULT / 100.);
+      *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_VALUE,
+			   _("Parse error at line %d: width must be in %d-%d.\n"),
+			   position, 0, 500);
       return FALSE;
     }
-
   OpenGLViewSet_precision((float)val / 100.);
   return TRUE;
 }
-gboolean exportParametersOpenGLView(GString *data, int *nbLinesWritten,
-				    VisuData *dataObj)
+static gboolean exportParametersOpenGLView(GString *data, int *nbLinesWritten,
+					   VisuData *dataObj _U_)
 {
   g_string_append_printf(data, "# %s\n", DESC_PARAMETER_OPENGL_DETAILS);
   g_string_append_printf(data, "%s: %d\n\n", FLAG_PARAMETER_OPENGL_DETAILS,
@@ -890,127 +1110,73 @@ gboolean exportParametersOpenGLView(GString *data, int *nbLinesWritten,
   *nbLinesWritten = 3;
   return TRUE;
 }
-gboolean readOpenGLThetaPhi(gchar **lines, int nbLines,
-			    int position, GString *errorMessage)
+static gboolean readOpenGLThetaPhi(gchar **lines, int nbLines, int position,
+				   VisuData *dataObj, GError **error)
 {
-  int res;
   float val[3];
-  GList *tmpLst;
   
-  res = sscanf(lines[0],"%f %f %f", &val[0], &val[1], &val[2]);
-  if (res != 3)
-    {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d, 2 floating "
-					       "point values must appear"
-					       " after the %s markup.\n"),
-			       position, FLAG_RESOURCE_OPENGL_ANGLES);
-      /* Don't change current values. */
-      return FALSE;
-    }
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
+  if (!configFileRead_float(lines[0], position, val, 3, error))
+    return FALSE;
   anglesDefault[0] = val[0];
   anglesDefault[1] = val[1];
   anglesDefault[2] = val[2];
-  tmpLst = visuDataGet_allObjects();
-  while (tmpLst)
-    {
-      visuDataSet_angleOfView(VISU_DATA(tmpLst->data), val[0], val[1], val[2],
-			      MASK_THETA | MASK_PHI | MASK_OMEGA);
-      tmpLst = g_list_next(tmpLst);
-    }
-
+  if (dataObj)
+    visuDataSet_angleOfView(dataObj, val[0], val[1], val[2],
+			    MASK_THETA | MASK_PHI | MASK_OMEGA);
+  
   return TRUE;
 }
-gboolean readOpenGLXsYs(gchar **lines, int nbLines,
-			int position, GString *errorMessage)
+static gboolean readOpenGLXsYs(gchar **lines, int nbLines, int position,
+			       VisuData *dataObj, GError **error)
 {
-  int res;
   float val[2];
-  GList *tmpLst;
   
-  res = sscanf(lines[0],"%f %f", &val[0], &val[1]);
-  if (res != 2)
-    {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d, 2 floating "
-					       "point values must appear"
-					       " after the %s markup.\n"),
-			       position, FLAG_RESOURCE_OPENGL_TRANSLAT);
-      /* Don't change current values. */
-      return FALSE;
-    }
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
+  if (!configFileRead_float(lines[0], position, val, 2, error))
+    return FALSE;
   translatDefault[0] = val[0];
   translatDefault[1] = val[1];
-  tmpLst = visuDataGet_allObjects();
-  while (tmpLst)
-    {
-      visuDataSet_positionOfView(VISU_DATA(tmpLst->data), val[0], val[1],
-				 MASK_XS | MASK_YS);
-      tmpLst = g_list_next(tmpLst);
-    }
-
+  if (dataObj)
+    visuDataSet_positionOfView(dataObj, val[0], val[1],
+			       MASK_XS | MASK_YS);
+  
   return TRUE;
 }
-gboolean readOpenGLGross(gchar **lines, int nbLines,
-			 int position, GString *errorMessage)
+static gboolean readOpenGLGross(gchar **lines, int nbLines, int position,
+				VisuData *dataObj, GError **error)
 {
-  int res;
   float val;
-  GList *tmpLst;
   
-  res = sscanf(lines[0],"%f", &val);
-  if (res != 1)
-    {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d, 1 floating "
-					       "point value (0. < v <= 10)"
-					       " must appear after the %s"
-					       " markup.\n"), position,
-			       FLAG_RESOURCE_OPENGL_GROSS);
-      /* Don't change current values. */
-      return FALSE;
-    }
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
+  if (!configFileRead_float(lines[0], position, &val, 1, error))
+    return FALSE;
   grossDefault = val;
-  tmpLst = visuDataGet_allObjects();
-  while (tmpLst)
-    {
-      visuDataSet_zoomOfView(VISU_DATA(tmpLst->data), val);
-      tmpLst = g_list_next(tmpLst);
-    }
+  if (dataObj)
+    visuDataSet_zoomOfView(dataObj, val);
 
   return TRUE;
 }
-gboolean readOpenGLPersp(gchar **lines, int nbLines,
-			 int position, GString *errorMessage)
+static gboolean readOpenGLPersp(gchar **lines, int nbLines, int position,
+				VisuData *dataObj, GError **error)
 {
-  int res;
   float val;
-  GList *tmpLst;
   
-  res = sscanf(lines[0],"%f", &val);
-  if (res != 1)
-    {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d, 1 floating "
-					       "point value (0. < v <= 10)"
-					       " must appear after the %s"
-					       " markup.\n"), position,
-			       FLAG_RESOURCE_OPENGL_PERSP);
-      /* Don't change current values. */
-      return FALSE;
-    }
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
+  if (!configFileRead_float(lines[0], position, &val, 1, error))
+    return FALSE;
   perspDefault = val;
-  tmpLst = visuDataGet_allObjects();
-  while (tmpLst)
-    {
-      visuDataSet_perspectiveOfView(VISU_DATA(tmpLst->data), val);
-      tmpLst = g_list_next(tmpLst);
-    }
+  if (dataObj)
+    visuDataSet_perspectiveOfView(dataObj, val);
 
   return TRUE;
 }
-gboolean exportResourcesOpenGLView(GString *data, int *nbLinesWritten,
-				   VisuData *dataObj)
+static gboolean exportResourcesOpenGLView(GString *data, int *nbLinesWritten,
+					  VisuData *dataObj _U_)
 {
   GList *tmpLst;
   OpenGLView *view;
