@@ -49,21 +49,23 @@
 #include <visu_object.h>
 #include <visu_tools.h>
 #include <gtk_pairs.h>
+#include <extraGtkFunctions/gtk_stippleComboBoxWidget.h>
 
 #include "wire.h"
 
 #define COLOUR_DESCR_RGB 0
 #define COLOUR_DESCR_HSV 1
 
-GtkWidget *colorLengthCurve[3];
-/* GtkWidget *labelCoord[3]; */
-GtkWidget *labelColourDescr[3];
-GtkWidget *vboxCurve[3];
-GtkWidget *spinThickness;
-GtkWidget *checkNonLinear;
-int colorForCurve;
+static GtkWidget *colorLengthCurve[3];
+/* static GtkWidget *labelCoord[3]; */
+static GtkWidget *labelColourDescr[3];
+static GtkWidget *vboxCurve[3];
+static GtkWidget *spinThickness;
+static GtkWidget *checkNonLinear;
+static GtkWidget *comboStipple;
+static int colorForCurve;
 int colorDescr;
-gulong signalSpinWidthId;
+static gulong signalSpinWidthId, signalComboStippleId;
 
 /* Callbacks */
 void changeWireThickness(GtkSpinButton *spin, gpointer data);
@@ -71,18 +73,20 @@ void changeWireColorLength(GtkToggleButton *button, gpointer data);
 void changeColorForCurve(GtkToggleButton *button, gpointer data);
 void changeColorDescr(GtkToggleButton *button, gpointer data);
 void applyChangesInCurves(GtkButton *button, gpointer data);
-static void wireValuesChangedOnResources(GObject *obj, gpointer data);
-/* gboolean drawCoordinates(GtkWidget *widget, GdkEvent *event, gpointer user_data); */
+static void onStippleChanged(StippleComboBox *combo, guint value, gpointer data);
+/* static void wireValuesChangedOnResources(GObject *obj, gpointer data); */
+/* static gboolean drawCoordinates(GtkWidget *widget, GdkEventMotion *event, */
+/* 				gpointer user_data); */
 
 void setValueForColorLength();
 
-void initGtkWireExtension(void)
+void gtkPairsWireInit(void)
 {
-  g_signal_connect(G_OBJECT(visu), "resourcesLoaded",
-		   G_CALLBACK(wireValuesChangedOnResources), (gpointer)0);
+/*   g_signal_connect(G_OBJECT(visu), "resourcesLoaded", */
+/* 		   G_CALLBACK(wireValuesChangedOnResources), (gpointer)0); */
 }
 
-GtkWidget* buildWireParameters()
+GtkWidget* gtkPairsWireBuild_widgets()
 {
   GtkWidget *hbox1;
   GtkWidget *vbox4;
@@ -113,21 +117,25 @@ GtkWidget* buildWireParameters()
   hbox1 = gtk_hbox_new (FALSE, 0);
 
   vbox4 = gtk_vbox_new (FALSE, 0);
-  gtk_widget_show (vbox4);
   gtk_box_pack_start (GTK_BOX (hbox1), vbox4, FALSE, FALSE, 0);
 
-  label1 = gtk_label_new (_("Thickness :"));
-  gtk_widget_show (label1);
+  label1 = gtk_label_new (_("Thickness:"));
   gtk_box_pack_start (GTK_BOX (vbox4), label1, FALSE, FALSE, 0);
   gtk_misc_set_alignment (GTK_MISC (label1), 1, 0.5);
-
   spinThickness_adj = gtk_adjustment_new (1, 1, 50, 1, 2, 0);
   spinThickness = gtk_spin_button_new (GTK_ADJUSTMENT (spinThickness_adj), 1, 0);
-  gtk_widget_show (spinThickness);
   gtk_box_pack_start (GTK_BOX (vbox4), spinThickness, FALSE, FALSE, 3);
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinThickness), TRUE);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinThickness), getWireGeneralWidth());
 
+  label1 = gtk_label_new (_("Pattern:"));
+  gtk_box_pack_start(GTK_BOX(vbox4), label1, FALSE, FALSE, 0);
+  gtk_misc_set_alignment(GTK_MISC (label1), 1, 0.5);
+  comboStipple = stippleComboBox_new();
+  gtk_box_pack_start(GTK_BOX(vbox4), comboStipple, FALSE, FALSE, 0);
+
+  gtk_widget_show_all(vbox4);
+
   frame = gtk_frame_new("");
 /*   gtk_widget_set_size_request(frame, -1, 75); */
   gtk_widget_show(frame);
@@ -196,7 +204,7 @@ GtkWidget* buildWireParameters()
   gtk_box_pack_start (GTK_BOX (hbox5), radiobutton1, FALSE, FALSE, 0);
   gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton1), radiobutton1_group);
   radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton1));
-  labelColourDescr[0] = gtk_label_new("R");
+  labelColourDescr[0] = gtk_label_new(_("R"));
   gtk_widget_show(labelColourDescr[0]);
   gtk_container_add(GTK_CONTAINER(radiobutton1), labelColourDescr[0]);
 
@@ -207,7 +215,7 @@ GtkWidget* buildWireParameters()
   gtk_box_pack_start (GTK_BOX (hbox5), radiobutton2, FALSE, FALSE, 0);
   gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton2), radiobutton1_group);
   radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton2));
-  labelColourDescr[1] = gtk_label_new("G");
+  labelColourDescr[1] = gtk_label_new(_("G"));
   gtk_widget_show(labelColourDescr[1]);
   gtk_container_add(GTK_CONTAINER(radiobutton2), labelColourDescr[1]);
 
@@ -218,7 +226,7 @@ GtkWidget* buildWireParameters()
   gtk_box_pack_start (GTK_BOX (hbox5), radiobutton3, FALSE, FALSE, 0);
   gtk_radio_button_set_group (GTK_RADIO_BUTTON (radiobutton3), radiobutton1_group);
   radiobutton1_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radiobutton3));
-  labelColourDescr[2] = gtk_label_new("B");
+  labelColourDescr[2] = gtk_label_new(_("B"));
   gtk_widget_show(labelColourDescr[2]);
   gtk_container_add(GTK_CONTAINER(radiobutton3), labelColourDescr[2]);
 
@@ -232,10 +240,11 @@ GtkWidget* buildWireParameters()
   gtk_widget_show (vboxCurve[0]);
   gtk_box_pack_start (GTK_BOX (hbox6), vboxCurve[0], TRUE, TRUE, 0);
   colorLengthCurve[0] = gtk_curve_new ();
+/*   gtk_widget_set_events(colorLengthCurve[0], GDK_ALL_EVENTS_MASK); */
   gtk_widget_show (colorLengthCurve[0]);
   gtk_box_pack_start (GTK_BOX (vboxCurve[0]), colorLengthCurve[0], TRUE, TRUE, 0);
   gtk_curve_set_range (GTK_CURVE (colorLengthCurve[0]), 0, 1, 0, 1);
-/*   labelCoord[0] = gtk_label_new (_("0.1")); */
+/*   labelCoord[0] = gtk_label_new ("0.1"); */
 /*   gtk_widget_show (labelCoord[0]); */
 /*   gtk_box_pack_start (GTK_BOX (vboxCurve[0]), labelCoord[0], FALSE, FALSE, 0); */
 
@@ -271,6 +280,9 @@ GtkWidget* buildWireParameters()
   g_signal_connect(G_OBJECT(buttonApply), "clicked",
 		   G_CALLBACK(applyChangesInCurves), (gpointer)0);
 
+  signalComboStippleId = g_signal_connect(G_OBJECT(comboStipple), "stipple-selected",
+					  G_CALLBACK(onStippleChanged), (gpointer)0);
+
   g_signal_connect(G_OBJECT(radiobutton1), "toggled",
 		   G_CALLBACK(changeColorForCurve), GINT_TO_POINTER(0));
   g_signal_connect(G_OBJECT(radiobutton2), "toggled",
@@ -282,62 +294,94 @@ GtkWidget* buildWireParameters()
 		   G_CALLBACK(changeColorDescr), GINT_TO_POINTER(COLOUR_DESCR_RGB));
   g_signal_connect(G_OBJECT(radiobuttonHSV), "toggled",
 		   G_CALLBACK(changeColorDescr), GINT_TO_POINTER(COLOUR_DESCR_HSV));
-/*   g_signal_connect(G_OBJECT(colorLengthCurve[0]), "event", */
+/*   g_signal_connect(G_OBJECT(colorLengthCurve[0]), "motion-notify-event", */
 /* 		   G_CALLBACK(drawCoordinates), GINT_TO_POINTER(0)); */
 
   return hbox1;
 }
 
-gchar* buildWireLabels(PairsData *data)
+gchar* gtkPairsWireGet_valuesAsLabel(VisuPairData *data)
 {
-  GString *str;
-  int *width;
-  int val;
+  int width;
+  guint16 stipple;
+  gchar *str;
 
-  width = (int*)getPairsProperty(data, "width");
-  if (width)
-    val = *width;
-  else
-    val = getWireGeneralWidth();
-      
-  str = g_string_new("");
-  g_string_printf(str, _("%2dpx"), val);
+  width = getWireWidth(data);
+  stipple = wireGet_stipple(data);
 
-  return g_string_free(str, FALSE);
+  /* px is for pixels and pat. for pattern. */
+  str = g_strdup_printf("%2d%s, %s %d", width, _("px"), _("pat."), stipple);
+
+  return str;
 }
 
-void signalWireSelected(PairsData *data)
+void gtkPairsWireSet_values(VisuPairData *data)
 {
-  int *width;
-  int val;
-
-  width = (int*)getPairsProperty(data, "width");
-  if (width)
-    val = *width;
-  else
-    val = getWireGeneralWidth();
+  int width;
+  guint16 stipple;
 
+  width = getWireWidth(data);
   g_signal_handler_block(G_OBJECT(spinThickness), signalSpinWidthId);
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinThickness), val);
+  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinThickness), width);
   g_signal_handler_unblock(G_OBJECT(spinThickness), signalSpinWidthId);
+
+  stipple = wireGet_stipple(data);
+  g_signal_handler_block(G_OBJECT(comboStipple), signalComboStippleId);
+  if (!stippleComboBoxSet_selectionByStipple(STIPPLE_COMBOX(comboStipple), stipple))
+    {
+      stippleComboBoxAdd_pattern(STIPPLE_COMBOX(comboStipple), stipple);
+      stippleComboBoxSet_selectionByStipple(STIPPLE_COMBOX(comboStipple), stipple);
+    }
+  g_signal_handler_unblock(G_OBJECT(comboStipple), signalComboStippleId);
 }
 
-int changeWidthValue(PairsData *data, gpointer userData)
-{
-  return setWireWidth(data, (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(userData)));
-}
-
-
-void changeWireThickness(GtkSpinButton *spin, gpointer data)
+void changeWireThickness(GtkSpinButton *spin, gpointer data _U_)
 {
   int res;
+  GtkPairsIter iter;
+  gchar* label;
+
+  res = FALSE;
+  /* We run on all selected pairs. */
+  for (gtkPairsIter_startSelected(&iter); iter.data;
+       gtkPairsIter_nextSelected(&iter))
+    {
+      /* We change the value for this pair. */
+      res = setWireWidth(iter.data, (float)gtk_spin_button_get_value(spin)) || res;
+      label = gtkPairsWireGet_valuesAsLabel(iter.data);
+      /* We change the drawn label. */
+      gtkPairsSet_specificLabels(&(iter.iter), label);
+      g_free(label);
+    }
+
+  if (res)
+    gtkPairs_createPairs();
+}
+static void onStippleChanged(StippleComboBox *combo _U_, guint value, gpointer data _U_)
+{
+  int res;
+  GtkPairsIter iter;
+  gchar* label;
+
+  DBG_fprintf(stderr, "Gtk Wire: caught 'stipple-selected' signal.\n");
+  res = FALSE;
+  /* We run on all selected pairs. */
+  for (gtkPairsIter_startSelected(&iter); iter.data;
+       gtkPairsIter_nextSelected(&iter))
+    {
+      /* We change the value for this pair. */
+      res = wireSet_stipple(iter.data, (guint16)value) || res;
+      label = gtkPairsWireGet_valuesAsLabel(iter.data);
+      /* We change the drawn label. */
+      gtkPairsSet_specificLabels(&(iter.iter), label);
+      g_free(label);
+    }
 
-  res = gtkPairsSet_specificLabels(buildWireLabels, changeWidthValue, (gpointer)spin);
   if (res)
     gtkPairs_createPairs();
 }
 
-void applyChangesInCurves(GtkButton *button, gpointer data)
+void applyChangesInCurves(GtkButton *button _U_, gpointer data _U_)
 {
   if (getWireNonLinear())
     {
@@ -347,7 +391,7 @@ void applyChangesInCurves(GtkButton *button, gpointer data)
     }
 }
 
-void changeWireColorLength(GtkToggleButton *button, gpointer data)
+void changeWireColorLength(GtkToggleButton *button, gpointer data _U_)
 {
   int res;
   gboolean checked;
@@ -366,11 +410,7 @@ void changeColorForCurve(GtkToggleButton *button, gpointer data)
     return;
   
   color = GPOINTER_TO_INT(data);
-  if ( color < 0 || color > 2)
-    {
-      fprintf(stderr, "WARNING! Wrong parameters in call of changeColorForCurve.\n");
-      return;
-    }
+  g_return_if_fail(color >= 0 && color < 3);
 
   gtk_widget_hide(vboxCurve[colorForCurve]);
   gtk_widget_show(vboxCurve[color]);
@@ -385,7 +425,7 @@ void setValueForColorLength()
   int i;
   gfloat var_h, var_i, var_0, var_1, var_2, var_3;
 
-  setPairsAreOutOfDate();
+  visuPairSet_outOfDate();
   if (colorDescr == 0)
     {
       DBG_fprintf(stderr, "Gtk Wire : set color with RGB description.\n");
@@ -407,13 +447,7 @@ void setValueForColorLength()
 	  values[color] = (gfloat*)getLengthSampleForColor(color, nb + color);
 	  if (!values[color] || nb[color] <= 0)
 	    break;
-	  tmp[color] = (gfloat*)0;
-	  tmp[color] = malloc(sizeof(gfloat) * nb[color]);
-	  if (!tmp[color])
-	    {
-	      allocationProblems();
-	      exit(1);
-	    }
+	  tmp[color] = g_malloc(sizeof(gfloat) * nb[color]);
 	  gtk_curve_get_vector(GTK_CURVE(colorLengthCurve[color]), nb[color], tmp[color]);
 	}
       if (nb[0] != nb[1] || nb[1] != nb[2] || nb[2] != nb[0])
@@ -476,7 +510,7 @@ void setValueForColorLength()
 	}
       for (color = 0; color < 3; color++)
 	if (tmp[color])
-	  free(tmp[color]);
+	  g_free(tmp[color]);
     }
 }
 
@@ -488,11 +522,8 @@ void changeColorDescr(GtkToggleButton *button, gpointer data)
     return;
 
   val = GPOINTER_TO_INT(data);
-  if ( val != COLOUR_DESCR_RGB && val != COLOUR_DESCR_HSV)
-    {
-      fprintf(stderr, "WARNING! Wrong parameters in call of changeColorDescr.\n");
-      return;
-    }
+  g_return_if_fail(val == COLOUR_DESCR_RGB || val == COLOUR_DESCR_HSV);
+
   DBG_fprintf(stderr, "Gtk Wire : set color description to %d.\n", val);
   colorDescr = val;
   
@@ -510,28 +541,25 @@ void changeColorDescr(GtkToggleButton *button, gpointer data)
     }
 }
 
-static void wireValuesChangedOnResources(GObject *obj, gpointer data)
-{
-  if (!gtkPairs_isPairExtensionInUse(pointerToPairExtension_wire))
-    return;
+/* static void wireValuesChangedOnResources(GObject *obj, gpointer data) */
+/* { */
+/*   if (!gtkPairs_isPairExtensionInUse(pointerToPairExtension_wire)) */
+/*     return; */
 
-  DBG_fprintf(stderr, "Gtk Wire : catch the 'resourcesLoaded' signal, updating values.\n");
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinThickness), getWireGeneralWidth());
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkNonLinear), getWireNonLinear());
-}
+/*   DBG_fprintf(stderr, "Gtk Wire : catch the 'resourcesLoaded' signal, updating values.\n"); */
+/*   gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinThickness), getWireGeneralWidth()); */
+/*   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkNonLinear), getWireNonLinear()); */
+/* } */
 
 
-/* gboolean drawCoordinates(GtkWidget *widget, GdkEvent *event, gpointer user_data) */
+/* static gboolean drawCoordinates(GtkWidget *widget, GdkEventMotion *event, */
+/* 				gpointer user_data) */
 /* { */
 /*   int curve; */
 
 /*   curve = GPOINTER_TO_INT(user_data); */
-/*   if (curve < 0 || curve > 2) */
-/*     { */
-/*       fprintf(stderr, "WARNING! Wrong parameters in call of drawCoordinates.\n"); */
-/*       return FALSE; */
-/*     } */
+/*   g_return_val_if_fail(curve >= 0 && curve < 3, FALSE); */
 
-/*   fprintf(stderr, "%d\n", event->type); */
+/*   fprintf(stderr, "%gx%g\n", event->x, event->y); */
 /*   return FALSE; */
 /* } */
