@@ -51,6 +51,18 @@
 #include <gtk_main.h>
 #include <renderingBackend/visu_windowInterface.h>
 
+/**
+ * SECTION:gtkAtomic
+ * @short_description: The gtk interface elements that are specific to
+ * the atomic rendering method.
+ *
+ * <para>This part gathers all the routine specific to the widgets
+ * related to the atomic rendering method. For the method itself,
+ * there is no specific widgets. For the #VisuElement part, one can
+ * tune the shape, the radius and the elipsoid orientation (when
+ * selected). There is no specific #GtkFileChooser widget.</para>
+ */
+
 #define SPIN_ATOMIC_RADIUS_UPPER (gdouble)5.
 #define SPIN_ATOMIC_RADIUS_LOWER (gdouble)0.001
 #define SPIN_ATOMIC_RADIUS_STEP  (gdouble)0.05
@@ -77,13 +89,13 @@ enum
 /* Pointer on a list to all selected elements. */
 GList *currentListElement;
 
-GtkWidget* createAtomicSpecificOpenWidget(void);
-
 /* Callbacks */
 void paramChanged(GtkSpinButton* button, gpointer data);
 void shapeChanged(GtkComboBox *box, gpointer data);
 void onSpinBoundsChanged(GObject *obj, gfloat val, gpointer data);
 
+/* Local routines. */
+
 
 /***************/
 /* Public part */
@@ -91,38 +103,27 @@ void onSpinBoundsChanged(GObject *obj, gfloat val, gpointer data);
 
 /* Initialise the specific area in the element panel
    for the atomic rendering method. */
-void initAtomic_gtkPanel()
+void panelElementAtomicInit_gtkPanel()
 {
   panelElements_setInterfaceMethods(pointerOnRenderingAtomicMethod,
-				   onElementChange_atomicMethod,
-				   createGtkInterfaceForAtomicMethod);
+				    panelElementAtomicCallback_onElementChange,
+				    panelElementAtomicCreate_gtkInterface);
   currentListElement = (GList*)0;
 }
 
 /* Initialise the gtk methods associated with
    the atomic rendering method. */
-void initAtomic_gtkMain()
+void panelElementAtomicInit_gtkMain()
 {
-  gtkMainSet_renderingSpecificMethods(pointerOnRenderingAtomicMethod,
-				      createAtomicSpecificOpenWidget,
-				      (createGtkLoadWidgetFunc)0);
+  gtkMainClassSet_renderingSpecificMethods(pointerOnRenderingAtomicMethod,
+					   (GtkMainSetFilesFunc)0);
 }
 
-GtkWidget* createAtomicSpecificOpenWidget(void)
-{
-  GtkWidget *label;
-
-  label = gtk_label_new("Coucou");
-  gtk_widget_show(label);
-  return label;
-}
-
-
 
 
 /* Create the gtk widgets (a hbox with a spin with
    positive values) and return it. */
-GtkWidget* createGtkInterfaceForAtomicMethod()
+GtkWidget* panelElementAtomicCreate_gtkInterface()
 {
   GtkWidget* hbox, *vbox;
   GtkWidget* label;
@@ -177,7 +178,7 @@ GtkWidget* createGtkInterfaceForAtomicMethod()
   g_signal_connect((gpointer)spinRadius, "value-changed",
 		   G_CALLBACK(paramChanged), GINT_TO_POINTER(paramRadius));
 
-  g_signal_connect(G_OBJECT(visuGtkObject), "spinBoundsChanged",
+  g_signal_connect(G_OBJECT(gtkMainClassGet_currentPanel()), "spinBoundsChanged",
 		   G_CALLBACK(onSpinBoundsChanged), (gpointer)0);
 
   /* Set widgets for the elipsoid parameters. */
@@ -217,7 +218,7 @@ GtkWidget* createGtkInterfaceForAtomicMethod()
 }
 
 /* This function is called whenever an element is changed. */
-void onElementChange_atomicMethod(GList *eleList)
+void panelElementAtomicCallback_onElementChange(GList *eleList)
 {
   float radius, ratio, phi, theta;
   int shape;
@@ -281,10 +282,11 @@ void onElementChange_atomicMethod(GList *eleList)
 /* Private part */
 /****************/
 
-void onSpinBoundsChanged(GObject *obj, gfloat mult, gpointer data)
+void onSpinBoundsChanged(GObject *obj _U_, gfloat mult, gpointer data _U_)
 {
   float val;
-  int i, id;
+  unsigned int i;
+  int id;
   VisuData *dataObj;
 
   if (getRenderingMethodInUse() != pointerOnRenderingAtomicMethod)
@@ -316,7 +318,7 @@ void onSpinBoundsChanged(GObject *obj, gfloat mult, gpointer data)
       }
 }
    
-void shapeChanged(GtkComboBox *box, gpointer data)
+void shapeChanged(GtkComboBox *box, gpointer data _U_)
 {
   int shape;
   GList *tmpLst;
@@ -343,8 +345,7 @@ void shapeChanged(GtkComboBox *box, gpointer data)
       tmpLst = g_list_next(tmpLst);
     }
   if (refresh)
-    g_signal_emit(visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
-		  0 , NULL);
+    g_idle_add(visuObjectRedraw, (gpointer)0);
 }
 
 void paramChanged(GtkSpinButton* button, gpointer data)
@@ -396,6 +397,5 @@ void paramChanged(GtkSpinButton* button, gpointer data)
       tmpLst = g_list_next(tmpLst);
     }
   if (refresh)
-    g_signal_emit(visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
-		  0 , NULL);
+    g_idle_add(visuObjectRedraw, (gpointer)0);
 }
