@@ -45,6 +45,8 @@
 */
 #include "gtkSpin.h"
 
+#include <visu_gtk.h>
+#include <visu_elements.h>
 #include <renderingMethods/renderingSpin.h>
 #include <openGLFunctions/view.h>
 #include <coreTools/toolMatrix.h>
@@ -57,6 +59,20 @@
 #include <string.h>
 #include <math.h>
 
+/**
+ * SECTION:gtkSpin
+ * @short_description: The gtk interface elements that can interfere
+ * with renderingSpin parameters.
+ *
+ * <para>This is the gtk interface for all #RenderingSpin module
+ * parameters. They are split in two parts. The first part is placed
+ * under the config tab and allows the user to modify the "global
+ * properties" of the renderingSpin module such as cone's axe
+ * orientation, and color repartition. The second part is under the
+ * element panel and allows to modify each element resource. Size of
+ * the arrows, and their shape can be modified.</para>
+ */
+
 #define LABEL_ARROW_SIZE      _("<i>Shape size and color properties:</i>")
 #define LABEL_ARROW_SIZE_MULT _("<i>Shape size and color properties <span style=\"italic\" size=\"smaller\" foreground=\"red\">\303\227 mult.</span>:</i>")
 #define LABEL_SPIN_POLICY     _("Drawing policy for spins with null modulus:")
@@ -81,6 +97,7 @@ static GtkWidget *gtkw_x = NULL, *gtkw_y = NULL, *gtkw_z = NULL;
 static GtkWidget *gtkw_set_view;
 static GtkWidget *labelSizeProp;
 static GtkWidget *vboxArrowShape, *vboxElipsoidShape;
+static GtkWidget *radioPerTypeMod, *radioGlobalMod;
 
 static guint element_callback_ids[10];
 
@@ -95,18 +112,19 @@ struct FileChooserSpin_struct
   GtkLabel* label[2];
   GtkRadioButton* radio[2];
   gchar* files[2];
-  GtkFileChooser *fileChooser;
-  GtkDialog *dialog;
+  GtkFileChooserDialog *fileChooserDialog;
   int fileKind;
+
+  VisuGtkPreview preview;
 };
 /* A special widget to select two kind of files at a time. */
-int gtkSpinCreate_fileChooser(VisuData *data, GtkFileChooser **fileChooser,
-			      GtkWidget **fileWidget);
+static gboolean gtkSpinCreate_fileChooser(VisuData *data, GtkWindow *parent);
 /* Its callbacks. */
-void fileSelectedForSpinMethod(GtkButton *button, gpointer data);
-void fileActivatedForSpinMethod(GtkFileChooser *file, gpointer data);
-void fileKindChangedForPosition(GtkToggleButton *button, gpointer data);
-void fileKindChangedForSpin(GtkToggleButton *button, gpointer data);
+static void fileSelectedForSpinMethod(GtkButton *button, gpointer data);
+static void fileKindChangedForPosition(GtkToggleButton *button, gpointer data);
+static void fileKindChangedForSpin(GtkToggleButton *button, gpointer data);
+static void fileUpdatePreview(GtkFileChooser *chooser,
+			      struct FileChooserSpin_struct *fileChooserSpin);
 
 /* Callbacks */
 void global_resource_callback(GtkWidget* widget, gpointer data);
@@ -123,7 +141,7 @@ void destroyGtkConfigForSpinMethod();
 
 void redraw(int i);
 
-static void sync_global_resources(GObject *object, gpointer data);
+static void sync_global_resources(GObject *object, VisuData *dataObj, gpointer data);
 static void sync_local_resources(GObject *trash, gpointer data);
 
 
@@ -132,7 +150,7 @@ static void sync_local_resources(GObject *trash, gpointer data);
 /***************/
 
 /* See h file for more info */
-void initSpin_gtkPanel()
+void panelElementSpinInit_gtkPanel()
 {
   panelElements_setInterfaceMethods(pointerOnRenderingSpinMethod,
 				   changeSomethingSpin,
@@ -141,7 +159,7 @@ void initSpin_gtkPanel()
 }
 
 /* See h file for more info */
-void initSpin_gtkConfig()
+void panelElementSpinInit_gtkConfig()
 {
   panelMethodSet_renderingInterface(pointerOnRenderingSpinMethod,
 				    createGtkConfigForSpinMethod,
@@ -149,12 +167,18 @@ void initSpin_gtkConfig()
 }
 
 /* Creates the hbox containing x,y,z spin buttons */
-static GtkWidget *gtk_spin_create_direction(float *cartesian_values)
+static GtkWidget *gtk_spin_create_direction(float *cartesian_values,
+					    float *spherical_values)
 {
-  GtkWidget* hbox = gtk_hbox_new(FALSE, 0);
-  GtkWidget* label_x = gtk_label_new(_("x :"));
-  GtkWidget* label_y = gtk_label_new(_(" y :"));
-  GtkWidget* label_z = gtk_label_new(_(" z :"));
+  GtkWidget *table;
+  GtkWidget* label_x = gtk_label_new(_("x:"));
+  GtkWidget* label_y = gtk_label_new(_("y:"));
+  GtkWidget* label_z = gtk_label_new(_("z:"));
+  GtkWidget* label_theta = gtk_label_new("\316\270:");
+  GtkWidget* label_phi = gtk_label_new("\317\206:");
+  GtkTooltips *tooltips;
+
+  table = gtk_table_new(2, 6, FALSE);
 
   gtk_misc_set_alignment(GTK_MISC(label_x), 1., 0.5);
   gtk_misc_set_alignment(GTK_MISC(label_y), 1., 0.5);
@@ -175,46 +199,25 @@ static GtkWidget *gtk_spin_create_direction(float *cartesian_values)
   g_signal_connect(gtkw_z, "value-changed", 
 		   G_CALLBACK(global_resource_callback), GINT_TO_POINTER(-3));   
 
-  gtk_box_pack_start(GTK_BOX(hbox), label_x, TRUE, TRUE, 1);
-  gtk_box_pack_start(GTK_BOX(hbox), gtkw_x, FALSE, FALSE, 1);
-  gtk_box_pack_start(GTK_BOX(hbox), label_y, TRUE, TRUE, 1);
-  gtk_box_pack_start(GTK_BOX(hbox), gtkw_y, FALSE, FALSE, 1);
-  gtk_box_pack_start(GTK_BOX(hbox), label_z, TRUE, TRUE, 1);
-  gtk_box_pack_start(GTK_BOX(hbox), gtkw_z, FALSE, FALSE, 1);
+  gtk_table_attach(GTK_TABLE(table), label_x, 0, 1, 0, 1,
+		   GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 2);
+  gtk_table_attach(GTK_TABLE(table), label_y, 2, 3, 0, 1,
+		   GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 2);
+  gtk_table_attach(GTK_TABLE(table), label_z, 4, 5, 0, 1,
+		   GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 2);
 
-  return hbox;
-}
-
-/* Creates the hbox containing the color_wheel_angle button */
-static GtkWidget *gtk_spin_create_color_wheel(float color_wheel_angle)
-{
-  GtkWidget* hbox = gtk_hbox_new(FALSE, 0);
-  GtkWidget* label = gtk_label_new(_("Rotate color wheel:"));
-
-  gtk_misc_set_alignment(GTK_MISC(label), 0.,0.5);
-
-  gtkw_color_wheel_angle = gtk_spin_button_new_with_range(0, 360., 3);
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtkw_color_wheel_angle), color_wheel_angle);
-  g_signal_connect(gtkw_color_wheel_angle, "value-changed",
-		   G_CALLBACK(global_resource_callback), GINT_TO_POINTER(spin_globalColorWheel));
-  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(gtkw_color_wheel_angle), TRUE);
-
-  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 1);
-  gtk_box_pack_start(GTK_BOX(hbox), gtkw_color_wheel_angle, TRUE, TRUE, 1);
-
-  return hbox;
-}
-
-/* Creates the hbox containing the color cone parameters buttons */
-static GtkWidget *gtk_spin_create_rotate_axe(float *spherical_values)
-{
-  GtkWidget* hbox = gtk_hbox_new(FALSE, 0);
-  GtkWidget* label_theta = gtk_label_new("Theta :");
-  GtkWidget* label_phi = gtk_label_new(" Phi : ");
-  GtkTooltips *tooltips;
+  gtk_table_attach(GTK_TABLE(table), gtkw_x, 1, 2, 0, 1,
+		   GTK_EXPAND | GTK_FILL, GTK_SHRINK, 1, 2);
+  gtk_table_attach(GTK_TABLE(table), gtkw_y, 3, 4, 0, 1,
+		   GTK_EXPAND | GTK_FILL, GTK_SHRINK, 1, 2);
+  gtk_table_attach(GTK_TABLE(table), gtkw_z, 5, 6, 0, 1,
+		   GTK_EXPAND | GTK_FILL, GTK_SHRINK, 1, 2);
 
   tooltips = gtk_tooltips_new();
 
+  gtk_misc_set_alignment(GTK_MISC(label_theta), 1., 0.5);
+  gtk_misc_set_alignment(GTK_MISC(label_phi), 1., 0.5);
+
   gtkw_cone_theta_angle = gtk_spin_button_new_with_range(0, 180., 3);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtkw_cone_theta_angle), spherical_values[1]);
   g_signal_connect(gtkw_cone_theta_angle, "value-changed",
@@ -232,11 +235,42 @@ static GtkWidget *gtk_spin_create_rotate_axe(float *spherical_values)
   g_signal_connect(gtkw_set_view, "clicked",
 		   G_CALLBACK(set_view), NULL);
 
-  gtk_box_pack_start(GTK_BOX(hbox), label_theta, FALSE, FALSE, 1);
-  gtk_box_pack_start(GTK_BOX(hbox), gtkw_cone_theta_angle, FALSE, FALSE, 1);
-  gtk_box_pack_start(GTK_BOX(hbox), label_phi, FALSE, FALSE, 1);
-  gtk_box_pack_start(GTK_BOX(hbox), gtkw_cone_phi_angle, FALSE, FALSE, 1);
-  gtk_box_pack_start(GTK_BOX(hbox), gtkw_set_view, TRUE, FALSE, 1);
+  gtk_table_attach(GTK_TABLE(table), label_theta, 0, 1, 1, 2,
+		   GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 2);
+  gtk_table_attach(GTK_TABLE(table), label_phi,   2, 3, 1, 2,
+		   GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 2);
+
+  gtk_table_attach(GTK_TABLE(table), gtkw_cone_theta_angle, 1, 2, 1, 2,
+		   GTK_EXPAND | GTK_FILL, GTK_SHRINK, 1, 2);
+  gtk_table_attach(GTK_TABLE(table), gtkw_cone_phi_angle, 3, 4, 1, 2,
+		   GTK_EXPAND | GTK_FILL, GTK_SHRINK, 1, 2);
+
+  gtk_table_attach(GTK_TABLE(table), gtkw_set_view, 4, 6, 1, 2,
+		   GTK_EXPAND, GTK_SHRINK, 1, 2);
+
+  return table;
+}
+
+/* Creates the hbox containing the color_wheel_angle button */
+static GtkWidget *gtk_spin_create_color_wheel(float color_wheel_angle)
+{
+  GtkWidget* hbox = gtk_hbox_new(FALSE, 0);
+  GtkWidget* label = gtk_label_new(_("Rotate color wheel:"));
+
+  gtk_misc_set_alignment(GTK_MISC(label), 0.,0.5);
+
+  gtkw_color_wheel_angle = gtk_spin_button_new_with_range(0, 360., 3);
+  gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtkw_color_wheel_angle), color_wheel_angle);
+  g_signal_connect(gtkw_color_wheel_angle, "value-changed",
+		   G_CALLBACK(global_resource_callback), GINT_TO_POINTER(spin_globalColorWheel));
+  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(gtkw_color_wheel_angle), TRUE);
+
+  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 1);
+  gtk_box_pack_start(GTK_BOX(hbox), gtkw_color_wheel_angle, FALSE, FALSE, 1);
+
+  /* Degrees. */
+  label = gtk_label_new(_("deg."));
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 1);
 
   return hbox;
 }
@@ -303,6 +337,8 @@ GtkWidget* createGtkConfigForSpinMethod()
   float spherical[3];
   GtkWidget *labelPolicy;
   GtkWidget *hbox, *check, *label, *align;
+  GSList *radiobutton_group;
+  SpinModulusPolicy modPol;
 
   DBG_fprintf(stderr, "Gtk Spin : building specific spin rendering method config widget.\n");
 
@@ -313,6 +349,7 @@ GtkWidget* createGtkConfigForSpinMethod()
   matrix_sphericalToCartesian(cartesian, spherical);
 
   /* Use modulus. */
+  modPol = rspin_getGlobalResource_uint(spin_globalModulus);
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   label = gtk_label_new(_("Spin lenght is proportional to modulus: "));
@@ -322,9 +359,34 @@ GtkWidget* createGtkConfigForSpinMethod()
   check = gtk_check_button_new();
   gtk_box_pack_start(GTK_BOX(hbox), check, FALSE, FALSE, 3);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),
-			       rspin_getGlobalResource_boolean(spin_globalModulus));
+			       (modPol != policyNoneModulus));
   g_signal_connect(G_OBJECT(check), "toggled",
-		   G_CALLBACK(onPolicyModulusChanged), (gpointer)0);
+		   G_CALLBACK(onPolicyModulusChanged),
+		   GINT_TO_POINTER(policyNoneModulus));
+
+  hbox = gtk_hbox_new(FALSE, 0);
+  radioPerTypeMod = gtk_radio_button_new_with_label(NULL, _("per node type"));
+  gtk_radio_button_set_group(GTK_RADIO_BUTTON(radioPerTypeMod), (GSList*)0);
+  radiobutton_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radioPerTypeMod));
+  gtk_box_pack_start(GTK_BOX(hbox), radioPerTypeMod, TRUE, TRUE, 1);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radioPerTypeMod),
+			       modPol == policyPerTypeModulus);
+  gtk_widget_set_sensitive(radioPerTypeMod, (modPol != policyNoneModulus));
+  g_signal_connect(G_OBJECT(radioPerTypeMod), "toggled",
+		   G_CALLBACK(onPolicyModulusChanged),
+		   GINT_TO_POINTER(policyPerTypeModulus));
+  radioGlobalMod = gtk_radio_button_new_with_label(NULL, _("globaly"));
+  gtk_radio_button_set_group(GTK_RADIO_BUTTON(radioGlobalMod), radiobutton_group);
+  radiobutton_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radioGlobalMod));
+  gtk_box_pack_start(GTK_BOX(hbox), radioGlobalMod, TRUE, TRUE, 1);
+  g_signal_connect(G_OBJECT(radioGlobalMod), "toggled",
+		   G_CALLBACK(onPolicyModulusChanged),
+		   GINT_TO_POINTER(policyGlobalModulus));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radioGlobalMod),
+			       modPol == policyGlobalModulus);
+  gtk_widget_set_sensitive(radioGlobalMod, (modPol != policyNoneModulus));
+  gtk_widget_show_all(hbox);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
   /* Drawing atomic. */
   hbox = gtk_hbox_new(FALSE, 0);
@@ -351,7 +413,7 @@ GtkWidget* createGtkConfigForSpinMethod()
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
   label = gtk_label_new("");
-  gtk_label_set_markup(GTK_LABEL(label), _("<i>Color distribution options</i>"));
+  gtk_label_set_markup(GTK_LABEL(label), _("Color distribution options:"));
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
   gtk_misc_set_padding(GTK_MISC(label), 3, 0);
   gtk_widget_set_name(label, "label_head_2");
@@ -367,8 +429,7 @@ GtkWidget* createGtkConfigForSpinMethod()
 
   gtk_box_pack_start(GTK_BOX(vbox), label2, FALSE, FALSE, 3);
   gtk_misc_set_alignment(GTK_MISC(label2), 0., 0.5);
-  gtk_box_pack_start(GTK_BOX(vbox), gtk_spin_create_direction(cartesian), FALSE, FALSE, 3);
-  gtk_box_pack_start(GTK_BOX(vbox), gtk_spin_create_rotate_axe(spherical), FALSE, FALSE, 3);
+  gtk_box_pack_start(GTK_BOX(vbox), gtk_spin_create_direction(cartesian, spherical), FALSE, FALSE, 3);
 
   gtk_widget_show_all(vbox);
 
@@ -390,7 +451,7 @@ static void onPolicyChanged(GtkToggleButton *toggle, gpointer data)
   if (rspin_setGlobalResource_uint(spin_globalHidingMode, (guint)GPOINTER_TO_INT(data)))
     redraw(2);
 }
-static void onPolicyAtomicChanged(GtkToggleButton *toggle, gpointer data)
+static void onPolicyAtomicChanged(GtkToggleButton *toggle, gpointer data _U_)
 {
   VisuData *dataObj;
 
@@ -408,9 +469,24 @@ static void onPolicyAtomicChanged(GtkToggleButton *toggle, gpointer data)
 static void onPolicyModulusChanged(GtkToggleButton *toggle, gpointer data)
 {
   VisuData *dataObj;
+  SpinModulusPolicy modPol;
 
-  if (rspin_setGlobalResource_boolean(spin_globalModulus,
-				      gtk_toggle_button_get_active(toggle)))
+  if ((SpinModulusPolicy)data == policyNoneModulus)
+    {
+      if (!gtk_toggle_button_get_active(toggle))
+	modPol = policyNoneModulus;
+      else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radioPerTypeMod)))
+	modPol = policyPerTypeModulus;
+      else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radioGlobalMod)))
+	modPol = policyGlobalModulus;
+      else
+	modPol = policyNoneModulus;
+    }
+  else
+    modPol = (SpinModulusPolicy)data;
+  gtk_widget_set_sensitive(radioGlobalMod, gtk_toggle_button_get_active(toggle));
+  gtk_widget_set_sensitive(radioPerTypeMod, gtk_toggle_button_get_active(toggle));
+  if (rspin_setGlobalResource_uint(spin_globalModulus, modPol))
     {
       dataObj = toolPanelGet_visuData(TOOL_PANEL(panelElements));
       if (!dataObj)
@@ -423,21 +499,19 @@ static void onPolicyModulusChanged(GtkToggleButton *toggle, gpointer data)
 
 /* Initialise the gtk methods associated with
    the spin rendering method. */
-void initSpin_gtkMain()
+void panelElementSpinInit_gtkMain()
 {
-  gtkMainSet_renderingSpecificMethods(pointerOnRenderingSpinMethod,
-				      (createGtkWidgetFunc)0,
-				      gtkSpinCreate_fileChooser);
+  gtkMainClassSet_renderingSpecificMethods(pointerOnRenderingSpinMethod,
+					   gtkSpinCreate_fileChooser);
 }
 
 
 /* Creates a custom file chooser which allows the user to select a spin and a position file */
-int gtkSpinCreate_fileChooser(VisuData *data, GtkFileChooser **fileChooser,
-			      GtkWidget **fileWidget)
+static gboolean gtkSpinCreate_fileChooser(VisuData *data, GtkWindow *parent)
 {
   GtkWidget *fileSelection;
   GList *tmpLst, *tmpLst2;
-  GtkWidget *dialog;
+  GtkWidget *vbox;
   gchar* directory;
   GtkFileFilter *filterPos, *filterSpin, *filterAll;
   GtkWidget *table, *hbox, *align;
@@ -445,27 +519,26 @@ int gtkSpinCreate_fileChooser(VisuData *data, GtkFileChooser **fileChooser,
   GtkWidget *labelPosition, *labelSpin;
   GtkWidget *radioPosition, *radioSpin;
   GtkWidget *hiding_mode;
-
   struct FileChooserSpin_struct fileChooserSpin;
-  gint returnVal;
+  gint status;
 
 
   if (!data)
     return 0;
 
-  dialog = gtk_dialog_new_with_buttons(_("Load session"),
-				       GTK_WINDOW(mainWindow),
-				       GTK_DIALOG_MODAL,
-				       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				       GTK_STOCK_OPEN, GTK_RESPONSE_OK,
-				       NULL);
-  *fileWidget = dialog;
+  fileSelection = gtk_file_chooser_dialog_new(_("Load session"),
+					      GTK_WINDOW(parent),
+					      GTK_FILE_CHOOSER_ACTION_OPEN,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+					      NULL);
+  directory = gtkMainClassGet_lastOpenDirectory();
+  if (directory)
+    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(fileSelection), directory);
 
-  label = gtk_label_new(_("Select a new file:"));
-  gtk_widget_set_name(label, "label_head_2");
-  gtk_misc_set_alignment(GTK_MISC(label), 0., 0.);
-  gtk_widget_show(label);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, FALSE, FALSE, 2);
+  gtk_widget_set_name(fileSelection, "filesel");
+  gtk_window_set_position(GTK_WINDOW(fileSelection), GTK_WIN_POS_CENTER_ON_PARENT);
+  gtk_window_set_modal(GTK_WINDOW(fileSelection), TRUE);
 
   tmpLst = renderingMethodGet_fileType(pointerOnRenderingSpinMethod, FILE_KIND_POSITION);
   filterPos = gtk_file_filter_new ();
@@ -502,94 +575,88 @@ int gtkSpinCreate_fileChooser(VisuData *data, GtkFileChooser **fileChooser,
   gtk_file_filter_add_pattern (filterAll, "*");
   gtk_file_filter_set_name(filterAll, _("All files"));
   
-  fileSelection = gtk_file_chooser_widget_new(GTK_FILE_CHOOSER_ACTION_OPEN);
-  *fileChooser = GTK_FILE_CHOOSER(fileSelection);
-  gtk_widget_set_size_request(fileSelection, 650, 300);
-  gtk_widget_show(fileSelection);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), fileSelection, TRUE, TRUE, 2);
-  g_signal_connect(G_OBJECT(fileSelection), "file-activated",
-		   G_CALLBACK(fileSelectedForSpinMethod), (gpointer)&fileChooserSpin);
+/*   g_signal_connect(G_OBJECT(fileSelection), "file-activated", */
+/* 		   G_CALLBACK(fileSelectedForSpinMethod), (gpointer)&fileChooserSpin); */
 
-  directory = getLastOpenDirectory();
-  if (directory)
-    gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(fileSelection), directory);
-  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fileSelection),
-			      filterPos);
-  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fileSelection),
-			      filterSpin);
-  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fileSelection),
-			      filterAll);
+  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fileSelection), filterPos);
+  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fileSelection), filterSpin);
+  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fileSelection), filterAll);
   gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(fileSelection), filterPos);
 
-  label = gtk_label_new(_("Current files :"));
+  /* The preview widget. */
+  visuGtkPreview_add(&fileChooserSpin.preview, GTK_FILE_CHOOSER(fileSelection));
+  g_signal_connect(GTK_FILE_CHOOSER(fileSelection), "update-preview",
+		   G_CALLBACK(fileUpdatePreview), &fileChooserSpin);
+
+  /* The additional widgets. */
+  vbox = gtk_vbox_new(FALSE, 0);
+  gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(fileSelection), vbox);
+  
+  label = gtk_label_new(_("Current files:"));
   gtk_widget_set_name(label, "label_head_2");
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.);
-  gtk_widget_show(label);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, FALSE, FALSE, 2);
+  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 2);
 
   hbox = gtk_hbox_new(FALSE, 0);
-  gtk_widget_show(hbox);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 2);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
 
   /* The table for the two file types selections (spin & position). */
   table = gtk_table_new(2, 3, FALSE);
-  gtk_widget_show(table);
   gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 2);
   
   /* Position button. */
   radioPosition = gtk_radio_button_new_with_label(NULL, _("position"));
-  gtk_widget_show(radioPosition);
-  gtk_table_attach(GTK_TABLE(table), radioPosition, 0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 2, 0);
+  gtk_table_attach(GTK_TABLE(table), radioPosition, 0, 1, 0, 1,
+		   GTK_SHRINK, GTK_SHRINK, 2, 0);
   g_signal_connect(G_OBJECT(radioPosition), "toggled",
-		   G_CALLBACK(fileKindChangedForPosition), (gpointer)&fileChooserSpin);
+		   G_CALLBACK(fileKindChangedForPosition),
+		   (gpointer)&fileChooserSpin);
 
   /* Spin button */
-  radioSpin = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radioPosition),
-							  _("spin"));
-  gtk_widget_show(radioSpin);
-  gtk_table_attach(GTK_TABLE(table), radioSpin, 0, 1, 1, 2, GTK_SHRINK, GTK_SHRINK, 2, 0);
+  radioSpin = gtk_radio_button_new_with_label_from_widget
+    (GTK_RADIO_BUTTON(radioPosition), _("spin"));
+  gtk_table_attach(GTK_TABLE(table), radioSpin, 0, 1, 1, 2,
+		   GTK_SHRINK, GTK_SHRINK, 2, 0);
   g_signal_connect(G_OBJECT(radioSpin), "toggled",
-		   G_CALLBACK(fileKindChangedForSpin), (gpointer)&fileChooserSpin);
+		   G_CALLBACK(fileKindChangedForSpin),
+		   (gpointer)&fileChooserSpin);
 
   /* Layout labels */
   label = gtk_label_new(":");
-  gtk_widget_show(label);
   gtk_table_attach(GTK_TABLE(table), label, 1, 2, 0, 1, GTK_SHRINK, GTK_SHRINK, 5, 0);
 
   label = gtk_label_new(":");
-  gtk_widget_show(label);
   gtk_table_attach(GTK_TABLE(table), label, 1, 2, 1, 2, GTK_SHRINK, GTK_SHRINK, 5, 0);
 
   labelPosition = gtk_label_new(_("None"));
   gtk_misc_set_alignment(GTK_MISC(labelPosition), 0., 0.5);
-  gtk_widget_show(labelPosition);
-  gtk_table_attach(GTK_TABLE(table), labelPosition, 2, 3, 0, 1, GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
+  gtk_table_attach(GTK_TABLE(table), labelPosition, 2, 3, 0, 1,
+		   GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
 
   labelSpin = gtk_label_new(_("None"));
   gtk_misc_set_alignment(GTK_MISC(labelSpin), 0., 0.5);
-  gtk_widget_show(labelSpin);
-  gtk_table_attach(GTK_TABLE(table), labelSpin, 2, 3, 1, 2, GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
+  gtk_table_attach(GTK_TABLE(table), labelSpin, 2, 3, 1, 2,
+		   GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
 
   /* Right aligned apply button. */
   align = gtk_alignment_new(0.5, 1.0, 0., 0.);
-  gtk_widget_show(align);
   gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 2);
   button = gtk_button_new_from_stock(GTK_STOCK_APPLY);
-  gtk_widget_show(button);
   gtk_container_add(GTK_CONTAINER(align), button);
   g_signal_connect(G_OBJECT(button), "clicked",
-		   G_CALLBACK(fileSelectedForSpinMethod), (gpointer)&fileChooserSpin);
+		   G_CALLBACK(fileSelectedForSpinMethod),
+		   (gpointer)&fileChooserSpin);
 
   /* Hiding mode button */
   hbox = gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, FALSE, FALSE, 2);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
   label = gtk_label_new(LABEL_SPIN_POLICY);
   gtk_widget_set_name(label, "label_head_2");
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
   hiding_mode = createHidingModeRadioWidgets();
-  gtk_widget_show(hiding_mode);
   gtk_box_pack_start(GTK_BOX(hbox), hiding_mode, FALSE, FALSE, 2);
-  gtk_widget_show_all(hbox);
+
+  gtk_widget_show_all(vbox);
   
 /*   frame = gtk_frame_new(NULL); */
 /*   gtk_widget_show (frame); */
@@ -612,60 +679,57 @@ int gtkSpinCreate_fileChooser(VisuData *data, GtkFileChooser **fileChooser,
 /*   gtk_widget_show(label); */
 /*   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0); */
   
-
-  gtk_widget_set_name(dialog, "filesel");
-  gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
-
   fileChooserSpin.label[FILE_KIND_POSITION] = GTK_LABEL(labelPosition);
   fileChooserSpin.label[FILE_KIND_SPIN] = GTK_LABEL(labelSpin);
   fileChooserSpin.radio[FILE_KIND_POSITION] = GTK_RADIO_BUTTON(radioPosition);
   fileChooserSpin.radio[FILE_KIND_SPIN] = GTK_RADIO_BUTTON(radioSpin);
-  fileChooserSpin.dialog = GTK_DIALOG(dialog);
-  fileChooserSpin.fileChooser = GTK_FILE_CHOOSER(fileSelection);
+  fileChooserSpin.fileChooserDialog = GTK_FILE_CHOOSER_DIALOG(fileSelection);
   fileChooserSpin.fileKind = FILE_KIND_POSITION;
   fileChooserSpin.files[FILE_KIND_POSITION] = (gchar*)0;
   fileChooserSpin.files[FILE_KIND_SPIN] = (gchar*)0;
-  
 
   do
     {
-      returnVal = gtk_dialog_run(GTK_DIALOG(dialog));
-      if (returnVal == GTK_RESPONSE_OK)
+      status = gtk_dialog_run(GTK_DIALOG(fileSelection));
+      if (status != GTK_RESPONSE_OK)
 	{
-	  fileSelectedForSpinMethod((GtkButton*)0, (gpointer)&fileChooserSpin);
-	  if (!fileChooserSpin.files[FILE_KIND_POSITION] || !fileChooserSpin.files[FILE_KIND_SPIN])
-	    {
-	      raiseAlertDialog(_("One position AND one spin file have to be chosen."));
-	      returnVal = 0;
-	    }
-	  else
-	    returnVal = 1;
+	  if (fileChooserSpin.files[FILE_KIND_POSITION])
+	    g_free(fileChooserSpin.files[FILE_KIND_POSITION]);
+	  if (fileChooserSpin.files[FILE_KIND_SPIN])
+	    g_free(fileChooserSpin.files[FILE_KIND_SPIN]);
+
+	  gtk_widget_destroy(fileSelection);
+	  
+	  return FALSE;
 	}
       else
-	returnVal = -1;
-    }
-  while (returnVal == 0);
+	{
+	  fileSelectedForSpinMethod((GtkButton*)0, &fileChooserSpin);
+	  if (fileChooserSpin.files[FILE_KIND_POSITION] &&
+	      fileChooserSpin.files[FILE_KIND_SPIN])
+	    {
+	      visuDataAdd_file(data, fileChooserSpin.files[FILE_KIND_POSITION],
+			       FILE_KIND_POSITION, (FileFormat*)0);
+	      visuDataAdd_file(data, fileChooserSpin.files[FILE_KIND_SPIN],
+			       FILE_KIND_SPIN, (FileFormat*)0);
+	      g_free(fileChooserSpin.files[FILE_KIND_POSITION]);
+	      g_free(fileChooserSpin.files[FILE_KIND_SPIN]);
 
-  if (returnVal > 0)
-    {
-      visuDataAdd_file(data, fileChooserSpin.files[FILE_KIND_POSITION],
-		       FILE_KIND_POSITION, (FileFormat*)0);
-      visuDataAdd_file(data, fileChooserSpin.files[FILE_KIND_SPIN],
-		       FILE_KIND_SPIN, (FileFormat*)0);
-      g_free(fileChooserSpin.files[FILE_KIND_POSITION]);
-      g_free(fileChooserSpin.files[FILE_KIND_SPIN]);
-      return 1;
-    }
-  else
-    return 0;
+	      directory = gtk_file_chooser_get_current_folder
+		(GTK_FILE_CHOOSER(fileSelection));
+	      gtkMainClassSet_lastOpenDirectory((char*)directory);
+	      g_free(directory);
+	      
+	      gtk_widget_destroy(fileSelection);
+	  
+	      return TRUE;
+	    }
+	}
+    } while (TRUE);
+  return FALSE;
 }
 
-void fileActivatedForSpinMethod(GtkFileChooser *file, gpointer data)
-{
-  fileSelectedForSpinMethod((GtkButton*)0, data);
-}
-
-void fileSelectedForSpinMethod(GtkButton *button, gpointer data)
+static void fileSelectedForSpinMethod(GtkButton *button _U_, gpointer data)
 {
   gchar *filename = NULL, *filenameUTF8;
   struct FileChooserSpin_struct *values;
@@ -673,13 +737,16 @@ void fileSelectedForSpinMethod(GtkButton *button, gpointer data)
   GString *message;
 
   values = (struct FileChooserSpin_struct*)data;
-  filename = gtk_file_chooser_get_filename(values->fileChooser);
+  filename = gtk_file_chooser_get_filename
+    (GTK_FILE_CHOOSER(values->fileChooserDialog));
   if (!g_file_test(filename, G_FILE_TEST_IS_DIR) && filename != NULL)
     {
       DBG_fprintf(stderr, "Gtk Spin : select '%s' as file %d\n", filename, values->fileKind);
       res = 0;
       values->files[values->fileKind] = filename;
-      filenameUTF8 = getStringInUTF8(filename);
+      filenameUTF8 = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
+      g_return_if_fail(filenameUTF8);
+
       message = g_string_new("");
       if (g_utf8_strlen(filenameUTF8, 51) > 50)
 	g_string_printf(message, "(...)%s",
@@ -696,11 +763,39 @@ void fileSelectedForSpinMethod(GtkButton *button, gpointer data)
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(values->radio[FILE_KIND_POSITION]), TRUE);
       
       if (values->files[FILE_KIND_POSITION] && values->files[FILE_KIND_SPIN])
-	gtk_dialog_response(values->dialog, GTK_RESPONSE_OK);
+	gtk_dialog_response(GTK_DIALOG(values->fileChooserDialog), GTK_RESPONSE_OK);
     }
 }
 
-void fileKindChangedForPosition(GtkToggleButton *button, gpointer data)
+static void fileUpdatePreview(GtkFileChooser *chooser,
+			      struct FileChooserSpin_struct *fileChooserSpin)
+{
+  const char *filenames[3];
+
+  if (fileChooserSpin->files[FILE_KIND_POSITION])
+    filenames[0] = g_strdup(fileChooserSpin->files[FILE_KIND_POSITION]);
+  else
+    filenames[0] = gtk_file_chooser_get_preview_filename(chooser);
+  if (fileChooserSpin->files[FILE_KIND_POSITION] &&
+      !fileChooserSpin->files[FILE_KIND_SPIN])
+    filenames[1] = gtk_file_chooser_get_preview_filename(chooser);
+  else if (fileChooserSpin->files[FILE_KIND_SPIN])
+    filenames[1] = g_strdup(fileChooserSpin->files[FILE_KIND_SPIN]);
+  else
+    filenames[1] = (char*)0;
+  filenames[2] = (char*)0;
+
+  /* We test if the selected filename is a directory or not. */
+  if (filenames[0] && !g_file_test(filenames[0], G_FILE_TEST_IS_DIR) &&
+      filenames[1] && !g_file_test(filenames[1], G_FILE_TEST_IS_DIR))
+    visuGtkPreview_update(&fileChooserSpin->preview, filenames);
+  if (filenames[0])
+    g_free((gchar*)filenames[0]);
+  if (filenames[1])
+    g_free((gchar*)filenames[1]);
+}
+
+static void fileKindChangedForPosition(GtkToggleButton *button, gpointer data)
 {
   struct FileChooserSpin_struct* param = data;
   GSList *tmpLst;
@@ -708,13 +803,14 @@ void fileKindChangedForPosition(GtkToggleButton *button, gpointer data)
   if (!gtk_toggle_button_get_active(button))
     return;
 
-  tmpLst = gtk_file_chooser_list_filters(param->fileChooser);
-  gtk_file_chooser_set_filter(param->fileChooser, (GtkFileFilter*)(tmpLst->data));
+  tmpLst = gtk_file_chooser_list_filters(GTK_FILE_CHOOSER(param->fileChooserDialog));
+  gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(param->fileChooserDialog),
+			      (GtkFileFilter*)(tmpLst->data));
   param->fileKind = FILE_KIND_POSITION;
 
 }
 
-void fileKindChangedForSpin(GtkToggleButton *button, gpointer data)
+static void fileKindChangedForSpin(GtkToggleButton *button, gpointer data)
 {
   struct FileChooserSpin_struct* param = data;
   GSList *tmpLst;
@@ -722,8 +818,9 @@ void fileKindChangedForSpin(GtkToggleButton *button, gpointer data)
   if (!gtk_toggle_button_get_active(button))
     return;
 
-  tmpLst = gtk_file_chooser_list_filters(param->fileChooser);
-  gtk_file_chooser_set_filter(param->fileChooser, (GtkFileFilter*)(tmpLst->next->data));
+  tmpLst = gtk_file_chooser_list_filters(GTK_FILE_CHOOSER(param->fileChooserDialog));
+  gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(param->fileChooserDialog),
+			      (GtkFileFilter*)(tmpLst->next->data));
   param->fileKind = FILE_KIND_SPIN;
 }
 
@@ -771,19 +868,19 @@ GtkWidget* createGtkInterfaceForSpinMethod()
   gtk_widget_show(labelSizeProp);
 
   /* Size labels. */
-  hLabel = gtk_label_new(_("Hat length : "));
+  hLabel = gtk_label_new(_("Hat length:"));
   gtk_misc_set_alignment(GTK_MISC(hLabel), 1.0, 0.5);
   gtk_widget_show(hLabel);
 
-  rhLabel = gtk_label_new(_("Tail length : "));
+  rhLabel = gtk_label_new(_("Tail length:"));
   gtk_misc_set_alignment(GTK_MISC(rhLabel), 1.0, 0.5);
   gtk_widget_show(rhLabel);
 
-  lLabel = gtk_label_new(_("Hat radius : "));
+  lLabel = gtk_label_new(_("Hat radius:"));
   gtk_misc_set_alignment(GTK_MISC(lLabel), 1.0, 0.5);
   gtk_widget_show(lLabel);
 
-  rlLabel = gtk_label_new(_("Tail radius : "));
+  rlLabel = gtk_label_new(_("Tail radius:"));
   gtk_misc_set_alignment(GTK_MISC(rlLabel), 1.0, 0.5);
   gtk_widget_show(rlLabel);
 
@@ -814,7 +911,7 @@ GtkWidget* createGtkInterfaceForSpinMethod()
   gtk_widget_show(rlengthResourceSpin);
 
   /* This callback is for multiplier */
-  g_signal_connect(G_OBJECT(visuGtkObject), "spinBoundsChanged",
+  g_signal_connect(G_OBJECT(gtkMainClassGet_currentPanel()), "spinBoundsChanged",
 		   G_CALLBACK(onSpinBoundsChangeArrowSize), (gpointer)0);
 
 
@@ -874,7 +971,7 @@ GtkWidget* createGtkInterfaceForSpinMethod()
   expand = gtk_expander_new(_("Atomic rendering options"));
   gtk_expander_set_expanded(GTK_EXPANDER(expand), FALSE);
   gtk_widget_show(expand);
-  vboxAtomic = createGtkInterfaceForAtomicMethod();
+  vboxAtomic = panelElementAtomicCreate_gtkInterface();
   gtk_container_add(GTK_CONTAINER(expand), vboxAtomic);
 
   /* Displaying every widget created so far */
@@ -988,14 +1085,14 @@ void changeSomethingSpin(GList *eleList)
   g_signal_handler_unblock (useElementForElipsoid, element_callback_ids[9] );  
 
   DBG_fprintf(stderr, "Gtk Spin : give the element change to the atomic part.\n");
-  onElementChange_atomicMethod(eleList);
+  panelElementAtomicCallback_onElementChange(eleList);
 
   need_redraw = 1;
 }
 
-void onSpinBoundsChangeArrowSize(GObject *obj, gfloat mult, gpointer data)
+void onSpinBoundsChangeArrowSize(GObject *obj _U_, gfloat mult, gpointer data _U_)
 {
-  int i, j;
+  unsigned int i, j;
   float *val;
   GType type;
   SpinElementResources props[6] = {spin_elementHatLength, spin_elementHatRadius,
@@ -1181,7 +1278,7 @@ void global_resource_callback(GtkWidget* button, gpointer data)
     redraw(2);
 }
 
-void set_view(GtkWidget* button, gpointer data)
+void set_view(GtkWidget* button _U_, gpointer data _U_)
 {
   float theta;
   float phi;
@@ -1242,11 +1339,11 @@ void redraw(int i)
   if (i == 2)
     g_signal_emit (visu, VISU_GET_CLASS (visu)->SpinAxes_signal_id,
 		   0 , dataObj, NULL);
-  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
-		 0 , NULL); 
+  g_idle_add(visuObjectRedraw, (gpointer)0);
 }
 
-static void sync_global_resources(GObject *object, gpointer data)
+static void sync_global_resources(GObject *object _U_, VisuData *dataObj _U_,
+				  gpointer data _U_)
 {
   float spherical[3];
   float cartesian[3];
@@ -1281,7 +1378,7 @@ static void sync_global_resources(GObject *object, gpointer data)
   need_redraw = 1;  
 }      
 
-static void sync_local_resources(GObject *trash, gpointer data)
+static void sync_local_resources(GObject *trash _U_, gpointer data _U_)
 {
   VisuElement *ele;
 
