@@ -42,20 +42,32 @@
 	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
 */
 #include "panelElements.h"
-#include "externalRenderingMethods_gtk.h"
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <support.h>
 #include <visu_object.h>
 #include <visu_tools.h>
 #include <visu_data.h>
+#include <visu_elements.h>
 #include <extraGtkFunctions/gtk_colorComboBoxWidget.h>
 #include <extraGtkFunctions/gtk_toolPanelWidget.h>
 #include <coreTools/toolColor.h>
 #include <openGLFunctions/light.h>
 
+#include "gtkAtomic.h"
+#include "gtkSpin.h"
+
+/* Functions used to initialise the
+ * specific area in the panel of elements.
+ */
+static VisuToolsInitFunc listInitRendenringGtkPanelFunc[] = {
+  panelElementAtomicInit_gtkPanel,
+  panelElementSpinInit_gtkPanel,
+  (VisuToolsInitFunc)0};
+
 GtkListStore *listStoreVisuElements;
 enum
   {
@@ -67,7 +79,8 @@ enum
 GtkWidget *vBoxElements;
 GtkWidget *renderingMethodElements;
 GtkWidget *elementsCombo;
-GtkWidget *checkRendered;
+static GtkWidget *checkRendered;
+static GtkWidget *checkMasked;
 GtkWidget *widgetVisuElementColor;
 gint sortVisuElements(gpointer a, gpointer b);
 char *getLabelOfElement(VisuElement *element);
@@ -105,23 +118,25 @@ void getRenderingInterface(RenderingMethod *method,
 
 /* Callbacks */
 guint idRefreshComboElements;
-void comboVisuElementsChanged(GtkComboBox *combobox, gpointer data);
-static void resourcesChanged(GObject *object, gpointer data);
+static void comboVisuElementsChanged(GtkComboBox *combobox, gpointer data);
+static void resourcesChanged(GObject *object, VisuData *dataObj, gpointer data);
 void colorVisuElementChanged(ColorComboBox *colorWd, Color *color, gpointer userData);
-void checkRenderedChanged(GtkToggleButton *button, gpointer data);
+static void checkRenderedChanged(GtkToggleButton *button, gpointer data);
+static void checkMaskedChanged(GtkToggleButton *button, gpointer data);
 void refreshComboElements(GObject *obj, VisuData *dataObj, gpointer userData);
-void renderingMethodChanged(GObject *obj, gpointer userData);
+static void onRenderingMethodChanged(GObject *obj, RenderingMethod *method,
+				     gpointer userData);
 static void onColorChanged(ColorComboBox *colorComboBox, guint colorId, gpointer data);
 static void onMaterialChanged(ColorComboBox *colorComboBox, Material mat, gpointer data);
 
-ToolPanel* initPanelElements()
+ToolPanel* panelElementsInit()
 {
   gchar *cl = _("Set elements caracteristics");
   gchar *tl = _("Elements");
-  gchar *ipn = "stock_palette_20.png";
   int i;
 
-  panelElements = toolPanelNew_withIconFromPath("Panel_elements", cl, tl, ipn);
+  panelElements = toolPanelNew_withIconFromPath("Panel_elements", cl, tl,
+						"stock-elements_20.png");
   createInteriorElements(panelElements);
   toolPanelSet_dockable(TOOL_PANEL(panelElements), TRUE);
 
@@ -136,11 +151,8 @@ ToolPanel* initPanelElements()
   listOfRenderingInterfaces = g_hash_table_new_full(g_direct_hash,
 						    g_direct_equal,
 						    NULL, NULL);
-  if (!listOfRenderingInterfaces)
-    {
-      allocationProblems();
-      exit(1);
-    }
+  g_return_val_if_fail(listOfRenderingInterfaces, (ToolPanel*)0);
+
   for (i = 0; listInitRendenringGtkPanelFunc[i]; i++)
     listInitRendenringGtkPanelFunc[i]();
 
@@ -149,22 +161,24 @@ ToolPanel* initPanelElements()
 
 void createInteriorElements(GtkWidget *toolPanel)
 {
-  GtkWidget *label, *expand;
+  GtkWidget *label, *expand, *image;
   GtkWidget *hbox;
   GtkWidget *align;
   GtkWidget *scrollView;
   GtkCellRenderer *renderer;
+  GtkTooltips *tooltips;
+
+  tooltips = gtk_tooltips_new ();
 
   /* We create the structure that store the VisuElements */
   listStoreVisuElements = gtk_list_store_new (N_COLUMNS,
 					      G_TYPE_STRING,
-					      G_TYPE_POINTER,
 					      G_TYPE_POINTER);
   
   scrollView = gtk_scrolled_window_new((GtkAdjustment*)0,
 				       (GtkAdjustment*)0);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollView),
-				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrollView), GTK_SHADOW_NONE);
   vBoxElements = gtk_vbox_new(FALSE, 0);
   gtk_widget_set_sensitive(vBoxElements, FALSE);
@@ -173,57 +187,57 @@ void createInteriorElements(GtkWidget *toolPanel)
 
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vBoxElements), hbox, FALSE, FALSE, 5);
-  gtk_widget_show(hbox);
-  label = gtk_label_new(_("Set caracteristics for :"));
+  label = gtk_label_new(_("<b>Set caracteristics of: </b>"));
   gtk_widget_set_name(label, "label_head");
-  align = gtk_alignment_new(0., 0.5, 0., 0.);
-  gtk_box_pack_start(GTK_BOX(hbox), align, TRUE, TRUE, 2);
-  gtk_widget_show(align);
-  gtk_container_add(GTK_CONTAINER(align), label);
-  gtk_widget_show(label);
+  gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
+  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
   /* We create the tree widget that show the methods. */
   elementsCombo = gtk_combo_box_new_with_model(GTK_TREE_MODEL(listStoreVisuElements));
   renderer = gtk_cell_renderer_text_new();
   gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(elementsCombo), renderer, FALSE);
   gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(elementsCombo), renderer, "text", 0);
   gtk_box_pack_start(GTK_BOX(hbox), elementsCombo, TRUE, TRUE, 2);
-  gtk_widget_show(elementsCombo);
 
-  label = gtk_label_new(_("Standard resources"));
+  label = gtk_label_new("");
+  gtk_label_set_markup(GTK_LABEL(label), _("<b>Standard resources</b>"));
+  gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
   gtk_widget_set_name(label, "label_head_2");
   gtk_box_pack_start(GTK_BOX(vBoxElements), label, FALSE, FALSE, 5);
-  gtk_widget_show(label);
 
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vBoxElements), hbox, FALSE, FALSE, 0);
-  gtk_widget_show(hbox);
-  label = gtk_label_new(_("Color: "));
-  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
-  gtk_widget_show(label);
+/*   label = gtk_label_new(_("Color: ")); */
+/*   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2); */
   widgetVisuElementColor = colorComboBox_newWithRanges(TRUE);
   gtk_box_pack_start(GTK_BOX(hbox), widgetVisuElementColor, FALSE, FALSE, 2);
-  gtk_widget_show(widgetVisuElementColor);
   align = gtk_alignment_new(1., 0.5, 0., 0.);
-  gtk_widget_show(align);
   gtk_box_pack_start(GTK_BOX(hbox), align, TRUE, TRUE, 2);
   checkRendered = gtk_check_button_new_with_label(_("rendered"));
-  gtk_widget_show(checkRendered);
   gtk_container_add(GTK_CONTAINER(align), checkRendered);
+  align = gtk_alignment_new(0.5, 0.5, 0., 0.);
+  gtk_box_pack_start(GTK_BOX(hbox), align, FALSE, FALSE, 2);
+  checkMasked = gtk_check_button_new();
+  gtk_container_add(GTK_CONTAINER(align), checkMasked);
+  image = create_pixmap((GtkWidget*)0, "stock-masking.png");
+  gtk_container_add(GTK_CONTAINER(checkMasked), image);
+  gtk_tooltips_set_tip(tooltips, checkMasked,
+		       _("Make nodes sensitive to the masking effect of planes."),
+		       NULL);
 
   expand = colorComboBoxGet_rangesWidgets(COLOR_COMBOX(widgetVisuElementColor));
-  gtk_widget_show_all(expand);
   gtk_box_pack_start(GTK_BOX(vBoxElements), expand, FALSE, FALSE, 0);
 
-  gtk_widget_show(vBoxElements);
-  gtk_widget_show(scrollView);
-
-  label = gtk_label_new(_("Rendering specific resources"));
+  label = gtk_label_new(_("<b>Rendering specific resources</b>"));
   gtk_widget_set_name(label, "label_head_2");
+  gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
+  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
   gtk_box_pack_start(GTK_BOX(vBoxElements), label, FALSE, FALSE, 5);
-  gtk_widget_show(label);
 
   renderingMethodElements = (GtkWidget*)0;
 
+  gtk_widget_show_all(scrollView);
+
   gtk_container_add(GTK_CONTAINER(toolPanel), scrollView);
 }
 
@@ -239,13 +253,15 @@ void createCallBacksElements()
 		   G_CALLBACK(onColorChanged), (gpointer)0);
   g_signal_connect(G_OBJECT(checkRendered), "toggled",
 		   G_CALLBACK(checkRenderedChanged), (gpointer)0);
+  g_signal_connect(G_OBJECT(checkMasked), "toggled",
+		   G_CALLBACK(checkMaskedChanged), (gpointer)0);
   /* Connect the signal emitted by visu when a new file is loaded. */
   g_signal_connect(G_OBJECT(visu), "dataReadyForRendering",
 		   G_CALLBACK(refreshComboElements), (gpointer)0);
   /* Connect the signal emitted by visu when
      a new rendering method is selected. */
   g_signal_connect(G_OBJECT(visu), "renderingChanged",
-		   G_CALLBACK(renderingMethodChanged), (gpointer)0);
+		   G_CALLBACK(onRenderingMethodChanged), (gpointer)0);
   /* Connect the signal emitted by visu when
      a new rendering method is selected. */
   g_signal_connect(G_OBJECT(visu), "resourcesLoaded",
@@ -263,9 +279,9 @@ gint sortVisuElements(gpointer a, gpointer b)
 /*************/
 /* Callbacks */
 /*************/
-void refreshComboElements(GObject *obj, VisuData *dataObj, gpointer userData)
+void refreshComboElements(GObject *obj _U_, VisuData *dataObj, gpointer userData _U_)
 {
-  int i;
+  unsigned int i;
   GList *tmpLst, *currentListOfElements;
   char *label;
   GtkTreeIter iter, selectedIter;
@@ -341,7 +357,7 @@ void refreshComboElements(GObject *obj, VisuData *dataObj, gpointer userData)
 			 COLUMN_NAME, label,
 			 COLUMN_POINTER_TO_DATA, tmpLst->data,
 			 -1);
-      free(label);
+      g_free(label);
       if (tmpLst == currentListOfElements) /* If first. */
 	selectedIter = iter;
       if (tmpLst->data == (gpointer)prevSelectedElement)
@@ -354,16 +370,15 @@ void refreshComboElements(GObject *obj, VisuData *dataObj, gpointer userData)
 
   DBG_fprintf(stderr,"Panel Element : Refresh OK.\n");
 }
-void renderingMethodChanged(GObject *obj, gpointer userData)
+static void onRenderingMethodChanged(GObject *obj _U_, RenderingMethod *method,
+				     gpointer userData _U_)
 {
-  RenderingMethod *method;
 /*   setSensitiveFunc sensitive; */
   panelElements_changeFunc change;
   createGtkWidgetFunc create;
   int sans;
 
   sans = 1;
-  method = getRenderingMethodInUse();
   if (method)
     {
       getRenderingInterface(method, &change, &create);
@@ -383,7 +398,7 @@ void renderingMethodChanged(GObject *obj, gpointer userData)
       changeRenderingMethodInterface((GtkWidget*)0);
     }
 }
-void comboVisuElementsChanged(GtkComboBox *combobox, gpointer data)
+static void comboVisuElementsChanged(GtkComboBox *combobox, gpointer data _U_)
 {
   GtkTreeModel *tree;
   gboolean selected;
@@ -413,12 +428,15 @@ void comboVisuElementsChanged(GtkComboBox *combobox, gpointer data)
       colorComboBoxSet_material(COLOR_COMBOX(widgetVisuElementColor), ele->material, FALSE);
       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkRendered),
 				   visuElementGet_rendered(ele));
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkMasked),
+				   visuElementGet_sensitiveToPlanes(ele));
     }
 
   /* Force refresh on the specific area. */
   sendElementToSpecificArea();
 }
-void colorVisuElementChanged(ColorComboBox *colorWd, Color *color, gpointer userData)
+void colorVisuElementChanged(ColorComboBox *colorWd _U_, Color *color,
+			     gpointer userData _U_)
 {
   int res;
   int selected, refresh;
@@ -475,7 +493,8 @@ void colorVisuElementChanged(ColorComboBox *colorWd, Color *color, gpointer user
 		     0 , NULL);
     }
 }
-static void onColorChanged(ColorComboBox *colorComboBox, guint colorId, gpointer data)
+static void onColorChanged(ColorComboBox *colorComboBox, guint colorId,
+			   gpointer data _U_)
 {
   int res;
   gboolean selected, refresh;
@@ -536,7 +555,8 @@ static void onColorChanged(ColorComboBox *colorComboBox, guint colorId, gpointer
     }
   g_free(rgbVal);
 }
-static void onMaterialChanged(ColorComboBox *colorComboBox, Material mat, gpointer data)
+static void onMaterialChanged(ColorComboBox *colorComboBox, Material mat,
+			      gpointer data _U_)
 {
   int res;
   int selected, refresh;
@@ -596,7 +616,8 @@ static void onMaterialChanged(ColorComboBox *colorComboBox, Material mat, gpoint
   g_free(matVal);
 }
 
-static void resourcesChanged(GObject *object, gpointer data)
+static void resourcesChanged(GObject *object _U_, VisuData *dataObj _U_,
+			     gpointer data _U_)
 {
   gboolean validIter;
   GtkTreeIter iter;
@@ -623,7 +644,56 @@ static void resourcesChanged(GObject *object, gpointer data)
   /* Force refresh on the specific area. */
   sendElementToSpecificArea();
 }
-void checkRenderedChanged(GtkToggleButton *button, gpointer data)
+static void checkMaskedChanged(GtkToggleButton *button, gpointer data _U_)
+{
+  gboolean validIter, setValue, refresh;
+  GtkTreeIter iter;
+  VisuData *dataObj;
+  VisuElement *ele;
+
+  dataObj = toolPanelGet_visuData(TOOL_PANEL(panelElements));
+
+  /* Get the selected iter. */
+  validIter = gtk_combo_box_get_active_iter(GTK_COMBO_BOX(elementsCombo), &iter);
+  g_return_if_fail(validIter);
+
+  /* Get a pointer on the current visuElement */
+  gtk_tree_model_get(GTK_TREE_MODEL(listStoreVisuElements), &iter,
+		     COLUMN_POINTER_TO_DATA, &ele, -1);
+  setValue = gtk_toggle_button_get_active(button);
+
+  refresh = FALSE;
+  if (ele)
+    /* We change one specific element. */
+    refresh = visuElementSet_sensitiveToPlanes(ele, setValue);
+  else
+    {
+      /* We change for all elements. */
+      validIter = gtk_tree_model_get_iter_first
+	(GTK_TREE_MODEL(listStoreVisuElements), &iter);
+      while (validIter)
+	{
+	  gtk_tree_model_get(GTK_TREE_MODEL(listStoreVisuElements), &iter,
+			     COLUMN_POINTER_TO_DATA, &ele, -1);
+	  if (ele)
+	    refresh = visuElementSet_sensitiveToPlanes(ele, setValue) || refresh;
+	  validIter = gtk_tree_model_iter_next
+	    (GTK_TREE_MODEL(listStoreVisuElements), &iter);
+	}
+    }
+  if (refresh)
+    {
+      visuDataEmit_askForShowHideNodes(dataObj, &refresh);
+
+      if (refresh)
+	{
+	  visuDataEmit_nodeRenderedChange(dataObj);
+	  visuData_createAllNodes(dataObj);
+	  g_idle_add(visuObjectRedraw, (gpointer)0);
+	}
+    }
+}
+static void checkRenderedChanged(GtkToggleButton *button, gpointer data _U_)
 {
   int res;
   gboolean validIter, setValue, refresh;
@@ -674,8 +744,7 @@ void checkRenderedChanged(GtkToggleButton *button, gpointer data)
 	}
     }
   if (refresh)
-    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
-		   0 /* details */, NULL);
+    g_idle_add(visuObjectRedraw, (gpointer)0);
 }
 
 
@@ -688,11 +757,6 @@ char *getLabelOfElement(VisuElement *element)
   char *res;
 
   str = g_string_new("");
-  if (!str)
-    {
-      allocationProblems();
-      exit(1);
-    }
   g_string_append_printf(str, _("Element '%s'"), element->name);
   res = str->str;
   g_string_free(str, FALSE);
@@ -803,12 +867,7 @@ void panelElements_setInterfaceMethods(RenderingMethod *method,
     }
   else
     {
-      interface = malloc(sizeof(GtkRenderingInterface));
-      if (!interface)
-	{
-	  allocationProblems();
-	  exit(1);
-	}
+      interface = g_malloc(sizeof(GtkRenderingInterface));
       interface->newElementInterface = change;
       interface->createInterface = create;
       g_hash_table_insert(listOfRenderingInterfaces,
