@@ -59,6 +59,17 @@
 #include <extraGtkFunctions/gtk_shadeComboBoxWidget.h>
 #include <openGLFunctions/objectList.h>
 #include <openGLFunctions/view.h>
+#include <openGLFunctions/text.h>
+
+/**
+ * SECTION:panelMap
+ * @short_description: The widget to create coloured map.
+ *
+ * <para>This is the user interface for the coloured maps. For a
+ * plane, a scalar field and a shade, it is possible to draw one
+ * coloured plane. The available planes are taken from the #panelPlane
+ * subpanel and the scalar field for the #panelSurfaces.</para>
+ */
 
 /* Local variables. */
 static GtkWidget *panelMap;
@@ -68,6 +79,8 @@ static GtkWidget *comboPlane;
 static GtkWidget *comboField;
 static GtkWidget *comboShade;
 static GtkWidget *spinPrecision;
+static GtkWidget *checkLogScale;
+static GtkWidget *spinIsoLines;
 static GtkWidget *removeButton, *buildButton;
 static OpenGLExtension *extensionMap;
 static OpenGLExtension *extensionLegend;
@@ -90,6 +103,9 @@ static void onPlaneMoved(Plane *plane, gpointer data);
 static void onDataNew(VisuObject *visu, VisuData *dataObj, gpointer data);
 static void onViewportChanged(VisuData *dataObj, OpenGLView *view, gpointer data);
 static void onZoomChanged(VisuData *dataObj, gpointer data);
+static void onSizeChanged(VisuData *dataObj, gpointer data);
+static void onIsoChanged(GtkSpinButton *button, gpointer data);
+static void onLogChanged(GtkToggleButton *button, gpointer data);
 
 static void panelMapCreate_gtkInterface(ToolPanel *panel)
 {
@@ -100,11 +116,25 @@ static void panelMapCreate_gtkInterface(ToolPanel *panel)
   vbox = gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(panel), vbox);
   
+  label = gtk_label_new(_("<b>Map sources</b>"));
+  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+  gtk_widget_set_name(label, "label_head");
+  gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
+  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 3);
+  
   /* The plane selector. */
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   label = gtk_label_new(_("Cutting plane:"));
   gtk_misc_set_padding(GTK_MISC(label), 5, 0);
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
-  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  label = gtk_label_new(_("<span size='small'>"
+			  "<i>Create elements in the 'planes' tab</i></span>"));
+  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+  gtk_misc_set_padding(GTK_MISC(label), 5, 0);
+  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
+  gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   
   align = gtk_alignment_new(0.5, 0.5, 1.0, 1.0);
   gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 0, 15, 15);
@@ -128,10 +158,18 @@ static void panelMapCreate_gtkInterface(ToolPanel *panel)
 		   G_CALLBACK(onComboPlaneChanged), (gpointer)0);
 
   /* The scalar field selector. */
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   label = gtk_label_new(_("Scalar field:"));
   gtk_misc_set_padding(GTK_MISC(label), 5, 0);
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
-  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+  label = gtk_label_new(_("<span size='small'>"
+			  "<i>Import fields in the 'isosurfaces' tab</i></span>"));
+  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+  gtk_misc_set_padding(GTK_MISC(label), 5, 0);
+  gtk_misc_set_alignment(GTK_MISC(label), 1., 0.5);
+  gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
   
   align = gtk_alignment_new(0.5, 0.5, 1.0, 1.0);
   gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 0, 15, 15);
@@ -166,17 +204,18 @@ static void panelMapCreate_gtkInterface(ToolPanel *panel)
 		   G_CALLBACK(onComboChanged), (gpointer)0);
 
   /* Options. */
-  label = gtk_label_new(_("Options"));
+  label = gtk_label_new(_("<b>Options</b>"));
+  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
   gtk_widget_set_name(label, "label_head");
-  gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 3);
   
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-  label = gtk_label_new(_("Precision"));
+  label = gtk_label_new(_("Precision:"));
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
-  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 5);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
   spinPrecision = gtk_spin_button_new_with_range(10, 500, 10);
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spinPrecision), 0);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinPrecision), 100);
@@ -187,6 +226,23 @@ static void panelMapCreate_gtkInterface(ToolPanel *panel)
   g_signal_connect(G_OBJECT(spinPrecision), "value-changed",
 		   G_CALLBACK(onPrecisionChanged), (gpointer)0);
 
+  checkLogScale = gtk_check_button_new_with_mnemonic(_("_Logarithmic scale"));
+  gtk_box_pack_end(GTK_BOX(hbox), checkLogScale, FALSE, FALSE, 0);
+  g_signal_connect(G_OBJECT(checkLogScale), "toggled",
+		   G_CALLBACK(onLogChanged), (gpointer)0);
+
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+
+  label = gtk_label_new(_("Number of isolines:"));
+  gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
+  spinIsoLines = gtk_spin_button_new_with_range(0, 20, 1);
+  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spinIsoLines), 0);
+  gtk_box_pack_start(GTK_BOX(hbox), spinIsoLines, FALSE, FALSE, 0);
+  g_signal_connect(G_OBJECT(spinIsoLines), "value-changed",
+		   G_CALLBACK(onIsoChanged), (gpointer)0);
+
   /* The action buttons. */
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
@@ -197,7 +253,7 @@ static void panelMapCreate_gtkInterface(ToolPanel *panel)
   g_signal_connect(G_OBJECT(removeButton), "clicked",
 		   G_CALLBACK(onRemoveClicked), (gpointer)0);
 
-  buildButton = gtk_button_new_with_label(_("Build"));
+  buildButton = gtk_button_new_with_mnemonic(_("_Build"));
   gtk_box_pack_end(GTK_BOX(hbox), buildButton, FALSE, FALSE, 15);
   g_signal_connect(G_OBJECT(buildButton), "clicked",
 		   G_CALLBACK(onBuildClicked), (gpointer)0);
@@ -205,26 +261,44 @@ static void panelMapCreate_gtkInterface(ToolPanel *panel)
   gtk_widget_show_all(vbox);
 }
 
-static void onBuildClicked(GtkButton *button, gpointer data)
+void panelMapSet(guint planeId, guint scalarFieldId, guint shadeId)
+{
+  VisuData *dataObj;
+
+  extensionMap->used = FALSE;
+  gtk_combo_box_set_active(GTK_COMBO_BOX(comboPlane), planeId);
+  gtk_combo_box_set_active(GTK_COMBO_BOX(comboField), scalarFieldId);
+  gtk_combo_box_set_active(GTK_COMBO_BOX(comboShade), shadeId);
+  extensionMap->used = TRUE;
+
+  dataObj = toolPanelGet_visuData(TOOL_PANEL(panelMap));
+  panelMapRebuild_listMap(dataObj);
+  panelMapRebuild_listLegend(dataObj);
+
+  /* Update the interface. */
+  gtk_widget_set_sensitive(removeButton, TRUE);
+}
+
+static void onBuildClicked(GtkButton *button _U_, gpointer data _U_)
 {
   /* Add as an idle function the graphic update. */
   g_idle_add(onIdleRebuild, (gpointer)0);
 }
 
-static void onPrecisionChanged(GtkSpinButton *spin, gpointer data)
+static void onPrecisionChanged(GtkSpinButton *spin _U_, gpointer data _U_)
 {
   if (extensionMap->used)
     /* Add as an idle function the graphic update. */
     g_idle_add(onIdleRebuild, (gpointer)0);
 }
 
-static void onComboChanged(GtkComboBox *combo, gpointer data)
+static void onComboChanged(GtkComboBox *combo _U_, gpointer data _U_)
 {
   if (extensionMap->used)
     /* Add as an idle function the graphic update. */
     g_idle_add(onIdleRebuild, (gpointer)0);
 }
-static void onComboPlaneChanged(GtkComboBox *combo, gpointer data)
+static void onComboPlaneChanged(GtkComboBox *combo, gpointer data _U_)
 {
   GtkTreeModel *model;
   GtkTreeIter iter;
@@ -249,15 +323,25 @@ static void onComboPlaneChanged(GtkComboBox *combo, gpointer data)
     g_idle_add(onIdleRebuild, (gpointer)0);
 }
 
-static void onPlaneMoved(Plane *plane, gpointer data)
+static void onPlaneMoved(Plane *plane _U_, gpointer data _U_)
 {
   if (extensionMap->used)
     /* Add as an idle function the graphic update. */
     g_idle_add(onIdleRebuild, (gpointer)0);
 }
 
-static void onRemoveClicked(GtkButton *button, gpointer data)
+static void onRemoveClicked(GtkButton *button _U_, gpointer data _U_)
 {
+  gboolean valid;
+  Plane *plane;
+  Shade *shade;
+  ScalarField *field;
+
+  /* Return the visibility status of the plane to TRUE. */
+  valid = panelMapGet_elements(&plane, &field, &shade);
+  if (plane)
+    panelPlanesSet_planeRendered(plane, TRUE);
+
   /* Update the interface. */
   gtk_widget_set_sensitive(removeButton, FALSE);
 
@@ -270,7 +354,7 @@ static void onRemoveClicked(GtkButton *button, gpointer data)
 		 NULL);
 }
 
-static void onDataNew(VisuObject *visu, VisuData *dataObj, gpointer data)
+static void onDataNew(VisuObject *visu _U_, VisuData *dataObj, gpointer data _U_)
 {
   extensionMap->used = 0;
   extensionLegend->used = 0;
@@ -281,22 +365,31 @@ static void onDataNew(VisuObject *visu, VisuData *dataObj, gpointer data)
                    G_CALLBACK(onViewportChanged), (gpointer)0);
   g_signal_connect(G_OBJECT(dataObj), "OpenGLFacetteChanged",
                    G_CALLBACK(onZoomChanged), (gpointer)0);
+  g_signal_connect(G_OBJECT(dataObj), "OpenGLNearFar",
+                   G_CALLBACK(onSizeChanged), (gpointer)0);
 }
 
-static void onViewportChanged(VisuData *dataObj, OpenGLView *view, gpointer data)
+static void onViewportChanged(VisuData *dataObj, OpenGLView *view _U_,
+			      gpointer data _U_)
 {
   DBG_fprintf(stderr, "Panel Map: get the 'openglWidthHeight' signal.\n");
   if (extensionMap->used)
     panelMapRebuild_listLegend(dataObj);
 }
-static void onZoomChanged(VisuData *dataObj, gpointer data)
+static void onZoomChanged(VisuData *dataObj, gpointer data _U_)
 {
   DBG_fprintf(stderr, "Panel Map: get the 'OpenGLFacetteChanged' signal.\n");
   if (extensionMap->used)
     panelMapRebuild_listMap(dataObj);
 }
+static void onSizeChanged(VisuData *dataObj, gpointer data _U_)
+{
+  DBG_fprintf(stderr, "Panel Map: get the 'OpenGLNearFar' signal.\n");
+  if (extensionMap->used)
+    panelMapRebuild_listMap(dataObj);
+}
 
-static gboolean onIdleRebuild(gpointer data)
+static gboolean onIdleRebuild(gpointer data _U_)
 {
   VisuData *dataObj;
 
@@ -304,16 +397,31 @@ static gboolean onIdleRebuild(gpointer data)
   panelMapRebuild_listMap(dataObj);
   panelMapRebuild_listLegend(dataObj);
 
-  /* Update the interface. */
-  gtk_widget_set_sensitive(removeButton, TRUE);
-
   if (extensionMap->used)
-    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
-		   0 /* details */,
-		   NULL);
+    {
+      /* Update the interface. */
+      gtk_widget_set_sensitive(removeButton, TRUE);
+
+      g_idle_add(visuObjectRedraw, (gpointer)0);
+    }
   return FALSE;
 }
 
+static void onLogChanged(GtkToggleButton *button _U_, gpointer data _U_)
+{
+  if (extensionMap->used)
+    /* Add as an idle function the graphic update. */
+    g_idle_add(onIdleRebuild, (gpointer)0);
+}
+
+static void onIsoChanged(GtkSpinButton *button _U_, gpointer data _U_)
+{
+  if (extensionMap->used)
+    /* Add as an idle function the graphic update. */
+    g_idle_add(onIdleRebuild, (gpointer)0);
+}
+
+
 static gboolean panelMapGet_elements(Plane **plane, ScalarField **field, Shade **shade)
 {
   GtkTreeModel *model;
@@ -357,23 +465,27 @@ static void panelMapRebuild_listMap(VisuData *dataObj)
       extensionMap->used = 0;
       return;
     }
+  /* Change the visibility status of the plane to FALSE. */
+  panelPlanesSet_planeRendered(plane, FALSE);
   extensionMap->used = 1;
   
   glNewList(identifierMap, GL_COMPILE);
   scalarFieldDraw_map(field, plane, shade, visuDataGet_openGLView(dataObj),
                       gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinPrecision)),
-                      drawnMinMax);
+                      drawnMinMax, gtk_toggle_button_get_active
+		      (GTK_TOGGLE_BUTTON(checkLogScale)),
+		      gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinIsoLines)));
   glEndList();
 }
 
-static void panelMapRebuild_listLegend(VisuData *dataObj)
+static void panelMapRebuild_listLegend(VisuData *dataObj _U_)
 {
   Shade *shade;
   ScalarField *field;
   Plane *plane;
   gboolean valid;
   double minMax[2];
-  float fMinMax[2];
+  float fMinMax[2], smin;
 
   DBG_fprintf(stderr, "Panel Map : rebuilding object list for legend.\n");
 
@@ -389,20 +501,40 @@ static void panelMapRebuild_listLegend(VisuData *dataObj)
   scalarFieldGet_minMax(field, minMax);
   fMinMax[0] = minMax[0];
   fMinMax[1] = minMax[1];
+  smin       = minMax[0];
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkLogScale)))
+    smin = scalarFieldGet_secondaryMin(field);
+  
+  openGLText_rebuildFontList();
+
   glNewList(identifierLegend, GL_COMPILE);
-  shadeDraw_legend(shade, .05, -1., fMinMax, drawnMinMax, 2);
+  glDisable(GL_DEPTH_TEST);
+  glDisable(GL_ALPHA_TEST);
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkLogScale)))
+    shadeDraw_legend(shade, .05, -1., fMinMax, drawnMinMax, 2, &smin);
+  else
+    shadeDraw_legend(shade, .05, -1., fMinMax, drawnMinMax, 2, (float*)0);
+
   glEndList();
 }
+void panelMapSet_logScale(gboolean logScale)
+{
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkLogScale), logScale);
+}
+void panelMapSet_nIsoLines(guint nIsoLines)
+{
+  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinIsoLines), nIsoLines);
+}
 
 
-ToolPanel* initPanelMap()
+ToolPanel* panelMapInit()
 {
   /* Long description */
   char *cl = _("Map projections");
   /* Short description */
   char *tl = _("Maps");
 
-  panelMap = toolPanelNew("Panel_map", cl, tl);
+  panelMap = toolPanelNew_withIconFromPath("Panel_map", cl, tl, "stock-map_20.png");
   g_return_val_if_fail(panelMap, (ToolPanel*)0);
 
   toolPanelSet_dockable(TOOL_PANEL(panelMap), TRUE);
@@ -412,7 +544,8 @@ ToolPanel* initPanelMap()
   
   /* Extension to draw values. */
   identifierMap = openGLObjectList_new(1);
-  extensionMap = OpenGLExtension_new(_("Map"), _("Tmp drawing for maps."),
+  extensionMap = OpenGLExtension_new("Map", _("Map"),
+				     _("Drawing extension for maps."),
 				     identifierMap, panelMapRebuild_listMap);
   extensionMap->used = 0;
   OpenGLExtensionSet_priority(extensionMap, OPENGL_EXTENSION_PRIORITY_FIRST);
@@ -421,8 +554,9 @@ ToolPanel* initPanelMap()
 
   /* Extension to draw the legend. */
   identifierLegend = openGLObjectList_new(1);
-  extensionLegend = OpenGLExtension_new(_("Map legend"), _("Responsible for drawing"
-							   " the legend of the color map."),
+  extensionLegend = OpenGLExtension_new("MapLegend", _("Map legend"),
+					_("Responsible for drawing"
+					  " the legend of the color map."),
 					identifierLegend, panelMapRebuild_listLegend);
   extensionLegend->used = 0;
   OpenGLExtensionSet_priority(extensionLegend, OPENGL_EXTENSION_PRIORITY_LAST);
