@@ -43,11 +43,18 @@
 */
 
 #include "panelMethod.h"
-#include "externalRenderingMethods_gtk.h"
 
 #include <visu_tools.h>
 #include <extraGtkFunctions/gtk_toolPanelWidget.h>
 
+#include "gtkSpin.h"
+
+/* Functions used to initialise the
+ * specific area in the panel of rendering methods.
+ */
+VisuToolsInitFunc listInitRendenringConfigGtkFunc[] = {
+  panelElementSpinInit_gtkConfig,
+  (VisuToolsInitFunc)0};
 
 /* Local variables. */
 /* Store the toolpanel object, can be interesting to
@@ -92,18 +99,19 @@ static void setRenderingSpecificWidget(struct RenderingExpanderWidget_struct *st
 
 /* Callbacks */
 static void onMethodPanelEnter(ToolPanel *toolPanel, gpointer data);
-static void renderingHasChanged(GObject *obj, gpointer data);
+static void onRenderingChanged(GObject *obj, RenderingMethod *meth, gpointer data);
 static void comboInputMethodsChanged(GtkComboBox *combo, gpointer data);
 
 
-ToolPanel* initPanelMethod()
+ToolPanel* panelMethodInit()
 {
   char *cl = _("Rendering method");
   char *tl = _("Draw");
   int i;
 
   /* Now that everything has a value, we create the panel. */
-  panelMethod = toolPanelNew_withIconFromStock("Panel_method", cl, tl, GTK_STOCK_HARDDISK);
+  panelMethod = toolPanelNew_withIconFromPath("Panel_method", cl, tl,
+					      "stock-method_20.png");
   if (!panelMethod)
     return (ToolPanel*)0;
   toolPanelSet_dockable(TOOL_PANEL(panelMethod), TRUE);
@@ -113,7 +121,7 @@ ToolPanel* initPanelMethod()
   currentDestroyFunc = (destroyFunc)0;
   listOfRenderingSpecificConfigWidget = g_hash_table_new_full(g_direct_hash,
                                                               g_direct_equal,
-                                                              NULL, freeGPointer);
+                                                              NULL, g_free);
 
   /* Initialise the rendering specific widgets */
   for (i = 0; listInitRendenringConfigGtkFunc[i]; i++)
@@ -126,7 +134,7 @@ ToolPanel* initPanelMethod()
   return TOOL_PANEL(panelMethod);
 }
 
-static void onMethodPanelEnter(ToolPanel *toolPanel, gpointer data)
+static void onMethodPanelEnter(ToolPanel *toolPanel _U_, gpointer data _U_)
 {
   GtkWidget *label, *vbox, *align, *scrollView, *hbox;
   int iterSelected;
@@ -196,7 +204,7 @@ static void onMethodPanelEnter(ToolPanel *toolPanel, gpointer data)
   g_signal_connect(G_OBJECT(comboMethod), "changed",
          		   G_CALLBACK(comboInputMethodsChanged), (gpointer)0);
   g_signal_connect(G_OBJECT(visu), "renderingChanged",
-		           G_CALLBACK(renderingHasChanged), (gpointer)1);
+		           G_CALLBACK(onRenderingChanged), (gpointer)1);
 
   /* Set the default values. */
   gtk_combo_box_set_active(GTK_COMBO_BOX(comboMethod), iterSelected);
@@ -215,10 +223,10 @@ static GtkWidget* createTreeListRenderingMethod(int *iterSelected)
 
   /* We create the structure that store the methods */
   listStoreRenderingMethods = gtk_list_store_new (N_COLUMNS,
-						                          GDK_TYPE_PIXBUF,
-                         						  G_TYPE_STRING,
-						                          G_TYPE_POINTER,
-						                          G_TYPE_POINTER);
+						  GDK_TYPE_PIXBUF,
+						  G_TYPE_STRING,
+						  G_TYPE_POINTER,
+						  G_TYPE_POINTER);
   *iterSelected = 0;
   i = 0;
   methodInUse = getRenderingMethodInUse();
@@ -255,7 +263,7 @@ static GtkWidget* createTreeListRenderingMethod(int *iterSelected)
   return methodsWd;
 }
 
-static void comboInputMethodsChanged(GtkComboBox *combo, gpointer data)
+static void comboInputMethodsChanged(GtkComboBox *combo, gpointer data _U_)
 {
   GtkTreeModel *tree;
   gboolean selected;
@@ -289,8 +297,8 @@ static void comboInputMethodsChanged(GtkComboBox *combo, gpointer data)
     }
   else
     {
-      fprintf(stderr, "WARNING! Can't find the selected method in the"
-	      " combobox of the config panel.\n");
+      g_warning("Can't find the selected method in the"
+		" combobox of the config panel.\n");
       gtk_label_set_text(GTK_LABEL(descrLabel), _("None"));
       setRenderingSpecificWidget((struct RenderingExpanderWidget_struct*)0);
     }
@@ -329,15 +337,14 @@ static void setRenderingSpecificWidget(struct RenderingExpanderWidget_struct *st
   gtk_container_add(GTK_CONTAINER(viewport), wd);
   gtk_widget_show(wd);
 }
-static void renderingHasChanged(GObject *obj, gpointer data)
+static void onRenderingChanged(GObject *obj _U_, RenderingMethod *meth,
+			       gpointer data _U_)
 {
-  RenderingMethod *meth;
   GtkTreeIter iter;
   gboolean valid;
   RenderingMethod* method;
   int i;
 
-  meth = getRenderingMethodInUse();
   if (meth)
     {
       DBG_fprintf(stderr, "Panel Method : changing rendering method to '%s'.\n", meth->name);
