@@ -43,6 +43,13 @@
 */
 #include "panelOpenGL.h"
 
+#include <gtk/gtk.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include <visu_gtk.h>
+#include <visu_tools.h>
 #include <visu_object.h>
 #include <support.h>
 #include <gtk_main.h>
@@ -52,31 +59,58 @@
 #include <openGLFunctions/renderingMode.h>
 #include <openGLFunctions/view.h>
 
+/* Sensitive widget in this subpanel. */
+static GtkWidget *checkImmediateDrawing;
+static GtkWidget *comboOpenGLRendering;
+static GtkWidget *checkAntialiasing;
+static GtkWidget *checkTransparency;
+static GtkWidget *checkStereo;
+static GtkWidget *spinPrecision;
+static GtkWidget *spinStereoAngle;
+
 static int disableCallbacksOpenGL;
 static GtkListStore *light_list_store;
 static GtkWidget *panelOpenGL;
 
+#if GTK_MINOR_VERSION > 5
+static GtkListStore *renderingListStore;
+#endif
+
 #define OPENGL_FOLLOW_GLOBAL_RENDERING_MODE _("Follow global setting")
 
+/* Private functions. */
+GtkWidget *createInteriorOpenGL();
+void createCallBacksOpenGL();
+
 /* Local callbacks. */
+static void immediateDrawingCheckChanged(GtkToggleButton *button, gpointer data);
+static void stereoCheckChanged(GtkToggleButton *button, gpointer data);
+static void stereoSpinChanged(GtkSpinButton *button, gpointer data);
+static void antialisingCheckChanged(GtkToggleButton *button, gpointer data);
+static void onTransparencyChanged(GtkToggleButton *button, gpointer data);
+void precisionChanged(GtkButton *button, gpointer user_data);
 void comboOpenGLRenderingChanged(GtkComboBox *combobox, gpointer data);
 void addPresetOneLightClicked(GtkButton *button, gpointer data);
 void addPresetFourLightsClicked(GtkButton *button, gpointer data);
 static void selectionChanged(GtkTreeSelection *tree, gpointer data);
 static void addNewLightClicked(GtkButton *button, gpointer data);
 static void removeSelectedLightsClicked(GtkButton *button, gpointer data);
+#if GTK_MINOR_VERSION > 5
 static void renderingModeEdited(GtkCellRendererText *cellrenderertext,
 				gchar *path, gchar *text, gpointer user_data);
+#endif
 static void onOpenGLEnter(ToolPanel *toolPanel, gpointer data);
 
-ToolPanel* initPanelOpenGL()
+ToolPanel* panelOpenGL_init()
 {
   char *cl = _("Set OpenGL parameters");
   char *tl = _("OpenGL");
 
-  panelOpenGL = toolPanelNew_withIconFromStock("Panel_opengl", cl, tl, GTK_STOCK_EXECUTE);
+  panelOpenGL = toolPanelNew_withIconFromPath("Panel_opengl", cl, tl,
+					      "stock-opengl_20.png");
   if (!panelOpenGL)
     return (ToolPanel*)0;
+
   gtk_container_add(GTK_CONTAINER(panelOpenGL), createInteriorOpenGL());
   toolPanelSet_dockable(TOOL_PANEL(panelOpenGL), TRUE);
 
@@ -155,7 +189,7 @@ void light_tree_show_hide(GtkCellRendererToggle *cell_renderer,
     g_signal_emit(visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id, 0, NULL);
 }
 
-void light_update(GtkCellRendererText *cellrenderertext,
+void light_update(GtkCellRendererText *cellrenderertext _U_,
 		  gchar *path, gchar *text, gpointer user_data)
 {
   GtkTreeIter iter;
@@ -261,7 +295,6 @@ GtkWidget* make_renderingTreeView()
   GtkTreeViewColumn* column; 
   GtkCellRenderer *renderer;
   GtkWidget *renderingTreeView;
-  GtkListStore *renderingListStore;
   GtkListStore *namesListStore;
   GtkTreeIter iter;
   const char** names;
@@ -292,14 +325,15 @@ GtkWidget* make_renderingTreeView()
     {
       if (((OpenGLExtension*)list->data)->isSensitiveToRenderingMode)
 	{
-	  if (((OpenGLExtension*)list->data)->preferedRenderingMode == -1)
+	  if (((OpenGLExtension*)list->data)->preferedRenderingMode ==
+	      followGeneralSetting)
 	    modeName = OPENGL_FOLLOW_GLOBAL_RENDERING_MODE;
 	  else
 	    modeName = names[((OpenGLExtension*)list->data)->preferedRenderingMode];
 	  gtk_list_store_append(renderingListStore, &iter);
 	  gtk_list_store_set(renderingListStore, &iter,
 			     RENDERING_POINTER_TO, list->data,
-			     RENDERING_NAME, ((OpenGLExtension*)list->data)->name,
+			     RENDERING_NAME, ((OpenGLExtension*)list->data)->nameI18n,
 			     RENDERING_MODE, modeName, -1);
 	}
       list = g_list_next(list);
@@ -332,7 +366,7 @@ GtkWidget* make_renderingTreeView()
 #endif
 }
   
-void addNewLightClicked(GtkButton *button, gpointer data)
+void addNewLightClicked(GtkButton *button _U_, gpointer data)
 {
   Light *light0;
   GtkTreeIter iter;
@@ -342,8 +376,9 @@ void addNewLightClicked(GtkButton *button, gpointer data)
 
   if(gtk_tree_model_iter_n_children(GTK_TREE_MODEL(data), (GtkTreeIter*)0) >= GL_MAX_LIGHTS)
     {
-      raiseAlertDialog(_("The maximm number of lights allowed by OpenGL"
-			 " has been reached, can't add more."));
+      visuGtkRaise_warning(_("OpenGL"),
+			   _("The maximm number of lights allowed by OpenGL"
+			     " has been reached, can't add more."), (GtkWindow*)0);
       return;
     }
 
@@ -366,7 +401,7 @@ void addNewLightClicked(GtkButton *button, gpointer data)
   if (status)
     g_signal_emit(visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id, 0, NULL);
 }
-void removeSelectedLightsClicked(GtkButton *button, gpointer data)
+void removeSelectedLightsClicked(GtkButton *button _U_, gpointer data)
 {
   GList *tmpLst, *selectedRows;
   GtkTreeIter iter;
@@ -432,18 +467,18 @@ void selectionChanged(GtkTreeSelection *tree, gpointer data)
 GtkWidget *createInteriorOpenGL()
 {
   GtkWidget *vbox, *hbox;
-  GtkWidget *label;
-  GtkWidget *table;
-  GtkWidget *align;
-  GtkObject *adj;
-  GtkWidget *scrollLight, *scrollRendering;
+  GtkWidget *label, *align;
+  GtkWidget *scrollLight;
   GtkWidget *button;
   GtkWidget *image;
   GtkWidget *scroll, *viewport;
-  GtkWidget *expanderLight, *vbox2, *expanderRendering;
-  GtkWidget *light_tree_view, *renderingTreeView;
+  GtkWidget *expanderLight, *vbox2, *expand;
+  GtkWidget *light_tree_view;
   const char** names;
   int i;
+#if GTK_MINOR_VERSION > 5
+  GtkWidget *scrollRendering, *expanderRendering, *renderingTreeView;
+#endif
 
   scroll = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
@@ -455,27 +490,30 @@ GtkWidget *createInteriorOpenGL()
   vbox = gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(viewport), vbox);
 
+  /*************************/
+  /* Rendering parameters. */
+  /*************************/
+  expand = gtk_expander_new(_("<b>Rendering options:</b>"));
+  gtk_expander_set_expanded(GTK_EXPANDER(expand), TRUE);
+  label = gtk_expander_get_label_widget(GTK_EXPANDER(expand));
+  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+  gtk_widget_set_name(label, "label_head");
+  gtk_box_pack_start(GTK_BOX(vbox), expand, FALSE, FALSE, 5);
+  align = gtk_alignment_new(0.5, 0.5, 1., 1.);
+  gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 0, 5, 0);
+  gtk_container_add(GTK_CONTAINER(expand), align);
+  vbox2 = gtk_vbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(align), vbox2);
   hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new(_("Redraw immediately after changes :"));
-  gtk_widget_set_name(label, "label_head");
+  gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 2);
+  label = gtk_label_new(_("Precision:"));
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+  spinPrecision = gtk_spin_button_new_with_range(10, 500, 1);
+  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinPrecision),
+			    OpenGLViewGet_precision() * 100.);
+  gtk_box_pack_start(GTK_BOX(hbox), spinPrecision, FALSE, FALSE, 2);
+  label = gtk_label_new("%");
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
-  checkImmediateDrawing = gtk_check_button_new();
-/*   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkImmediateDrawing), */
-/* 			       (gboolean)get); */
-  gtk_box_pack_start(GTK_BOX(hbox), checkImmediateDrawing, FALSE, FALSE, 2);
-  gtk_widget_show(checkImmediateDrawing);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
-
-  table = gtk_table_new(2, 1, FALSE);
-  gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 5);
-  align = gtk_alignment_new(0., 0.5, 0., 0.);
-  gtk_table_attach(GTK_TABLE(table), align, 0, 1, 0, 1, GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
-  label = gtk_label_new(_("Choose a rendering option :"));
-  gtk_widget_set_name(label, "label_head");
-  gtk_container_add(GTK_CONTAINER(align), label);
-  align = gtk_alignment_new(1.0, 0.5, 0., 0.);
-  gtk_table_attach(GTK_TABLE(table), align, 0, 1, 1, 2, GTK_FILL | GTK_EXPAND, GTK_SHRINK, 2, 0);
-
   comboOpenGLRendering = gtk_combo_box_new_text();
   names = openGLGet_allRenderingModesI18n();
   if (names)
@@ -484,38 +522,84 @@ GtkWidget *createInteriorOpenGL()
   else
     g_warning("No OpenGL rendering mode available.");
   gtk_combo_box_set_active(GTK_COMBO_BOX(comboOpenGLRendering), 0);
-  gtk_container_add(GTK_CONTAINER(align), comboOpenGLRendering);
+  gtk_box_pack_end(GTK_BOX(hbox), comboOpenGLRendering, FALSE, FALSE, 2);
   g_signal_connect(G_OBJECT(comboOpenGLRendering), "changed",
 		   G_CALLBACK(comboOpenGLRenderingChanged), (gpointer)0);
-
+  label = gtk_label_new(_("Mode:"));
+  gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+  
   hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new(_("Precision (100 is normal) : "));
-  gtk_widget_set_name(label, "label_head");
-  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
-  adj = gtk_adjustment_new(5., 10., 500., 5., 1.0, 1.0);
-  spinPrecision = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.01, 0);
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinPrecision),
-			    OpenGLViewGet_precision() * 100.);
-  gtk_box_pack_start(GTK_BOX(hbox), spinPrecision, FALSE, FALSE, 2);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
-
-  hbox = gtk_hbox_new(FALSE, 0);
-  label = gtk_label_new(_("Antialiase lines :"));
-  gtk_widget_set_name(label, "label_head");
+  gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 2);
+  label = gtk_label_new(_("Antialiase lines:"));
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
   checkAntialiasing = gtk_check_button_new();
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkAntialiasing),
 			       (gboolean)getAntialias());
   gtk_box_pack_start(GTK_BOX(hbox), checkAntialiasing, FALSE, FALSE, 2);
+
+  checkTransparency = gtk_check_button_new();
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkTransparency),
+			       openGLGet_trueTransparency());
+  gtk_box_pack_end(GTK_BOX(hbox), checkTransparency, FALSE, FALSE, 2);
+  label = gtk_label_new(_("Enhanced transparency:"));
+  gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 5);
+  label = gtk_label_new(_("Use stereo rendering:"));
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+  checkStereo = gtk_check_button_new();
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkStereo),
+			       openGLGet_stereo());
+  gtk_box_pack_start(GTK_BOX(hbox), checkStereo, FALSE, FALSE, 2);
+  gtk_widget_set_sensitive(hbox, openGLGet_stereoCapability());
+  /* Degrees. */
+  label = gtk_label_new(_("deg."));
+  gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+  spinStereoAngle = gtk_spin_button_new_with_range(0.5,25.,0.1);
+  gtk_box_pack_end(GTK_BOX(hbox), spinStereoAngle, FALSE, FALSE, 2);
+  label = gtk_label_new(_("angle:"));
+  gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 1);
+
+  /******************************/
+  /* Rendering mode parameters. */
+  /******************************/
+#if GTK_MINOR_VERSION > 5
+  expanderRendering = gtk_expander_new(_("Per extension rendering mode:"));
+  label = gtk_expander_get_label_widget(GTK_EXPANDER(expanderRendering));
+  gtk_box_pack_start(GTK_BOX(vbox2), expanderRendering, TRUE, TRUE, 5);
+  scrollRendering = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollRendering),
+				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_container_add(GTK_CONTAINER(expanderRendering), scrollRendering);
+  renderingTreeView = make_renderingTreeView();
+  gtk_widget_set_size_request(renderingTreeView, -1, 125);
+  gtk_container_add(GTK_CONTAINER(scrollRendering), renderingTreeView);
+#endif
+  
+  /******************/
+  /* Other options. */
+  /******************/
+  hbox = gtk_hbox_new(FALSE, 0);
+  label = gtk_label_new(_("<b>Redraw immediately after changes:</b>"));
+  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
+  gtk_widget_set_name(label, "label_head");
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+  checkImmediateDrawing = gtk_check_button_new();
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkImmediateDrawing),
+			       openGLGet_immediate());
+  gtk_box_pack_start(GTK_BOX(hbox), checkImmediateDrawing, FALSE, FALSE, 2);
+  gtk_widget_show(checkImmediateDrawing);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
 
   /*********************/
   /* Light parameters. */
   /*********************/
-  expanderLight = gtk_expander_new(_("Light sources"));
+  expanderLight = gtk_expander_new(_("<b>Light sources:</b>"));
   label = gtk_expander_get_label_widget(GTK_EXPANDER(expanderLight));
+  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
   gtk_widget_set_name(label, "label_head");
-  gtk_box_pack_start(GTK_BOX(vbox), expanderLight, TRUE, TRUE, 5);
+  gtk_box_pack_start(GTK_BOX(vbox), expanderLight, FALSE, FALSE, 5);
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(expanderLight), hbox);
   scrollLight = gtk_scrolled_window_new (NULL, NULL);
@@ -547,62 +631,67 @@ GtkWidget *createInteriorOpenGL()
 		   "changed", G_CALLBACK(selectionChanged), (gpointer)button);
 
   button = gtk_button_new();
-  image = create_pixmap(mainWindow, "stock-one-light_20.png");
+  image = create_pixmap((GtkWidget*)0, "stock-one-light_20.png");
   gtk_container_add(GTK_CONTAINER(button), image);
   g_signal_connect(G_OBJECT(button), "clicked",
 		   G_CALLBACK(addPresetOneLightClicked),
 		   (gpointer)gtk_tree_view_get_model(GTK_TREE_VIEW(light_tree_view)));
   gtk_box_pack_end(GTK_BOX(vbox2), button, FALSE, FALSE, 1);
   button = gtk_button_new();
-  image = create_pixmap(mainWindow, "stock-four-lights_20.png");
+  image = create_pixmap((GtkWidget*)0, "stock-four-lights_20.png");
   gtk_container_add(GTK_CONTAINER(button), image);
   g_signal_connect(G_OBJECT(button), "clicked",
 		   G_CALLBACK(addPresetFourLightsClicked),
 		   (gpointer)gtk_tree_view_get_model(GTK_TREE_VIEW(light_tree_view)));
   gtk_box_pack_end(GTK_BOX(vbox2), button, FALSE, FALSE, 1);
 
-  /******************************/
-  /* Rendering mode parameters. */
-  /******************************/
-#if GTK_MINOR_VERSION > 5
-  expanderRendering = gtk_expander_new(_("Per extension rendering mode"));
-  label = gtk_expander_get_label_widget(GTK_EXPANDER(expanderRendering));
-  gtk_widget_set_name(label, "label_head");
-  gtk_box_pack_start(GTK_BOX(vbox), expanderRendering, TRUE, TRUE, 5);
-  scrollRendering = gtk_scrolled_window_new(NULL, NULL);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollRendering),
-				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_container_add(GTK_CONTAINER(expanderRendering), scrollRendering);
-  renderingTreeView = make_renderingTreeView();
-  gtk_widget_set_size_request(renderingTreeView, -1, 120);
-  gtk_container_add(GTK_CONTAINER(scrollRendering), renderingTreeView);
-#endif
-  
   gtk_widget_show_all(scroll);
   return scroll;
 }
 
 void createCallBacksOpenGL()
 {
-  g_signal_connect((gpointer)spinPrecision, "value-changed",
+  g_signal_connect(G_OBJECT(spinPrecision), "value-changed",
 		   G_CALLBACK(precisionChanged), (gpointer)0);
-  g_signal_connect((gpointer)checkAntialiasing, "toggled",
+  g_signal_connect(G_OBJECT(checkAntialiasing), "toggled",
 		   G_CALLBACK(antialisingCheckChanged), (gpointer)0);
-  g_signal_connect((gpointer)checkImmediateDrawing, "toggled",
+  g_signal_connect(G_OBJECT(checkTransparency), "toggled",
+		   G_CALLBACK(onTransparencyChanged), (gpointer)0);
+  g_signal_connect(G_OBJECT(checkImmediateDrawing), "toggled",
 		   G_CALLBACK(immediateDrawingCheckChanged), (gpointer)0);
+  g_signal_connect(G_OBJECT(checkStereo), "toggled",
+		   G_CALLBACK(stereoCheckChanged), (gpointer)0);
+  g_signal_connect(G_OBJECT(spinStereoAngle), "value-changed",
+		   G_CALLBACK(stereoSpinChanged), (gpointer)0);
 }
 
 /*************/
 /* Callbacks */
 /*************/
-void immediateDrawingCheckChanged(GtkToggleButton *button, gpointer data)
+static void immediateDrawingCheckChanged(GtkToggleButton *button, gpointer data _U_)
 {
   if (disableCallbacksOpenGL)
     return;
 
-  setImmediateDrawing(gtk_toggle_button_get_active(button));
+  openGLSet_immediate(gtk_toggle_button_get_active(button));
 }
-void comboOpenGLRenderingChanged(GtkComboBox *combobox, gpointer data)
+static void stereoCheckChanged(GtkToggleButton *button, gpointer data _U_)
+{
+  if (disableCallbacksOpenGL)
+    return;
+
+  openGLSet_stereo(gtk_toggle_button_get_active(button));
+  g_idle_add(visuObjectRedraw, (gpointer)0);
+}
+static void stereoSpinChanged(GtkSpinButton *button, gpointer data _U_)
+{
+  if (disableCallbacksOpenGL)
+    return;
+
+  if (openGLSet_stereoAngle((float)gtk_spin_button_get_value(button)))
+    g_idle_add(visuObjectRedraw, (gpointer)0);
+}
+void comboOpenGLRenderingChanged(GtkComboBox *combobox, gpointer data _U_)
 {
   int mode;
   
@@ -612,19 +701,25 @@ void comboOpenGLRenderingChanged(GtkComboBox *combobox, gpointer data)
   mode = (int)gtk_combo_box_get_active(combobox);
 
   if (openGLSet_globalRenderingMode(mode))
-    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
-		   0 , NULL);
+    g_idle_add(visuObjectRedraw, (gpointer)0);
 }
-void antialisingCheckChanged(GtkToggleButton *button, gpointer data)
+static void antialisingCheckChanged(GtkToggleButton *button, gpointer data _U_)
 {
   if (disableCallbacksOpenGL)
     return;
 
   if (setAntialias(gtk_toggle_button_get_active(button)))
-    g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
-		   0 , NULL);
+    g_idle_add(visuObjectRedraw, (gpointer)0);
 }
-void precisionChanged(GtkButton *button, gpointer user_data)
+static void onTransparencyChanged(GtkToggleButton *button, gpointer data _U_)
+{
+  if (disableCallbacksOpenGL)
+    return;
+
+  if (openGLSet_trueTransparency(gtk_toggle_button_get_active(button)))
+    g_idle_add(visuObjectRedraw, (gpointer)0);
+}
+void precisionChanged(GtkButton *button _U_, gpointer user_data _U_)
 {
   float prec;
   VisuData *dataObj;
@@ -638,24 +733,62 @@ void precisionChanged(GtkButton *button, gpointer user_data)
       dataObj = toolPanelGet_visuData(TOOL_PANEL(panelOpenGL));
       visuDataEmit_facettesChanged(dataObj);
       visuData_createAllElements(dataObj); 
-      g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
-		     0 , NULL);
+      g_idle_add(visuObjectRedraw, (gpointer)0);
     }
 }
-static void onOpenGLEnter(ToolPanel *toolPanel, gpointer data)
+static void onOpenGLEnter(ToolPanel *toolPanel _U_, gpointer data _U_)
 {
+#if GTK_MINOR_VERSION > 5
+  const gchar **names;
+  gchar *name;
+  GtkTreeIter iter;
+  gboolean valid;
+  OpenGLExtension *ext;
+#endif
+
   disableCallbacksOpenGL = 1;
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkImmediateDrawing),
-			       (gboolean)getImmediateDrawing());
+			       openGLGet_immediate());
   gtk_combo_box_set_active(GTK_COMBO_BOX(comboOpenGLRendering),
 			   openGLGet_globalRenderingOption());
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinPrecision),
 			    OpenGLViewGet_precision() * 100.);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkAntialiasing),
 			       (gboolean)getAntialias());
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkTransparency),
+			       openGLGet_trueTransparency());
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkStereo),
+			       openGLGet_stereo());
+  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinStereoAngle),
+			    openGLGet_stereoAngle());
+
+#if GTK_MINOR_VERSION > 5
+  names = openGLGet_allRenderingModesI18n();
+  valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(renderingListStore), &iter);
+  while(valid)
+    {
+      gtk_tree_model_get(GTK_TREE_MODEL(renderingListStore), &iter,
+			 RENDERING_POINTER_TO, &ext,
+			 RENDERING_MODE, &name, -1);
+      if (ext->isSensitiveToRenderingMode)
+	{
+	  if (ext->preferedRenderingMode < nb_renderingModes &&
+	      strcmp(names[ext->preferedRenderingMode], name))
+	    gtk_list_store_set(renderingListStore, &iter,
+			       RENDERING_MODE,
+			       names[ext->preferedRenderingMode], -1);
+	  else if (ext->preferedRenderingMode == followGeneralSetting &&
+		   strcmp(OPENGL_FOLLOW_GLOBAL_RENDERING_MODE, name))
+	    gtk_list_store_set(renderingListStore, &iter,
+			       RENDERING_MODE,
+			       OPENGL_FOLLOW_GLOBAL_RENDERING_MODE, -1);
+	}
+      valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(renderingListStore), &iter);
+    }
+#endif
   disableCallbacksOpenGL = 0;
 }
-void addPresetOneLightClicked(GtkButton *button, gpointer data)
+void addPresetOneLightClicked(GtkButton *button _U_, gpointer data)
 {
   Light *light;
   gboolean status;
@@ -673,7 +806,7 @@ void addPresetOneLightClicked(GtkButton *button, gpointer data)
   if (status)
     g_signal_emit(visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id, 0, NULL);
 }
-void addPresetFourLightsClicked(GtkButton *button, gpointer data)
+void addPresetFourLightsClicked(GtkButton *button _U_, gpointer data)
 {
   Light *light;
   gboolean status;
@@ -708,7 +841,8 @@ void addPresetFourLightsClicked(GtkButton *button, gpointer data)
   if (status)
     g_signal_emit(visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id, 0, NULL);
 }
-static void renderingModeEdited(GtkCellRendererText *cellrenderertext,
+#if GTK_MINOR_VERSION > 5
+static void renderingModeEdited(GtkCellRendererText *cellrenderertext _U_,
 				gchar *path, gchar *text, gpointer user_data)
 {
   GtkListStore *list;
@@ -718,6 +852,9 @@ static void renderingModeEdited(GtkCellRendererText *cellrenderertext,
   int mode;
   OpenGLExtension *ext;
 
+  if (disableCallbacksOpenGL)
+    return;
+
   list = GTK_LIST_STORE(user_data);
   g_return_if_fail(list);
 
@@ -728,7 +865,7 @@ static void renderingModeEdited(GtkCellRendererText *cellrenderertext,
   if (!strcmp(text, OPENGL_FOLLOW_GLOBAL_RENDERING_MODE))
     {
       valid = TRUE;
-      mode = -1;
+      mode = followGeneralSetting;
     }
   else
     {
@@ -736,23 +873,25 @@ static void renderingModeEdited(GtkCellRendererText *cellrenderertext,
       names = openGLGet_allRenderingModesI18n();
       for (mode = 0; names[mode] && !valid; mode++)
 	valid = !strcmp(text, names[mode]);
+      mode -= 1;
     }
   g_return_if_fail(valid);
 
   gtk_tree_model_get(GTK_TREE_MODEL(list), &iter, 
 		     RENDERING_POINTER_TO, &ext, -1);
-  valid = OpenGLExtensionSet_preferedRenderingMode(ext, mode - 1);
+  valid = OpenGLExtensionSet_preferedRenderingMode(ext, mode);
 
-  if (mode == -1)
+  if (mode == followGeneralSetting)
     gtk_list_store_set(list, &iter,
 		       RENDERING_MODE, OPENGL_FOLLOW_GLOBAL_RENDERING_MODE, -1);
   else
     {
       names = openGLGet_allRenderingModesI18n();
       gtk_list_store_set(list, &iter,
-			 RENDERING_MODE, names[mode - 1], -1);
+			 RENDERING_MODE, names[mode], -1);
     }
 
-  if (valid)
+  if (valid && ext->used)
     g_signal_emit(visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id, 0, NULL);
 }
+#endif
