@@ -49,10 +49,23 @@
 #include <gtk/gtk.h>
 #include <visu_data.h>
 #include <coreTools/toolOptions.h>
-
+#include <extraFunctions/plane.h>
+#include <extraGtkFunctions/gtk_toolPanelWidget.h>
 
 GtkWidget *panelSurfaces;
 
+/**
+ * panelIsosurfacesInit:
+ *
+ * Should be used in the list declared in externalModules.h to be loaded by
+ * V_Sim on start-up. This routine will create the #ToolPanel where the iso-surfaces
+ * stuff can be done, such as creating a surface, loading a scalar field,
+ * changing the properties...
+ *
+ * Returns: a newly created #ToolPanel object.
+ */
+ToolPanel* panelIsosurfacesInit();
+
 /**
  * panelIsosurfacesSet_used:
  * @dataObj: the current #VisuData (can be NULL) ;
@@ -83,7 +96,7 @@ int panelIsosurfacesLoad_file(const char* file_name, gboolean fit_to_box,
 			      VisuData *data, OptionTable *table);
 
 /**
- * panelIsosurfacesSet_showAll:
+ * panelIsosurfacesShow_all:
  * @show: TRUE to show all surfaces, FALSE to hide them.
  * 
  * Shows or hides all surfaces and check their "draw" status in the panel accordingly.
@@ -104,13 +117,11 @@ gboolean isosurfaces_show_next();
 
 /**
  * panelIsosurfacesEdit_surfaceProperties:
- * @flag: if TRUE, edit the properties of the selected surface only (or all
- *        surfaces of one selected file only). If FALSE, don't take into
- *        account the selection and change properties of all surfaces.
+ * @iter: the currently selected row iter (or NULL).
  *
  * Opens a new window allowing to edit surface properties.
  **/
-void panelIsosurfacesEdit_surfaceProperties(gboolean flag);
+void panelIsosurfacesEdit_surfaceProperties(GtkTreeIter *iter);
 
 /**
  * panelIsosurfacesAdd_withValue:
@@ -151,4 +162,48 @@ typedef enum
  */
 GtkListStore* panelIsosurfacesGet_listStore();
 
+/**
+ * panelIsosurfacesHide:
+ * @planes: an array of planes to be applied (NULL terminated).
+ *
+ * Must be called after the initialisation of the plane subpanel. It applies the
+ * masking scheme of planes on current surfaces.
+ *
+ * Returns: TRUE if the surface should be rebuilt.
+ */
+gboolean panelIsosurfacesHide(Plane **planes);
+
+/**
+ * SURFACE_NAME_STR:
+ *
+ * The default string used to name surfaces that are not associated to
+ * any public surface ressource.
+ */
+#define SURFACE_NAME_STR    "Choose an id name"
+/**
+ * SURFACE_NAME_CHOOSE:
+ *
+ * The string used in the tree view to represent the surfaces that
+ * don't share surface resources.
+ */
+#define SURFACE_NAME_CHOOSE "<span size=\"smaller\"><i>"SURFACE_NAME_STR"</i></span>"
+
+/**
+ * panelIsosurfacesGenerate_isoValues:
+ * @nbValues: a location of an integer to store the number of generated values ;
+ * @values: a pointer on a float array. The target of this pointer must be
+ *          NULL and it will be allocated after a call to this method. Use
+ *          g_free() after use to free it.
+ * @name: a pointer to store a name. The target of this pointer must be
+ *        NULL on enter. It is associated only if a name is given.
+ * @minVal: the minimum value for the range ;
+ * @maxVal: the maximum value for the range.
+ * 
+ * This method opens a little dialog window that is made to help the
+ * user enter a list of values for creation of iso-surfaces. These values
+ * are generated between @minVal and @maxVal.
+ */
+void panelIsosurfacesGenerate_isoValues(int *nbValues, float **values, gchar **name,
+					float minVal, float maxVal);
+
 #endif
