@@ -49,6 +49,8 @@
 #include <math.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
+#include <visu_gtk.h>
 #include <extraFunctions/pot2surf.h>
 #include <extraFunctions/surfaces.h>
 #include <gtk_renderingWindowWidget.h>
@@ -270,7 +272,7 @@ void surf_open_file_chooser()
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 				NULL);
-  gchar *directory = getLastOpenDirectory();
+  gchar *directory = gtkMainClassGet_lastOpenDirectory();
 
 
   if (directory)
@@ -301,7 +303,9 @@ void surf_open_file_chooser()
 	    {
 	      gtk_entry_set_text(GTK_ENTRY(entry_current_file), "");
 	      gtk_list_store_clear(list_store); 
-	      raiseAlertDialog(_("Unable to parse the selected file."));
+	      visuGtkRaise_warning(_("Loading a file"),
+				   _("Unable to parse the selected file."),
+				   (GtkWindow*)0);
 	    }
 	  break;
 	case GTK_RESPONSE_CANCEL:
@@ -314,7 +318,7 @@ void surf_open_file_chooser()
 
 /* Opens a file chooser asking the user to select a surf file and 
    set the correspondig entry to selected file. */
-void surfmerge_target_file_chooser(GtkWidget *widget, gpointer data)
+void surfmerge_target_file_chooser(GtkWidget *widget _U_, gpointer data _U_)
 {
   GtkFileFilter *filter_surf = gtk_file_filter_new();
   GtkFileFilter *filter_none = gtk_file_filter_new();
@@ -324,7 +328,7 @@ void surfmerge_target_file_chooser(GtkWidget *widget, gpointer data)
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 				NULL);
-  gchar *directory = getLastOpenDirectory();
+  gchar *directory = gtkMainClassGet_lastOpenDirectory();
 
   if (directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_chooser), directory);
@@ -359,18 +363,18 @@ void surfmerge_target_file_chooser(GtkWidget *widget, gpointer data)
 
 /* Callback for when a user specifies a custom name for a surface.
    Register the given name as the new name of the surface. */
-void surfmerge_update_surf_name(GtkCellRendererText *cellrenderertext,
-			       gchar *path, gchar *text, gpointer user_data)
+void surfmerge_update_surf_name(GtkCellRendererText *cellrenderertext _U_,
+			       gchar *path, gchar *text, gpointer user_data _U_)
 {
   GtkTreeIter iter;
   if(gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(list_store_target),
 					 &iter, path))
-    gtk_list_store_set(list_store_target, &iter,
-		       MERGE_NEW_SURF_NAME, g_strdelimit(text, " ", '_'), -1);
+      gtk_list_store_set(list_store_target, &iter,
+			 MERGE_NEW_SURF_NAME, g_strdelimit(g_strstrip(text), " ", '_'), -1);
 }
 
 /* Callback for when a user removes a surface from the list */  
-void surfmerge_remove_surf(GtkWidget *button, gpointer data)
+void surfmerge_remove_surf(GtkWidget *button _U_, gpointer data _U_)
 {
   GtkTreeIter iter, iter2;
   GtkTreeModel *model;
@@ -395,7 +399,7 @@ void surfmerge_remove_surf(GtkWidget *button, gpointer data)
    data == GINT_TO_POINTER(1) : move up
    data == GINT_TO_POINTER(-1) : move down
 */
-void surfmerge_move_surf(GtkWidget *button, gpointer data)
+void surfmerge_move_surf(GtkWidget *button _U_, gpointer data)
 {
   GtkTreeIter iter, iter2;
   GtkTreeModel *model;
@@ -561,7 +565,7 @@ void surf_add_surf_to_target()
     
 /* Only used with a gtk_tree_model_foreach()
    Used to count the total of surfaces, nb_vertex, and nb_points to build a correct .surf file. */
-gboolean surfmerge_init_export(GtkTreeModel *model, GtkTreePath *path,
+gboolean surfmerge_init_export(GtkTreeModel *model, GtkTreePath *path _U_,
 			  GtkTreeIter *iter, gpointer data)
 {
   int *total = data;
@@ -581,7 +585,7 @@ gboolean surfmerge_init_export(GtkTreeModel *model, GtkTreePath *path,
 
 /* Only used with a gtk_tree_model_foreach()
    Used to write a surface in the file the user wants to write. */
-gboolean surf_export_surf(GtkTreeModel *model, GtkTreePath *path,
+gboolean surf_export_surf(GtkTreeModel *model, GtkTreePath *path _U_,
 			  GtkTreeIter *iter, gpointer data)
 {
   FILE *file_to_write = data;
@@ -616,7 +620,9 @@ gboolean surf_export_surf(GtkTreeModel *model, GtkTreePath *path,
 	  g_free(old_surf_name);
 	  g_string_free(line, TRUE);
 	  g_io_channel_unref(file_to_read);
-	  raiseAlertDialog(_("An unknown error occured. Your surf file is corrupted.\n"));
+	  visuGtkRaise_warning(_("Loading a file"),
+			       _("An unknown error occured. Your surf"
+				 " file is corrupted.\n"), (GtkWindow*)0);
 	  return TRUE;
 	}
     }
@@ -634,14 +640,16 @@ gboolean surf_export_surf(GtkTreeModel *model, GtkTreePath *path,
 	  g_free(old_surf_name);
 	  g_string_free(line, TRUE);
 	  g_io_channel_unref(file_to_read);
-	  raiseAlertDialog(_("An unknown error occured. Your surf file is corrupted.\n"));
+	  visuGtkRaise_warning(_("Loading a file"),
+			       _("An unknown error occured. Your surf"
+				 " file is corrupted.\n"), (GtkWindow*)0);
 	  return TRUE;
 	}
       cur_surf_name = g_strdup(line->str);
       g_strdelimit(cur_surf_name, "\n", ' ');       
       g_strstrip(cur_surf_name);
       
-      fprintf(stderr, "%s, old:%s\n", cur_surf_name, old_surf_name);
+      DBG_fprintf(stderr, "%s, old:%s\n", cur_surf_name, old_surf_name);
       if(strcmp(cur_surf_name, old_surf_name) == 0)
 	  fprintf(file_to_write, "%s\n", surf_name);
 
@@ -655,10 +663,12 @@ gboolean surf_export_surf(GtkTreeModel *model, GtkTreePath *path,
 	  g_free(old_surf_name);
 	  g_string_free(line, TRUE);
 	  g_io_channel_unref(file_to_read);
-	  raiseAlertDialog(_("An unknown error occured. Your surf file is corrupted.\n"));
+	  visuGtkRaise_warning(_("Loading a file"),
+			       _("An unknown error occured. Your surf"
+				 " file is corrupted.\n"), (GtkWindow*)0);
 	  return TRUE;
 	}
-      fprintf(stderr, "%s, old:%s\n", cur_surf_name, old_surf_name);
+      DBG_fprintf(stderr, "%s, old:%s\n", cur_surf_name, old_surf_name);
       if(strcmp(cur_surf_name, old_surf_name) == 0)
       	fprintf(file_to_write, "%s", line->str);
 
@@ -673,20 +683,22 @@ gboolean surf_export_surf(GtkTreeModel *model, GtkTreePath *path,
 	      g_free(old_surf_name);
 	      g_string_free(line, TRUE);
 	      g_io_channel_unref(file_to_read);
-	      raiseAlertDialog(_("An unknown error occured. Your surf file is corrupted.\n"));
+	      visuGtkRaise_warning(_("Loading a file"),
+				   _("An unknown error occured. Your surf"
+				     " file is corrupted.\n"), (GtkWindow*)0);
 	      return TRUE;
 	    }
 	  if(strcmp(cur_surf_name, old_surf_name) == 0)
 	    fprintf(file_to_write, "%s", line->str);
 	}
-      fprintf(stderr, "%s, old:%s\n", cur_surf_name, old_surf_name);
+      DBG_fprintf(stderr, "%s, old:%s\n", cur_surf_name, old_surf_name);
       if(strcmp(cur_surf_name, old_surf_name) == 0)
 	{
 	  g_free(cur_surf_name);
 	  break;
 	}
       g_free(cur_surf_name);
-      fprintf(stderr, "%s, old:%s\n", cur_surf_name, old_surf_name);
+      DBG_fprintf(stderr, "%s, old:%s\n", cur_surf_name, old_surf_name);
     }
   g_free(file_name);
   g_free(old_surf_name);
@@ -706,7 +718,8 @@ void surf_gogogo()
 
   if(file_to_write == NULL)
     {
-      raiseAlertDialog(_("Please choose a surf file to write\n"));
+      visuGtkRaise_warning(_("Saving a file"),
+			   _("Please choose a surf file to write\n"), (GtkWindow*)0);
       return;
     }
 
@@ -718,7 +731,8 @@ void surf_gogogo()
 
   if(total[0] == 0)
     {
-      raiseAlertDialog(_("No surface to write\n"));
+      visuGtkRaise_warning(_("Saving a file"),
+			   _("No surface to write\n"), (GtkWindow*)0);
       return;
     }
 
@@ -745,7 +759,8 @@ void surf_gogogo()
 
   fclose(file_to_write);
 
-  fprintf(stderr, "Surface Merger : wrote file %s\n", gtk_entry_get_text(GTK_ENTRY(entry_target_file)));
+  DBG_fprintf(stderr, "Surface Merger : wrote file %s\n",
+	      gtk_entry_get_text(GTK_ENTRY(entry_target_file)));
 }
 
 /* See header file for more info */
@@ -953,8 +968,6 @@ static GtkWidget *pot2surf_entry_target_surf_file = NULL;
 static GtkWidget *pot2surf_potmax = NULL;
 static GtkWidget *pot2surf_potmin = NULL;
 
-static int pot2surf_counter = 0;
-
 enum {
   POT2SURF_NAME,
   POT2SURF_VALUE,
@@ -966,8 +979,6 @@ enum {
 void pot2surf_popup_help()
 {
   GtkWidget *window;
-  GIOChannel* help_file = NULL;
-  GError *error = NULL;
   gsize length;
   gchar *text = NULL;
   GtkTextBuffer *text_buf;
@@ -975,25 +986,19 @@ void pot2surf_popup_help()
   GtkWidget *scrolled_window;
   gchar *chemin = g_build_filename(v_sim_legal_dir, "pot2surf_help", NULL);
 
-  if((help_file = g_io_channel_new_file(chemin, "r", &error)) == NULL)
+  if(!g_file_get_contents(chemin, &text, &length, (GError**)0))
     {
       g_free(chemin);
       return;
     }
-
   g_free(chemin);
-  if(g_io_channel_read_to_end(help_file, &text, &length, &error) != G_IO_STATUS_NORMAL)
-    return;
 
   text_buf = gtk_text_buffer_new(NULL);
   text_view = gtk_text_view_new_with_buffer(text_buf);
   scrolled_window = gtk_scrolled_window_new(NULL, NULL);
 
-  
-  g_io_channel_shutdown (help_file, FALSE, &error);
-  g_io_channel_unref (help_file);
-
   gtk_text_buffer_insert_at_cursor(text_buf, text, -1);
+  g_free(text);
 
   /* Create a new dialog window for the scrolled window to be
    * packed into.  */
@@ -1039,7 +1044,8 @@ gboolean pot2surf_quick_parse(char *file_name, float *pot_min, float *pot_max)
 
   if(file_to_parse == NULL)
     {
-      raiseAlertDialog(_("Can't open given file for reading"));
+      visuGtkRaise_warning(_("Loading a file"),
+			   _("Can't open given file for reading"), (GtkWindow*)0);
       g_io_channel_unref(file_to_parse);
       return FALSE;
     }
@@ -1051,7 +1057,9 @@ gboolean pot2surf_quick_parse(char *file_name, float *pot_min, float *pot_max)
 	g_io_channel_read_line_string (file_to_parse, line, &terminator_pos, NULL);
       if(last_op_status != G_IO_STATUS_NORMAL)
 	{
-	  raiseAlertDialog(_("This file doesn't seem to be a correct pot file"));	 
+	  visuGtkRaise_warning(_("Loading a file"),
+			       _("This file doesn't seem to be a correct pot file"),
+			       (GtkWindow*)0);	 
 	  g_io_channel_unref(file_to_parse);
 	  return FALSE;
 	}
@@ -1060,7 +1068,9 @@ gboolean pot2surf_quick_parse(char *file_name, float *pot_min, float *pot_max)
 	  sscanf(line->str, "%d %d %d", &na, &nb, &nc);
 	  if(na <= 0 || nb <= 0 || nc <= 0)
 	    {
-	      raiseAlertDialog(_("Second line seem to contain incorrect values"));	  
+	      visuGtkRaise_warning(_("Loading a file"),
+				   _("Second line seem to contain incorrect values"),
+				   (GtkWindow*)0);	  
 	      g_io_channel_unref(file_to_parse);
 	      return FALSE;
 	    }
@@ -1075,7 +1085,9 @@ gboolean pot2surf_quick_parse(char *file_name, float *pot_min, float *pot_max)
 	g_io_channel_read_line_string (file_to_parse, line, &terminator_pos, NULL);
       if(last_op_status != G_IO_STATUS_NORMAL)
 	{
-	  raiseAlertDialog(_("This file doesn't seem to be a correct pot file"));	  
+	  visuGtkRaise_warning(_("Loading a file"),
+			       _("This file doesn't seem to be a correct pot file"),
+			       (GtkWindow*)0);	  
 	  g_io_channel_unref(file_to_parse);
 	  return FALSE;
 	}
@@ -1097,7 +1109,6 @@ gboolean pot2surf_quick_parse(char *file_name, float *pot_min, float *pot_max)
     }
   g_io_channel_unref(file_to_parse);
 
-  pot2surf_counter = 0;
   return TRUE;
 }
 
@@ -1105,7 +1116,7 @@ gboolean pot2surf_quick_parse(char *file_name, float *pot_min, float *pot_max)
    This file is parsed using  pot2surf_quick_parse() to try to retrieve pot_in and pot_max.
    If parsing is successful, set entries to default values (ie : file_selected.surf and
    file_selected.instruc) */
-void pot2surf_select_pot_file(GtkWidget *widget, gpointer data)
+void pot2surf_select_pot_file(GtkWidget *widget _U_, gpointer data _U_)
 {
   GtkFileFilter *filter_pot = gtk_file_filter_new();
   GtkFileFilter *filter_none = gtk_file_filter_new();
@@ -1115,13 +1126,14 @@ void pot2surf_select_pot_file(GtkWidget *widget, gpointer data)
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 /* 				GTK_STOCK_HELP, GTK_RESPONSE_HELP, */
 				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
-  gchar *directory = getLastOpenDirectory();
+  gchar *directory = gtkMainClassGet_lastOpenDirectory();
 
   if (directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_chooser), directory);
 
   gtk_file_filter_add_pattern(filter_pot, "*.pot");
-  gtk_file_filter_set_name(filter_pot, "Potential files (*.pot)");  
+  gtk_file_filter_add_pattern(filter_pot, "*.dat");
+  gtk_file_filter_set_name(filter_pot, "Potential files (*.pot, *.dat)");  
   gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(file_chooser), filter_pot);
 
   gtk_file_filter_add_pattern(filter_none, "*");
@@ -1156,7 +1168,7 @@ void pot2surf_select_pot_file(GtkWidget *widget, gpointer data)
 	      g_string_free(other_files, TRUE);
 
 	      directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser));
-	      setLastOpenDirectory(directory);
+	      gtkMainClassSet_lastOpenDirectory(directory);
 	      g_free(directory);
 	      gtk_list_store_clear(pot2surf_list_store);
 	      gtk_widget_destroy(file_chooser);
@@ -1168,7 +1180,7 @@ void pot2surf_select_pot_file(GtkWidget *widget, gpointer data)
 	case GTK_RESPONSE_CANCEL:
 	default:
 	  directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser));
-	  setLastOpenDirectory(directory);
+	  gtkMainClassSet_lastOpenDirectory(directory);
 	  g_free(directory);
 	  gtk_widget_destroy (file_chooser);
 	  return;
@@ -1187,7 +1199,7 @@ void pot2surf_select_pot_file(GtkWidget *widget, gpointer data)
 /* 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, */
 /* /\* 				GTK_STOCK_HELP, GTK_RESPONSE_HELP, *\/ */
 /* 				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL); */
-/*   gchar *directory = getLastOpenDirectory(); */
+/*   gchar *directory = gtkMainClassGet_lastOpenDirectory(); */
 
 /*   if (directory) */
 /*     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_chooser), directory); */
@@ -1209,7 +1221,7 @@ void pot2surf_select_pot_file(GtkWidget *widget, gpointer data)
 /* 	    gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser)); */
 /* 	  gtk_entry_set_text(GTK_ENTRY(pot2surf_entry_source_instruc_file), selected_file); */
 /*       directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser)); */
-/*       setLastOpenDirectory(directory); */
+/*       gtkMainClassSet_lastOpenDirectory(directory); */
 /*       g_free(directory); */
 /* 	  gtk_widget_destroy(file_chooser); */
 /* 	  return; */
@@ -1218,7 +1230,7 @@ void pot2surf_select_pot_file(GtkWidget *widget, gpointer data)
 /* 	case GTK_RESPONSE_CANCEL: */
 /* 	default: */
 /*       directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser)); */
-/*       setLastOpenDirectory(directory); */
+/*       gtkMainClassSet_lastOpenDirectory(directory); */
 /*       g_free(directory); */
 /* 	  gtk_widget_destroy (file_chooser); */
 /* 	  return; */
@@ -1227,7 +1239,7 @@ void pot2surf_select_pot_file(GtkWidget *widget, gpointer data)
 /* } */
 
 /* Opens a file chooser to select a surf file. */
-void pot2surf_select_surf_file(GtkWidget *widget, gpointer data)
+void pot2surf_select_surf_file(GtkWidget *widget _U_, gpointer data _U_)
 {
   GtkFileFilter *filter_pot = gtk_file_filter_new ();
   GtkFileFilter *filter_none = gtk_file_filter_new();
@@ -1237,7 +1249,7 @@ void pot2surf_select_surf_file(GtkWidget *widget, gpointer data)
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 /* 				GTK_STOCK_HELP, GTK_RESPONSE_HELP, */
 				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
-  gchar *directory = getLastOpenDirectory();
+  gchar *directory = gtkMainClassGet_lastOpenDirectory();
 
   if (directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_chooser), directory);
@@ -1261,7 +1273,7 @@ void pot2surf_select_surf_file(GtkWidget *widget, gpointer data)
 	    gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser));
 	  gtk_entry_set_text(GTK_ENTRY(pot2surf_entry_target_surf_file), selected_file);
       directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser));
-      setLastOpenDirectory(directory);
+      gtkMainClassSet_lastOpenDirectory(directory);
       g_free(directory);
 	  gtk_widget_destroy(file_chooser);
 	  return;
@@ -1270,7 +1282,7 @@ void pot2surf_select_surf_file(GtkWidget *widget, gpointer data)
 	case GTK_RESPONSE_CANCEL:
 	default:
       directory = (char*)gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser));
-      setLastOpenDirectory(directory);
+      gtkMainClassSet_lastOpenDirectory(directory);
       g_free(directory);
 	  gtk_widget_destroy (file_chooser);
 	  return;
@@ -1279,19 +1291,29 @@ void pot2surf_select_surf_file(GtkWidget *widget, gpointer data)
 }
 
 /* Callback for when a surf name is changed in the gtk_tree_view */
-void pot2surf_update_surf_name(GtkCellRendererText *cellrenderertext,
-			       gchar *path, gchar *text, gpointer user_data)
+void pot2surf_update_surf_name(GtkCellRendererText *cellrenderertext _U_,
+			       gchar *path, gchar *text, gpointer user_data _U_)
 {
   GtkTreeIter iter;
-  if(gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(pot2surf_list_store),
-					 &iter, path))
-      gtk_list_store_set(pot2surf_list_store, &iter,
-			 POT2SURF_NAME, g_strdelimit(g_strstrip(text), " ", '_'), -1);
+
+  g_return_if_fail(gtk_tree_model_get_iter_from_string
+		   (GTK_TREE_MODEL(pot2surf_list_store), &iter, path));
+
+  DBG_fprintf(stderr, "Panel SurfacesTools: entered text '%s'.\n", text);
+  if (!strcmp(text, SURFACE_NAME_STR))
+    gtk_list_store_set(pot2surf_list_store, &iter,
+		       POT2SURF_NAME, SURFACE_NAME_CHOOSE, -1);
+  else if (text && text[0])
+    gtk_list_store_set(pot2surf_list_store, &iter,
+		       POT2SURF_NAME, g_strdelimit(text, " ", '_'), -1);
+  else
+    gtk_list_store_set(pot2surf_list_store, &iter,
+		       POT2SURF_NAME, SURFACE_NAME_CHOOSE, -1);
 }
 
 /* Callback for when the pot of a surf to build is changed in the gtk_tree_view */
-void pot2surf_update_surf_value(GtkCellRendererText *cellrenderertext,
-				gchar *path, gchar *text, gpointer user_data)
+void pot2surf_update_surf_value(GtkCellRendererText *cellrenderertext _U_,
+				gchar *path, gchar *text, gpointer user_data _U_)
 {
   GtkTreeIter iter;
   float value;
@@ -1318,7 +1340,7 @@ void pot2surf_create_tree_view()
   GtkCellRenderer* surf_value = gtk_cell_renderer_text_new ();  
   GtkTreeViewColumn* column1 
     = gtk_tree_view_column_new_with_attributes(_("Surface name"), surf_name,
-					      "text", POT2SURF_NAME, NULL);
+					      "markup", POT2SURF_NAME, NULL);
   GtkTreeViewColumn* column2
     = gtk_tree_view_column_new_with_attributes(_("Pot value"), surf_value,
 					       "text", POT2SURF_VALUE, NULL);
@@ -1326,8 +1348,8 @@ void pot2surf_create_tree_view()
   pot2surf_list_store = gtk_list_store_new (POT2SURF_N_COLUMNS, G_TYPE_STRING, G_TYPE_FLOAT);
   pot2surf_tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pot2surf_list_store));
 
-  g_object_set(G_OBJECT(surf_name), "editable", TRUE, NULL);
-  g_object_set(G_OBJECT(surf_value), "editable", TRUE, NULL);
+  g_object_set(G_OBJECT(surf_name), "editable", TRUE, "foreground", "blue", NULL);
+  g_object_set(G_OBJECT(surf_value), "editable", TRUE, "foreground", "blue", NULL);
 
   gtk_tree_view_column_set_expand (column1, TRUE);
   gtk_tree_view_column_set_expand(column2, FALSE);
@@ -1341,21 +1363,18 @@ void pot2surf_create_tree_view()
 }
   
 /* Adds a new surface to the list of surfaces to build from a .pot file. */
-void pot2surf_add_surf(GtkWidget *button, gpointer data)
+void pot2surf_add_surf(GtkWidget *button _U_, gpointer data _U_)
 {
   GtkTreeIter iter, new_row;
   GtkTreeModel *model;
   GtkTreeSelection* tree_selection = 
     gtk_tree_view_get_selection(GTK_TREE_VIEW(pot2surf_tree_view));
-  gchar *surface;
 
   if(strcmp("", gtk_entry_get_text(GTK_ENTRY(pot2surf_entry_source_pot_file))) == 0)
     return;
 
   tree_selection = 
-    gtk_tree_view_get_selection(GTK_TREE_VIEW(pot2surf_tree_view));  
-  surface = g_strdup_printf("Surface %d", pot2surf_counter);
-  
+    gtk_tree_view_get_selection(GTK_TREE_VIEW(pot2surf_tree_view));    
 
   if (gtk_tree_selection_get_selected (tree_selection, &model, &iter))
     gtk_list_store_insert_after(pot2surf_list_store, &new_row, &iter);
@@ -1364,13 +1383,10 @@ void pot2surf_add_surf(GtkWidget *button, gpointer data)
 
   gtk_tree_selection_select_iter(tree_selection, &new_row);
   gtk_list_store_set(pot2surf_list_store, &new_row, 
-		     POT2SURF_NAME, surface,
+		     POT2SURF_NAME, SURFACE_NAME_CHOOSE,
 		     POT2SURF_VALUE, 
 		     (atof(gtk_entry_get_text(GTK_ENTRY(pot2surf_potmin))) 
 		      + atof(gtk_entry_get_text(GTK_ENTRY(pot2surf_potmax)))) / 2, -1);
-  
-  g_free(surface);  
-  pot2surf_counter++;
 }
 
 void onSpecialModeToggled(GtkToggleButton *button, gpointer data)
@@ -1378,119 +1394,45 @@ void onSpecialModeToggled(GtkToggleButton *button, gpointer data)
   gtk_widget_set_sensitive(GTK_WIDGET(data), gtk_toggle_button_get_active(button));
 }
 
-void pot2surf_special_add_surf(GtkWidget *button, gpointer data)
+void pot2surf_special_add_surf(GtkWidget *button _U_, gpointer data _U_)
 {
-  GtkWidget *dialog, *table;
-  float pot_min;
-  float pot_max;
-  float value, incr;
-
-  GtkWidget *spin_start;
-  GtkWidget *spin_end;
-  GtkWidget *spin_step;
-  GtkWidget *spin_delta;
-  GtkWidget *radioStep, *radioDelta;
-
-  GSList *radiobuttonCycle_group;
+  float *values, pot_min, pot_max;
+  int nbValues, i;
+  gchar *name;
+  GtkTreeIter new_row;
+  GtkTreeSelection* tree_selection = 
+    gtk_tree_view_get_selection(GTK_TREE_VIEW(pot2surf_tree_view));
 
   if(strcmp("", gtk_entry_get_text(GTK_ENTRY(pot2surf_entry_source_pot_file))) == 0)
     return;
   
-  dialog = gtk_dialog_new_with_buttons (_("Special add surfs"), NULL, GTK_DIALOG_MODAL,
-					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
   pot_min = g_ascii_strtod(gtk_entry_get_text(GTK_ENTRY(pot2surf_potmin)), NULL);
   pot_max = g_ascii_strtod(gtk_entry_get_text(GTK_ENTRY(pot2surf_potmax)), NULL);
-  spin_start = gtk_spin_button_new_with_range(pot_min, pot_max, 0.0000001);
-  spin_end = gtk_spin_button_new_with_range(pot_min, pot_max, 0.0000001);
-  spin_step = gtk_spin_button_new_with_range(2, 99, 1);
-  spin_delta = gtk_spin_button_new_with_range(0.000001, pot_max - pot_min,
-					      (pot_max - pot_min) / 200);
 
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_end), pot_max);
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_delta), (pot_max - pot_min) / 20);
+  values = (float*)0;
+  name = (gchar*)0;
+  panelIsosurfacesGenerate_isoValues(&nbValues, &values, &name,
+				     pot_min, pot_max);
 
-  table = gtk_table_new(2, 2, FALSE);
-
-  radioStep = gtk_radio_button_new_with_label(NULL, _("Number of steps:"));
-  gtk_radio_button_set_group(GTK_RADIO_BUTTON(radioStep), (GSList*)0);
-  radiobuttonCycle_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radioStep));
-  gtk_table_attach(GTK_TABLE(table), radioStep, 0, 1, 0, 1,
-                   (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0);
-  g_signal_connect(G_OBJECT(radioStep), "toggled",
-		   G_CALLBACK(onSpecialModeToggled), (gpointer)spin_step);
-
-  radioDelta = gtk_radio_button_new_with_label(NULL, _("Delta of steps:"));
-  gtk_radio_button_set_group(GTK_RADIO_BUTTON(radioDelta), radiobuttonCycle_group);
-  radiobuttonCycle_group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(radioDelta));
-  gtk_table_attach(GTK_TABLE(table), radioDelta, 1, 2, 0, 1,
-                   (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0);
-  g_signal_connect(G_OBJECT(radioDelta), "toggled",
-		   G_CALLBACK(onSpecialModeToggled), (gpointer)spin_delta);
-
-  gtk_table_attach(GTK_TABLE(table), spin_step, 0, 1, 1, 2,
-                   GTK_FILL | GTK_EXPAND, (GtkAttachOptions)0, 0, 0);
-  gtk_table_attach(GTK_TABLE(table), spin_delta, 1, 2, 1, 2,
-                   GTK_FILL | GTK_EXPAND, (GtkAttachOptions)0, 0, 0);
-
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), gtk_label_new(_("Start :")), FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), spin_start, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), gtk_label_new(_("End :")), FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), spin_end, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), table, FALSE, FALSE, 0);
-
-  gtk_widget_set_sensitive(spin_delta, FALSE);
-
-  gtk_widget_show_all(dialog);
-
-  switch(gtk_dialog_run(GTK_DIALOG(dialog)))
+  for (i = 0; i < nbValues; i++)
     {
-    case GTK_RESPONSE_ACCEPT:
-      {
-	GtkTreeIter new_row;
-	GtkTreeSelection* tree_selection = 
-	  gtk_tree_view_get_selection(GTK_TREE_VIEW(pot2surf_tree_view));
-	float start = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin_start));
-	float end = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin_end));
-	int step = floor(gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin_step)));
-
-	value = start;
-	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radioStep)))
-	  {
-	    if (start == pot_min)
-	      step += 1;
-	    incr = (end - start) / step;
-	  }
-	else
-	  incr = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin_delta));
-	while ((incr > 0 && value < end) || (incr < 0 && value > end))
-	  {
-	    gchar *surface;
-	    
-	    value += incr;
-	    if (value > pot_min && value < pot_max)
-	      {
-		surface = g_strdup_printf("Surface %d", pot2surf_counter);
-		gtk_list_store_append(pot2surf_list_store, &new_row);
-		gtk_list_store_set(pot2surf_list_store, &new_row, 
-				   POT2SURF_NAME, surface,
-				   POT2SURF_VALUE, value, -1);
-		gtk_tree_selection_select_iter(tree_selection, &new_row);
-		g_free(surface);  
-		pot2surf_counter++;
-	      }
-	  }
-  
-      }
-    default:
-      gtk_widget_destroy(dialog);
-      return;
+      gtk_list_store_append(pot2surf_list_store, &new_row);
+      gtk_list_store_set(pot2surf_list_store, &new_row, 
+			 POT2SURF_NAME, (name && name[0])?name:SURFACE_NAME_CHOOSE,
+			 POT2SURF_VALUE, values[i], -1);
     }
+  if (nbValues)
+    gtk_tree_selection_select_iter(tree_selection, &new_row);
+
+  if (values)
+    g_free(values);
+  if (name)
+    g_free(name);
 }
   
 
 /* Removes selected surface from the list of surfaces to build from a .pot file. */
-void pot2surf_remove_surf(GtkWidget *button, gpointer data)
+void pot2surf_remove_surf(GtkWidget *button _U_, gpointer data _U_)
 {
   GtkTreeIter iter, iter2;
   GtkTreeModel *model;
@@ -1515,7 +1457,7 @@ void pot2surf_remove_surf(GtkWidget *button, gpointer data)
    data == GINT_TO_POINTER(1) : move up
    data == GINT_TO_POINTER(-1) : move down
 */
-void pot2surf_move_surf(GtkWidget *button, gpointer data)
+void pot2surf_move_surf(GtkWidget *button _U_, gpointer data)
 {
   GtkTreeIter iter, iter2;
   GtkTreeModel *model;
@@ -1549,8 +1491,8 @@ void pot2surf_move_surf(GtkWidget *button, gpointer data)
 
 /* Only used through a gtk_tree_model_foreach()
    count the number of element present in a GtkTreeModel */
-gboolean pot2surf_count_surf(GtkTreeModel *model, GtkTreePath *path, 
-			     GtkTreeIter *iter, gpointer data)
+gboolean pot2surf_count_surf(GtkTreeModel *model _U_, GtkTreePath *path _U_,
+			     GtkTreeIter *iter _U_, gpointer data)
 {
   int *n = data;
   (*n)++;
@@ -1559,7 +1501,7 @@ gboolean pot2surf_count_surf(GtkTreeModel *model, GtkTreePath *path,
 
 /* Only used through a gtk_tree_model_foreach()
    Writes a surface name and value in an instruc file. */
-gboolean pot2surf_write_surf(GtkTreeModel *model, GtkTreePath *path, 
+gboolean pot2surf_write_surf(GtkTreeModel *model, GtkTreePath *path _U_,
 			     GtkTreeIter *iter, gpointer data)
 {
   FILE *instruc = data;
@@ -1586,16 +1528,20 @@ gboolean pot2surf_build_instruc_file(const gchar *pot_file, const gchar *surf_fi
   FILE *instruc = NULL;
   if(instruc_file == NULL || strcmp(instruc_file, "") == 0)
     {
-      raiseAlertDialog("Please select an instruc file to write");
+      visuGtkRaise_warning(_("Loading a file"),
+			   _("Please select an instruc file to write"),
+			   (GtkWindow*)0);
       return FALSE;
     }
   if((instruc = fopen(instruc_file, "w")) == NULL)
     {
-      raiseAlertDialog("Can't open selected instruc file for writing");
+      visuGtkRaise_warning(_("Loading a file"),
+			   _("Can't open selected instruc file for writing"),
+			   (GtkWindow*)0);
       return FALSE;
     }
 
-  fprintf(stderr, "pot2surf : writing/overwriting %s file...\n", instruc_file);
+  DBG_fprintf(stderr, "pot2surf : writing/overwriting %s file...\n", instruc_file);
 
   fprintf(instruc, "%s\n", pot_file);
   fprintf(instruc, "%s\n", surf_file);
@@ -1603,7 +1549,7 @@ gboolean pot2surf_build_instruc_file(const gchar *pot_file, const gchar *surf_fi
   
   gtk_tree_model_foreach(GTK_TREE_MODEL(pot2surf_list_store), pot2surf_write_surf, instruc);
 
-  fprintf(stderr, "pot2surf : ...finished to write file\n");
+  DBG_fprintf(stderr, "pot2surf : ...finished to write file\n");
 
   fclose(instruc);
 
@@ -1615,7 +1561,7 @@ void pot2surf_select_instruc_file_to_save()
   GtkFileFilter *filter_instruc;
   GtkFileFilter *filter_none;
   GtkWidget* file_chooser;
-  char *directory = getLastOpenDirectory();
+  char *directory = gtkMainClassGet_lastOpenDirectory();
   int n=0;
   const gchar *pot_file = gtk_entry_get_text(GTK_ENTRY(pot2surf_entry_source_pot_file));
   const gchar *surf_file = gtk_entry_get_text(GTK_ENTRY(pot2surf_entry_target_surf_file));
@@ -1624,17 +1570,20 @@ void pot2surf_select_instruc_file_to_save()
   gtk_tree_model_foreach(GTK_TREE_MODEL(pot2surf_list_store), pot2surf_count_surf, &n);
   if(n == 0)
     {
-      raiseAlertDialog("Please specify surfaces to draw");
+      visuGtkRaise_warning(_("Saving a file"),
+			   _("Please specify surfaces to draw"), (GtkWindow*)0);
       return;
     }
   if(strcmp(pot_file, "") == 0)
     {
-      raiseAlertDialog("Please select a source pot file");
+      visuGtkRaise_warning(_("Saving a file"),
+			   _("Please select a source pot file"), (GtkWindow*)0);
       return;
     }
   if(surf_file == NULL || strcmp(surf_file, "") == 0)
     {
-      raiseAlertDialog("Please select a target surf file");
+      visuGtkRaise_warning(_("Saving a file"),
+			   _("Please select a target surf file"), (GtkWindow*)0);
       return;
     }
 
@@ -1669,7 +1618,7 @@ void pot2surf_select_instruc_file_to_save()
 					gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser)), n))
 	   
 	  break;
-	setLastOpenDirectory(gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser)));
+	gtkMainClassSet_lastOpenDirectory(gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser)));
 	g_string_free(default_file, TRUE);
 	gtk_widget_destroy(file_chooser);
 	return;
@@ -1683,7 +1632,7 @@ void pot2surf_select_instruc_file_to_save()
 
 /* Only used through a gtk_tree_model_foreach()
    Writes a surface name and value in a struct surf array */
-gboolean pot2surf_get_surfs(GtkTreeModel *model, GtkTreePath *path, 
+gboolean pot2surf_get_surfs(GtkTreeModel *model, GtkTreePath *path _U_,
 			     GtkTreeIter *iter, gpointer data)
 {
   struct surf {
@@ -1703,7 +1652,7 @@ gboolean pot2surf_get_surfs(GtkTreeModel *model, GtkTreePath *path,
 }
 
 /* Tries to build a valid .instruc file and then to executes this file. */
-gboolean pot2surf_build_surf_file(GtkWidget *button, gpointer data)
+gboolean pot2surf_build_surf_file(GtkWidget *button _U_, gpointer data)
 {
   struct surf {
     char *name;
@@ -1719,17 +1668,20 @@ gboolean pot2surf_build_surf_file(GtkWidget *button, gpointer data)
   gtk_tree_model_foreach(GTK_TREE_MODEL(pot2surf_list_store), pot2surf_count_surf, &n);
   if(n == 0)
     {
-      raiseAlertDialog("Please specify surfaces to draw");
+      visuGtkRaise_warning(_("Saving a file"),
+			   _("Please specify surfaces to draw"), (GtkWindow*)0);
       return FALSE;
     }
   if(pot_file == NULL || strcmp(pot_file, "") == 0)
     {
-      raiseAlertDialog("Please select a source pot file");
+      visuGtkRaise_warning(_("Saving a file"),
+			   _("Please select a source pot file"), (GtkWindow*)0);
       return FALSE;
     }
   if(surf_file == NULL || strcmp(surf_file, "") == 0)
     {
-      raiseAlertDialog("Please select a target surf file");
+      visuGtkRaise_warning(_("Saving a file"),
+			   _("Please select a target surf file"), (GtkWindow*)0);
       return FALSE;
     }
 
@@ -1746,12 +1698,16 @@ gboolean pot2surf_build_surf_file(GtkWidget *button, gpointer data)
     {
 /*       fprintf(stderr, "%s %f\n", surfaces[i].name, surfaces[i].value); */
       values[i] = surfaces[i].value;
-      names[i] = surfaces[i].name;
+      if (strcmp(surfaces[i].name, SURFACE_NAME_CHOOSE))
+	names[i] = surfaces[i].name;
+      else
+	names[i] = (gchar*)0;
     }  
 
   if(pot2surf_direct(surf_file, pot_file, n, values, (const gchar **)names) != 0)
     {
-      raiseAlertDialog("Error");
+      visuGtkRaise_warning(_("Saving a file"),
+			   _("Error"), (GtkWindow*)0);
       for(i=0; i<n; i++)
 	g_free(names[i]);
       g_free(names);
@@ -1841,14 +1797,17 @@ gboolean pot2surf_load_instruc_file(const gchar *instruc_filename)
   line = g_string_new("");
   if(instruc_file == NULL)
     {
-      raiseAlertDialog(_("Can't open selected file"));
+      visuGtkRaise_warning(_("Loading a file"),
+			   _("Can't open selected file"), (GtkWindow*)0);
       return FALSE;
     }
 
   if(g_io_channel_read_line_string(instruc_file, line, NULL, NULL) 
      != G_IO_STATUS_NORMAL)
     {
-      raiseAlertDialog(_("Line 1 must contain the full path to the .pot file to read\n"));
+      visuGtkRaise_warning(_("Loading a file"),
+			   _("Line 1 must contain the full path to the .pot"
+			     " file to read\n"), (GtkWindow*)0);
       g_string_free(line, TRUE);
       return FALSE;
     }
@@ -1858,7 +1817,9 @@ gboolean pot2surf_load_instruc_file(const gchar *instruc_filename)
   if(g_io_channel_read_line_string(instruc_file, line, NULL, NULL) 
      != G_IO_STATUS_NORMAL)
     {
-      raiseAlertDialog(_("Line 2 must contain the full path to the .surf file to read\n"));
+      visuGtkRaise_warning(_("Loading a file"),
+			   _("Line 2 must contain the full path to the"
+			     " .surf file to read\n"), (GtkWindow*)0);
       g_string_free(line, TRUE);
       g_free(instruc_file);
       return FALSE;
@@ -1869,7 +1830,9 @@ gboolean pot2surf_load_instruc_file(const gchar *instruc_filename)
   if((g_io_channel_read_line_string(instruc_file, line, NULL, NULL) != G_IO_STATUS_NORMAL)
      || (sscanf(line->str, "%d", &n) != 1))
     {
-      raiseAlertDialog(_("Line 3 must contain the number of surfaces to build\n"));
+      visuGtkRaise_warning(_("Loading a file"),
+			   _("Line 3 must contain the number of"
+			     " surfaces to build\n"), (GtkWindow*)0);
       g_string_free(line, TRUE);
       g_free(instruc_file);
       g_free(surf_file);
@@ -1884,7 +1847,9 @@ gboolean pot2surf_load_instruc_file(const gchar *instruc_filename)
       if((g_io_channel_read_line_string(instruc_file, line, NULL, NULL) != G_IO_STATUS_NORMAL)
 	 || (sscanf(line->str, "%f %s", &surf_values[i], surf_names[i]) != 2))
 	{      
-	  raiseAlertDialog(_("Lines must contain the value of the surface to build and its name\n"));
+	  visuGtkRaise_warning(_("Loading a file"),
+			       _("Lines must contain the value of the"
+				 " surface to build and its name\n"), (GtkWindow*)0);
 	  g_string_free(line, TRUE);
 	  g_free(instruc_file);
 	  g_free(surf_file);
@@ -1933,7 +1898,7 @@ void pot2surf_select_instruc_file_to_load()
 				GTK_FILE_CHOOSER_ACTION_OPEN,
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
-  char *directory = getLastOpenDirectory();
+  char *directory = gtkMainClassGet_lastOpenDirectory();
 
   if(directory)
     gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_chooser), directory);
@@ -1952,7 +1917,7 @@ void pot2surf_select_instruc_file_to_load()
       case GTK_RESPONSE_ACCEPT:
 	if(!pot2surf_load_instruc_file(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(file_chooser))))
 	  break;
-	setLastOpenDirectory(gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser)));
+	gtkMainClassSet_lastOpenDirectory(gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER (file_chooser)));
 	gtk_widget_destroy(file_chooser);
 	return;
       case GTK_RESPONSE_CANCEL:
