@@ -42,8 +42,8 @@
 	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
 */
 #include "visuConfig.h"
-#include "externalRenderingMethods_gtk.h"
 
+#include <visu_gtk.h>
 #include <visu_tools.h>
 #include <visu_object.h>
 #include <visu_basic.h>
@@ -54,6 +54,7 @@
 #include <renderingBackend/visu_windowInterface.h>
 #include <support.h>
 #include <coreTools/toolColor.h>
+#include <coreTools/toolConfigFile.h>
 
 #include <unistd.h>
 #include <time.h>
@@ -65,7 +66,7 @@
 #define FLAG_PARAMETER_SKIN_CONFIG      "config_skin"
 #define FLAG_PARAMETER_REFRESH_CONFIG   "config_refreshIsOn"
 #define FLAG_PARAMETER_PERIOD_CONFIG    "config_refreshPeriod"
-#define DESC_PARAMETER_TABVIEW_CONFIG   "See or not the tab for the subpanel view ; boolean 0 or 1"
+#define DESC_PARAMETER_TABVIEW_CONFIG   "See or not the labels on tabs ; boolean 0 or 1"
 #define DESC_PARAMETER_SKIN_CONFIG      "Path to a gtkrc file ; chain"
 #define DESC_PARAMETER_REFRESH_CONFIG   "When on V_Sim reloads the file at periodic time ; boolean 0 or 1"
 #define DESC_PARAMETER_PERIOD_CONFIG    "The period of reloading in ms ; integer (10 < v < 10000)"
@@ -73,17 +74,15 @@
 #define PARAMETER_CONFIG_SKIN_DEFAULT      "None"
 #define PARAMETER_CONFIG_REFRESH_DEFAULT   0
 #define PARAMETER_CONFIG_PERIOD_DEFAULT    250
-int config_subPanelTabView;
-gchar *config_skin;
-char *config_resourcesPath;
-int config_refreshIsOn;
-float config_refreshPeriod;
-gdouble config_spinBoundsValue;
+static int config_subPanelTabView;
+static gchar *config_skin;
+static int config_refreshIsOn;
+static float config_refreshPeriod;
+static gdouble config_spinBoundsValue;
 
 
 /* Private functions. */
 GtkWidget *createInteriorConfig();
-void createCallBacksConfig();
 void setSkin(char* label);
 void setAutomaticRefresh(int bool);
 void setRefreshPeriod(float val);
@@ -91,18 +90,16 @@ gboolean automaticReload(gpointer data);
 
 /* This function details how to read the preferedNumberOfViews
    parameter. */
-gboolean readConfigTabView(gchar **lines, int nbLines,
-			   int position, GString *errorMessage);
-gboolean readConfigSkin(gchar **lines, int nbLines,
-			int position, GString *errorMessage);
-gboolean readConfigResPath(gchar **lines, int nbLines,
-			   int position, GString *errorMessage);
-gboolean readConfigRefresh(gchar **lines, int nbLines,
-			   int position, GString *errorMessage);
-gboolean readConfigPeriod(gchar **lines, int nbLines,
-			   int position, GString *errorMessage);
+static gboolean readConfigTabView(gchar **lines, int nbLines, int position,
+				  VisuData *dataObj, GError **error);
+static gboolean readConfigSkin(gchar **lines, int nbLines, int position,
+			       VisuData *dataObj, GError **error);
+static gboolean readConfigRefresh(gchar **lines, int nbLines, int position,
+				  VisuData *dataObj, GError **error);
+static gboolean readConfigPeriod(gchar **lines, int nbLines, int position,
+				 VisuData *dataObj, GError **error);
 /* These functions write all the element list to export there associated resources. */
-gboolean exportParametersPanelConfig(GString *data, int *nbLinesWritten,
+static gboolean exportParametersPanelConfig(GString *data, int *nbLinesWritten,
 				     VisuData* dataObj);
 
 void initPanelConfigGtkPart();
@@ -112,36 +109,31 @@ static char *defaultSkinPath;
 gboolean isPanelConfigInitialized;
 
 /* Specific widgets used in this panel. */
-GtkWidget *visuConfigData;
-GtkWidget *entryFactorSpin;
-GtkWidget *checkShowTab;
-GtkWidget *buttonRcFiles;
-GtkWidget *spinRefresh;
-GtkWidget *checkRefreshAuto;
-GtkWidget *checkStorePositions;
-GtkWidget *buttonMoveUpPath;
-GtkWidget *buttonMoveDownPath;
-GtkWidget *buttonAddPath;
-GtkWidget *buttonRemovePath;
+static GtkWidget *visuConfigData;
+static GtkWidget *entryFactorSpin;
+static GtkWidget *checkShowTab;
+static GtkWidget *checkRefreshAuto;
+static GtkWidget *checkStorePositions;
 
 /* Callbacks */
-void checkShowTabToggled(GtkToggleButton *button, gpointer data);
+static void checkShowTabToggled(GtkToggleButton *button, gpointer data);
 void checkRememberToggled(GtkToggleButton *button, gpointer data);
 void entryRcFilesChanged(GtkEntry *entry, gpointer data);
 void directorySelectedForResources(GtkButton *button, gpointer user_data);
 void openRcFileSelector(GtkButton *button, gpointer data);
 void checkRefreshToggled(GtkToggleButton *button, gpointer data);
-void refreshPeriodChanged(GtkSpinButton* button, gpointer data);
+static void onRefreshPeriodChanged(GtkSpinButton* button, gpointer data);
 void onDataReadySignal(GObject *obj, VisuData *dataObj, gpointer data);
 gboolean treePathClicked(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
 void directoryRemoveFromResources(GtkButton *button, gpointer user_data);
-void onReturnEntrySpinBounds(NumericalEntry *entry, double value, gpointer data);
+static void onReturnEntrySpinBounds(NumericalEntry *entry, double oldValue,
+				    gpointer data);
 static void onConfigEnter(ToolPanel *toolPanel, gpointer data);
 
-ToolPanel* initVisuConfig()
+ToolPanel* panelConfigInit()
 {
   char *cl = _("Configure the interface");
-  char *tl = _("Config");
+  char *tl = _("Configuration");
   VisuConfigFileEntry *resourceEntry;
 
   resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_PARAMETER,
@@ -215,6 +207,7 @@ GtkWidget *createInteriorConfig()
   GtkWidget *align;
   GtkObject *adj;
   GtkWidget *scrollView;
+  GtkWidget *spinRefresh;
   
   GtkTooltips *tooltips;
 
@@ -232,7 +225,7 @@ GtkWidget *createInteriorConfig()
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
   gtk_widget_show(hbox);
-  label = gtk_label_new(_("Show tabs in subpanel :"));
+  label = gtk_label_new(_("Always show labels in tabs: "));
   gtk_widget_set_name(label, "label_head");
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
   gtk_widget_show(label);
@@ -251,7 +244,7 @@ GtkWidget *createInteriorConfig()
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_widget_show(hbox);
   gtk_container_add(GTK_CONTAINER(align), hbox);
-  label = gtk_label_new(_("Automatic refresh : "));
+  label = gtk_label_new(_("Automatic refresh: "));
   gtk_widget_set_name(label, "label_head");
   gtk_widget_show(label);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
@@ -266,7 +259,7 @@ GtkWidget *createInteriorConfig()
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_widget_show(hbox);
   gtk_container_add(GTK_CONTAINER(align), hbox);
-  label = gtk_label_new(_("Refresh period : "));
+  label = gtk_label_new(_("Refresh period: "));
   gtk_widget_set_name(label, "label_head");
   gtk_widget_show(label);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
@@ -287,7 +280,7 @@ GtkWidget *createInteriorConfig()
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);
-  label = gtk_label_new(_("Remember windows positions :"));
+  label = gtk_label_new(_("Remember windows positions:"));
   gtk_widget_set_name(label, "label_head");
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
@@ -311,7 +304,7 @@ GtkWidget *createInteriorConfig()
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
   gtk_widget_show(hbox);
-  label = gtk_label_new(_("Set factor for spin buttons :"));
+  label = gtk_label_new(_("Set factor for spin buttons: "));
   gtk_widget_set_name(label, "label_head");
   gtk_misc_set_alignment(GTK_MISC(label), 0., 0.5);
   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
@@ -334,46 +327,41 @@ GtkWidget *createInteriorConfig()
   gtk_widget_show(scrollView);
 
   /* Set the callbacks. */
-  createCallBacksConfig();
-  g_signal_connect((gpointer)checkRefreshAuto, "toggled",
-		   GTK_SIGNAL_FUNC(checkRefreshToggled), (gpointer)0);
-  g_signal_connect((gpointer)spinRefresh, "value-changed",
-		   G_CALLBACK(refreshPeriodChanged), (gpointer)0);
+  g_signal_connect(G_OBJECT(checkShowTab), "toggled",
+		   G_CALLBACK(checkShowTabToggled), (gpointer)0);
+  g_signal_connect(G_OBJECT(checkStorePositions), "toggled",
+		   G_CALLBACK(checkRememberToggled), (gpointer)0);
+  g_signal_connect(G_OBJECT(entryFactorSpin), "value-changed",
+		   G_CALLBACK(onReturnEntrySpinBounds), (gpointer)1);
+  g_signal_connect(G_OBJECT(checkRefreshAuto), "toggled",
+		   G_CALLBACK(checkRefreshToggled), (gpointer)0);
+  g_signal_connect(G_OBJECT(spinRefresh), "value-changed",
+		   G_CALLBACK(onRefreshPeriodChanged), (gpointer)0);
   
   
   return scrollView;
 }
 
-void createCallBacksConfig()
-{
-  g_signal_connect((gpointer)checkShowTab, "toggled",
-		   GTK_SIGNAL_FUNC(checkShowTabToggled), (gpointer)0);
-  g_signal_connect((gpointer)checkStorePositions, "toggled",
-		   GTK_SIGNAL_FUNC(checkRememberToggled), (gpointer)0);
-  g_signal_connect(G_OBJECT(entryFactorSpin), "value-changed",
-		   G_CALLBACK(onReturnEntrySpinBounds), (gpointer)1);
-}
-
 /*************/
 /* Callbacks */
 /*************/
-void checkShowTabToggled(GtkToggleButton *button, gpointer data)
+static void checkShowTabToggled(GtkToggleButton *button _U_, gpointer data _U_)
 {
   panelConfigSet_tabView(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkShowTab)));
 }
-void checkRememberToggled(GtkToggleButton *button, gpointer data)
+void checkRememberToggled(GtkToggleButton *button, gpointer data _U_)
 {
-  setRememberPosition((int)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)));
+  gtkMainClassSet_rememberPosition(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)));
 }
-static void onConfigEnter(ToolPanel *toolPanel, gpointer data)
+static void onConfigEnter(ToolPanel *toolPanel _U_, gpointer data _U_)
 {
   if (!isPanelConfigInitialized)
     initPanelConfigGtkPart();
 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkStorePositions),
-			       getRememberPosition());
+			       gtkMainClassGet_rememberPosition());
 }
-void checkRefreshToggled(GtkToggleButton *button, gpointer data)
+void checkRefreshToggled(GtkToggleButton *button, gpointer data _U_)
 {
   RenderingWindow *window;
 
@@ -382,7 +370,7 @@ void checkRefreshToggled(GtkToggleButton *button, gpointer data)
 
   panelConfigSet_automaticRefresh(window, gtk_toggle_button_get_active(button));
 }
-void refreshPeriodChanged(GtkSpinButton* button, gpointer data)
+static void onRefreshPeriodChanged(GtkSpinButton* button, gpointer data _U_)
 {
   RenderingWindow *window;
 
@@ -391,15 +379,16 @@ void refreshPeriodChanged(GtkSpinButton* button, gpointer data)
 
   panelConfigSet_refreshPeriod(window, (float)gtk_spin_button_get_value(button));
 }
-void openRcFileSelector(GtkButton *button, gpointer data)
+void openRcFileSelector(GtkButton *button _U_, gpointer data _U_)
 {
 }
-void onReturnEntrySpinBounds(NumericalEntry *entry, double value, gpointer data)
+static void onReturnEntrySpinBounds(NumericalEntry *entry, double oldValue _U_,
+				    gpointer data _U_)
 {
   DBG_fprintf(stderr, "Panel Config : grep value from GtkEntry for"
 	      " the bounding values of spins.\n");
 
-  panelConfigSet_spinBoundsValue(value);
+  panelConfigSet_spinBoundsValue(numericalEntryGet_value(entry));
 }
 void setSkin(char* label)
 {
@@ -436,7 +425,8 @@ void autoReloadAddTimeout(VisuData *data, guint time, gpointer user_data)
   if (!timeoutId)
     {
       timeoutId = g_malloc(sizeof(guint));
-      visuDataSet_property(data, "autoLoad_timeoutId", (gpointer)timeoutId);
+      visuDataSet_propertyWithDestroyFunc(data, "autoLoad_timeoutId",
+					  (gpointer)timeoutId, g_free);
       *timeoutId = 0;
     }
   if (!*timeoutId)
@@ -506,10 +496,12 @@ gboolean automaticReload(gpointer data)
       DBG_fprintf(stderr, "Panel Config : automatic reloading in progress...\n");
       visuDataFree_population(dataObj);
       error = (GError*)0;
-      ok = visuBasicLoad_dataFromFile(dataObj, (FileFormat*)0, &error);
+      ok = visuBasicLoad_dataFromFile(dataObj, (FileFormat*)0,
+				      visuDataGet_setId(dataObj), &error);
       if (!ok)
 	{
-	  raiseAlertDialog(error->message);
+	  visuGtkRaise_warning(_("Automatic file reloading"), error->message,
+			       (GtkWindow*)0);
 	  g_error_free(error);
 	  visuRenderingWindowSet_visuData(window, (VisuData*)0);
 	}
@@ -524,7 +516,7 @@ gboolean automaticReload(gpointer data)
     }
   return TRUE;
 }
-void onDataReadySignal(GObject *obj, VisuData *dataObj, gpointer data)
+void onDataReadySignal(GObject *obj _U_, VisuData *dataObj, gpointer data _U_)
 {
   struct stat statBuf;
   int res;
@@ -535,7 +527,7 @@ void onDataReadySignal(GObject *obj, VisuData *dataObj, gpointer data)
   if (!dataObj)
     return;
 
-  DBG_fprintf(stderr, "Panel Config : caught the 'dataReadyForRendering' signal,"
+  DBG_fprintf(stderr, "Panel Config: caught the 'dataReadyForRendering' signal,"
 	      " set acces time for VisuData %p.\n", (gpointer)dataObj);
   nbKind = renderingMethodGet_nbFileType(getRenderingMethodInUse());
   lastReadTime = g_malloc(sizeof(time_t) * nbKind);
@@ -553,7 +545,8 @@ void onDataReadySignal(GObject *obj, VisuData *dataObj, gpointer data)
 	    lastReadTime[kind] = 0;
 	}
     }
-  visuDataSet_property(dataObj, "autoLoad_accessTime", lastReadTime);
+  visuDataSet_propertyWithDestroyFunc(dataObj, "autoLoad_accessTime",
+				      lastReadTime, g_free);
   if (config_refreshIsOn)
     autoReloadAddTimeout(dataObj, config_refreshPeriod,
 			 (gpointer)visuDataGet_renderingWindow(dataObj));
@@ -568,17 +561,12 @@ void onDataReadySignal(GObject *obj, VisuData *dataObj, gpointer data)
 
 void panelConfigSet_tabView(gboolean viewed)
 {
-  GtkWidget *tabSubPanel;
-  DockWindow *dock;
-
   DBG_fprintf(stderr,"Panel Config : toggle tab view -> %d (previously %d)\n",
 	      (int)viewed, config_subPanelTabView);
   if (viewed == config_subPanelTabView)
     return;
 
-  dock = toolPanelGet_container(TOOL_PANEL(visuConfigData));
-  tabSubPanel = dockWindowGet_notebook(dock);
-  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(tabSubPanel), viewed);
+  toolPanelClassSet_headerVisibility(viewed);
   
   /* set the associated parameter. */
   config_subPanelTabView = viewed;
@@ -603,6 +591,8 @@ void panelConfigSet_automaticRefresh(RenderingWindow *window, int bool)
 }
 void panelConfigSet_refreshPeriod(RenderingWindow *window, float val)
 {
+  VisuData *dataObj;
+
   DBG_fprintf(stderr, "Panel Config : set the refresh period to %f (previously %f).\n",
 	      val, config_refreshPeriod);
 
@@ -610,14 +600,14 @@ void panelConfigSet_refreshPeriod(RenderingWindow *window, float val)
     return;
 
   config_refreshPeriod = val;
-  if (window && renderingWindowGet_visuData(window) && config_refreshIsOn)
+  dataObj = (window)?renderingWindowGet_visuData(window):(VisuData*)0;
+  if (dataObj && config_refreshIsOn)
     {
       DBG_fprintf(stderr, "Panel Config : stopping the callback of refresh "
 		  "to change the period.\n");
-      autoReloadRemoveTimeout(renderingWindowGet_visuData(window));
-      autoReloadAddTimeout(renderingWindowGet_visuData(window),
-			   config_refreshPeriod, (gpointer)window);
-    }  
+      autoReloadRemoveTimeout(dataObj);
+      autoReloadAddTimeout(dataObj, config_refreshPeriod, (gpointer)window);
+    }
 }
 void panelConfigSet_spinBoundsValue(gdouble val)
 {
@@ -631,13 +621,12 @@ void panelConfigSet_spinBoundsValue(gdouble val)
   prec = config_spinBoundsValue;
   config_spinBoundsValue = val;
   /* Emitting appropriated signal. */
-  g_signal_emit(visuGtkObject,
-		VISU_GTK_GET_CLASS(visuGtkObject)->spinBoundsChanged_signal_id,
-		0 , (gfloat)val / prec, NULL);
+/*   g_signal_emit(visuGtkObject, */
+/* 		VISU_GTK_GET_CLASS(visuGtkObject)->spinBoundsChanged_signal_id, */
+/* 		0 , (gfloat)val / prec, NULL); */
   
   /* Asking for redraw */
-  g_signal_emit (visu, VISU_GET_CLASS (visu)->OpenGLAskForReDraw_signal_id,
-		 0 , NULL);
+  g_idle_add(visuObjectRedraw, (gpointer)0);
 }
 gdouble panelConfigGet_spinBoundsValue()
 {
@@ -651,39 +640,32 @@ gdouble panelConfigGet_spinBoundsValue()
 
 /* This function details how to read the preferedNumberOfViews
    parameter. */
-gboolean readConfigTabView(gchar **lines, int nbLines,
-			   int position, GString *errorMessage)
+static gboolean readConfigTabView(gchar **lines, int nbLines, int position,
+				  VisuData *dataObj _U_, GError **error)
 {
-  int res;
-  int viewed;
+  gboolean viewed;
 
-  res = sscanf(lines[0],"%d", &viewed);
-  if (res != 1)
-    {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d,"
-					       " 1 boolean value must appear"
-					       " after the %s markup.\n"),
-			       position, FLAG_PARAMETER_TABVIEW_CONFIG);
-      panelConfigSet_tabView(PARAMETER_CONFIG_TABVIEW_DEFAULT);
-      return FALSE;
-    }
-  panelConfigSet_tabView((gboolean)viewed);
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
+  if (!configFileRead_boolean(lines[0], position, &viewed, 1, error))
+    return FALSE;
+  panelConfigSet_tabView(viewed);
   
   return TRUE;
 }
-gboolean readConfigSkin(gchar **lines, int nbLines,
-			int position, GString *errorMessage)
+static gboolean readConfigSkin(gchar **lines, int nbLines, int position,
+			       VisuData *dataObj _U_, GError **error)
 {
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
   lines[0] = g_strstrip(lines[0]);
 
   if (!lines[0][0])
     {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d,"
-					       " 1 string value must appear after the"
-					       " %s markup.\n"),
-			       position, FLAG_PARAMETER_SKIN_CONFIG);
+      *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_VALUE,
+			   _("Parse error at line %d: 1 string value must appear"
+			     " after the %s markup.\n"), position,
+			   FLAG_PARAMETER_SKIN_CONFIG);
       config_skin = g_strdup(PARAMETER_CONFIG_SKIN_DEFAULT);
       setSkin(config_skin);
       return FALSE;
@@ -694,54 +676,41 @@ gboolean readConfigSkin(gchar **lines, int nbLines,
 
   return TRUE;
 }
-gboolean readConfigRefresh(gchar **lines, int nbLines,
-			   int position, GString *errorMessage)
+static gboolean readConfigRefresh(gchar **lines, int nbLines, int position,
+				  VisuData *dataObj _U_, GError **error)
 {
-  int res, bool;
+  gboolean bool;
   RenderingWindow *window;
 
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
   window = RENDERING_WINDOW(visuRenderingWindowGet_current());
-  res = sscanf(lines[0],"%d", &bool);
-  if (res != 1)
-    {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d,"
-					       " 1 boolean value must appear"
-					       " after the %s markup.\n"),
-			       position, FLAG_PARAMETER_REFRESH_CONFIG);
-      panelConfigSet_automaticRefresh(window, PARAMETER_CONFIG_REFRESH_DEFAULT);
-      return FALSE;
-    }
+  if (!configFileRead_boolean(lines[0], position, &bool, 1, error))
+    return FALSE;
   panelConfigSet_automaticRefresh(window, bool);
 
   return TRUE;
 }
-gboolean readConfigPeriod(gchar **lines, int nbLines,
-			  int position, GString *errorMessage)
+static gboolean readConfigPeriod(gchar **lines, int nbLines, int position,
+				 VisuData *dataObj _U_, GError **error)
 {
-  int res;
+  float val;
   RenderingWindow *window;
 
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
   window = RENDERING_WINDOW(visuRenderingWindowGet_current());
-  res = sscanf(lines[0],"%f", &config_refreshPeriod);
-  if (res != 1)
-    {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d,"
-					       " 1 string value must appear after"
-					       " the %s markup.\n"),
-			       position, FLAG_PARAMETER_PERIOD_CONFIG);
-      panelConfigSet_refreshPeriod(window, PARAMETER_CONFIG_PERIOD_DEFAULT);
-      return FALSE;
-    }
+  if (!configFileRead_float(lines[0], position, &val, 1, error))
+    return FALSE;
+  config_refreshPeriod = val;
   panelConfigSet_refreshPeriod(window, config_refreshPeriod);
 
   return TRUE;
 }
 
 /* These functions write all the element list to export there associated resources. */
-gboolean exportParametersPanelConfig(GString *data, int *nbLinesWritten,
-				     VisuData *dataObj)
+static gboolean exportParametersPanelConfig(GString *data, int *nbLinesWritten,
+					    VisuData *dataObj _U_)
 {
   g_string_append_printf(data, "# %s\n", DESC_PARAMETER_TABVIEW_CONFIG);
   g_string_append_printf(data, "%s[gtk]: %d\n\n", FLAG_PARAMETER_TABVIEW_CONFIG,
