@@ -51,6 +51,8 @@
 #include <openGLFunctions/view.h>
 #include <openGLFunctions/objectList.h>
 #include <opengl.h>
+#include <coreTools/toolConfigFile.h>
+#include <coreTools/toolElements.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -59,11 +61,27 @@
 #include <GL/gl.h>
 #include <GL/glu.h> 
 
+#include <math.h>
+
 
 /* The OpenGL identifier to store the glObjectLists that
    describe the spheres. */
 /* int identifierSpheres; */
 
+/* The icosahedron drawing. */
+#define X .525731112119133606 
+#define Z .850650808352039932
+static GLfloat vdata[12][3] = {    
+   {X, 0.0, -Z}, {-X, 0.0, -Z}, {X, 0.0, Z}, {-X, 0.0, Z},    
+   {0.0, -Z, -X}, {0.0, -Z, X}, {0.0, Z, -X}, {0.0, Z, X},    
+   {-Z, -X, 0.0}, {Z, -X, 0.0}, {-Z, X, 0.0}, {Z, X, 0.0} 
+};
+static GLuint tindices[20][3] = { 
+   {0,4,1}, {0,9,4}, {9,5,4}, {4,5,8}, {4,8,1},    
+   {8,10,1}, {8,3,10}, {5,3,8}, {5,2,3}, {2,7,3},    
+   {7,10,3}, {7,6,10}, {7,11,6}, {11,0,6}, {0,1,6}, 
+   {6,1,10}, {9,0,11}, {9,11,2}, {9,2,5}, {7,2,11} };
+
 #define RADIUS_DEFAULT 1.
 #define SHAPE_DEFAULT shapeSphere
 #define RATIO_DEFAULT 2.
@@ -71,12 +89,26 @@
 #define THETA_DEFAULT 90.
 /* Dealing with the radius resource. */
 #define FLAG_RESOURCE_RADIUS_SHAPE "atomic_radius_shape"
-#define DESC_RESOURCE_RADIUS_SHAPE "The radius of the element and its shape, a real > 0. & [Sphere Cube Elipsoid]"
-gboolean renderingAtomicRead_atomicRadiusShape(gchar **lines, int nbLines,
-					       int position, GString *errorMessage);
+#define DESC_RESOURCE_RADIUS_SHAPE "The radius of the element and its shape, a real > 0. & [Sphere Cube Elipsoid Point]"
+static gboolean readAtomicRadiusShape(gchar **lines, int nbLines, int position,
+			      VisuData *dataObj, GError **error);
 /* These functions write all the element list to export there associated resources. */
-gboolean renderingAtomicExport_atomicRadiusShape(GString *data, int *nbLinesWritten,
-						 VisuData* dataObj);
+static gboolean exportAtomicRadiusShape(GString *data, int *nbLinesWritten,
+					VisuData* dataObj);
+#define FLAG_PARAMETER_SHAPE "atomic_sphere_method"
+#define DESC_PARAMETER_SHAPE "The sphere drawing method, [GluSphere Icosahedron]"
+static gboolean readAtomicShape(gchar **lines, int nbLines, int position,
+				VisuData *dataObj, GError **error);
+static gboolean exportAtomicShape(GString *data, int *nbLinesWritten,
+				  VisuData* dataObj);
+enum
+  {
+    sphere_glu,
+    sphere_icosahedron,
+    sphere_nb
+  };
+static guint sphereMethod = sphere_icosahedron;
+const char* sphereName[sphere_nb + 1] = {"GluSphere", "Icosahedron", (const char*)0};
 
 const char* shapeName[nbAtomicShapes + 1] = {"Sphere", "Cube", "Elipsoid", "Point", (const char*)0};
 const char* shapeNameI18n[nbAtomicShapes + 1];
@@ -104,8 +136,7 @@ struct atomicResources_struct
 /* This list contains all the method than can load an atomic file. */
 static GList *allAtomicLoadingMethods;
 
-
-void freeAtomicStruct(gpointer data);
+static struct atomicResources_struct *getRadiusAndShape(VisuElement *ele);
 
 RenderingMethod* initAtomic()
 {
@@ -140,7 +171,8 @@ RenderingMethod* initAtomic()
       tmpLst = g_list_next(tmpLst);
     }
 
-  atom = renderingMethod_new("Atom visualisation", name, descr, 1, renderingAtomicLoad);
+  atom = renderingMethod_new("Atom visualisation", name, descr,
+			     1, renderingAtomicLoad);
   renderingMethodSet_fileType(atom, formatList, 0, _("Position files"));
   iconPath = g_build_filename(v_sim_pixmaps_dir, "stock-atomic.png", NULL);
   renderingMethodSet_icon(atom, iconPath);
@@ -153,10 +185,16 @@ RenderingMethod* initAtomic()
   resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_RESOURCE,
 					  FLAG_RESOURCE_RADIUS_SHAPE,
 					  DESC_RESOURCE_RADIUS_SHAPE,
-					  1,
-					  renderingAtomicRead_atomicRadiusShape);
+					  1, readAtomicRadiusShape);
   visuConfigFileAdd_exportFunction(VISU_CONFIGFILE_RESOURCE,
-				   renderingAtomicExport_atomicRadiusShape);
+				   exportAtomicRadiusShape);
+  resourceEntry = visuConfigFileAdd_entry(VISU_CONFIGFILE_PARAMETER,
+					  FLAG_PARAMETER_SHAPE,
+					  DESC_PARAMETER_SHAPE,
+					  1, readAtomicShape);
+  visuConfigFileSet_version(resourceEntry, 3.4f);
+  visuConfigFileAdd_exportFunction(VISU_CONFIGFILE_PARAMETER,
+				   exportAtomicShape);
 					  
 
   /* Initialise the OpenGl part. */
@@ -174,7 +212,8 @@ RenderingMethod* initAtomic()
   return atom;
 }
 
-gboolean renderingAtomicLoad(VisuData *data, FileFormat *format, GError **error)
+gboolean renderingAtomicLoad(VisuData *data, FileFormat *format _U_,
+			     int nSet, GError **error)
 {
   gchar *file;
   gboolean loadOk;
@@ -197,7 +236,7 @@ gboolean renderingAtomicLoad(VisuData *data, FileFormat *format, GError **error)
   if (g_file_test(file, G_FILE_TEST_IS_DIR))
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
-			   _("WARNING! The specified file is"
+			   _("The specified file is"
 			     " a directory indeed. Can't load"
 			     " a directory!\n"));
       return FALSE;
@@ -217,7 +256,7 @@ gboolean renderingAtomicLoad(VisuData *data, FileFormat *format, GError **error)
 	  DBG_fprintf(stderr,"Rendering Atomic : testing '%s' with format : %s.\n",
 		      file, ((RenderingFormatLoad*)tmpLst->data)->name);
 	  loadOk = ((RenderingFormatLoad*)tmpLst->data)->load
-	    (data, file, ((RenderingFormatLoad*)tmpLst->data)->fmt, error);
+	    (data, file, ((RenderingFormatLoad*)tmpLst->data)->fmt, nSet, error);
 	}
       tmpLst = g_list_next(tmpLst);
     }
@@ -226,37 +265,36 @@ gboolean renderingAtomicLoad(VisuData *data, FileFormat *format, GError **error)
     {
       if (!*error)
 	*error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
-			     _("WARNING : impossible to load this file.\n"));
+			     _("Impossible to load this file.\n"));
       return FALSE;
     }
+  else if (*error)
+    return FALSE;
 
   return TRUE;
 }
 
-void freeAtomicStruct(gpointer data)
-{
-  if (data)
-    free(data);
-}
-
-struct atomicResources_struct* createDefaultAtomicStruct()
+static struct atomicResources_struct *getRadiusAndShape(VisuElement *ele)
 {
   struct atomicResources_struct *str;
 
-  str = malloc(sizeof(struct atomicResources_struct));
+  g_return_val_if_fail(ele, (struct atomicResources_struct*)0);
+
+  str = (struct atomicResources_struct*)visuElementGet_property
+    (ele, "radiusAndShape");
   if (!str)
     {
-      allocationProblems();
-      exit(1);
+      str = g_malloc(sizeof(struct atomicResources_struct));
+      if (!toolElementsGet_elementFromSymbol((int*)0, &str->radius, ele->name))
+	str->radius = RADIUS_DEFAULT;
+      str->shape  = SHAPE_DEFAULT;
+      str->ratio  = RATIO_DEFAULT;
+      str->phi    = PHI_DEFAULT;
+      str->theta  = THETA_DEFAULT;
+      str->openGLIdentifier = openGLObjectList_new(1);
+
+      visuElementSet_property(ele, "radiusAndShape", (gpointer)str);
     }
-  str->radius = RADIUS_DEFAULT;
-  str->shape  = SHAPE_DEFAULT;
-  str->ratio  = RATIO_DEFAULT;
-  str->phi    = PHI_DEFAULT;
-  str->theta  = THETA_DEFAULT;
-
-  str->openGLIdentifier = openGLObjectList_new(1);
-
   return str;
 }
 
@@ -264,17 +302,10 @@ float renderingAtomicGet_radius(VisuElement *ele)
 {
   struct atomicResources_struct *str;
 
-  if (!ele)
-    return RADIUS_DEFAULT;
+  g_return_val_if_fail(ele, RADIUS_DEFAULT);
 
-  str = (struct atomicResources_struct*)visuElementGet_property(ele, "radiusAndShape");
-  /* g_hash_table_lookup(shapeAndRadiusResource, (gpointer)ele); */
-  if (!str)
-    {
-      str = createDefaultAtomicStruct();
-      visuElementSet_property(ele, "radiusAndShape", (gpointer)str);
-/*       g_hash_table_insert(shapeAndRadiusResource, (gpointer)ele, (gpointer)str); */
-    }
+  str = getRadiusAndShape(ele);
+  g_return_val_if_fail(str, -1.f);
   return str->radius;
 }
 float renderingAtomicGet_radiusDefault()
@@ -285,22 +316,12 @@ gboolean renderingAtomicSet_radius(VisuElement* ele, float value)
 {
   struct atomicResources_struct *str;
 
-  if (!ele)
-    return FALSE;
+  g_return_val_if_fail(ele && value > 0.f, FALSE);
   
-  str = (struct atomicResources_struct*)visuElementGet_property(ele, "radiusAndShape");
-  if (!str)
-    {
-      str = createDefaultAtomicStruct();
-      visuElementSet_property(ele, "radiusAndShape", (gpointer)str);
-    }
-  if (value <= 0.)
-    {
-      fprintf(stderr, "WARNING! Trying to set a negative value (%f)"
-	      " for the radius of VisuElement '%s'.\n", value, ele->name);
-      str->radius = 1.;
-    }
-  else if (value == str->radius)
+  str = getRadiusAndShape(ele);
+  g_return_val_if_fail(str, FALSE);
+
+  if (value == str->radius)
     return FALSE;
   else
     str->radius = value;
@@ -313,12 +334,10 @@ int renderingAtomicGet_shape(VisuElement *ele)
 {
   struct atomicResources_struct *str;
 
-  str = (struct atomicResources_struct*)visuElementGet_property(ele, "radiusAndShape");
-/*   str = g_hash_table_lookup(shapeAndRadiusResource, (gpointer)ele); */
-  if (!str)
-    return -1;
-  else
-    return str->shape;
+  str = getRadiusAndShape(ele);
+  g_return_val_if_fail(str, FALSE);
+
+  return str->shape;
 }
 const char* renderingAtomicGet_shapeName(int shape)
 {
@@ -339,20 +358,11 @@ int renderingAtomicSet_shape(VisuElement *ele, int shape)
 {
   struct atomicResources_struct *str;
 
-  if (!ele || shape < 0 || shape >= nbAtomicShapes)
-    {
-      fprintf(stderr, "WARNING! Wrong parameters to the call of renderingAtomicSet_shape.\n");
-      return 0;
-    }
+  g_return_val_if_fail(ele && shape >= 0 && shape < nbAtomicShapes, 0);
   
-  str = (struct atomicResources_struct*)visuElementGet_property(ele, "radiusAndShape");
-/*   str = g_hash_table_lookup(shapeAndRadiusResource, (gpointer)ele); */
-  if (!str)
-    {
-      str = createDefaultAtomicStruct();
-      visuElementSet_property(ele, "radiusAndShape", (gpointer)str);
-/*       g_hash_table_insert(shapeAndRadiusResource, (gpointer)ele, (gpointer)str); */
-    }
+  str = getRadiusAndShape(ele);
+  g_return_val_if_fail(str, FALSE);
+
   if (shape == str->shape)
     return 0;
   else
@@ -373,7 +383,7 @@ int renderingAtomicSet_shapeFromName(VisuElement *ele, char* shape)
     }
   if (res < 0)
     {
-      fprintf(stderr, "WARNING! Unknown shape name in the call of renderingAtomicSet_shapeFromName.\n");
+      g_warning("Unknown shape name in the call of renderingAtomicSet_shapeFromName.");
       return 0;
     }
   else
@@ -395,12 +405,9 @@ gboolean renderingAtomicSet_elipsoidParameters(VisuElement *ele, float ratio,
 
   g_return_val_if_fail(ele && (ratio >= 1.), 0);
   
-  str = (struct atomicResources_struct*)visuElementGet_property(ele, "radiusAndShape");
-  if (!str)
-    {
-      str = createDefaultAtomicStruct();
-      visuElementSet_property(ele, "radiusAndShape", (gpointer)str);
-    }
+  str = getRadiusAndShape(ele);
+  g_return_val_if_fail(str, FALSE);
+
   refresh = FALSE;
   if (ratio != str->ratio)
     {
@@ -426,12 +433,9 @@ gboolean renderingAtomicSet_elipsoidRatio(VisuElement *ele, float ratio)
 
   g_return_val_if_fail(ele && (ratio >= 1.), FALSE);
   
-  str = (struct atomicResources_struct*)visuElementGet_property(ele, "radiusAndShape");
-  if (!str)
-    {
-      str = createDefaultAtomicStruct();
-      visuElementSet_property(ele, "radiusAndShape", (gpointer)str);
-    }
+  str = getRadiusAndShape(ele);
+  g_return_val_if_fail(str, FALSE);
+
   if (ratio != str->ratio)
     {
       str->ratio = ratio;
@@ -445,12 +449,9 @@ gboolean renderingAtomicSet_elipsoidPhi(VisuElement *ele, float phi)
 
   g_return_val_if_fail(ele, FALSE);
   
-  str = (struct atomicResources_struct*)visuElementGet_property(ele, "radiusAndShape");
-  if (!str)
-    {
-      str = createDefaultAtomicStruct();
-      visuElementSet_property(ele, "radiusAndShape", (gpointer)str);
-    }
+  str = getRadiusAndShape(ele);
+  g_return_val_if_fail(str, FALSE);
+
   if (phi != str->phi)
     {
       str->phi = phi;
@@ -464,12 +465,9 @@ gboolean renderingAtomicSet_elipsoidTheta(VisuElement *ele, float theta)
 
   g_return_val_if_fail(ele, FALSE);
   
-  str = (struct atomicResources_struct*)visuElementGet_property(ele, "radiusAndShape");
-  if (!str)
-    {
-      str = createDefaultAtomicStruct();
-      visuElementSet_property(ele, "radiusAndShape", (gpointer)str);
-    }
+  str = getRadiusAndShape(ele);
+  g_return_val_if_fail(str, FALSE);
+
   if (theta != str->theta)
     {
       str->theta = theta;
@@ -515,65 +513,70 @@ float renderingAtomicGet_elipsoidTheta(VisuElement *ele)
 }
 
 
-gboolean renderingAtomicRead_atomicRadiusShape(gchar **lines, int nbLines,
-					       int position, GString *errorMessage)
+static gboolean readAtomicRadiusShape(gchar **lines, int nbLines, int position,
+				      VisuData *dataObj _U_, GError **error)
 {
   VisuElement* ele;
-  char *cursor;
-  int res, shapeNum, errorFlag, i;
+  int shapeNum, i, token;
   float radius;
-  char shape[MAX_LINE_LENGTH];
-  GenericRenderingWindow window;
+  gchar **shape;
+  gchar **tokens;
 
-  cursor = lines[0];
-  ele = visuElementGet_fromValueLine(&cursor, -1, &errorFlag);
-  if (!ele)
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
+  /* Tokenize the line of values. */
+  tokens = g_strsplit_set(g_strchug(lines[0]), " \n", MAX_LINE_LENGTH);
+  token = 0;
+
+  /* Get the two elements. */
+  if (!configFileRead_elementFromTokens(tokens, &token, &ele, 1, position, error))
     {
-      if (errorMessage)
-	visuElementPrint_errorFromValueLine(errorMessage, position, errorFlag);
+      g_strfreev(tokens);
       return FALSE;
     }
-  res = sscanf(cursor, "%f %s", &radius, shape);
-  if (res != 2 || radius <=0.)
+
+  /* Read 1 int. */
+  if (!configFileRead_floatFromTokens(tokens, &token, &radius, 1, position, error))
     {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d,"
-					       " 1 floating point value (v > 0.) and a shape name "
-					       "must appear after the %s markup.\n"),
-			       position, FLAG_RESOURCE_RADIUS_SHAPE);
+      g_strfreev(tokens);
       return FALSE;
     }
+  radius = CLAMP(radius, 0., G_MAXFLOAT);
+
+  /* Read 1 string. */
+  if (!configFileRead_stringFromTokens(tokens, &token, &shape, 1, position, error))
+    {
+      g_strfreev(tokens);
+      return FALSE;
+    }
+  g_strfreev(tokens);
+
   shapeNum = -1;
   for (i = 0; shapeName[i] && shapeNum < 0; i++)
     {
-      if (!strcmp(shapeName[i], shape))
+      if (!strcmp(shapeName[i], shape[0]))
 	shapeNum = i;
     }
+  g_free(shape);
   if (shapeNum < 0)
     {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d,"
-					       " the shape '%s' is unknown.\n"),
-			       position, shape);
-      return 1;
+      *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_VALUE,
+			   _("Parse error at line %d: the shape '%s' is unknown.\n"),
+			   position, shape[0]);
+      return FALSE;
     }
-  res = renderingAtomicSet_radius(ele, radius);
-  res = renderingAtomicSet_shape(ele, shapeNum) || res;
+  renderingAtomicSet_radius(ele, radius);
+  renderingAtomicSet_shape(ele, shapeNum);
 
-  window = visuRenderingWindowGet_current();
-  if (res && window && visuRenderingWindowGet_visuData(window))
-    renderingAtomic_createShape(visuRenderingWindowGet_visuData(window),
-				ele);
-  
   return TRUE;
 }
 
 /* These functions write all the element list to export there associated resources. */
-gboolean renderingAtomicExport_atomicRadiusShape(GString *data, int *nbLinesWritten,
-						 VisuData *dataObj)
+static gboolean exportAtomicRadiusShape(GString *data, int *nbLinesWritten,
+					VisuData *dataObj)
 {
   GList *pos, *eleList;
-  int i;
+  unsigned int i;
   struct atomicResources_struct *str;
 
   g_string_append_printf(data, "# %s\n", DESC_RESOURCE_RADIUS_SHAPE);
@@ -610,6 +613,45 @@ gboolean renderingAtomicExport_atomicRadiusShape(GString *data, int *nbLinesWrit
   return TRUE;
 }
 
+static gboolean readAtomicShape(gchar **lines, int nbLines, int position,
+				VisuData *dataObj _U_, GError **error)
+{
+  gchar **vals;
+  guint i;
+
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
+  if (!configFileRead_string(lines[0], position, &vals, 1, FALSE, error))
+    return FALSE;
+
+  for (i = 0; i < sphere_nb; i++)
+    if (!strcmp(vals[0], sphereName[i]))
+      {
+	sphereMethod = i;
+	break;
+      }
+  if (i == sphere_nb)
+    {
+      *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_VALUE,
+			   _("Parse error at line %d: the sphere method '%s' is unknown.\n"),
+			   position, vals[0]);
+      g_strfreev(vals);
+      return FALSE;
+    }
+  g_strfreev(vals);
+  return TRUE;
+}
+static gboolean exportAtomicShape(GString *data, int *nbLinesWritten,
+				  VisuData* dataObj _U_)
+{
+  g_string_append_printf(data, "# %s\n", DESC_PARAMETER_SHAPE);
+  g_string_append_printf(data, "%s: %s\n\n", FLAG_PARAMETER_SHAPE,
+			 sphereName[sphereMethod]);
+  *nbLinesWritten += 3;
+
+  return TRUE;
+}
+
 
 /***************/
 /* OpenGL part */
@@ -619,16 +661,11 @@ void renderingAtomic_positionShape(VisuData *visuData, VisuNode *node, VisuEleme
 {
   float rgba[4];
   float xyz[3];
-  OpenGLView *view;
 
   visuDataGet_nodePosition(visuData, node, xyz);
-  view = visuDataGet_openGLView(visuData);
-  g_return_if_fail(view);
 
   glPushMatrix();
-  glTranslated(xyz[0] - view->box->dxxs2,
-	       xyz[1] - view->box->dyys2,
-	       xyz[2] - view->box->dzzs2);
+  glTranslated(xyz[0], xyz[1], xyz[2]);
   if (visuData->setColor)
     {
       visuData->setColor(visuData, rgba, ele, node);
@@ -637,21 +674,54 @@ void renderingAtomic_positionShape(VisuData *visuData, VisuNode *node, VisuEleme
   glCallList(ele->openGLIdentifier);
   glPopMatrix();
 }
+static void drawtriangle(float *v1, float *v2, float *v3) 
+{ 
+   glBegin(GL_TRIANGLES); 
+      glNormal3fv(v1); glVertex3fv(v1);    
+      glNormal3fv(v2); glVertex3fv(v2);    
+      glNormal3fv(v3); glVertex3fv(v3);    
+   glEnd(); 
+}
+static void normalize(float v[3]) {    
+   GLfloat d = sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]); 
 
+   g_return_if_fail(d > 0.);
+   v[0] /= d; v[1] /= d; v[2] /= d; 
+}
+static void subdivide(float *v1, float *v2, float *v3, int depth) 
+{ 
+  GLfloat v12[3], v23[3], v31[3];    
+  GLint i;
+
+  if (depth == 0)
+    {
+      drawtriangle(v1, v2, v3);
+      return;
+    }
+  for (i = 0; i < 3; i++)
+    { 
+      v12[i] = v1[i]+v2[i]; 
+      v23[i] = v2[i]+v3[i];     
+      v31[i] = v3[i]+v1[i];    
+    } 
+  normalize(v12);    
+  normalize(v23); 
+  normalize(v31); 
+  subdivide(v1, v12, v31, depth - 1);    
+  subdivide(v2, v23, v12, depth - 1);    
+  subdivide(v3, v31, v23, depth - 1);    
+  subdivide(v12, v23, v31, depth - 1); 
+}
 int renderingAtomic_createShape(VisuData *visuData, VisuElement* ele)
 {
   struct atomicResources_struct *str;
   OpenGLView *view;
-  int nlat;
+  int nlat, i, nfac;
 
   g_return_val_if_fail(visuData && ele, -1);
 
-  str = (struct atomicResources_struct*)visuElementGet_property(ele, "radiusAndShape");
-  if (!str)
-    {
-      str = createDefaultAtomicStruct();
-      visuElementSet_property(ele, "radiusAndShape", (gpointer)str);
-    }
+  str = getRadiusAndShape(ele);
+  g_return_val_if_fail(str, -1);
 
   view = visuDataGet_openGLView(visuData);
   nlat = OpenGLViewGet_numberOfFacettes(view, str->radius);
@@ -667,7 +737,26 @@ int renderingAtomic_createShape(VisuData *visuData, VisuElement* ele)
   switch (str->shape)
     {
     case shapeSphere:
-      gluSphere(obj, (double)str->radius, nlat, nlat);
+      DBG_fprintf(stderr, " | use sphere method %d\n", sphereMethod);
+      if (sphereMethod == sphere_glu)
+	gluSphere(obj, (double)str->radius, nlat, nlat);
+      else if (sphereMethod == sphere_icosahedron)
+	{
+	  nfac = (int)(log((float)(nlat + 2) / 4.f) / log(2.f));
+	  DBG_fprintf(stderr, " | glusphere vs. dodecahedron %dx%d\n",
+		      nlat * nlat, 20 * (int)pow(4, nfac));
+	  glPushMatrix();
+	  glScalef(str->radius, str->radius, str->radius);
+	  glBegin(GL_TRIANGLES);    
+	  for (i = 0; i < 20; i++)
+	    subdivide(&vdata[tindices[i][0]][0],       
+		      &vdata[tindices[i][1]][0],       
+		      &vdata[tindices[i][2]][0], nfac);
+	  glEnd();
+	  glPopMatrix();
+	}
+      else
+	g_warning("Wrong sphere method.");
       break;
     case shapeElipsoid:
       glPushMatrix();
