@@ -48,10 +48,34 @@
 
 #include "renderingAtomic.h"
 #include <visu_rendering.h>
+#include <visu_data.h>
 
+/**
+ * SECTION:renderingAtomic_ascii
+ * @short_description: Method to load ascii position file.
+ *
+ * <para>Ascii format is a plain text format to store elements and
+ * their position. The first line is a commentary and is not
+ * read. Then, the next two lines must contains each three numbers,
+ * describing the box containing the data. The first number is the
+ * length on x direction. The next two numbers are on one hand the
+ * component on the same x direction and on the other hand the other
+ * component on a direction orthogonal to the x direction, setting
+ * thus the y direction. The three other numbers are the component of
+ * the z direction on first, the x direction, then the y direction and
+ * finally in a direction orthogonal both to x and y directions. After
+ * these three first lines, all other lines can be blank, beginning
+ * with a '#' character or containing three numbers and a label name
+ * for the element.</para>
+ *
+ * <para>The read_ascii_file() method is used to try to load an ascii
+ * file in memory. Other methods are quite private and just put here
+ * for other loader which wants to share the code of this loader (see
+ * read_xyz_file() for instance).</para>
+ */
 
 static gboolean loadAscii(VisuData *data, const gchar* filename,
-			  FileFormat *format, GError **error);
+			  FileFormat *format, int nSet, GError **error);
 static int read_ascii_file(VisuData *data, FILE *flux, GError **error);
 
 
@@ -65,7 +89,9 @@ RenderingFormatLoad* atomicAsciiInit()
   meth->name = "Plain text native format of V_Sim.";
   meth->fmt = fileFormatNew(descrASCII, typeASCII);
   if (!meth->fmt)
-    g_error("Can't initialize the rendering atomic method, aborting...\n");
+    {
+      g_error("Can't initialize the rendering atomic method, aborting...");
+    }
   meth->priority = 50;
   meth->load = loadAscii;
 
@@ -94,9 +120,9 @@ int readFile_is_comment(char *str) {
 
 void freeDataAscii(gpointer data)
 {
-  free((struct dataAscii*)data);
+  g_free((struct dataAscii*)data);
 }
-void putValTypeInAsciiType(gpointer key, gpointer value, gpointer data)
+void putValTypeInAsciiType(gpointer key _U_, gpointer value, gpointer data)
 {
   VisuElement **tab;
   struct dataAscii *infos;
@@ -105,7 +131,7 @@ void putValTypeInAsciiType(gpointer key, gpointer value, gpointer data)
   infos = (struct dataAscii *)value;
   tab[infos->pos] = infos->ele;
 }
-void putValNbInAsciiType(gpointer key, gpointer value, gpointer data)
+void putValNbInAsciiType(gpointer key _U_, gpointer value, gpointer data)
 {
   int *tab;
   struct dataAscii *infos;
@@ -117,7 +143,7 @@ void putValNbInAsciiType(gpointer key, gpointer value, gpointer data)
 /******************************************************************************/
 
 static gboolean loadAscii(VisuData *data, const gchar* filename,
-			  FileFormat *format, GError **error)
+			  FileFormat *format _U_, int nSet _U_, GError **error)
 {
   int res;
   FILE *readFrom;
@@ -129,7 +155,7 @@ static gboolean loadAscii(VisuData *data, const gchar* filename,
   if (!readFrom)
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
-			   _("WARNING : impossible to open this file.\n"));
+			   _("Impossible to open this file.\n"));
       return FALSE;
     }
   /* Do the job of parsing. */
@@ -156,15 +182,16 @@ static int read_ascii_file(VisuData *data, FILE *flux, GError **error)
 {
   char line[MAX_LINE_LENGTH], info[MAX_LINE_LENGTH];
   double xu, yu, zu;
-  int i, res;
+  unsigned int i;
+  int res;
   char nomloc[9], *ptChar;
   VisuElement *type;
   VisuElement **types;
-  int *nattyp;
-  int ntype, nbLine;
+  unsigned int *nattyp;
+  unsigned int ntype, nbLine;
   GHashTable *elements;
   struct dataAscii *infos;
-  float boxGeometry[6];
+  float boxGeometry[6], xyz[3];
   gchar *infoUTF8;
     
   g_return_val_if_fail(error && *error == (GError*)0, FALSE);
@@ -174,8 +201,8 @@ static int read_ascii_file(VisuData *data, FILE *flux, GError **error)
   if(feof(flux))
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
-			   _("WARNING : the ASCII file should"
-			     " contain more than 3 lines\n"));
+			   _("The ASCII file should"
+			     " contain more than 3 lines.\n"));
       /* The file doesn't conform to ascii norm, we exit with a wrong type flag. */
       return -1;
     }
@@ -184,16 +211,16 @@ static int read_ascii_file(VisuData *data, FILE *flux, GError **error)
   if(feof(flux))
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
-			   _("WARNING : the ASCII file should"
-			     " contain more than 3 lines\n"));
+			   _("The ASCII file should"
+			     " contain more than 3 lines.\n"));
       /* The file doesn't conform to ascii norm, we exit with a wrong type flag. */
       return -1;
     }
   if(sscanf(line, "%f %f %f", &(boxGeometry[0]), &(boxGeometry[1]), &(boxGeometry[2])) != 3)
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
-			   _("WARNING : cannot read dxx dyx dyy"
-			     " on 2nd line of ASCII file\n"));
+			   _("Cannot read dxx dyx dyy"
+			     " on 2nd line of ASCII file.\n"));
       /* The file doesn't conform to ascii norm, we exit with a wrong type flag. */
       return -1;
     }
@@ -202,20 +229,19 @@ static int read_ascii_file(VisuData *data, FILE *flux, GError **error)
   if(feof(flux))
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
-			   _("WARNING : the ASCII file should"
-			     " contain more than 3 lines\n"));
+			   _("The ASCII file should"
+			     " contain more than 3 lines.\n"));
       /* The file doesn't conform to ascii norm, we exit with a wrong type flag. */
       return -1;
     }
   if(sscanf(line, "%f %f %f", &boxGeometry[3], &boxGeometry[4], &boxGeometry[5]) != 3)
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
-			   _("WARNING : cannot read dzx dzy dzz"
-			     " on 3rd line of ASCII file\n"));
+			   _("Cannot read dzx dzy dzz"
+			     " on 3rd line of ASCII file.\n"));
       /* The file doesn't conform to ascii norm, we exit with a wrong type flag. */
       return -1;
     }
-  visuDataSet_boxGeometry(data, boxGeometry, TRUE);
    
   /* Ok, from this point we assert that the file is in ascii format.
      All following errors are treated as syntax errors and the corresponding
@@ -224,12 +250,9 @@ static int read_ascii_file(VisuData *data, FILE *flux, GError **error)
   /* 1st pass to count ntype */
   nbLine = 4;
   ntype = 0;
-  elements = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, freeDataAscii);
-  if (!elements)
-    {
-      allocationProblems();
-      exit(1);
-    }
+  elements = g_hash_table_new_full(g_direct_hash, g_direct_equal,
+				   NULL, freeDataAscii);
+  g_return_val_if_fail(elements, -1);
   for(;;)
     {
       (void)fgets(line, MAX_LINE_LENGTH, flux);
@@ -244,7 +267,7 @@ static int read_ascii_file(VisuData *data, FILE *flux, GError **error)
             &xu, &yu, &zu, nomloc) != 4)
 	{
 	  *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
-			       _("WARNING : cannot read x, y, z,"
+			       _("Cannot read x, y, z,"
 				 " name in ASCII file at line %d.\n\n"
 				 " Quoting '%s'.\n"), nbLine, line);
 	  return 1;
@@ -254,11 +277,10 @@ static int read_ascii_file(VisuData *data, FILE *flux, GError **error)
       type = visuElementGet_fromName(nomloc);
       if (!type)
 	{
-	  type = visuElementNewWithName(nomloc);
+	  type = visuElementNew_withName(nomloc);
 	  if (!type)
 	    {
-	      g_warning("Cannot create a new"
-			" type for '%s'\n", nomloc);
+	      g_warning("Cannot create a new type for '%s'.", nomloc);
 	      return 1;
 	    }
 	  res = visuElementAdd(type);
@@ -271,12 +293,7 @@ static int read_ascii_file(VisuData *data, FILE *flux, GError **error)
       infos = (struct dataAscii*)g_hash_table_lookup(elements, (gconstpointer)type);
       if (!infos)
 	{
-	  infos = malloc(sizeof(struct dataAscii));
-	  if (!infos)
-	    {
-	      allocationProblems();
-	      exit(1);
-	    }
+	  infos = g_malloc(sizeof(struct dataAscii));
 	  infos->ele = type;
 	  infos->pos = ntype;
 	  infos->nbNodes = 1;
@@ -287,27 +304,17 @@ static int read_ascii_file(VisuData *data, FILE *flux, GError **error)
 	infos->nbNodes += 1;
     }
   /* Allocate the space for the nodes. */
-  types = malloc(sizeof(VisuElement*) * ntype);
-  if (!types)
-    {
-      allocationProblems();
-      exit(1);
-    }
-  nattyp = malloc(sizeof(int) * ntype);
-  if(!nattyp)
-    {
-      allocationProblems();
-      exit(1);
-    }
+  types = g_malloc(sizeof(VisuElement*) * ntype);
+  nattyp = g_malloc(sizeof(int) * ntype);
   g_hash_table_foreach(elements, (GHFunc)putValTypeInAsciiType, (gpointer)&types);
   g_hash_table_foreach(elements, (GHFunc)putValNbInAsciiType, (gpointer)&nattyp);
   g_hash_table_destroy(elements);
   if (ntype <= 0)
     {
-      free(nattyp);
-      free(types);
+      g_free(nattyp);
+      g_free(types);
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
-			   _("WARNING : the file contains no atom coordinates.\n"));
+			   _("The file contains no atom coordinates.\n"));
       return 1;
     }
 
@@ -317,17 +324,24 @@ static int read_ascii_file(VisuData *data, FILE *flux, GError **error)
   if (ptChar)
     *ptChar = '\0';
   if (info[0] == '#')
-    infoUTF8 = getStringInUTF8(info + 1);
+    infoUTF8 = g_locale_to_utf8(info + 1, -1, NULL, NULL, NULL);
   else
-    infoUTF8 = getStringInUTF8(info);
-  visuDataSet_fileCommentary(data, infoUTF8);
-  g_free(infoUTF8);
+    infoUTF8 = g_locale_to_utf8(info, -1, NULL, NULL, NULL);
+  if (infoUTF8)
+    {
+      visuDataSet_fileCommentary(data, infoUTF8, 0);
+      g_free(infoUTF8);
+    }
+  else
+    g_warning("Can't convert '%s' to UTF8.\n", info);
 
   res = visuDataSet_population(data, ntype, nattyp, types);
   if (!res)
-    g_error("Can't store the nodes in the VisuData object.");
-  free(nattyp);
-  free(types);
+    {
+      g_error("Can't store the nodes in the VisuData object.");
+    }
+  g_free(nattyp);
+  g_free(types);
          
   /* reread the file to store the coordinates */
   
@@ -355,10 +369,16 @@ static int read_ascii_file(VisuData *data, FILE *flux, GError **error)
 		    " seems to be new.\n", nomloc);
 	  return 1;
 	}
-      visuDataAdd_VisuElement(data, type, xu, yu, zu, i);
+      xyz[0] = (float)xu;
+      xyz[1] = (float)yu;
+      xyz[2] = (float)zu;
+      visuDataAdd_nodeFromElement(data, type, xyz);
       i++;
     }
+
+  /* We finish with the box description. */
+  visuDataSet_boxGeometry(data, boxGeometry, TRUE);
    
-   return 0;
+  return 0;
 }
 
