@@ -46,6 +46,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <GL/gl.h>
+#include <GL/glu.h> 
+
 #include "renderingSpin.h"
 #include "renderingAtomic.h" /* To get the position file format
 				(and posibly in the future, the read
@@ -70,12 +73,6 @@
 #define FLAG_RESOURCES_SPIN "spin_resources"
 #define DESC_RESOURCES_SPIN "Global or element resource for rendering spin module"
 
-#define SPINVALUES_ID     "spinRendering_values"
-#define SPINMAXMODULUS_ID "spinRendering_maxModulus"
-#define SPIN_THETA   0
-#define SPIN_PHI     1
-#define SPIN_MODULUS 2
-
 /* An identifier for the different shapes to draw elements. */
 enum
   {
@@ -130,14 +127,14 @@ static GType spinElementResourcesTypes[spin_nbElementResources] =
 /* Global resources for spin rendering and associated flag. */
 static GType spinGlobalResourcesTypes[spin_nbGlobalResources] =
   {G_TYPE_FLOAT, G_TYPE_FLOAT, G_TYPE_FLOAT,
-   G_TYPE_UINT, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN};
+   G_TYPE_UINT, G_TYPE_BOOLEAN, G_TYPE_UINT};
 float coneOrientation[2];
 float colorWheel;
 SpinDrawingPolicy spinPolicy;
 gboolean spinAndAtomicRendering;
-gboolean spinModulusUsage;
-
+SpinModulusPolicy spinModulusUsage;
 /* Internal variables. */
+static GValue spinValue = {0, {{0}, {0}}};
 static DataNode *dataNode;
 static GList *allSpinMethods;
 
@@ -161,22 +158,28 @@ const char** policyNameSpin_UTF8;
 int identifierSpheresSpin;
 
 /* Local methods. */
-static gboolean loadSpin(VisuData *data, FileFormat *format, GError **error);
-gboolean readSpinResources(gchar **lines, int nbLines,
-			   int position, GString *errorMessage);
+static gboolean loadSpin(VisuData *data, FileFormat *format,
+			 int nSet, GError **error);
+static gboolean readSpinResources(gchar **lines, int nbLines,
+				  int position, VisuData *dataObj, GError **error);
 gboolean exportResourcesRenderingSpin(GString *data, int *nbLinesWritten,
 				      VisuData *dataObj);
-void setVariablesForSpinMethod(GObject *visu, gpointer data);
+static void onRenderingChanged(GObject *visu, RenderingMethod *method,
+			       gpointer data);
 static struct spinResources_struct* getSpinResources(VisuElement *ele);
 static void onSpinParametersChanged(VisuData *dataObj, VisuNode *node, gpointer data);
+static void freeSpin(gpointer obj, gpointer data);
+static gpointer newOrCopySpin(gconstpointer obj, gpointer data);
 
 /* Local loading methods. */
+static const char* rspin_shape_number_to_name(int n);
+static int rspin_shape_name_to_number(const char *name);
 static RenderingFormatLoad* spinAsciiInit();
 static RenderingFormatLoad* spinBinaryInit();
 static gboolean read_spin_file(VisuData *data, const char* fileName,
-			       FileFormat *format, GError **error);
+			       FileFormat *format, int nSet, GError **error);
 static gboolean read_binary_file(VisuData *data, const char* fileName,
-				 FileFormat *format, GError **error);
+				 FileFormat *format, int nSet, GError **error);
 
 
 int rspin_get_number_of_shapes()
@@ -184,7 +187,7 @@ int rspin_get_number_of_shapes()
   return nb_of_shapes;
 }
 
-int rspin_shape_name_to_number(const char *name)
+static int rspin_shape_name_to_number(const char *name)
 {
   int i=0;
   if(name == NULL) 
@@ -197,7 +200,7 @@ int rspin_shape_name_to_number(const char *name)
   return -1;
 }
 
-const char* rspin_shape_number_to_name(int n)
+static const char* rspin_shape_number_to_name(int n)
 {
   if(n>=nb_of_shapes && n < 0)
     return NULL;
@@ -244,7 +247,7 @@ const char* rspin_hiding_number_to_translated_name(int n)
 
 float returnOneF()
 {
-  return 1.0;
+  return 1.0f;
 }
 
 /*****************************************************************************/
@@ -280,8 +283,10 @@ RenderingMethod* initSpin()
   /* We check that the atomic rendering has been already defined, to use its own
      loader and file formats. */
   if (!pointerOnRenderingAtomicMethod)
-    g_error("Can't initialize the RenderingSpin method,"
+    {
+      g_error("Can't initialize the RenderingSpin method,"
 	      " renderingAtomic must be initialized first. Aborting...\n");
+    }
   formatListPos = renderingMethodGet_fileType(pointerOnRenderingAtomicMethod, FILE_KIND_POSITION);
 
   /* We create the list of spin file formats. */
@@ -297,12 +302,14 @@ RenderingMethod* initSpin()
   tmpLst = allSpinMethods;
   while (tmpLst)
     {
-      formatListSpin = g_list_append(formatListSpin, ((RenderingFormatLoad*)tmpLst->data)->fmt);
+      formatListSpin = g_list_append(formatListSpin,
+				     ((RenderingFormatLoad*)tmpLst->data)->fmt);
       tmpLst = g_list_next(tmpLst);
     }
   
   /* Calling the constructor to create a new renderingMethod. */
-  spin = renderingMethod_new("Spin visualisation", methodName, methodDescr, 2, loadSpin);
+  spin = renderingMethod_new("Spin visualisation", methodName, methodDescr,
+			     2, loadSpin);
   renderingMethodSet_fileType(spin, formatListPos, FILE_KIND_POSITION, _("Position files"));
   renderingMethodSet_fileType(spin, formatListSpin, FILE_KIND_SPIN, _("Spin files"));
 
@@ -320,6 +327,7 @@ RenderingMethod* initSpin()
 					  FLAG_RESOURCES_SPIN,
 					  DESC_RESOURCES_SPIN,
 					  1, readSpinResources);
+  visuConfigFileSet_version(resourceEntry, 3.1f);
   visuConfigFileAdd_exportFunction(VISU_CONFIGFILE_RESOURCE,
 				   exportResourcesRenderingSpin);
   
@@ -331,21 +339,24 @@ RenderingMethod* initSpin()
 
 
   /* Initialise the OpenGl part by reserving space in the display list to draw objects. */
-  identifierSpheresSpin = openGLObjectList_new(NMAX_TP);
-  spinPolicy = commandLineGet_spinHidingMode();
+  identifierSpheresSpin  = openGLObjectList_new(NMAX_TP);
+  spinPolicy             = commandLineGet_spinHidingMode();
   spinAndAtomicRendering = commandLineGet_spinAndAtomic();
-  spinModulusUsage = FALSE;
-  coneOrientation[0] = 0.;
-  coneOrientation[1] = 0.;
-  colorWheel = 0.;
+  spinModulusUsage       = policyNoneModulus;
+  coneOrientation[0]     = 0.;
+  coneOrientation[1]     = 0.;
+  colorWheel             = 0.;
 
   /* We set a general pointer to retrieve this method later. */
   pointerOnRenderingSpinMethod = spin;
 
+  /* We initialise the container GValue for the spin property. */
+  g_value_init(&spinValue, G_TYPE_POINTER);
+
   /* Connect to the signal method changed to set or unset some variables
      when spin is current or not. */
   g_signal_connect(G_OBJECT(visu), "renderingChanged",
-		   G_CALLBACK(setVariablesForSpinMethod), (gpointer)spin);
+		   G_CALLBACK(onRenderingChanged), (gpointer)spin);
 
   /* Register a new NodeData. */
   dataNode = nodeDataNew(SPINVALUES_ID, G_TYPE_FLOAT);
@@ -366,7 +377,9 @@ static RenderingFormatLoad* spinAsciiInit()
   meth->name = "Plain text native spin format of V_Sim.";
   meth->fmt = fileFormatNew(descrSpin, typeSpin);
   if (!meth->fmt)
-    g_error("Can't initialize the rendering spin method, aborting...\n");
+    {
+      g_error("Can't initialize the rendering spin method, aborting...\n");
+    }
   meth->priority = 50;
   meth->load = read_spin_file;
 
@@ -384,16 +397,19 @@ static RenderingFormatLoad* spinBinaryInit()
   meth->name = "Binary native spin format of V_Sim.";
   meth->fmt = fileFormatNew(descrSpin, typeSpin);
   if (!meth->fmt)
-    g_error("Can't initialize the rendering spin method, aborting...\n");
+    {
+      g_error("Can't initialize the rendering spin method, aborting...\n");
+    }
   meth->priority = 10;
   meth->load = read_binary_file;
 
   return meth;
 }
 
-void setVariablesForSpinMethod(GObject *visu, gpointer data)
+static void onRenderingChanged(GObject *visu _U_, RenderingMethod *method,
+			       gpointer data)
 {
-  if (getRenderingMethodInUse() == (RenderingMethod*)data)
+  if (method == (RenderingMethod*)data)
     visuElementSet_updateNodesOnMaterialChange();
   else
     visuElementUnset_updateNodesOnMaterialChange();
@@ -410,7 +426,7 @@ void setVariablesForSpinMethod(GObject *visu, gpointer data)
 
 /* This is the public method to load a spin file.
    The VisuData should be already allocated and nodes should exist. */
-gboolean rspin_load(VisuData *data, FileFormat *format, GError **error)
+gboolean rspin_load(VisuData *data, FileFormat *format _U_, int nSet, GError **error)
 {
   gchar *file;
   gboolean loadOk;
@@ -433,8 +449,7 @@ gboolean rspin_load(VisuData *data, FileFormat *format, GError **error)
   if (g_file_test(file, G_FILE_TEST_IS_DIR))
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
-			   _("WARNING! The specified file is"
-			     " a directory indeed. Can't load"
+			   _("The specified file is a directory indeed. Can't load"
 			     " a directory!\n"));
       return FALSE;
     }
@@ -454,7 +469,7 @@ gboolean rspin_load(VisuData *data, FileFormat *format, GError **error)
 		      file, ((RenderingFormatLoad*)tmpLst->data)->name);
 	  if (((RenderingFormatLoad*)tmpLst->data)->load)
 	    loadOk = ((RenderingFormatLoad*)tmpLst->data)->load
-	      (data, file, ((RenderingFormatLoad*)tmpLst->data)->fmt, error);
+	      (data, file, ((RenderingFormatLoad*)tmpLst->data)->fmt, nSet, error);
 	  else
 	    loadOk = FALSE;
 	}
@@ -465,23 +480,51 @@ gboolean rspin_load(VisuData *data, FileFormat *format, GError **error)
     {
       if (!*error)
 	*error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
-			     _("WARNING : impossible to load this file.\n"));
+			     _("impossible to load this file.\n"));
       return FALSE;
     }
 
+  nodeDataSet_used(dataNode, data, 3);
   return TRUE;
 }
 
+static void freeSpin(gpointer obj, gpointer data _U_)
+{
+#if GLIB_MINOR_VERSION > 9
+  g_slice_free1(sizeof(float) * 3, obj);
+#else
+  g_free(obj);
+#endif
+}
+static gpointer newOrCopySpin(gconstpointer obj, gpointer data _U_)
+{
+  float *spinData;
+
+#if GLIB_MINOR_VERSION > 9
+  spinData = g_slice_alloc(sizeof(float) * 3);
+#else
+  spinData = g_malloc(sizeof(float) * 3);
+#endif
+  if (obj)
+    memcpy(spinData, obj, sizeof(float) * 3);
+  else
+    memset(spinData, 0, sizeof(float) * 3);
+    
+  return (gpointer)spinData;
+}
+
 /* This is the method associated to the reading of supported spin files. */
 static gboolean read_spin_file(VisuData *data, const char* fileName,
-			       FileFormat *format, GError **error)
+			       FileFormat *format _U_, int nSet _U_, GError **error)
 {
   char line[MAX_LINE_LENGTH] = "\0";
-  float theta=0, phi=0, module = 0;
+  float vals[3];
   float *svgSpinValues, *svgMaxSpinModulus;
-  int i=0, itrash = 0, iele;
-  VisuNode *currentNode = NULL;
+  int itrash, iLine;
+  VisuDataIter iter;
   FILE *readFrom;
+  gboolean readContinue;
+  VisuNodeProperty *spin;
 
   g_return_val_if_fail(error && *error == (GError*)0, FALSE);
 
@@ -489,81 +532,82 @@ static gboolean read_spin_file(VisuData *data, const char* fileName,
   if (!readFrom)
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
-			   _("WARNING : impossible to open this spin file.\n"));
+			   _("impossible to open this spin file.\n"));
       return FALSE;
     }
+  iLine = 1;
 
   /* The first line is a commentry. */
   fgets(line, MAX_LINE_LENGTH, readFrom);
   if(feof(readFrom))
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
-			   _("WARNING : spin file should have one line at least.\n"));
+			   _("spin file should have one line at least.\n"));
       fclose(readFrom);
       return FALSE;
     }
+  iLine += 1;
    
   /* Create a storage for max values of spin modulus for each element. */
   svgMaxSpinModulus = g_malloc(sizeof(float) * data->ntype);
-  visuDataSet_property(data, SPINMAXMODULUS_ID, (gpointer)svgMaxSpinModulus);
-  visuDataAdd_nodeProperty(data, SPINVALUES_ID, freeFloat);
-  nodeDataSet_used(dataNode, data, 3);
-  for(i=1;i<=data->nbOfAllStoredNodes;i++)
+  visuDataSet_propertyWithDestroyFunc(data, SPINMAXMODULUS_ID,
+				      (gpointer)svgMaxSpinModulus, g_free);
+  spin = visuNodeNew_pointerProperty(visuDataGet_nodeArray(data), SPINVALUES_ID,
+				     freeSpin, newOrCopySpin, (gpointer)0);
+  
+  readContinue = TRUE;
+  visuDataIter_new(data, &iter);
+  for(visuDataIter_startNumber(data, &iter); iter.node;
+      visuDataIter_nextNodeNumber(data, &iter))
     {
-      fgets(line, MAX_LINE_LENGTH, readFrom);
-      if(feof(readFrom)) break;
-
-      currentNode = visuDataGet_nodeFromNumber(data, i);
-      if(currentNode == 0) 
+      if (readContinue)
 	{
-	  g_warning("Information for node number %d can't be retrieved.", i);
-	  continue;
+	  fgets(line, MAX_LINE_LENGTH, readFrom);
+	  if(feof(readFrom))
+	    readContinue = FALSE;
+	  else
+	    {
+	      if(sscanf(line, "%d %f %f %f", &itrash, vals + SPIN_MODULUS,
+			vals + SPIN_THETA, vals + SPIN_PHI) != 4)
+		{
+		  g_warning("line number #%d is invalid."
+			    " Setting node parameters to default ones...", iLine);
+		  vals[SPIN_THETA]   = 0.f;
+		  vals[SPIN_PHI]     = 0.f;
+		  vals[SPIN_MODULUS] = 0.f;
+		}
+	    }
+	  iLine += 1;
 	}
-      
-      if(sscanf(line, "%d %f %f %f", &itrash, &module, &theta, &phi) != 4)
+      else
 	{
-	  g_warning("line number #%d is invalid."
-		    " Setting node parameters to default ones...", i);
-	  theta = 0.;
-	  phi = 0.;
-	  module = 0.;
-	}      
-      svgSpinValues = g_malloc(sizeof(float) * 3);
-      svgSpinValues[SPIN_THETA] = theta;
-      svgSpinValues[SPIN_PHI] = phi;
-      svgSpinValues[SPIN_MODULUS] = module;
-      visuDataSet_nodeProperty(data, currentNode, SPINVALUES_ID,
-			       (gpointer)svgSpinValues);
-      iele = currentNode->posElement;
-      svgMaxSpinModulus[iele] = (module > svgMaxSpinModulus[iele])?module:svgMaxSpinModulus[iele];
+	  vals[SPIN_THETA]   = 0.f;
+	  vals[SPIN_PHI]     = 0.f;
+	  vals[SPIN_MODULUS] = 0.f;
+	}
+      svgSpinValues = newOrCopySpin(vals, (gpointer)0);
+      g_value_set_pointer(&spinValue, svgSpinValues);
+      visuNodePropertySet_value(spin, iter.node, &spinValue);
+      svgMaxSpinModulus[iter.iElement] = MAX(vals[SPIN_MODULUS],
+					     svgMaxSpinModulus[iter.iElement]);
     }
   fclose(readFrom);
-  itrash = i;
-  for(i = itrash; i <= data->nbOfAllStoredNodes; i++)
-    {
-      currentNode = visuDataGet_nodeFromNumber(data, i);
-      svgSpinValues = g_malloc(sizeof(float) * 3);
-      svgSpinValues[SPIN_THETA] = 0.;
-      svgSpinValues[SPIN_PHI] = 0.;
-      svgSpinValues[SPIN_MODULUS] = 0.;
-      visuDataSet_nodeProperty(data, currentNode, SPINVALUES_ID,
-			       (gpointer)svgSpinValues);
-    }
   return TRUE;
 }
 
 /* This is the method associated to the reading of binary spin files. */
 static gboolean read_binary_file(VisuData *data, const char* fileName,
-				 FileFormat *format, GError **error)
+				 FileFormat *format _U_, int nSet _U_, GError **error)
 {
   FILE *readFrom;
   gboolean valid;
   EndianId endian;
   unsigned int nspins;
   double *spins;
-  int i, iele;
-  float *svgSpinValues, *svgMaxSpinModulus;
-  VisuNode *currentNode;
+  int i;
+  float *svgSpinValues, *svgMaxSpinModulus, vals[3];
+  VisuDataIter iter;
+  VisuNodeProperty *spin;
 
   g_return_val_if_fail(error && *error == (GError*)0, FALSE);
 
@@ -571,7 +615,7 @@ static gboolean read_binary_file(VisuData *data, const char* fileName,
   if (!readFrom)
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FILE,
-			   _("WARNING : impossible to open this spin file.\n"));
+			   _("impossible to open this spin file.\n"));
       return FALSE;
     }
 
@@ -592,10 +636,11 @@ static gboolean read_binary_file(VisuData *data, const char* fileName,
     }
 
   /* From now on, we consider to a have valid spin file. */
-  if ((int)nspins != data->nbOfAllStoredNodes)
+  visuDataIter_new(data, &iter);
+  if (nspins != iter.nAllStoredNodes)
     {
       *error = g_error_new(VISU_ERROR_RENDERING, RENDERING_ERROR_FORMAT,
-			   _("WARNING : number of spin differs from number of nodes.\n"));
+			   _("number of spin differs from number of nodes.\n"));
       fclose(readFrom);
       return TRUE;
     }
@@ -630,26 +675,23 @@ static gboolean read_binary_file(VisuData *data, const char* fileName,
 
   /* Create a storage for max values of spin modulus for each element. */
   svgMaxSpinModulus = g_malloc(sizeof(float) * data->ntype);
-  visuDataSet_property(data, SPINMAXMODULUS_ID, (gpointer)svgMaxSpinModulus);
-  visuDataAdd_nodeProperty(data, SPINVALUES_ID, freeFloat);
-  nodeDataSet_used(dataNode, data, 3);
-  for(i = 0; i < data->nbOfAllStoredNodes; i++)
+  visuDataSet_propertyWithDestroyFunc(data, SPINMAXMODULUS_ID,
+				      (gpointer)svgMaxSpinModulus, g_free);
+  spin = visuNodeNew_pointerProperty(visuDataGet_nodeArray(data), SPINVALUES_ID,
+				     freeSpin, newOrCopySpin, (gpointer)0);
+
+  i = 0;
+  for(visuDataIter_start(data, &iter); iter.node; visuDataIter_next(data, &iter))
     {
-      currentNode = visuDataGet_nodeFromNumber(data, i + 1);
-      if(currentNode == 0) 
-	{
-	  g_warning("Information for node number %d can't be retrieved.", i);
-	  continue;
-	}
-      
-      svgSpinValues = g_malloc(sizeof(float) * 3);
-      svgSpinValues[SPIN_MODULUS] = (float)spins[i];
-      svgSpinValues[SPIN_THETA]   = (float)spins[nspins + i];
-      svgSpinValues[SPIN_PHI]     = (float)spins[2 * nspins + i];
-      visuDataSet_nodeProperty(data, currentNode, SPINVALUES_ID,
-			       (gpointer)svgSpinValues);
-      iele = currentNode->posElement;
-      svgMaxSpinModulus[iele] = MAX((float)spins[i], svgMaxSpinModulus[iele]);
+      vals[SPIN_MODULUS] = (float)spins[i];
+      vals[SPIN_THETA]   = (float)spins[nspins + i];
+      vals[SPIN_PHI]     = (float)spins[2 * nspins + i];
+      svgSpinValues = newOrCopySpin(vals, (gpointer)0);
+      g_value_set_pointer(&spinValue, svgSpinValues);
+      visuNodePropertySet_value(spin, iter.node, &spinValue);
+      svgMaxSpinModulus[iter.iElement] = MAX((float)spins[i],
+					     svgMaxSpinModulus[iter.iElement]);
+      i += 1;
     }
 
   g_free(spins);
@@ -664,10 +706,11 @@ void rspin_addLoadMethod(RenderingFormatLoad* meth)
   allSpinMethods = g_list_prepend(allSpinMethods, meth);
   allSpinMethods = g_list_sort(allSpinMethods, renderingMethodCompare_priority);
   if (meth->fmt)
-    renderingMethodAdd_fileFormat(pointerOnRenderingSpinMethod, meth->fmt, 0);
+    renderingMethodAdd_fileFormat(pointerOnRenderingSpinMethod,
+				  meth->fmt, FILE_KIND_SPIN);
 }
 
-static gboolean loadSpin(VisuData *data, FileFormat *format, GError **error)
+static gboolean loadSpin(VisuData *data, FileFormat *format, int nSet, GError **error)
 {
   g_return_val_if_fail(error && *error == (GError*)0, FALSE);
 
@@ -676,13 +719,13 @@ static gboolean loadSpin(VisuData *data, FileFormat *format, GError **error)
   
   DBG_fprintf(stderr, "Rendering Spin : starting loadSpin procedure...\n");
 
-  if(!renderingAtomicLoad(data, format, error))
+  if(!renderingAtomicLoad(data, format, nSet, error))
     return FALSE;
   /* If we have errors, we return. */
   if (*error)
     return TRUE;
 
-  if (!rspin_load(data, format, error))
+  if (!rspin_load(data, format, nSet, error))
     return FALSE;
   /* If we have errors, we return. */
   if (*error)
@@ -1001,6 +1044,7 @@ static gboolean readElementArrow(gchar **lines, int nbLines,
   int id;
 
   g_return_val_if_fail(error && (*error == (GError*)0), FALSE);
+  g_return_val_if_fail(nbLines == 1, FALSE);
 
   tokens = g_strsplit_set(lines[0], " \n", MAX_LINE_LENGTH);
   id = 0;
@@ -1053,6 +1097,7 @@ static gboolean readElementElipsoid(gchar **lines, int nbLines,
   int id;
 
   g_return_val_if_fail(error && (*error == (GError*)0), FALSE);
+  g_return_val_if_fail(nbLines == 1, FALSE);
 
   tokens = g_strsplit_set(lines[0], " \n", MAX_LINE_LENGTH);
   id = 0;
@@ -1100,6 +1145,7 @@ static gboolean readElementShape(gchar **lines, int nbLines,
   struct spinResources_struct *str;
 
   g_return_val_if_fail(error && (*error == (GError*)0), FALSE);
+  g_return_val_if_fail(nbLines == 1, FALSE);
 
   /* Read a string.*/
   res = configFileRead_stringWithElement(lines[0], position, &shapes, 1, &ele, error);
@@ -1112,8 +1158,7 @@ static gboolean readElementShape(gchar **lines, int nbLines,
       if (shapeId == -1)
 	{
 	  *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_READ,
-			       _("WARNING! Parse error at line %d," 
-				 " the shape '%s' is unknown.\n"), 
+			       _("Parse error at line %d, the shape '%s' is unknown.\n"), 
 			       position, shapes[0]);
 	  g_strfreev(shapes);
 	  return FALSE;
@@ -1129,12 +1174,14 @@ static gboolean readElementShape(gchar **lines, int nbLines,
 
 #define FLAG_SPIN_CONE_ANGLE "spin_global_color_cone"
 static gboolean readSpinColorCone(gchar **lines, int nbLines,
-				   int position, VisuData *dataObj, GError **error)
+				  int position, VisuData *dataObj _U_,
+				  GError **error)
 {
   float angles[2];
   gboolean res;
 
   g_return_val_if_fail(error && (*error == (GError*)0), FALSE);
+  g_return_val_if_fail(nbLines == 1, FALSE);
 
   /* Read one floating point values.*/
   res = configFileRead_float(lines[0], position, angles, 2, error);
@@ -1150,12 +1197,14 @@ static gboolean readSpinColorCone(gchar **lines, int nbLines,
 }
 #define FLAG_SPIN_WHEEL_ANGLE "spin_global_color_wheel"
 static gboolean readSpinColorWheel(gchar **lines, int nbLines,
-				   int position, VisuData *dataObj, GError **error)
+				   int position, VisuData *dataObj _U_,
+				   GError **error)
 {
   float angle;
   gboolean res;
 
   g_return_val_if_fail(error && (*error == (GError*)0), FALSE);
+  g_return_val_if_fail(nbLines == 1, FALSE);
 
   /* Read one floating point values.*/
   res = configFileRead_float(lines[0], position, &angle, 1, error);
@@ -1168,16 +1217,18 @@ static gboolean readSpinColorWheel(gchar **lines, int nbLines,
 }
 #define FLAG_SPIN_HIDING_MODE "spin_global_hiding_mode"
 static gboolean readSpinHidingMode(gchar **lines, int nbLines,
-				   int position, VisuData *dataObj, GError **error)
+				   int position, VisuData *dataObj _U_,
+				   GError **error)
 {
   gchar **mode;
   int modeId;
   gboolean res;
 
   g_return_val_if_fail(error && (*error == (GError*)0), FALSE);
+  g_return_val_if_fail(nbLines == 1, FALSE);
 
   /* Read a string.*/
-  res = configFileRead_string(lines[0], position, &mode, 1, error);
+  res = configFileRead_string(lines[0], position, &mode, 1, FALSE, error);
   if (*error)
     return FALSE;
   if (res)
@@ -1187,7 +1238,7 @@ static gboolean readSpinHidingMode(gchar **lines, int nbLines,
       if (modeId == -1)
 	{
 	  *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_READ,
-			       _("WARNING! Parse error at line %d," 
+			       _("Parse error at line %d," 
 				 " the hiding mode '%s' is unknown.\n"), 
 			       position, mode[0]);
 	  g_strfreev(mode);
@@ -1200,12 +1251,14 @@ static gboolean readSpinHidingMode(gchar **lines, int nbLines,
 }
 #define FLAG_SPIN_AND_ATOMIC "spin_global_atomic"
 static gboolean readSpinAndAtomic(gchar **lines, int nbLines,
-				  int position, VisuData *dataObj, GError **error)
+				  int position, VisuData *dataObj _U_,
+				  GError **error)
 {
   gboolean use;
   gboolean res;
 
   g_return_val_if_fail(error && (*error == (GError*)0), FALSE);
+  g_return_val_if_fail(nbLines == 1, FALSE);
 
   /* Read one floating point values.*/
   res = configFileRead_boolean(lines[0], position, &use, 1, error);
@@ -1218,54 +1271,48 @@ static gboolean readSpinAndAtomic(gchar **lines, int nbLines,
 }
 #define FLAG_SPIN_MODULUS "spin_global_modulus"
 static gboolean readSpinModulus(gchar **lines, int nbLines,
-				   int position, VisuData *dataObj, GError **error)
+				int position, VisuData *dataObj _U_,
+				GError **error)
 {
-  gboolean use;
+  int modeId;
   gboolean res;
 
   g_return_val_if_fail(error && (*error == (GError*)0), FALSE);
+  g_return_val_if_fail(nbLines == 1, FALSE);
 
-  /* Read one floating point values.*/
-  res = configFileRead_boolean(lines[0], position, &use, 1, error);
+  /* Read a string.*/
+  res = configFileRead_integer(lines[0], position, &modeId, 1, error);
   if (*error)
     return FALSE;
   if (res)
-    /* All is OK, we store the values. */
-    spinModulusUsage = use;
+    {
+      if (modeId < 0 || modeId >= policyNbModulusModes)
+	{
+	  *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_READ,
+			       _("Parse error at line %d," 
+				 " the modulus mode '%d' is unknown.\n"), 
+			       position, modeId);
+	  return FALSE;
+	}
+      spinModulusUsage = (SpinModulusPolicy)modeId;
+    }
   return res;
 }
 
-/** 
- * readSpinResources:
- * @lines: an array of strings ;
- * @nbLines: an integer ;
- * @position: an integer ;
- * @errorMessage: a pointer to a GString or NULL.
- *
- * Returns: 1 if an error occured, 0 otherwise.
- *
- * This is the function called by the core when a spin resource flag is found
- * while reading a resource file. It tries to read, extract and set the resources 
- * it reads from the given file.
- */ 
-gboolean readSpinResources(gchar **lines, int nbLines,
-			   int position, GString *errorMessage)
+static gboolean readSpinResources(gchar **lines, int nbLines,
+				  int position, VisuData *dataObj, GError **error)
 {
-  GenericRenderingWindow window;
-  VisuData *dataObj;
-  GError *error;
   gchar **tokens;
-  int i;
+  int i, token;
   gboolean ok;
   VisuElement* ele;
-  char *cursor;
   char shape[MAX_LINE_LENGTH];
   float length, u_length, height, u_height;
   int use_color_element, use_element_color_hat;
   int shape_number;
   struct spinResources_struct *str;
   #define NB_SPIN_READ_METHODS 8
-  configFileReadFunc readFuncs[NB_SPIN_READ_METHODS] =
+  visuConfigFileReadFunc readFuncs[NB_SPIN_READ_METHODS] =
     {readElementArrow, readElementElipsoid, readElementShape,
      readSpinColorCone, readSpinColorWheel,
      readSpinHidingMode, readSpinAndAtomic, readSpinModulus};
@@ -1274,30 +1321,18 @@ gboolean readSpinResources(gchar **lines, int nbLines,
      FLAG_SPIN_CONE_ANGLE, FLAG_SPIN_WHEEL_ANGLE,
      FLAG_SPIN_HIDING_MODE, FLAG_SPIN_AND_ATOMIC, FLAG_SPIN_MODULUS};
 
-  window = visuRenderingWindowGet_current();
-  if (window)
-    dataObj = visuRenderingWindowGet_visuData(window);
-  else
-    dataObj = (VisuData*)0;
+  g_return_val_if_fail(nbLines == 1, FALSE);
 
   /* Read the first keyword. */
   tokens = g_strsplit_set(g_strchug(lines[0]), " \n", 2);
 
   /* Try to find an element/global resource flag. */
-  error = (GError*)0;
   for (i = 0; i < NB_SPIN_READ_METHODS; i++)
     {
       if (!strcmp(g_strstrip(tokens[0]), readFlags[i]))
 	{
-	  ok = readFuncs[i](tokens + 1, 1, position, dataObj, &error);
-	  if (error)
-	    {
-	      if (errorMessage)
-		g_string_append_printf(errorMessage, error->message);
-	      g_error_free(error);
-	      g_strfreev(tokens);
-	      return FALSE;
-	    }
+	  ok = readFuncs[i](tokens + 1, 1, position, dataObj, error);
+	  g_strfreev(tokens);
 	  return ok;
 	}
     }
@@ -1309,41 +1344,36 @@ gboolean readSpinResources(gchar **lines, int nbLines,
       return TRUE;
     }
 
-  g_strfreev(tokens);
-
-  /* Finaly read the old format "ele + values". */
-  cursor = lines[0];
-  error = (GError*)0;
-  ele = visuElementRead_fromLine(&cursor, MAX_LINE_LENGTH, position, &error);
-  if (error)
+  /* Finaly fallback the old format "ele + values". */
+  token = 0;
+  ok = configFileRead_elementFromTokens(tokens, &token, &ele, 1, nbLines, error);
+  if (!ok)
     {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, error->message);
-      g_error_free(error);
+      g_strfreev(tokens);
       return FALSE;
     }
   
   /* Retrieving element's resources */
-  if(sscanf(cursor, "%s %f %f %f %f %d %d", shape, 
+  if(sscanf(tokens[1], "%s %f %f %f %f %d %d", shape, 
 	     &height, &u_height, &length, &u_length, 
 	     &use_color_element, &use_element_color_hat) != 7
      || length <= 0. || u_length <= 0. 
      || height <= 0. || u_height <= 0.)
     {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, 
-			       _("WARNING! Parse error at line %d : 4 valid floating point "
-				 "values and a shape name must appear after the %s markup.\n"),
-			       position, FLAG_RESOURCES_SPIN);
+      *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_MISSING,
+			   _("Parse error at line %d: a shape with 4 floating points "
+			     "and 2 booleans must appear after the %s markup.\n"),
+			   position, FLAG_RESOURCES_SPIN);
+      g_strfreev(tokens);
       return FALSE;
     }
+  g_strfreev(tokens);
   if((shape_number = rspin_shape_name_to_number(shape)) == -1)
     {
-      if (errorMessage)
-	g_string_append_printf(errorMessage, _("WARNING! Parse error at line %d," 
-					       " the shape '%s' is unknown.\n"), 
-			       position, shape);
-      shape_number = 0;
+      *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_VALUE,
+			   _("Parse error at line %d: the shape '%s' is unknown.\n"),
+			   position, shape);
+      return FALSE;
     }
   str = getSpinResources(ele);
   str->height                = height;
@@ -1354,9 +1384,7 @@ gboolean readSpinResources(gchar **lines, int nbLines,
   str->use_element_color_hat = use_element_color_hat;
   str->shape                 = shape_number;
   DBG_fprintf(stderr, "resources set successfully for element '%s'\n", ele->name);
-  if (dataObj)
-    rspin_createShapeSpin(dataObj, ele);
-  
+
   return TRUE;
 }
 
@@ -1376,7 +1404,7 @@ gboolean exportResourcesRenderingSpin(GString *data, int *nbLinesWritten,
 				      VisuData* dataObj)
 {
   GList *pos, *eleList;
-  int i;
+  unsigned int i;
   struct spinResources_struct *str;
   VisuElement *ele;
 
@@ -1468,7 +1496,8 @@ gboolean exportResourcesRenderingSpin(GString *data, int *nbLinesWritten,
   return TRUE;
 }
 
-static void onSpinParametersChanged(VisuData *dataObj, VisuNode *node, gpointer data)
+static void onSpinParametersChanged(VisuData *dataObj, VisuNode *node,
+				    gpointer data _U_)
 {
   g_return_if_fail(dataObj && node);
 
@@ -1581,7 +1610,7 @@ static void drawEdgedTail(float tail_length, float tail_height)
 int rspin_createShapeSpin(VisuData *visuData, VisuElement* ele)
 {
   int nlatl=0, nlatul=0, nlatoh=0, iele;
-  float hatLength, hatRadius, tailLength, tailRadius, *maxModulus, ratio;
+  float hatLength, hatRadius, tailLength, tailRadius, *maxModulus, ratio, globalMax;
   struct spinResources_struct *str;
   OpenGLView *view;
 
@@ -1602,8 +1631,17 @@ int rspin_createShapeSpin(VisuData *visuData, VisuElement* ele)
 	      " nlatul = %d, nlatoh = %d\n", 
 	      ele->name, nlatl, nlatul, nlatoh);
 
-  if (spinModulusUsage)
+  globalMax = 0.f;
+  if (spinModulusUsage == policyNoneModulus)
+    maxModulus = (float*)0;
+  else if (spinModulusUsage == policyPerTypeModulus)
     maxModulus = (float*)visuDataGet_property(visuData, SPINMAXMODULUS_ID);
+  else if (spinModulusUsage == policyGlobalModulus)
+    {
+      maxModulus = (float*)visuDataGet_property(visuData, SPINMAXMODULUS_ID);
+      for (iele = 0; iele < (int)visuData->ntype; iele++)
+	globalMax = MAX(maxModulus[iele], globalMax);
+    }
   else
     maxModulus = (float*)0;
 
@@ -1614,7 +1652,10 @@ int rspin_createShapeSpin(VisuData *visuData, VisuElement* ele)
   if (maxModulus)
     {
       iele = *(int*)g_hash_table_lookup(visuData->fromVisuElementToInt, (gpointer)ele);
-      ratio = 1. / maxModulus[iele];
+      if (spinModulusUsage == policyGlobalModulus)
+	ratio = 1.f / globalMax;
+      else
+	ratio = 1.f / maxModulus[iele];
       glScalef(ratio, ratio, ratio);
     }
   switch (str->shape)
@@ -1703,10 +1744,6 @@ int rspin_createShapeSpin(VisuData *visuData, VisuElement* ele)
 /* This is a method called before the drawing of each node. It's needed to set the
    right positions of each atom but you can apply other treatments if you need.
    Here we also rotate our atom according to the spin theta and phi parameters. */
-#ifndef PI
-#define PI 3.1415926
-#endif
-
 #ifndef DEG2RAD
 #define DEG2RAD(x) (0.01745329251994329509 * x)    /* pi / 180 * x */
 #endif
@@ -1732,6 +1769,7 @@ void rspin_placeNodeSpin(VisuData *visuData, VisuNode *node, VisuElement* ele)
   float cartesian_prime[3];
   float cartesian_second[3];
   OpenGLView *view;
+  float cosCone, sinCone;
 
   struct spinResources_struct *str;
 
@@ -1739,37 +1777,45 @@ void rspin_placeNodeSpin(VisuData *visuData, VisuNode *node, VisuElement* ele)
   g_return_if_fail(view);
 
   /* Test the modulus. */
-  spinValues = (float*)visuDataGet_nodeProperty(visuData, node,
-						SPINVALUES_ID);
-  if (spinValues && (spinValues[SPIN_MODULUS] != 0. || spinPolicy == policyAlwaysSpin))
+  visuNodeGet_propertyValue(visuDataGet_nodeArray(visuData), node,
+			    SPINVALUES_ID, &spinValue);
+  spinValues = (float*)g_value_get_pointer(&spinValue);
+  if (spinValues && (spinValues[SPIN_MODULUS] != 0. ||
+		     spinPolicy == policyAlwaysSpin))
     {
       /* We draw a spin shape. */
       spherical[0] = 1;
       spherical[1] = spinValues[SPIN_THETA];
       spherical[2] = spinValues[SPIN_PHI];
 
-      matrix_rot_theta[0][0] = cos(DEG2RAD(coneOrientation[0]));
+      cosCone = cos(DEG2RAD(coneOrientation[0]));
+      sinCone = sin(DEG2RAD(coneOrientation[0]));
+      matrix_rot_theta[0][0] = cosCone;
       matrix_rot_theta[0][1] = 0;
-      matrix_rot_theta[0][2] = -sin(DEG2RAD(coneOrientation[0]));
+      matrix_rot_theta[0][2] = -sinCone;
       matrix_rot_theta[1][0] = 0;
       matrix_rot_theta[1][1] = 1;
       matrix_rot_theta[1][2] = 0;
-      matrix_rot_theta[2][0] = sin(DEG2RAD(coneOrientation[0]));
+      matrix_rot_theta[2][0] = sinCone;
       matrix_rot_theta[2][1] = 0;
-      matrix_rot_theta[2][2] = cos(DEG2RAD(coneOrientation[0]));
+      matrix_rot_theta[2][2] = cosCone;
 
-      matrix_rot_phi[0][0] = cos(DEG2RAD(-coneOrientation[1]));
-      matrix_rot_phi[0][1] = -sin(DEG2RAD(-coneOrientation[1]));
+      cosCone = cos(DEG2RAD(-coneOrientation[1]));
+      sinCone = sin(DEG2RAD(-coneOrientation[1]));
+      matrix_rot_phi[0][0] = cosCone;
+      matrix_rot_phi[0][1] = -sinCone;
       matrix_rot_phi[0][2] = 0;
-      matrix_rot_phi[1][0] = sin(DEG2RAD(-coneOrientation[1]));
-      matrix_rot_phi[1][1] = cos(DEG2RAD(-coneOrientation[1]));
+      matrix_rot_phi[1][0] = sinCone;
+      matrix_rot_phi[1][1] = cosCone;
       matrix_rot_phi[1][2] = 0;
       matrix_rot_phi[2][0] = 0;
       matrix_rot_phi[2][1] = 0;
       matrix_rot_phi[2][2] = 1; 
 
-      cartesian[0] = sin(DEG2RAD(spinValues[SPIN_THETA]))*cos(DEG2RAD(spinValues[SPIN_PHI]));
-      cartesian[1] = sin(DEG2RAD(spinValues[SPIN_THETA]))*sin(DEG2RAD(spinValues[SPIN_PHI]));
+      cartesian[0] = sin(DEG2RAD(spinValues[SPIN_THETA])) *
+	cos(DEG2RAD(spinValues[SPIN_PHI]));
+      cartesian[1] = sin(DEG2RAD(spinValues[SPIN_THETA])) *
+	sin(DEG2RAD(spinValues[SPIN_PHI]));
       cartesian[2] = cos(DEG2RAD(spinValues[SPIN_THETA]));
 
       /*   fprintf(stderr, "%f %f %f --> ", spherical[0], spherical[1], spherical[2]); */
@@ -1821,9 +1867,7 @@ void rspin_placeNodeSpin(VisuData *visuData, VisuNode *node, VisuElement* ele)
       glPushMatrix();
 
       /* Translate to the rendering position. */
-      glTranslated(xyz[0] - view->box->dxxs2,
-		   xyz[1] - view->box->dyys2,
-		   xyz[2] - view->box->dzzs2);
+      glTranslated(xyz[0], xyz[1], xyz[2]);
 
 
       /* If we need to draw also an atom shape. */
@@ -1839,7 +1883,7 @@ void rspin_placeNodeSpin(VisuData *visuData, VisuNode *node, VisuElement* ele)
       glRotated(spinValues[SPIN_THETA], 0, 1, 0); 
 
       /* We scale the shape if required. */
-      if (spinModulusUsage)
+      if (spinModulusUsage != policyNoneModulus)
 	{
 	  glScalef(spinValues[SPIN_MODULUS],
 		   spinValues[SPIN_MODULUS],
@@ -1870,9 +1914,7 @@ void rspin_placeNodeSpin(VisuData *visuData, VisuNode *node, VisuElement* ele)
 
 	  glPushMatrix();
   
-	  glTranslated(xyz[0] - view->box->dxxs2,
-		       xyz[1] - view->box->dyys2,
-		       xyz[2] - view->box->dzzs2);
+	  glTranslated(xyz[0], xyz[1], xyz[2]);
 
 	  glCallList(visuElementGet_identifierMaterial(ele));
 	  str = (struct spinResources_struct*)getSpinResources(ele);
