@@ -52,6 +52,47 @@
 #include <string.h>
 #include <unistd.h> /* For the access markers R_OK, W_OK ... */
 
+/**
+ * SECTION:visu_configFile
+ * @short_description: Defines methods to access (read/write) to
+ * config files and to create different entries.
+ *
+ * <para>V_Sim uses two different configuration files. The first
+ * stores the configuration of the program itself and is called
+ * parameters file. The second stores differents values that control
+ * the way files are rendered. It is called resources file. For
+ * example, their is an entry in the parameters file that controls
+ * your favorite rendering method ; and there is an entry in the
+ * resources file that codes that vacancy elements are rendered by
+ * cube in the atomic rendering method. Most methods of this part uses
+ * a first argument usually called 'kind', that control if the method
+ * will apply on the parameters file or on the resources
+ * file. #VISU_CONFIGFILE_PARAMETER and #VISU_CONFIGFILE_RESOURCE are
+ * the two flags that should be used for the 'kind' argument.</para>
+ *
+ * <para>There are different paths where these files can be
+ * stored. These paths are stored in V_Sim with an order : for example
+ * parameters file are first looked for in the current working
+ * directory, then in the $HOME/.v_sim directory and finally in the
+ * installation directory. This is transparent for the user and
+ * visuConfigFileGet_validPath() is the right method to access to the
+ * best readable configuration file.</para>
+ *
+ * <para>Different part of V_Sim can add entries in these files. The
+ * method visuConfigFileAdd_entry() is designed to this purpose. The
+ * entries are defined by their name and they appear in the
+ * configuration file as 'name:' followed by the data associated to
+ * this entry. In the parameters file, the data are on the same
+ * line. In the resources file, the data begin the line after and can
+ * be longer that one line. When a configuration file is read, the
+ * method associated to each entry (visuConfigFileReadFunc()) is
+ * called with a copy of their data lines. The method
+ * visuConfigFileAdd_exportFunction() should be used to add a callback
+ * when the configurations files are written, then each part of V_Sim
+ * that have entries can put some lines in the configuration
+ * files.</para>
+ */
+
 #define PARAMETER_HEADER     "#V_Sim parameters file v"
 #define RESOURCE_HEADER      "#V_Sim resources file v"
 #define VERSION_HEADER       "3.0"
@@ -61,17 +102,21 @@
 #define FLAG_RESOURCES_PATH "main_resourcesPath"
 #define DESC_RESOURCES_PATH "Favorite paths to find and save the resources file ; chain[:chain]"
 #define DEFAULT_RESOURCES_PATH ""
-gboolean readResourcesPaths(gchar **lines, int nbLines,
-			    int position, GString *errorMessage);
-gboolean exportResourcesPaths(GString *data, int *nbLinesWritten,
-			      VisuData *dataObj);
+static gboolean readResourcesPaths(gchar **lines, int nbLines, int position _U_,
+				   VisuData *dataObj _U_, GError **error _U_);
+static gboolean exportResourcesPaths(GString *data, int *nbLinesWritten,
+				     VisuData *dataObj);
 
 
-struct VisuConfigFileEntry_struct
+struct _VisuConfigFileEntry
 {
   /* Name of the key. */
   gchar *key;
   gchar *description;
+  /* Version, default is 3.0. */
+  float version;
+  /* If set, entry is obsolete and newKey should replaces it. */
+  gchar *newKey;
 
   /* A parameter or a resource */
   int kind;
@@ -99,56 +144,41 @@ struct writeFunc_struct
 /* This hashtable stores all the known entries.
    The keys are the name of the entry (its key), and
    the value is a pointer to a VisuConfigFileEntry. */
-GHashTable *visuConfigFile_entryList;
-GList *registeredResources, *registeredParameters;
-GList *visuConfigFile_exportResourcesList, *visuConfigFile_exportParametersList;
-GHashTable *knownTags;
+static GHashTable *visuConfigFile_entryList;
+static GList *registeredResources, *registeredParameters;
+static GList *exportResourcesList, *exportParametersList;
+static GHashTable *knownTags;
 
 /* Store the paths to where it is possible to store
    and/or read resource files. This list is ordered
    and first element is the most prefered path. */
-GList *resourcesPath;
+static GList *resourcesPath;
 /* Store the paths to where it is possible to store
    parameters files. This list is ordered and first element is
    the most prefered path.*/
-GList *parametersPath;
+static GList *parametersPath;
 
 
-gint compareStringsInGList(gconstpointer a, gconstpointer b)
+static gint compareStringsInGList(gconstpointer a, gconstpointer b)
 {
   return strcmp((char*)a, (char*)b);
 }
 
-VisuConfigFileEntry* visuConfigFileAdd_entry(int kind, gchar *key, gchar* description,
-					     int nbLines, visuConfigFileReadFunc readFunc)
+VisuConfigFileEntry* visuConfigFileAdd_entry(int kind, const gchar *key,
+					     const gchar* description, int nbLines,
+					     visuConfigFileReadFunc readFunc)
 {
   VisuConfigFileEntry *entry;
 
-  if (!key)
-    {
-      fprintf(stderr, "WARNING! 'visuConfigFileAdd_entry' has been called with"
-	      " a null 'key' argument, it should not.\n");
-      return (VisuConfigFileEntry*)0;
-    }
-  if (nbLines < 1 || (kind != VISU_CONFIGFILE_PARAMETER && kind != VISU_CONFIGFILE_RESOURCE))
-    {
-      fprintf(stderr, "WARNING! 'visuConfigFileAdd_entry' has been called with"
-	      " a wrong 'nbLines' or 'kind' arguments.\n");
-      return (VisuConfigFileEntry*)0;
-    }
-  if (g_hash_table_lookup(visuConfigFile_entryList, (gpointer)key))
-    {
-      fprintf(stderr, "WARNING! Can't add this new entry, it's label"
-	      " ('%s') already exists.\n", key);
-      return (VisuConfigFileEntry*)0;
-    }
+  g_return_val_if_fail(key && *key, (VisuConfigFileEntry*)0);
+  g_return_val_if_fail(description && *description, (VisuConfigFileEntry*)0);
+  g_return_val_if_fail(nbLines > 0 && (kind == VISU_CONFIGFILE_PARAMETER ||
+				       kind == VISU_CONFIGFILE_RESOURCE),
+		       (VisuConfigFileEntry*)0);
+  g_return_val_if_fail(!g_hash_table_lookup(visuConfigFile_entryList, (gpointer)key),
+		       (VisuConfigFileEntry*)0);
 
-  entry = malloc(sizeof(VisuConfigFileEntry));
-  if (!entry)
-    {
-      allocationProblems();
-      return (VisuConfigFileEntry*)0;
-    }
+  entry = g_malloc(sizeof(VisuConfigFileEntry));
   entry->key = g_strdup(key);
   entry->description = g_strdup(description);
   entry->kind = kind;
@@ -158,8 +188,11 @@ VisuConfigFileEntry* visuConfigFileAdd_entry(int kind, gchar *key, gchar* descri
     entry->nbLines = nbLines;
   entry->read = readFunc;
   entry->tag = (gchar*)0;
+  entry->newKey = (gchar*)0;
+  entry->version = 3.0f;
 
-  g_hash_table_insert(visuConfigFile_entryList, (gpointer)entry->key, (gpointer)entry);
+  g_hash_table_insert(visuConfigFile_entryList,
+		      (gpointer)entry->key, (gpointer)entry);
 
   if (kind == VISU_CONFIGFILE_RESOURCE)
     registeredResources = g_list_append(registeredResources, (gpointer)entry);
@@ -168,14 +201,21 @@ VisuConfigFileEntry* visuConfigFileAdd_entry(int kind, gchar *key, gchar* descri
   return entry;
 }
 
+GList* visuConfigFileGet_entries(int kind)
+{
+  g_return_val_if_fail(kind == VISU_CONFIGFILE_PARAMETER ||
+		       kind == VISU_CONFIGFILE_RESOURCE,
+		       (GList*)0);
+  if (kind == VISU_CONFIGFILE_PARAMETER)
+    return registeredParameters;
+  else
+    return registeredResources;
+}
+
 /* Method to add a new known tag. */
 void visuConfigFileAdd_knownTag(gchar* tag)
 {
-  if (!tag)
-    {
-      fprintf(stderr, "WARNING! 'addKnownTag' has just been called with a null string.\n");
-      return;
-    }
+  g_return_if_fail(tag && *tag);
 
   g_hash_table_insert(knownTags, (gpointer)tag, GINT_TO_POINTER(1));
 }
@@ -186,55 +226,67 @@ void visuConfigFileAdd_exportFunction(int kind, visuConfigFileExportFunc writeFu
 
   if (!writeFunc)
     return;
-  if (kind != VISU_CONFIGFILE_RESOURCE && kind != VISU_CONFIGFILE_PARAMETER)
-    {
-      fprintf(stderr, "WARNING! 'visuConfigFileAdd_exportFunction' has been called with"
-	      " a wrong 'kind' argument.\n");
-      return;
-    }
-  str = malloc(sizeof(struct writeFunc_struct));
-  if (!str)
-    {
-      allocationProblems();
-      return;
-    }
+  g_return_if_fail(kind == VISU_CONFIGFILE_PARAMETER ||
+		   kind == VISU_CONFIGFILE_RESOURCE);
+
+  str = g_malloc(sizeof(struct writeFunc_struct));
   str->writeFunc = writeFunc;
   if (kind == VISU_CONFIGFILE_RESOURCE)
-    visuConfigFile_exportResourcesList = g_list_append(visuConfigFile_exportResourcesList,
-						       (gpointer)str);
+    exportResourcesList =
+      g_list_append(exportResourcesList, (gpointer)str);
   else if (kind == VISU_CONFIGFILE_PARAMETER)
-    visuConfigFile_exportParametersList = g_list_append(visuConfigFile_exportParametersList,
-						       (gpointer)str);
+    exportParametersList =
+      g_list_append(exportParametersList, (gpointer)str);
 }
 
-void freeConfigEntry(gpointer entry)
+static void freeConfigEntry(gpointer data)
 {
-  if (!entry)
+  VisuConfigFileEntry *entry;
+
+  if (!data)
     return;
 
-  g_free(((VisuConfigFileEntry*)entry)->key);
-  if (((VisuConfigFileEntry*)entry)->tag)
-    g_free(((VisuConfigFileEntry*)entry)->tag);
-  free(entry);
+  entry = (VisuConfigFileEntry*)data;
+  g_free(entry->key);
+  g_free(entry->description);
+  if (entry->tag)
+    g_free(entry->tag);
+  if (entry->newKey)
+    g_free(entry->newKey);
+  g_free(entry);
 }
 
 void visuConfigFileSet_tag(VisuConfigFileEntry *entry, gchar *tag)
 {
-  if (!entry)
-    {
-      fprintf(stderr, "WARNING! 'visuConfigFileSet_tag' has been called with"
-	      " a null 'entry' argument, can't work.\n");
-      return;
-    }
+  g_return_if_fail(entry);
+
+  if (entry->tag)
+    g_free(entry->tag);
   entry->tag = g_strdup(tag);
 }
 
-gboolean visuConfigFileLoad(int kind, const char* fileName, GString **message)
+void visuConfigFileSet_version(VisuConfigFileEntry *entry, float version)
+{
+  g_return_if_fail(entry && version > 3.0f);
+
+  entry->version = version;
+}
+
+void visuConfigFileSet_replace(VisuConfigFileEntry *newEntry,
+			       VisuConfigFileEntry *oldEntry)
+{
+  g_return_if_fail(newEntry && oldEntry);
+
+  if (oldEntry->newKey)
+    g_free(oldEntry->newKey);
+  oldEntry->newKey = g_strdup(newEntry->key);
+}
+
+gboolean visuConfigFileLoad(int kind, const char* fileName,
+			    VisuData *dataObj, GError **error)
 {
   GIOChannel *ioFile;
-  gchar *fileUTF8;
-  GError *error;
-  GString *line, *errorMessage;
+  GString *line;
   GIOStatus status; 
   int nbLine, i;
   char *deuxPoints;
@@ -242,52 +294,26 @@ gboolean visuConfigFileLoad(int kind, const char* fileName, GString **message)
   gchar *key, *tag, *finTag, *finKey;
   VisuConfigFileEntry *entry;
   int withErrors;
+  GString *message;
 
   DBG_fprintf(stderr, "Visu ConfigFile : parsing '%s' file for"
 	      " resources/parameters...\n", fileName);
   
-  if (kind != VISU_CONFIGFILE_RESOURCE && kind != VISU_CONFIGFILE_PARAMETER)
-    {
-      fprintf(stderr, "WARNING! 'visuConfigFileLoad' has been called"
-	      " with a wrong 'kind' argument.\n");
-      *message = (GString*)0;
-      return FALSE;
-    }
+  g_return_val_if_fail(kind == VISU_CONFIGFILE_RESOURCE ||
+		       kind == VISU_CONFIGFILE_PARAMETER, FALSE);
 
-  if (message)
-    {
-      *message = g_string_new("");
-      fileUTF8 = getStringInUTF8(fileName);
-      if (kind == VISU_CONFIGFILE_RESOURCE)
-	g_string_append_printf(*message, _("Parsing '%s' file for resources...\n\n"), fileUTF8);
-      else if (kind == VISU_CONFIGFILE_PARAMETER)
-	g_string_append_printf(*message, _("Parsing '%s' file for parameters...\n\n"), fileUTF8);
-      g_free(fileUTF8);
-    }
-
-  error = (GError*)0;
-  ioFile = g_io_channel_new_file(fileName, "r", &error);
-  if (error)
-    {
-      if (*message)
-	g_string_append_printf(*message, _("Error : %s"), error->message);
-      g_error_free(error);
-      return FALSE;
-    }
+  ioFile = g_io_channel_new_file(fileName, "r", error);
+  if (*error)
+    return FALSE;
 
   line = g_string_new("");
   nbLine = 0;
-  error = (GError*)0;
-  status = g_io_channel_read_line_string(ioFile, line, NULL, &error);
-  if (error)
-    {
-      if (*message)
-	g_string_append_printf(*message, _("Error : %s"), error->message);
-      g_error_free(error);
-      return FALSE;
-    }
+  status = g_io_channel_read_line_string(ioFile, line, NULL, error);
+  if (*error)
+    return FALSE;
   nbLine += 1;
   withErrors = 0;
+  message = (GString*)0;
   while (status == G_IO_STATUS_NORMAL)
     {
       if (line->str[0] != '#' && line->str[0] != '\n')
@@ -311,51 +337,45 @@ gboolean visuConfigFileLoad(int kind, const char* fileName, GString **message)
 		    *finTag = '\0';
 		  else
 		    {
-		      if (*message)
-			g_string_append_printf(*message, _("WARNING! Parse error at line %d,"
-							   " the tag '%s' is not closed.\n"),
-					       nbLine, tag);
+		      g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_TAG,
+				  _("Parse error at line %d,"
+				    " the tag '%s' is not closed.\n"),
+				  nbLine, tag);
 		      withErrors = 1;
 		    }
-		  DBG_fprintf(stderr,"Visu ConfigFile : read a flag (tag): '%s' (%s).\n", key, tag);
+		  DBG_fprintf(stderr,"Visu ConfigFile: read a flag (tag): '%s' (%s).\n", key, tag);
 		}
 	      else
-		  DBG_fprintf(stderr,"Visu ConfigFile : read a flag (tag): '%s' (none).\n", key);
+		  DBG_fprintf(stderr,"Visu ConfigFile: read a flag (tag): '%s' (none).\n", key);
 
 	      if (tag && !g_hash_table_lookup(knownTags, (gpointer)tag))
 		{
-		  DBG_fprintf(stderr, "Visu ConfigFile : the entry '%s' has an unknown tag (%s),"
+		  DBG_fprintf(stderr, "Visu ConfigFile: the entry '%s' has an unknown tag (%s),"
 			      " it will be dismissed.\n", key, tag);
 		}
 	      else
 		{
-		  entry = (VisuConfigFileEntry*)g_hash_table_lookup(visuConfigFile_entryList,
-								    (gpointer)key);
+		  entry = (VisuConfigFileEntry*)
+		    g_hash_table_lookup(visuConfigFile_entryList, (gpointer)key);
 		  if (entry)
 		    {
-		      tokens = malloc(sizeof(gchar*) * (entry->nbLines + 1));
-		      if (!tokens)
-			{
-			  allocationProblems();
-			  exit(1);
-			}
+		      tokens = g_malloc(sizeof(gchar*) * (entry->nbLines + 1));
 		      if (kind == VISU_CONFIGFILE_RESOURCE)
 			for (i = 0; i < entry->nbLines; i++)
 			  {
-			    error= (GError*)0;
-			    status = g_io_channel_read_line_string(ioFile, line, NULL, &error);
+			    status = g_io_channel_read_line_string(ioFile, line,
+								   NULL, error);
 			    nbLine += 1;
 			    if (status != G_IO_STATUS_NORMAL)
 			      {
-				if (*message)
-				  {
-				    g_string_append_printf(*message, _("Error at line %d :\n"), nbLine);
-				    g_string_append_printf(*message, "%s", error->message);
-				  }
-				g_error_free(error);
 				tokens[i] = (gchar*)0;
 				g_strfreev(tokens);
 				tokens = (gchar**)0;
+				*error = g_error_new(CONFIG_FILE_ERROR,
+						     CONFIG_FILE_ERROR_MISSING,
+						     _("Parse error at line %d,"
+						       " '%s' needs %d lines but only %d were read.\n"),
+						     nbLine, key, entry->nbLines, nbLine);
 				withErrors = 1;
 			      }
 			    tokens[i] = g_strdup(line->str);
@@ -365,114 +385,99 @@ gboolean visuConfigFileLoad(int kind, const char* fileName, GString **message)
 		      if (tokens)
 			{
 			  tokens[entry->nbLines] = (gchar*)0;
-			  if (*message)
-			    errorMessage = g_string_new("");
-			  else
-			    errorMessage = (GString*)0;
-			  if (entry->read && !entry->read(tokens,
-							  entry->nbLines,
-							  nbLine,
-							  errorMessage))
+			  if (entry->read && !entry->read(tokens, entry->nbLines,
+							  nbLine, dataObj, error))
 			    {
-			      if (*message)
-				g_string_append_printf(*message, "%s", errorMessage->str);
+			      g_return_val_if_fail(*error, FALSE);
 			      withErrors = 1;
 			    }
-			  if (*message)
-			    g_string_free(errorMessage, TRUE);
 			  g_strfreev(tokens);
 			}
+		      if (entry->newKey)
+			g_warning(_("Parsing resource file, markup '%s' is"
+				    " obsolete, replaced by '%s'."),
+				  key, entry->newKey);
 		    }
 		  else
 		    {
-		      if (*message)
-			g_string_append_printf(*message, _("WARNING! Parse error at line %d,"
-							   " '%s' is an unknown markup.\n"),
-					       nbLine, key);
+		      *error = g_error_new(CONFIG_FILE_ERROR,
+					   CONFIG_FILE_ERROR_MARKUP,
+					   _("Parse error at line %d,"
+					     " '%s' is an unknown markup.\n"),
+					   nbLine, key);
 		      withErrors = 1;
 		    }
 		}
+	      if (withErrors && *error)
+		{
+		  if (!message)
+		    message = g_string_new("");
+		  g_string_append_printf(message, "[%s]: %s\n",
+					 key, (*error)->message);
+		  g_error_free(*error);
+		  *error = (GError*)0;
+		}
 	      g_free(key);
 	    }
 	}
 
-      error = (GError*)0;
-      status = g_io_channel_read_line_string(ioFile, line, NULL, &error);
-      if (error)
-	{
-	  if (*message)
-	    g_string_append_printf(*message, _("Error : %s"), error->message);
-	  g_error_free(error);
-	  return FALSE;
-	}
+      status = g_io_channel_read_line_string(ioFile, line, NULL, error);
       nbLine += 1;
     }
+  g_string_free(line, TRUE);
   if (status == G_IO_STATUS_ERROR)
     {
-      if (*message)
-	g_string_append_printf(*message, _("Error : %s"), error->message);
-      g_error_free(error);
-      withErrors = 1;
-    }
-  g_string_free(line, TRUE);
-
-  error = (GError*)0;
-  g_io_channel_shutdown(ioFile, FALSE, &error);
-  if (error)
-    {
-      if (*message)
-	g_string_append_printf(*message, _("Error : %s"), error->message);
-      g_error_free(error);
+      g_io_channel_shutdown(ioFile, FALSE, NULL);
       return FALSE;
     }
 
-  if (kind == VISU_CONFIGFILE_RESOURCE)
-    {
-      DBG_fprintf(stderr, "Visu ConfigFile : emitting 'resourcesLoaded' signal.\n");
-      g_signal_emit(visu, VISU_GET_CLASS(visu)->resourcesLoaded_signal_id,
-		    0, NULL);
-    }
+  status = g_io_channel_shutdown(ioFile, FALSE, error);
+  if (status != G_IO_STATUS_NORMAL)
+    return FALSE;
+  DBG_fprintf(stderr, "Visu ConfigFile: read OK.\n");
 
   if (withErrors)
-    return FALSE;
+    {
+      g_return_val_if_fail(message, FALSE);
+      *error = g_error_new(CONFIG_FILE_ERROR, CONFIG_FILE_ERROR_READ, message->str);
+      g_string_free(message, TRUE);
+      return FALSE;
+    }
   else
     return TRUE;
 }
 
 /* Routine to export the resources (color and
    material of elements ...) to file. */
-gboolean visuConfigFileSave(int kind, const char* fileName, GString **message,
-			    int *lines, VisuData *dataObj)
+gboolean visuConfigFileSave(int kind, const char* fileName, int *lines,
+			    VisuData *dataObj, GError **error)
 {
   gchar *fileUTF8;
-  GString *exportString, *data;
+  GString *exportString, *data, *message;
   int nbLine, nb;
   GList *pos;
   gboolean ok;
   int withErrors;
-  FILE *file;
+
+  g_return_val_if_fail(error && !*error, FALSE);
 
   DBG_fprintf(stderr, "Visu ConfigFile : exporting '%s' file for"
 	      " resources/parameters...\n", fileName);
   
-  if (kind != VISU_CONFIGFILE_RESOURCE && kind != VISU_CONFIGFILE_PARAMETER)
-    {
-      fprintf(stderr, "WARNING! 'visuConfigFileSave' has been called"
-	      " with a wrong 'kind' argument.\n");
-      *message = (GString*)0;
-      return FALSE;
-    }
+  g_return_val_if_fail(kind == VISU_CONFIGFILE_RESOURCE ||
+		       kind == VISU_CONFIGFILE_PARAMETER, FALSE);
 
-  if (message)
-    {
-      *message = g_string_new("");
-      fileUTF8 = getStringInUTF8(fileName);
-      if (kind == VISU_CONFIGFILE_RESOURCE)
-	g_string_append_printf(*message, _("Writing '%s' file for resources...\n\n"), fileUTF8);
-      else if (kind == VISU_CONFIGFILE_PARAMETER)
-	g_string_append_printf(*message, _("Writing '%s' file for parameters...\n\n"), fileUTF8);
-      g_free(fileUTF8);
-    }
+  fileUTF8 = g_filename_from_utf8(fileName, -1, NULL, NULL, NULL);
+  g_return_val_if_fail(fileUTF8, FALSE);
+
+  message = g_string_new("");
+  if (kind == VISU_CONFIGFILE_RESOURCE)
+    g_string_append_printf(message, _("Writing '%s' file for"
+				      " resources...\n\n"), fileUTF8);
+  else if (kind == VISU_CONFIGFILE_PARAMETER)
+    g_string_append_printf(message, _("Writing '%s' file for"
+				      " parameters...\n\n"), fileUTF8);
+  g_free(fileUTF8);
 
   withErrors = 0;
   nbLine = 0;
@@ -534,70 +539,49 @@ gboolean visuConfigFileSave(int kind, const char* fileName, GString **message,
   nbLine += 1;
   pos = (GList*)0;
   if (kind == VISU_CONFIGFILE_RESOURCE)
-    pos = visuConfigFile_exportResourcesList;
+    pos = exportResourcesList;
   else if (kind == VISU_CONFIGFILE_PARAMETER)
-    pos = visuConfigFile_exportParametersList;
+    pos = exportParametersList;
   while(pos)
     {
       data = g_string_new("");
       ok = ((struct writeFunc_struct*)(pos->data))->writeFunc(data, &nb, dataObj);
-      if (!ok && message)
-	g_string_append_printf(*message, _("Error on line %d.\n"), nbLine);
       if (ok)
 	{
 	  g_string_append_printf(exportString, "%s", data->str);
 	  nbLine += nb;
 	}
       else
-	withErrors = 1;
+	{
+	  g_string_append_printf(message, _("Error on line %d.\n"), nbLine);
+	  withErrors = 1;
+	}
       g_string_free(data, TRUE);
       pos = g_list_next(pos);
     }
 
-  file = fopen(fileName, "w");
-  if (!file)
+  if (!g_file_set_contents(fileName, exportString->str, -1, error))
     {
-      if (message)
-	g_string_append_printf(*message, _("WARNING! Unable to create the requested"
-					   " file. Check that you have enough"
-					   " permissions to write it.\n"));
       g_string_free(exportString, TRUE);
+      g_string_free(message, TRUE);
       return FALSE;
     }
-  fprintf(file, "%s", exportString->str);
-  fclose(file);
   g_string_free(exportString, TRUE);
 
-  if (message)
-    g_string_append_printf(*message, _(" %d line(s) written.\n\n"), nbLine);
   if (lines)
     *lines = nbLine;
   
   if (withErrors)
-    return FALSE;
+    {
+      g_error_new(G_FILE_ERROR, 0, message->str);
+      g_string_free(message, TRUE);
+      return FALSE;
+    }
   else
-    return TRUE;
-}
-
-int visuConfigFileSet_floatValue(float *variable, float value, float min, float max)
-{
-  int outOfBounds;
-
-  outOfBounds = 0;
-  if (max > min && value > max)
     {
-      *variable = max;
-      outOfBounds = 1;
+      g_string_free(message, TRUE);
+      return TRUE;
     }
-  if (min < max && value < min)
-    {
-      *variable = min;
-      outOfBounds = 1;
-    }
-  if (!outOfBounds)
-    *variable = value;
-
-  return outOfBounds;
 }
 
 gchar* getValidFileWithHeader(int mode, char* filename, char* header, GList **list)
@@ -628,9 +612,9 @@ gchar* getValidFileWithHeader(int mode, char* filename, char* header, GList **li
 	  file = fopen(res, "r");
 	  if (!file)
 	    {
-	      fprintf(stderr, _("INTERNAL ERROR! The file '%s' should be readable"
-				" but something goes nasty when one wants to open it.\n"), res);
-	      free(res);
+	      g_warning("The file '%s' should be readable but something goes"
+			" nasty when one wants to open it.\n", res);
+	      g_free(res);
 	      return (gchar*)0;
 	    }
 	  version = 0.;
@@ -660,12 +644,8 @@ gchar* visuConfigFileGet_validPath(int kind, int mode, int utf8)
   gchar* file;
   gchar* fileUTF8;
 
-  if (kind != VISU_CONFIGFILE_RESOURCE && kind != VISU_CONFIGFILE_PARAMETER)
-    {
-      fprintf(stderr, "WARNING! 'visuConfigFileGet_validPath' has been called"
-	      " with a wrong 'kind' argument.\n");
-      return (gchar*)0;
-    }
+  g_return_val_if_fail(kind == VISU_CONFIGFILE_RESOURCE ||
+		       kind == VISU_CONFIGFILE_PARAMETER, (gchar*)0);
 
   if (kind == VISU_CONFIGFILE_RESOURCE)
     {
@@ -685,7 +665,7 @@ gchar* visuConfigFileGet_validPath(int kind, int mode, int utf8)
 
   if (utf8)
     {
-      fileUTF8 = getStringInUTF8(file);
+      fileUTF8 = g_filename_from_utf8(file, -1, NULL, NULL, NULL);
       g_free(file);
       return fileUTF8;
     }
@@ -697,19 +677,10 @@ gchar* visuConfigFileGet_nextValidPath(int kind, int accessMode, GList **list, i
   gchar* file;
   gchar* fileUTF8;
 
-  if (kind != VISU_CONFIGFILE_RESOURCE && kind != VISU_CONFIGFILE_PARAMETER)
-    {
-      fprintf(stderr, "WARNING! 'visuConfigFileGet_nextValidPath' has been called"
-	      " with a wrong 'kind' argument.\n");
-      return (gchar*)0;
-    }
+  g_return_val_if_fail(kind == VISU_CONFIGFILE_RESOURCE ||
+		       kind == VISU_CONFIGFILE_PARAMETER, (gchar*)0);
+  g_return_val_if_fail(list, (gchar*)0);
 
-  if (!list)
-    {
-      fprintf(stderr, "WARNING! 'visuConfigFileGet_nextValidPath' has been called"
-	      " with a wrong 'list' argument.\n");
-      return (gchar*)0;
-    }
   if (!*list)
     return (gchar*)0;
 
@@ -728,7 +699,7 @@ gchar* visuConfigFileGet_nextValidPath(int kind, int accessMode, GList **list, i
 
   if (utf8)
     {
-      fileUTF8 = getStringInUTF8(file);
+      fileUTF8 = g_filename_from_utf8(file, -1, NULL, NULL, NULL);
       g_free(file);
       return fileUTF8;
     }
@@ -737,12 +708,8 @@ gchar* visuConfigFileGet_nextValidPath(int kind, int accessMode, GList **list, i
 }
 gchar* visuConfigFileGet_defaultFileName(int kind)
 {
-  if (kind != VISU_CONFIGFILE_RESOURCE && kind != VISU_CONFIGFILE_PARAMETER)
-    {
-      fprintf(stderr, "WARNING! 'visuConfigFileGet_defaultFileName' has been called"
-	      " with a wrong 'kind' argument.\n");
-      return (gchar*)0;
-    }
+  g_return_val_if_fail(kind == VISU_CONFIGFILE_RESOURCE ||
+		       kind == VISU_CONFIGFILE_PARAMETER, (gchar*)0);
 
   if (kind == VISU_CONFIGFILE_RESOURCE)
     return (gchar*)RESOURCES_FILE_NAME;
@@ -752,12 +719,8 @@ gchar* visuConfigFileGet_defaultFileName(int kind)
 
 GList* visuConfigFileGet_pathList(int kind)
 {
-  if (kind != VISU_CONFIGFILE_RESOURCE && kind != VISU_CONFIGFILE_PARAMETER)
-    {
-      fprintf(stderr, "WARNING! 'visuConfigFileGet_nextValidPath' has been called"
-	      " with a wrong 'kind' argument.\n");
-      return (GList*)0;
-    }
+  g_return_val_if_fail(kind == VISU_CONFIGFILE_RESOURCE ||
+		       kind == VISU_CONFIGFILE_PARAMETER, (GList*)0);
 
   if (kind == VISU_CONFIGFILE_RESOURCE)
     return resourcesPath;
@@ -784,35 +747,82 @@ GList* visuConfigFileAdd_resourcesPath(char* dir)
     return (GList*)element;
 }
 
+gboolean visuConfigFileExport_toXML(const gchar *filename, int kind, GError **error)
+{
+  GString *str;
+  GList *tmpLst;
+  VisuConfigFileEntry *entry;
+  gboolean status;
+  gchar *desc;
+
+  g_return_val_if_fail(filename && *filename, FALSE);
+  g_return_val_if_fail(kind == VISU_CONFIGFILE_RESOURCE ||
+		       kind == VISU_CONFIGFILE_PARAMETER, FALSE);
+
+  str = g_string_new("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
+  if (kind == VISU_CONFIGFILE_PARAMETER)
+    {
+      tmpLst = registeredParameters;
+      g_string_append_printf(str, "<configFile kind=\"parameters\">\n");
+    }
+  else
+    {
+      tmpLst = registeredResources;
+      g_string_append_printf(str, "<configFile kind=\"resources\">\n");
+    }
+  while (tmpLst)
+    {
+      entry = (VisuConfigFileEntry*)tmpLst->data;
+      if (entry->tag)
+	g_string_append_printf(str, "  <entry name=\"%s\" tag=\"%s\" "
+			       "version=\"%f3.1\">\n",
+			       entry->key, entry->tag, entry->version);
+      else
+	g_string_append_printf(str, "  <entry name=\"%s\" version=\"%3.1f\">\n",
+			       entry->key, entry->version);
+      desc = g_markup_escape_text(entry->description, -1);
+      g_string_append_printf(str, "    <description>%s</description>\n", desc);
+      g_free(desc);
+      if (entry->newKey)
+	g_string_append_printf(str, "    <obsolete replacedBy=\"%s\" />\n",
+			       entry->newKey);
+      g_string_append_printf(str, "  </entry>\n");
+      tmpLst = g_list_next(tmpLst);
+    }
+  g_string_append_printf(str, "</configFile>\n");
+
+  status = g_file_set_contents(filename, str->str, -1, error);
+  g_string_free(str, TRUE);
+  return status;
+}
+
 
 int visuConfigFile_init()
 {
-  gchar *homeCfgDir, *currentDir;
+  gchar *currentDir;
   VisuConfigFileEntry *resourceEntry;
 
   DBG_fprintf(stderr, "Visu ConfigFile : initialization process ...");
   CONFIG_FILE_ERROR = g_quark_from_string("visu_configFile");
 
-  visuConfigFile_entryList = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, freeConfigEntry);
-  if (!visuConfigFile_entryList)
-    {
-      allocationProblems();
-      return 0;
-    }
+  visuConfigFile_entryList = g_hash_table_new_full(g_str_hash, g_str_equal,
+						   NULL, freeConfigEntry);
+  g_return_val_if_fail(visuConfigFile_entryList, 0);
+
   registeredResources = (GList*)0;
   registeredParameters = (GList*)0;
-  visuConfigFile_exportResourcesList = (GList*)0;
-  visuConfigFile_exportParametersList = (GList*)0;
+  exportResourcesList = (GList*)0;
+  exportParametersList = (GList*)0;
   knownTags = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, NULL);
-  if (!knownTags)
-    {
-      allocationProblems();
-      return 0;
-    }
+  g_return_val_if_fail(knownTags, 0);
+
   resourcesPath = (GList*)0;
   parametersPath = (GList*)0;
   resourcesPath = g_list_prepend(resourcesPath, (gpointer)v_sim_data_dir);
-  parametersPath = g_list_prepend(parametersPath, (gpointer)v_sim_conf_dir);
+  parametersPath = g_list_prepend(parametersPath, (gpointer)v_sim_data_dir);
+
+  resourcesPath = g_list_prepend(resourcesPath, (gpointer)v_sim_old_local_conf_dir);
+  parametersPath = g_list_prepend(parametersPath, (gpointer)v_sim_old_local_conf_dir);
 
   resourcesPath = g_list_prepend(resourcesPath, (gpointer)v_sim_local_conf_dir);
   parametersPath = g_list_prepend(parametersPath, (gpointer)v_sim_local_conf_dir);
@@ -833,13 +843,15 @@ int visuConfigFile_init()
   return 1;
 }
 
-gboolean readResourcesPaths(gchar **lines, int nbLines,
-			    int position, GString *errorMessage)
+static gboolean readResourcesPaths(gchar **lines, int nbLines, int position _U_,
+				   VisuData *dataObj _U_, GError **error _U_)
 {
   int i;
   gchar **tokens;
   gchar *key;
 
+  g_return_val_if_fail(nbLines == 1, FALSE);
+
   tokens = g_strsplit_set(lines[0], ":", -1);
   for (i = 0; tokens[i]; i++)
     {
@@ -850,8 +862,8 @@ gboolean readResourcesPaths(gchar **lines, int nbLines,
   g_strfreev(tokens);
   return TRUE;
 }
-gboolean exportResourcesPaths(GString *data, int *nbLinesWritten,
-			      VisuData *dataObj)
+static gboolean exportResourcesPaths(GString *data, int *nbLinesWritten,
+				     VisuData *dataObj _U_)
 {
   GList *pnt;
   
