@@ -137,14 +137,6 @@ typedef void (*setColorFunc)(VisuData *visuData, float rgba[4], VisuElement *ele
  *                        its pointer is known.
  * @fromIntToVisuElement: give the pointer of a #ViosuElement when
  *                        its number is known.
- * @fromNumberToVisuNode: give the #VisuNode knowing its element number and
- *                        its own position.
- * @nbOfAllStoredNodes: number of registered #VisuNode in this object.
- * @numberOfNodes: give the number of allocated #VisuNode for each
- *                 #VisuElement.
- * @numberOfStoredNodes: give the number of stored #VisuNode for
- *                       each #VisuElement.
- * @nodes: array that stores all the nodes.
  * @setColor: method called before drawing each node (can be NULL).
  *
  * This structure describes a #VisuData object.
@@ -153,11 +145,11 @@ struct _VisuData
 {
   GObject parent;
 
-  /********************/
-  /* Nodes attributes */
-  /********************/
+  /***********************/
+  /* Elements attributes */
+  /***********************/
   /* Number of VisuElements used by these data. */
-  int ntype;
+  unsigned int ntype;
   /* This hashtable gives access to the good array  of VisuNodes
      when one has the VisuElement. */
   GHashTable *fromVisuElementToInt;
@@ -165,19 +157,6 @@ struct _VisuData
      to the associated VisuElement. */
   VisuElement **fromIntToVisuElement;
 
-  /* This array gives access to the good VisuNode
-     when one has its number. */
-  VisuNode **fromNumberToVisuNode;
-  /* The total of stored VisuNodes. */
-  int nbOfAllStoredNodes;
-
-  /* Number of nodes allocated (size of the nodes array) per VisuElement. */
-  int* numberOfNodes;
-  /* Number of nodes physically present in the array per VisuElement. */
-  int* numberOfStoredNodes;
-  /* Coordinates of all these nodes. */
-  VisuNode **nodes;
-
   /* Define a method to set the color of each node.
      If this method is NULL, the color of the element is used. */
   setColorFunc setColor;
@@ -190,17 +169,10 @@ struct _VisuData
  *
  * This method returns the type of #VisuData, use VISU_DATA_TYPE instead.
  *
- * Returns : the type of #VisuData.
+ * Returns: the type of #VisuData.
  */
 GType visu_data_get_type(void);
 
-/**
- * visuDataNew:
- *
- * This creates an empty #VisuData object.
- *
- * Returns: a newly created #VisuData object (its ref count is set to 1).
- */
 VisuData* visuDataNew();
 /**
  * visuDataNew_withOpenGLView:
@@ -229,6 +201,13 @@ VisuData* visuDataNew_withOpenGLView(OpenGLView *view);
  */
 int visuDataGet_objectList(VisuData *data);
 
+/**
+ * EXT_VISU_DATA_ID:
+ *
+ * The id used to identify this extension, see
+ * OpenGLExtensionRebuild_list() for instance.
+ */
+#define EXT_VISU_DATA_ID "AllElements"
 
 
 
@@ -256,7 +235,8 @@ enum visuData_boxVector
  *
  * Returns: 1 if everything goes right.
  */
-int visuDataSet_population(VisuData *data, int nbOfTypes, int* nbOfNodesPerVisuElement,
+int visuDataSet_population(VisuData *data, unsigned int nbOfTypes,
+			   unsigned int* nbOfNodesPerVisuElement,
 			   VisuElement **visuElementUsed);
 /**
  * visuDataFree_population:
@@ -302,22 +282,66 @@ gchar* visuDataGet_file(VisuData *data, int kind, FileFormat **format);
 /**
  * visuDataSet_fileCommentary:
  * @data: a #VisuData object ;
- * @commentary: the message to be stored (null terminated).
+ * @commentary: the message to be stored (null terminated) ;
+ * @iSet: an integer.
  *
  * This method is used to store a description of the given @data. This
- * string is copied and @commentary can be freed.
+ * string is copied and @commentary can be freed. Before using this
+ * method, the number of possible node sets must have been defined
+ * using visuDataSet_nSet(), if not, only iSet == 0 is allowed.
  */
-void visuDataSet_fileCommentary(VisuData *data, gchar* commentary);
+void visuDataSet_fileCommentary(VisuData *data, gchar* commentary, gint iSet);
 /**
  * visuDataGet_fileCommentary:
- * @data: a #VisuData object.
+ * @data: a #VisuData object ;
+ * @iSet: an integer (>= 0).
  *
- * Get the commentary associated to the given @data.
+ * Get the commentary associated to the given @data, for the given
+ * node set.
  *
- * Returns: a string description or NULL. This string is own by V_Sim
+ * Returns: a string description (possibly empty). This string is own by V_Sim
  *          and should not be freed.
  */
-gchar* visuDataGet_fileCommentary(VisuData *data);
+gchar* visuDataGet_fileCommentary(VisuData *data, gint iSet);
+/**
+ * visuDataSet_nSet:
+ * @data: a #VisuData object ;
+ * @nSet: an integer.
+ *
+ * Change the number of available sets of nodes for this
+ * #VisuData. This has a side effect to delete all previously saved
+ * file commentaries (see visuDataSet_fileCommentary()).
+ */
+void visuDataSet_nSet(VisuData *data, int nSet);
+/**
+ * visuDataGet_nSet:
+ * @data: a #VisuData object.
+ *
+ * Retrieve the number of available sets of nodes for this #VisuData,
+ * see visuDataSet_nSet().
+ *
+ * Returns: the number of set of nodes (1 is default).
+ */
+int visuDataGet_nSet(VisuData *data);
+/**
+ * visuDataSet_setId:
+ * @data: a #VisuData object ;
+ * @iSet: an integer.
+ *
+ * Change the current id of the set of data (ordered as in C,
+ * beginning at 0).
+ */
+void visuDataSet_setId(VisuData *data, int iSet);
+/**
+ * visuDataGet_setId:
+ * @data: a #VisuData object.
+ *
+ * Retrieve the id of the current set of data (ordered as in C,
+ * beginning at 0).
+ *
+ * Returns: the id of the set of nodes currently loaded, -1 if none.
+ */
+int visuDataGet_setId(VisuData *data);
 
 /**
  * visuData_compareElements:
@@ -353,40 +377,26 @@ void visuDataSet_changeElementFlag(VisuData *data, gboolean changeElement);
 gboolean visuDataGet_changeElementFlag(VisuData *data);
 
 /**
- * visuDataAdd_VisuElement:
+ * visuDataAdd_nodeFromElement:
  * @data: the #VisuData where to add the new #VisuNode ;
  * @ele: the #VisuElement kind of the new #VisuNode ;
- * @x: its x coordinate ;
- * @y: its y coordinate ;
- * @z: its z coordinate ;
- * @number: its position in the input file (or anything else that can uniqualy
- *          identify this new #VisuNode).
+ * @xyz: its coordinates.
  *
  * This method adds a new #VisuNode to the specified #VisuData.
- *
- * Returns: 0 if everything is allright.
  */
-int visuDataAdd_VisuElement(VisuData *data, VisuElement *ele,
-			    double x, double y, double z, int number);
+void visuDataAdd_nodeFromElement(VisuData *data, VisuElement *ele, float xyz[3]);
 /**
- * visuDataAdd_int:
+ * visuDataAdd_nodeFromIndex:
  * @data: the #VisuData where to add the new #VisuNode ;
  * @position: a integer corresponding to the position of
  *            a #VisuElement in the array **nodes in the structure;
- * @x: its x coordinate ;
- * @y: its y coordinate ;
- * @z: its z coordinate ;
- * @number: its position in the input file (or anything else that can uniqualy
- *          identify this new #VisuNode).
+ * @xyz: its coordinates.
  *
  * This method adds a new #VisuNode to the specified #VisuData. Position must be
  * chosen between 0 and (ntype - 1) and corresponds to the position of the array
  * of #VisuNodes of a #VisuElement.
- *
- * Returns: 0 if everything is alright.
  */
-int visuDataAdd_int(VisuData *data, int position,
-		    double x, double y, double z, int number);
+void visuDataAdd_nodeFromIndex(VisuData *data, unsigned int position, float xyz[3]);
 /**
  * visuDataGet_nodeFromNumber:
  * @data: a %VisuData structure which stores the nodes.
@@ -398,120 +408,18 @@ int visuDataAdd_int(VisuData *data, int position,
  *
  * Returns: the found #VisuNode or NULL if none corresponds to number.
  */
-VisuNode* visuDataGet_nodeFromNumber(VisuData *data, int number);
-/**
- * visuDataMatch_nodeFromId:
- * @data: a #VisuData object ;
- * @eleId: a positive integer ;
- * @posId: a positive integer.
- *
- * This method try to match a #VisuNode of the given @data that has the given
- * position, where @eleId describes the number of the element and @posId the position
- * of the node in the node array of this element. If no node matchs, no error is raised,
- * but the return value is NULL.
- *
- * Returns: the #VisuNode corresponding to the given id, if exists, NULL otherwise.
- */
-VisuNode* visuDataGet_nodeFromId(VisuData *data, int eleId, int posId);
+VisuNode* visuDataGet_nodeFromNumber(VisuData *data, unsigned int number);
 
 /**
- * visuDataSet_nodeProperty:
- * @data: a #VisuData object ;
- * @node: a #VisuNode object ;
- * @key: a string ;
- * @value: an allocated area to store data.
+ * visuDataGet_nodeArray:
+ * @data: a #VisuData object.
  *
- * This method is used to store some values associated with
- * the given @node of the given @data. These values are anything allocated and will be
- * freed (with free()) when the node is deleted. These values
- * are described by the @key, and can be retrieved with the
- * #visuDataGet_nodeProperty method. The @key is used by the #VisuData
- * and should not be freed. Moreover it is not freed when the node
- * is destroyed.
- */
-void visuDataSet_nodeProperty(VisuData* data, VisuNode* node, char* key, gpointer value);
-/**
- * visuDataGet_nodeProperty:
- * @data: a #VisuData object ;
- * @node: a #VisuNode object ;
- * @key: a string.
+ * This method retrieve the #VisuNodeArray associated to the given @data.
  *
- * This method is used to retrieve some data associated to
- * the specified @node, stored in the given @data. These return data
- * should not be freed after used.
- *
- * Returns: some data associated to the key.
+ * Returns: the associated #VisuNodeArray.
  */
-gpointer visuDataGet_nodeProperty(VisuData* data, VisuNode* node, char* key);
-/**
- * visuDataAdd_nodeProperty:
- * @data: a #VisuData object ;
- * @key: a string ;
- * @freeFunc: a method to free each token.
- * 
- * This method adds and allocates a new area to store nodes associated data that
- * could be retrieve with the @key. When the property is removed with the
- * #visuDataRemove_nodePropertry the area is freed and @freeFunc is called for
- * each token.
- */
-void visuDataAdd_nodeProperty(VisuData* data, char* key, GDestroyNotify freeFunc);
-/**
- * visuDataRemove_nodeProperty:
- * @data: a #VisuData object ;
- * @key: a string.
- * 
- * This method remove all the data associated with the string @key, of all nodes.
- * They are freed.
- */
-void visuDataRemove_nodeProperty(VisuData* data, char* key);
+VisuNodeArray* visuDataGet_nodeArray(VisuData *data);
 
-/**
- * visuDataSet_property:
- * @data: a #VisuData object ;
- * @key: a string ;
- * @value: an allocated area to store data.
- *
- * This method is used to store some values associated with
- * the given @data. These values are anything allocated and will be
- * freed (with free()) when the @data is deleted. These values
- * are described by the @key, and can be retrieved with the
- * #visuDataGet_property() method. The @key is used by the #VisuData
- * and should not be freed. Moreover it is not freed when the node
- * is destroyed.
- */
-void visuDataSet_property(VisuData* data, char* key, gpointer value);
-/**
- * visuDataSet_propertyWithDestroyFunc:
- * @data: a #VisuData object ;
- * @key: a string ;
- * @value: an allocated area to store data ;
- * @freeFunc: a method to free each token.
- *
- * Same method as visuDataSet_property() but giving a free method when
- * the data is removed.
- */
-void visuDataSet_propertyWithDestroyFunc(VisuData* data, char* key, gpointer value,
-					 GDestroyNotify freeFunc);
-/**
- * visuDataGet_property:
- * @data: a #VisuData object ;
- * @key: a string.
- *
- * This method is used to retrieve some data associated to
- * the specified @data. These return data should not be freed after used.
- *
- * Returns: some data associated to the key.
- */
-gpointer visuDataGet_property(VisuData* data, char* key);
-/**
- * visuDataRemove_property:
- * @data: a #VisuData object ;
- * @key: a string.
- * 
- * This method remove the data associated with the string @key.
- * They are freed.
- */
-void visuDataRemove_property(VisuData* data, char* key);
 
 /**
  * visuDataSet_ColorFunc:
@@ -618,10 +526,10 @@ gboolean visuDataGet_periodic(VisuData *data);
  * of the bounding box. The position of each node is the result of the
  * sum of their own position and of the box translation.
  *
- * Returns: 1 if visuData_createAllNodes() should be called and
+ * Returns: TRUE if visuData_createAllNodes() should be called and
  *          the 'NodePositionChanged' should be emitted.
  */
-int visuData_constrainedElementInTheBox(VisuData *data, VisuElement *element);
+gboolean visuData_constrainedElementInTheBox(VisuData *data, VisuElement *element);
 /**
  * visuData_constrainedInTheBox:
  * @data: a #VisuData object.
@@ -632,10 +540,10 @@ int visuData_constrainedElementInTheBox(VisuData *data, VisuElement *element);
  * The position of each node is the result of the
  * sum of their own position and of the box translation.
  *
- * Returns: 1 if visuData_createAllNodes() should be called and
+ * Returns: TRUE if visuData_createAllNodes() should be called and
  *          the 'NodePositionChanged' should be emitted.
  */
-int visuData_constrainedInTheBox(VisuData *data);
+gboolean visuData_constrainedInTheBox(VisuData *data);
 /**
  * visuData_constrainedFree:
  * @data: a #VisuData object.
@@ -643,10 +551,63 @@ int visuData_constrainedInTheBox(VisuData *data);
  * Return all the nodes to their original position, except for the
  * global translation.
  *
- * Returns: 1 if visuData_createAllNodes() should be called and
+ * Returns: TRUE if visuData_createAllNodes() should be called and
  *          the 'NodePositionChanged' should be emitted.
  */
-int visuData_constrainedFree(VisuData *data);
+gboolean visuData_constrainedFree(VisuData *data);
+/**
+ * visuDataGet_translationStatus:
+ * @data: a #VisuData object.
+ *
+ * When a translation is applied (even with a [0,0,0] vector), the
+ * nodes are shifted to be in the box. This routine returns the
+ * translation status of all nodes. If one of them is translated, then
+ * return value is TRUE.
+ *
+ * Returns: if one of the nodes is shifted.
+ */
+gboolean visuDataGet_translationStatus(VisuData *data);
+/**
+ * visuData_replicate:
+ * @data: a #VisuData object ;
+ * @extension: three floating point values ;
+ * @rebuild: a location to store a flag.
+ *
+ * This routine will create (or remove) nodes to expand the initial box to
+ * the required size. An extension of 0 means no extension, i.e. the initial
+ * box. The extension is done symmetrically in each direction toward negative
+ * and positive direction.
+ *
+ * If the @rebuild argument is TRUE, the nodes should be rebuilt (with
+ * visuData_createAllNodes()).
+ *
+ * To remove added nodes, see visuData_restore().
+ *
+ * returns: TRUE if the redraw should be done.
+ */
+gboolean visuData_replicate(VisuData *data, float extension[3], gboolean *rebuild);
+/**
+ * visuData_restore:
+ * @data: a #VisuData object.
+ *
+ * Remove all nodes that have been added by a visuData_replicate()
+ * call.
+ *
+ * Returns: TRUE if some nodes has been indeed removed.
+ */
+gboolean visuData_restore(VisuData *data);
+/**
+ * visuDataGet_extension:
+ * @dataObj: a #VisuData object ;
+ * @extension: an allocated array to store the values.
+ *
+ * Using visuData_replicate(), it is possible to duplicate the primitive box
+ * in each directions. Use this method to know the current extension. Returned
+ * values are positive floating point values. An extension of 0. means that
+ * only the primitive box exists, while a value of one means a duplication of
+ * one box in each direction of the coordinate.
+ */
+void visuDataGet_extension(VisuData *dataObj, float extension[3]);
 /**
  * visuDataGet_nodePosition:
  * @data: a #VisuData object ;
@@ -663,7 +624,17 @@ void visuDataGet_nodePosition(VisuData *data, VisuNode *node, float coord[3]);
 /***************/
 /* OpenGL Part */
 /***************/
-
+/**
+ * visuData_createNodeInfos:
+ * @data: a #VisuData object.
+ *
+ * This create is list with labels positions on nodes with some informations. These
+ * informations are taken from the #DataNode stored in the property "dataNode". If
+ * this property is 1, then the number of nodes are printed, if the value is 2,
+ * the name of the element is used, instead the value is a pointer to a #DataNode
+ * object and its method is used to get the label.
+ */
+void visuData_createNodeInfos(VisuData *data);
 /**
  * visuData_createNodes:
  * @data: a #VisuData object ;
@@ -911,17 +882,187 @@ void visuDataEmit_facettesChanged(VisuData *data);
  */
 GList* visuDataGet_allObjects();
 
+/**
+ * visuDataRemove_nodes:
+ * @data: a #VisuData object ;
+ * @nodeNumbers: an allocated int array with number identifying nodes.
+ *
+ * Delete some nodes using this method. The nodes are identified by their
+ * number and have not to be of the same element. This routine raises
+ * the "NodePopulationDecrease" signal. The @nodeNumbers argument must
+ * be terminated with a negative value.
+ */
+void visuDataRemove_nodes(VisuData *data, int *nodeNumbers);
 
+/**
+ * VisuDataIter_struct:
+ * @data: a pointer the iterator is associated to ;
+ * @nAllStoredNodes: the total number of stored nodes for the
+ *                   associated #VisuData ;
+ * @nElements: the number of #VisuElement for the associated #VisuData ;
+ * @nStoredNodes: the number of stored nodes for each element ;
+ * @iElement: the index corresponding to @element (or -1 if no set);
+ * @node: a pointer on a current node ;
+ * @element: a pointer on a current element.
+ *
+ * This structure is an iterator over the nodes of a #VisuData object.
+ * Create it with visuDataIter_new(). Then the numbers are allocated and
+ * correspond to the value of the #VisuData object. Use visuDataIter_start()
+ * to initialise the iterator for a run over the nodes, visuDataIter_next()
+ * to associate @node and @element to the next node, or NULL if there is no
+ * more node to run over.
+ */
+typedef struct VisuDataIter_struct
+{
+  VisuData *data;
 
+  unsigned int nAllStoredNodes;
+  unsigned int nElements;
+  unsigned int *nStoredNodes;
 
-/* Return the maxium size occupied by a VisuElement that has
-   at least one VisuNode. */
-/* float getMaxExtens(); */
-/* Store a new value in maxDrawingSizeOfVisuElements that controls
-   the maximum size occupied by a VisuElement. This function emits
-   a signal () that is tarpped by opengl.c. Value must be positive or
-   nul. */
-/* void setMaxExtends(float value); */
+  unsigned int iElement;
+  VisuNode *node;
+  VisuElement *element;
+} VisuDataIter;
+/**
+ * visuDataIter_new:
+ * @data: a #VisuData object ;
+ * @iter: an alocated iterator.
+ *
+ * Set values to a #VisuDataIter object to iterate over nodes.
+ * Its contain is initialised with the data size (number of elements,
+ * number of nodes per element...).
+ */
+void visuDataIter_new(VisuData *data, VisuDataIter *iter);
+/**
+ * visuDataIter_start:
+ * @data: a #VisuData object ;
+ * @iter: a #VisuDataIter object.
+ *
+ * Initialise the node and element internal pointers for a run over the nodes.
+ */
+void visuDataIter_start(VisuData *data, VisuDataIter *iter);
+/**
+ * visuDataIter_startVisible:
+ * @data: a #VisuData object ;
+ * @iter: a #VisuDataIter object.
+ *
+ * Initialise the node and element internal pointers for a run over the 
+ * visible nodes (see visuDataIter_nextVisible).
+ */
+void visuDataIter_startVisible(VisuData *data, VisuDataIter *iter);
+/**
+ * visuDataIter_startNumber:
+ * @data: a #VisuData object ;
+ * @iter: a #VisuDataIter object.
+ *
+ * Initialise the node and element internal pointers for a run
+ * following the node oder.
+ */
+void visuDataIter_startNumber(VisuData *data, VisuDataIter *iter);
+/**
+ * visuDataIter_restartNode:
+ * @data: a #VisuData object ;
+ * @iter: a #VisuDataIter object.
+ *
+ * The element internal pointer must be associated. Then, it returns the
+ * node pointer to the first node for this element.
+ */
+void visuDataIter_restartNode(VisuData *data, VisuDataIter *iter);
+/**
+ * visuDataIter_next:
+ * @data: a #VisuData object ;
+ * @iter: a #VisuDataIter object.
+ *
+ * Modify node and element internal pointers to the next node, or NULL if
+ * none remains.
+ */
+void visuDataIter_next(VisuData *data, VisuDataIter *iter);
+/**
+ * visuDataIter_nextNode:
+ * @data: a #VisuData object ;
+ * @iter: a #VisuDataIter object.
+ *
+ * Modify node internal pointer to the next node, or NULL if
+ * none remains. Contrary to visuDataIter_next() it does not go to the
+ * next element if one exists.
+ */
+void visuDataIter_nextNode(VisuData *data, VisuDataIter *iter);
+/**
+ * visuDataIter_nextNodeNumber:
+ * @data: a #VisuData object ;
+ * @iter: a #VisuDataIter object.
+ *
+ * Modify node internal pointer to the next node, increasing the id of
+ * the current node. The element internal pointer is also updated
+ * accordingly. If no more nodes exist after the given one, node and
+ * element internal pointers are set to NULL.
+ */
+void visuDataIter_nextNodeNumber(VisuData *data, VisuDataIter *iter);
+/**
+ * visuDataIter_nextElement:
+ * @data: a #VisuData object ;
+ * @iter: a #VisuDataIter object.
+ *
+ * Modify element internal pointer to the next element and set node
+ * to the first one, or NULL if none remains.
+ */
+void visuDataIter_nextElement(VisuData *data, VisuDataIter *iter);
+/**
+ * visuDataIter_nextVisible:
+ * @data: a #VisuData object ;
+ * @iter: a #VisuDataIter object.
+ *
+ * Go to the next rendered node (changing element if required).
+ */
+void visuDataIter_nextVisible(VisuData *data, VisuDataIter *iter);
+
+#ifndef V_SIM_DISABLE_DEPRECATED
+/**
+ * visuDataSet_property:
+ * @data: a #VisuData object ;
+ * @key: a string ;
+ * @value: an allocated area to store data.
+ *
+ * This method is used to store some values associated with
+ * the given @data. These values are anything but will not be freed
+ * when the property is changed or removed. These values
+ * are described by the @key, and can be retrieved with the
+ * visuDataGet_property() method.
+ *
+ * Deprecated: use directly g_object_set_data() instead.
+ */
+#define visuDataSet_property(data, key, value) \
+  g_object_set_data(G_OBJECT(data), key, value)
+/**
+ * visuDataSet_propertyWithDestroyFunc:
+ * @data: a #VisuData object ;
+ * @key: a string ;
+ * @value: an allocated area to store data ;
+ * @freeFunc: a method to free each token.
+ *
+ * Same method as visuDataSet_property() but giving a free method when
+ * the data is removed (like g_free()).
+ *
+ * Deprecated: use directly g_object_set_data_full() instead.
+ */
+#define visuDataSet_propertyWithDestroyFunc(data, key, value, freeFunc) \
+  g_object_set_data_full(G_OBJECT(data), key, value, freeFunc)
+/**
+ * visuDataGet_property:
+ * @data: a #VisuData object ;
+ * @key: a string.
+ *
+ * This method is used to retrieve some data associated to
+ * the specified @data. These return data should not be freed after used.
+ *
+ * Returns: some data associated to the key.
+ *
+ * Deprecated: use directly g_object_get_data() instead.
+ */
+#define visuDataGet_property(data, key) \
+  g_object_get_data(G_OBJECT(data), key)
+#endif
 
 G_END_DECLS
 
