@@ -47,6 +47,8 @@
 #include "visu_data.h"
 #include "coreTools/toolColor.h"
 
+G_BEGIN_DECLS
+
 /**
  * PAIRS_MIN
  *
@@ -86,30 +88,68 @@
 
 /**
  * pairsData_struct:
- * @name1: the name of the first #VisuElement ;
- * @name2: the name of the second #VisuElement ;
  * @minMax: storage for the length bounds for drawn pairs ;
- * @properties: a storage for userData ;
- * @drawn: a boolean to say if the pair is drawn or not.
+ * @drawn: a boolean to say if the pair is drawn or not ;
+ * @printLength: a boolean to say if length of pairs are drawn near
+ * them ;
+ * @properties: some key:value association for this link.
  *
- * This structure is used to describe a pair between two elements.
- * These elements are described by their name. A pair is drawn only
- * its length is between the minimum and the maximum value stored in
- * the minMax array.
+ * This structure is used to describe a link between two elements. A
+ * link is drawn only its length is between the minimum and the
+ * maximum value stored in the minMax array.
  */
 struct pairsData_struct
 {
-  char *name1, *name2;
   float minMax[2];
-  GHashTable *properties;
   gboolean drawn;
+  gboolean printLength;
+
+  GHashTable *properties;
 };
 /**
- * PairsData:
+ * VisuPairData:
  * 
  * A common way to name #pairsData_struct objects.
  */
-typedef struct pairsData_struct PairsData;
+typedef struct pairsData_struct VisuPairData;
+
+/**
+ * VisuPair_struct:
+ * 
+ * An opaque structure to store informations about links between two elements.
+ */
+struct VisuPair_struct;
+/**
+ * VisuPair:
+ * 
+ * A common way to name #pairsData_struct objects.
+ */
+typedef struct VisuPair_struct VisuPair;
+
+/**
+ * VisuPairDistribution_struct:
+ * @histo: an array containing the distribution ;
+ * @nValues: the size of the array ;
+ * @stepValue: the step increase in distance at each value ;
+ * @nNodesEle1: the number of nodes used during the computation ;
+ * @nNodesEle2: idem for #VisuElement 2.
+ *
+ * This structure stores for a given pair, the distance distribution
+ * on a given range [0;@nValues * @stepValue[.
+ */
+struct VisuPairDistribution_struct
+{
+  int *histo;
+  int nValues;
+  float stepValue;
+  int nNodesEle1, nNodesEle2;
+};
+/**
+ * VisuPairDistribution:
+ *
+ * A convenient name for #VisuPairDistribution_struct objects.
+ */
+typedef struct VisuPairDistribution_struct VisuPairDistribution;
 
 
 /**
@@ -125,7 +165,7 @@ typedef void (*initEndOpenGlPairsFunc)();
  * startEndPairsFunc:
  * @ele1: a #VisuElement object ;
  * @ele2: a #VisuElement object ;
- * @data: a #PairsData object.
+ * @data: a #VisuPairData object.
  *
  * Prototype of functions called at the beginning and
  * the end of drawing of each pairs types. @ele1 and @ele2
@@ -133,12 +173,12 @@ typedef void (*initEndOpenGlPairsFunc)();
  * This is useful to set some OpenGL definition specific to each pair, such
  * as the color for example.
  */
-typedef void (*startEndPairsFunc)(VisuElement *ele1, VisuElement *ele2, PairsData *data);
+typedef void (*startEndPairsFunc)(VisuElement *ele1, VisuElement *ele2, VisuPairData *data);
 /**
  * pairDefinitionFunc:
  * @ele1: a #VisuElement object ;
  * @ele2: a #VisuElement object ;
- * @data: a #PairsData object ;
+ * @data: a #VisuPairData object ;
  * @view: a #OpenGLView object, giving some constants describing
  *        the OpenGL scene ;
  * @x1: a floating point value ;
@@ -147,24 +187,31 @@ typedef void (*startEndPairsFunc)(VisuElement *ele1, VisuElement *ele2, PairsDat
  * @x2: a floating point value ;
  * @y2: a floating point value ;
  * @z2: a floating point value ;
- * @d2: a floating point value.
+ * @d2: a floating point value ;
+ * @alpha: a floating point value.
  *
  * Prototype of function to draw a pair. Such function are called each time a pair
  * is drawn between the two points (@x1, @y1, @z1) and (@x2, @y2, @z2). The @d2 argument
- * is the square distance between the two points.
+ * is the square distance between the two points. The @alpha argument
+ * is a proposed alpha colour from the main program, its value is in [0;1].
  */
-typedef void (*pairDefinitionFunc)(VisuElement *ele1, VisuElement *ele2, PairsData *data,
-				   OpenGLView *view, double x1, double y1, double z1,
-				   double x2, double y2, double z2, float d2);
+typedef void (*pairDefinitionFunc)(VisuElement *ele1, VisuElement *ele2,
+				   VisuPairData *data, OpenGLView *view,
+				   double x1, double y1, double z1,
+				   double x2, double y2, double z2,
+				   float d2, float alpha);
 /**
  * foreachPairsFunc:
- * @data: a #PairsData object ;
+ * @ele1: a #VisuElement object ;
+ * @ele2: a #VisuElement object ;
+ * @data: a #VisuPairData object ;
  * @userData: some user defined data.
  *
  * Prototype of functions called with the foreach method apply to each
  * pairs.
  */
-typedef void (*foreachPairsFunc)(PairsData *data, gpointer userData);
+typedef void (*foreachPairsFunc)(VisuElement *ele1, VisuElement *ele2,
+				 VisuPairData *data, gpointer userData);
 
 /**
  * PairsExtension_struct:
@@ -181,7 +228,7 @@ struct PairsExtension_struct;
 typedef struct PairsExtension_struct PairsExtension;
 
 /**
- * pairsExtension_new:
+ * visuPairExtensionNew:
  * @name: name of the extension (must be non null) ;
  * @printName: a string to label the method that can be safely translated ;
  * @description: a brief description of the extension (can be null) ;
@@ -198,38 +245,59 @@ typedef struct PairsExtension_struct PairsExtension;
  *
  * Returns: the new PairsExtension or null if something wrong happens.
  */
-PairsExtension* pairsExtension_new(char* name, char* printName, char* description, int sensitive,
-				   initEndOpenGlPairsFunc init, initEndOpenGlPairsFunc stop,
-				   startEndPairsFunc start, startEndPairsFunc end,
-				   pairDefinitionFunc draw);
+PairsExtension* visuPairExtensionNew(const char* name, const char* printName,
+				     const char* description, gboolean sensitive,
+				     initEndOpenGlPairsFunc init,
+				     initEndOpenGlPairsFunc stop,
+				     startEndPairsFunc start,
+				     startEndPairsFunc end,
+				     pairDefinitionFunc draw);
 /**
- * PairsExtension_free:
+ * visuPairExtensionFree:
  * @extension: the extension to delete.
  *
  * Free all the allocated attributes of the specified method.
  */
-void PairsExtension_free(PairsExtension* extension);
+void visuPairExtensionFree(PairsExtension* extension);
 /**
- * registerPairsExtension:
+ * visuPairExtensionAdd:
  * @extension: an extension.
  *
  * A method used by user to registered a new extension.
  */
-void registerPairsExtension(PairsExtension *extension);
-
+void visuPairExtensionAdd(PairsExtension *extension);
 /**
- * isValidPairsExtension:
- * @pairsModel: a #PairsExtension object.
+ * visuPairExtensionSet:
+ * @extension: a #PairsExtension object.
  *
- * Test if the given #PairsExtension has been already registered
- * by V_Sim.
+ * Choose the method used to draw pairs. If necessary, visuPairBuild() are called.
  *
- * Returns: 1 if the given extension is already stored.
+ * Returns: TRUE if the OpenGLAskForReDraw signal show be emitted or not.
  */
-int isValidPairsExtension(PairsExtension* pairsModel);
+gboolean visuPairExtensionSet(PairsExtension *extension);
+/**
+ * visuPairExtensionGet_allMethods:
+ * 
+ * Useful to know all #PairsExtension.
+ *
+ * Returns: a list of all the known #PairsExtension. This list should
+ *          be considered read-only.
+ */
+GList* visuPairExtensionGet_allMethods();
+/**
+ * visuPairExtensionGet_current:
+ *
+ * If some process need to know the current #PairsExtension. Such extension has
+ * been set with setPairsMethod().
+ *
+ * Returns: the current #PairsExtension, NULL if none has been set.
+ */
+PairsExtension* visuPairExtensionGet_current();
+
+
 
 /**
- * createPairs:
+ * visuPairBuild:
  * @dataObj: the #VisuData object to create pairs for.
  *
  * This methods recreates the OpenGL list of the #OpenGLExtension associated
@@ -241,99 +309,88 @@ int isValidPairsExtension(PairsExtension* pairsModel);
  * the pairDefinitionFunc() for these two nodes. WARNING! this method is very expensive
  * in computing cost.
  *
- * Returns: 1 if the OpenGLAskForReDraw signal show be emitted or not.
+ * Returns: TRUE if the OpenGLAskForReDraw signal show be emitted or not.
  */
-int createPairs(VisuData *dataObj);
+gboolean visuPairBuild(VisuData *dataObj);
 
 /**
- * setPairsOnOff:
+ * visuPairSet_status:
  * @onOff : a boolean 0 or 1.
  * 
  * Turn on or off the pairs.
  *
- * Returns: 1 if createPairs() should be called and then the 'OpenGLAskForReDraw'
+ * Returns: TRUE if visuPairBuild() should be called and then the 'OpenGLAskForReDraw'
  *          signal be emitted.
  */
-int setPairsOnOff(int onOff);
+gboolean visuPairSet_status(gboolean onOff);
 /**
- * setPairsMethod:
- * @extension: a #PairsExtension object.
- *
- * Choose the method used to draw pairs. If necessary, createPairs() are called.
- *
- * Returns: 1 if the OpenGLAskForReDraw signal show be emitted or not.
- */
-int setPairsMethod(PairsExtension *extension);
-/**
- * setPairsAll:
- * @data: a #PairsData object ;
- * @rgb: an array of three floating point values ;
- * @minMax: an array of two floating point values.
- *
- * This method sets length and color informations for the given pairs.
- *
- * Returns: 1 if createPairs() should be called and then the 'OpenGLAskForReDraw'
- *          signal be emitted.
- */
-int setPairsAll(PairsData *data, float rgb[3], float minMax[2]);
-/**
- * setPairsDrawn:
+ * visuPairSet_drawn:
  * @data: a #PairsData object ;
  * @drawn: a boolean.
  *
  * A pair can or cannot be drawn, use this method to tune it.
  *
- * Returns: 1 if createPairs() should be called.
+ * Returns: TRUE if visuPairBuild() should be called.
  */
-int setPairsDrawn(PairsData *data, gboolean drawn);
+gboolean visuPairSet_drawn(VisuPairData *data, gboolean drawn);
 /**
- * getPairsDrawn:
+ * visuPairGet_drawn:
  * @data: a #PairsData object ;
  *
  * A pair can or cannot be drawn, use this method to retrieve its state.
  *
  * Returns: TRUE if pairs can be drawn.
  */
-gboolean getPairsDrawn(PairsData *data);
+gboolean visuPairGet_drawn(VisuPairData *data);
 /**
- * setPairsDistance:
+ * visuPairSet_distance:
  * @val: a floating point value ;
  * @data: a #PairsData object ;
  * @minOrMax: #PAIRS_MAX or #PAIRS_MIN.
  *
  * Set the minimum or the maximum length for the given pair.
  *
- * Returns: 1 if createPairs() should be called or not.
+ * Returns: TRUE if visuPairBuild() should be called or not.
  */
-int setPairsDistance(float val, PairsData *data, int minOrMax);
+gboolean visuPairSet_distance(VisuPairData *data, float val, int minOrMax);
 /**
- * setPairsColor:
+ * visuPairSet_color:
  * @data: a #PairsData object ;
- * @color: a #Color object..
+ * @destColor: a #Color object.
  *
  * Set the color of the given pair.
  *
- * Returns: 1 if createPairs() should be called or not.
+ * Returns: TRUE if visuPairBuild() should be called or not.
  */
-int setPairsColor(PairsData *data, Color *color);
+gboolean visuPairSet_color(VisuPairData *data, Color* destColor);
 /**
- * setPairsAreOutOfDate:
+ * visuPairSet_printLength:
+ * @data: a #PairsData object ;
+ * @status: TRUE to print length near pairs.
+ *
+ * Set the attribute that controls if the length of pairs are drawn near pairs.
+ *
+ * Returns: TRUE if visuPairBuild() should be called or not.
+ */
+gboolean visuPairSet_printLength(VisuPairData *data, gboolean status);
+/**
+ * visuPairSet_outOfDate:
  *
  * Use this method to change the internal flag that pairs should be
  * rebuilt. It is useful when an extension of pairs has one of its
  * parameters that has changed.
  */
-void setPairsAreOutOfDate();
+void visuPairSet_outOfDate();
 /**
- * getPairsOnOff:
+ * visuPairGet_status:
  * 
  * Get the status of pairs, drawn or not.
  *
- * Returns: 1 if pairs are drawn.
+ * Returns: TRUE if pairs are drawn.
  */
-int getPairsOnOff();
+gboolean visuPairGet_status();
 /**
- * setPairsProperty:
+ * visuPairSet_linkProperty:
  * @data: a #PairsData object ;
  * @key: a static string ;
  * @value: a pointer to some allocated data.
@@ -341,39 +398,31 @@ int getPairsOnOff();
  * Each pair can store some informations that can be retrieve by a string key.
  * This method is used to registered a new value associated to the key @key.
  */
-void setPairsProperty(PairsData *data, const gchar* key, gpointer value);
+void visuPairSet_linkProperty(VisuPairData *data, const gchar* key, gpointer value);
 /**
- * getAllPairsMethods:
- * 
- * Useful to know all #PairsExtension.
+ * visuPairSet_property:
+ * @pair: a #Pairs object ;
+ * @key: a static string ;
+ * @value: a pointer to some allocated data ;
+ * @freeFunc: a destroying method (can be NULL).
  *
- * Returns: a list of all the known #PairsExtension. This list should
- *          be considered read-only.
+ * Each element/element can have associated data.
  */
-GList* getAllPairsMethods();
+void visuPairSet_property(VisuPair *pair, const gchar* key,
+			  gpointer value, GDestroyNotify freeFunc);
 /**
- * getCurrentPairsMethods:
+ * visuPairGet_property:
+ * @pair: a #VisuPair object ;
+ * @key: a string.
  *
- * If some process need to know the current #PairsExtension. Such extension has
- * been set with setPairsMethod().
+ * Retrieve the property associated to the @key or NULL if none exist.
  *
- * Returns: the current #PairsExtension, NULL if none has been set.
+ * Returns: the associated data.
  */
-PairsExtension* getCurrentPairsMethods();
+gpointer visuPairGet_property(VisuPair *pair, const gchar* key);
 /**
- * getNameOfPairsMethod:
- * @extension: a #PairsExtension object.
- *
- * All #PairsExtension has a name.
- *
- * Returns: a string that is associated to the given #PairsExtension. This string
- *          should not be freed.
- */
-char* getNameOfPairsMethod(PairsExtension *extension);
-/**
- * getPairsDistance:
- * @ele1: a #VisuElement object ;
- * @ele2: a #VisuElement object ;
+ * visuPairGet_distance:
+ * @data: a #PairsData object ;
  * @minOrMax: #PAIRS_MIN or #PAIRS_MAX.
  *
  * A pair between @ele1 and @ele2 is drawn only if its length is between
@@ -381,9 +430,19 @@ char* getNameOfPairsMethod(PairsExtension *extension);
  *
  * Returns: the minimum or the maximum value for the pair between @ele1 and @ele2.
  */
-float getPairsDistance(VisuElement *ele1, VisuElement *ele2, int minOrMax);
+float visuPairGet_distance(VisuPairData *data, int minOrMax);
 /**
- * getPairsProperty:
+ * visuPairGet_printLength:
+ * @data: a #PairsData object.
+ * 
+ * Get the print length parameter of a pair. This parameter is used to tell if
+ * length should be drawn near pairs of this kind.
+ *
+ * Returns: TRUE if length are printed.
+ */
+gboolean visuPairGet_printLength(VisuPairData *data);
+/**
+ * visuPairGet_linkProperty:
  * @data: a #PairsData object ;
  * @key: a static string.
  *
@@ -393,28 +452,113 @@ float getPairsDistance(VisuElement *ele1, VisuElement *ele2, int minOrMax);
  * Returns: a found value, or NULL if nothing is associated to the @key.
  *          If something is returned it should not be freed.
  */
-gpointer getPairsProperty(PairsData *data, const gchar* key);
+gpointer visuPairGet_linkProperty(VisuPairData *data, const gchar* key);
 /* float getPairsColor(VisuElement *ele1, VisuElement *ele2, int color); */
 /**
- * getPairsData:
+ * visuPairGet_pair:
  * @ele1: a #VisuElement object ;
  * @ele2: a #VisuElement object.
  *
- * The object #PairsData is used to characterized the link between two elements.
+ * The object #VisuPair is used to characterized links between two elements.
  *
- * Returns: the #PairsData object associated to the given two elements. If
+ * Returns: the #VisuPair object associated to the given two elements. If
  *          none exists it is created. The returned value should not be freed.
  */
-PairsData* getPairsData(VisuElement *ele1, VisuElement *ele2);
+VisuPair* visuPairGet_pair(VisuElement *ele1, VisuElement *ele2);
 /**
- * foreachPairsData:
+ * visuPairGet_distanceDistribution:
+ * @pair: a #VisuPair ;
+ * @dataObj: a #VisuData ;
+ * @step: a float for the distance mesh (negative value to use
+ * built-in default) ;
+ * @max: a float for the maximum scanning value (negative value to use
+ * built-in default).
+ * 
+ * This will compute the distnace distribution of nodes for the given
+ * @pair.
+ *
+ * Returns: a structure defining the distance distribution. This
+ * structure is private and should not be freed.
+ */
+VisuPairDistribution* visuPairGet_distanceDistribution(VisuPair *pair,
+						       VisuData *dataObj,
+						       float step, float max);
+
+/**
+ * visuPairGet_links:
+ * @ele1: a #VisuElement object ;
+ * @ele2: a #VisuElement object.
+ *
+ * There can be one or several links between elements, retrieve them
+ * with this routine.
+ *
+ * Returns: a list of #VisuPairData. The list is owned by V_Sim and
+ * should not be freed.
+ */
+GList* visuPairGet_links(VisuElement *ele1, VisuElement *ele2);
+/**
+ * visuPairGet_link:
+ * @ele1: a #VisuElement object ;
+ * @ele2: a #VisuElement object ;
+ * @minMax: the two min and max distances.
+ *
+ * A link between two elements is characterized by its boundary distances.
+ *
+ * Returns: the #VisuPairData object associated to the given two
+ *          elements and distances. If none exists it is created. The
+ *          returned value should not be freed.
+ */
+VisuPairData* visuPairGet_link(VisuElement *ele1, VisuElement *ele2, float minMax[2]);
+/**
+ * visuPairGet_linkFromId:
+ * @ele1: a #VisuElement object ;
+ * @ele2: a #VisuElement object ;
+ * @pos: the position in the list of links.
+ *
+ * A link can also be retrieved by its position.
+ *
+ * Returns: the #VisuPairData object associated to the given two
+ *          elements and distances. If none exists NULL is returned.
+ */
+VisuPairData* visuPairGet_linkFromId(VisuElement *ele1, VisuElement *ele2, guint pos);
+/**
+ * visuPairRemove_link:
+ * @ele1: a #VisuElement object ;
+ * @ele2: a #VisuElement object ;
+ * @data: a link object.
+ *
+ * Delete the given link.
+ *
+ * Returns: TRUE if the link exists and has been successfully removed.
+ */
+gboolean visuPairRemove_link(VisuElement *ele1, VisuElement *ele2, VisuPairData *data);
+/**
+ * visuPairForeach:
  * @whatToDo: a foreachPairsFunc() method ;
  * @userData: some user defined data.
  *
  * The way #PairsData are stored in V_Sim is private and could changed between version.
  * This method is used to apply some method each pairs.
  */
-void foreachPairsData(foreachPairsFunc whatToDo, gpointer userData);
+void visuPairForeach(foreachPairsFunc whatToDo, gpointer userData);
+/**
+ * visuPairRead_linkFromTokens:
+ * @tokens: array of tokens resulting from a call to g_strsplit() with " " as separator ;
+ * @index: IN, the position of the beginning in @tokens ; OUT, one token
+ *            after the last read ;
+ * @data: a pointer to return an allocated link object ;
+ * @position: the number of the line of the config
+ *            file which the @line argument is taken from ;
+ * @error: a location to store a possible reading error.
+ *
+ * This routine is used to read the resource file. Given @tokens, it
+ * associate a link object by reading the two elements and the two
+ * distances that characterised this link.
+ *
+ * Returns: TRUE if succeed.
+ */
+gboolean visuPairRead_linkFromTokens(gchar **tokens, int *index, VisuPairData **data,
+				     int position, GError **error);
 
 
 /****************/
@@ -446,7 +590,7 @@ struct PairsExtension_struct
   char* printName;
   char* description;
 
-  int sensitiveToFacette;
+  gboolean sensitiveToFacette;
 
   initEndOpenGlPairsFunc initOpenGl;
   initEndOpenGlPairsFunc stopOpenGl;
@@ -471,5 +615,6 @@ struct PairsExtension_struct
  */
 int initPairsModule();
 
+G_END_DECLS
 
 #endif
