@@ -50,43 +50,30 @@
 #include <unistd.h> /* For the access markers R_OK, W_OK ... */
 #include <math.h>
 
+/**
+ * SECTION:visu_tools
+ * @short_description: Basic tools and variables that are used all
+ * around the program.
+ *
+ * <para>This module gives tools to handle strings (trim, append,
+ * allocate...), GList and GHashTable (compare, delete functions), the
+ * file system (test for existing file, for read/write permissions),
+ * mathematical functions (floating point modulo, matrix product,
+ * coordinates handling)...</para>
+ */
 
-int fileSystemIsInUTF8;
+void freeProperty(gpointer prop)
+{
+  g_return_if_fail(prop);
 
-void setFileSystemInUTF8(int val)
-{
-  DBG_fprintf(stderr, "Visu Tools : the file system is in UTF8 : %d.\n", val);
-  fileSystemIsInUTF8 = val;
-}
-int getFileSystemInUTF8()
-{
-  return fileSystemIsInUTF8;
+  DBG_fprintf(stderr, "Visu tools: free the data property '%s' (%p).\n",
+	      ((Property*)prop)->name, ((Property*)prop)->data);
+  if (((Property*)prop)->freeFunc)
+    ((Property*)prop)->freeFunc(((Property*)prop)->data);
+  g_free(prop);
 }
 
 
-/* Used to free the keys or the values of a GHashTable
-   where gpointer are char*. */
-void freeString(gpointer ele)
-{
-  free((char*)ele);
-}
-/* Used to free the keys or the values of a GHashTable
-   where gpointer are float*. */
-void freeFloat(gpointer ele)
-{
-  free((float*)ele);
-}
-/* Used to free the keys or the values of a GHashTable
-   where gpointer are int*. */
-void freeInt(gpointer ele)
-{
-  free((int*)ele);
-}
-void freeGPointer(gpointer ele)
-{
-  free(ele);
-}
-
 gchar* getValidPath(GList **pathList, char *fileName, int accessMode)
 {
   gchar *validPath;
@@ -125,75 +112,6 @@ gchar* getValidPath(GList **pathList, char *fileName, int accessMode)
 }
 
 
-void allocationProblems()
-{
-  fprintf(stderr, "INTERNAL ERROR! Can't allocate the requested object.\n");
-}
-
-gchar* getStringInUTF8(const gchar* str)
-{
-  gchar* strTo;
-  const gchar *end;
-  gsize ecrit, lu;
-
-  if (fileSystemIsInUTF8)
-    {
-      /* We validate the UTF8 file name before using it. */
-      if (g_utf8_validate(str, -1, &end))
-	strTo = g_strdup(str);
-      else
-	{
-	  g_warning("Encoding error in name (%s), invalid UTF8.\n", str);
-	  strTo = g_strndup(str, (int)(end - str));
-	}
-      return strTo;
-    }
-  
-  DBG_fprintf(stderr, "Visu Tools : A conversion to UTF8 is required.\n");
-  strTo = g_filename_to_utf8(str, -1, &lu, &ecrit, NULL);
-  g_return_val_if_fail(strTo, (gchar*)0);
-
-  return strTo;
-}
-
-/* It puts in dest all the characters of value without ' ' or '\t'
-   at the begining at at the end. */
-int getTrimString(char **dest, const char* value)
-{
-  int debut, fin;
-
-  if (!value)
-    return 1;
-  *dest = realloc(*dest, sizeof(char) * (strlen(value) + 1));
-  if (!*dest)
-    {
-      allocationProblems();
-      exit(1);
-    }
-  debut = 0;
-  while (value[debut] == ' ' || value[debut] == '\t')
-    debut++;
-  /* retire les blancs  la fin */
-  fin = strlen(value) - 1;
-  while (fin >= 0 && (value[fin] == ' ' || value[fin] == '\t' || value[fin] == '\n'))
-    fin--;
-  strncpy(*dest, value + debut, fin - debut + 1);
-  (*dest)[fin - debut + 1] = '\0';
-  return 0;
-}
-
-/* Reallocates the pointer dest and strcpy value within. */
-void putString(char **dest, const char* value)
-{
-  *dest = realloc(*dest, sizeof(char) * (strlen(value) + 1));
-  if (!*dest)
-    {
-      allocationProblems();
-      exit(1);
-    }
-  strcpy(*dest, value);
-}
-
 float fModulo(float a, int b) {
   float fb = (float)b;
   while(a < fb) a += fb;
@@ -201,7 +119,7 @@ float fModulo(float a, int b) {
   return a;
 }
 
-gchar* normalize_path(gchar* path)
+gchar* normalize_path(const gchar* path)
 {
 #if SYSTEM_X11 == 1
 #define FILE_SYSTEM_SEP "/"
@@ -231,8 +149,8 @@ gchar* normalize_path(gchar* path)
   for (i = 0; tokens[i]; i++)
     {
       /* If tokens[i] == . or is empty (because of //), we ignore. */
-      if (!strcmp(tokens[i], "."))
-	continue;
+	if (!strcmp(tokens[i], "."))
+	  continue;
       if (!tokens[i][0])
 	continue;
       /* If token[i] == .. then we pop one element from lst. */
@@ -264,3 +182,20 @@ gchar* normalize_path(gchar* path)
 	      path, normPath->str);
   return g_string_free(normPath, FALSE);
 }
+
+#if GLIB_MINOR_VERSION < 5
+gboolean g_file_set_contents(const gchar *fileName, const gchar *str,
+			     gsize len _U_, GError **error _U_)
+{
+  FILE *f;
+
+  f = fopen(fileName, "w");
+  if (!f)
+    return FALSE;
+  if (fwrite(str, strlen(str), 1, f) != strlen(str))
+    return FALSE;
+  fclose(f);
+
+  return TRUE;
+}
+#endif
